/*

  File: InventorWriter.cpp

  (c) 2003 by Oliver Ehli

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

     See File LICENSE for more Information.

*/
#include <iostream.h>
#include <fstream.h>
#include "InventorWriter.h"

#ifdef DEBUG_BIN

void InventorWriter::writeRGBColor(ofstream & out, float r, float g, float b, float a)
{
  out << "Material { ambientColor " << r << " " << g << " " << b << 
    " diffuseColor " << r << " " << g << " " << b << " transparency " << a << " }\n";


  //  out << "BaseColor { rgb " << r << " " << g << " " << b << " }\n";
}



void InventorWriter::writeColor(ofstream & out, int whatColor)
{
    InventorWriter::writeRGBColor(out,
				  colors[whatColor][0],
				  colors[whatColor][1],
				  colors[whatColor][2], whatColor == 12 ? 0.8 : 0.5);
}



void InventorWriter::writeTranslation(ofstream & out, float x, float y, float z)
{
  out << "Translation { translation " << x << " " << y << " " << z << " }\n";

}



void InventorWriter::writeCube(ofstream & out, float xSize, float ySize, float zSize)
{
  out << "Cube { }\n";
}


void InventorWriter::writeBBox(ofstream & out, int indentLevel, int bbox[3])
{
  for (int l=0; l<indentLevel; l++)
    out << indent;
  out << "Coordinate3 {" << endl;

  for (int l=0; l<indentLevel+1; l++)
    out << indent;
  out << "point [" << endl;

  float origin[] = { -1.05, -1.05, -1.05 };
  float currentPoint[] = { -1.05, -1.05, -1.05 };

//   cout << bbox[0] << ", " << bbox[1] << ", " << bbox[2] << endl;

  for (int i=0; i < 2; i++) {
    for (int j=0; j < 3; j++) {
      if(j) currentPoint[j-1] = origin[j-1] + bbox[j-1]*baseUnit;

      if(i || j) out << "," << endl;

      for (int l=0; l<indentLevel+2; l++)
	out << indent;
      out << currentPoint[0] << " " << currentPoint[1] << " "
	  << currentPoint[2];
    }
    currentPoint[0] = -1.05;

    out << "," << endl;
    for (int l=0; l<indentLevel+2; l++)
      out << indent;
    for (int l=0; l<indentLevel+2; l++)
      out << indent;
    out << currentPoint[0] << " " << currentPoint[1] << " "
	<< currentPoint[2];

    currentPoint[1] = -1.05;
    currentPoint[2] = origin[2] + bbox[2]*baseUnit;
  }
  out << endl;

  for (int l=0; l<indentLevel+1; l++)
    out << indent;
  out << "]" << endl;

  for (int l=0; l<indentLevel; l++)
    out << indent;
  out << "}" << endl;



  for (int l=0; l<indentLevel; l++)
    out << indent;
  out << "IndexedLineSet {" << endl;

  for (int l=0; l<indentLevel+1; l++)
    out << indent;
  out << "coordIndex [" << endl;

  for (int l=0; l<indentLevel+1; l++)
    out << indent;
  out << "0, 1, -1," << endl;

  for (int l=0; l<indentLevel+1; l++)
    out << indent;
  out << "1, 2, -1," << endl;

  for (int l=0; l<indentLevel+1; l++)
    out << indent;
  out << "2, 3, -1," << endl;

  for (int l=0; l<indentLevel+1; l++)
    out << indent;
  out << "3, 0, -1," << endl;




  for (int l=0; l<indentLevel+1; l++)
    out << indent;
  out << "4, 5, -1," << endl;

  for (int l=0; l<indentLevel+1; l++)
    out << indent;
  out << "5, 6, -1," << endl;

  for (int l=0; l<indentLevel+1; l++)
    out << indent;
  out << "6, 7, -1," << endl;

  for (int l=0; l<indentLevel+1; l++)
    out << indent;
  out << "7, 4, -1," << endl;



  for (int l=0; l<indentLevel+1; l++)
    out << indent;
  out << "0, 4, -1," << endl;

  for (int l=0; l<indentLevel+1; l++)
    out << indent;
  out << "1, 5, -1," << endl;

  for (int l=0; l<indentLevel+1; l++)
    out << indent;
  out << "2, 6, -1," << endl;

  for (int l=0; l<indentLevel+1; l++)
    out << indent;
  out << "3, 7, -1" << endl;



  for (int l=0; l<indentLevel+1; l++)
    out << indent;
  out << "]" << endl;

  for (int l=0; l<indentLevel; l++)
    out << indent;
  out << "}" << endl;

}




char *InventorWriter::header = "#Inventor V2.1 ascii\n\nSeparator {\n";
char *InventorWriter::footer = "}";
char *InventorWriter::beginGroup = "Separator {\n";
char *InventorWriter::endGroup = "}\n";
char *InventorWriter::indent = "\t";

float InventorWriter::baseUnit = 2.1;


float InventorWriter::colors[][3] =
  {
    {1, 0, 0},
    {.7, .3, 0},
    {.5, 1, .7},
    {0, 1, 0},
    {0, .5, .2},
    {0, .8, .8},
    {0, 0, 1},
    {.5, 0, .8},
    {.7, 0, .5},
    {.7, .7, .7},
    {.8, .5, .8},
    {.8, .8, 0},
    {1, 1, 1}
  };

#endif
