/*

  File: Piece.h

  (c) 2003 by Oliver Ehli

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

     See File LICENSE for more Information.

*/
#ifndef PIECE_H_
#define PIECE_H_

#include <iostream.h>
#include <fstream.h>
#include "Puzzle.h"


//using namespace std;

class Piece  {


 public:

  Piece(int inId, int (*dimensions)[3], bool (*description)[PC_Z_DIM][PC_Y_DIM][PC_X_DIM],
	bool skipSymmetry = false, bool specialSymmetry = false);


  inline long long getPosition(int coords, int index) {
    return occupiedCoords[coords][index];
  }

  inline long long *getPositions(int coords) {
    return occupiedCoords[coords];
  }

  inline int getPositionIndex(int coords, int index) {
    return occupiedCoordsPositionIndex[coords][index];
  }

  inline int getNumPositions(int coords) {
    return occupiedCoordsIndex[coords];
  }

  int getNumRotations() {return numRotations;}

  int getNumPositions() {return numPositions;}

  inline long long getPosition(int i) {return positions[i];}

  inline int getPositionRotation(int i) {return positionRotation[i];}

  inline long long getRotation(int i) {return rotations[i];}

  inline int* getBBox() {return *_bbox;}

  long long mirror(long long piece, bool xAxis, bool yAxis, bool zAxis);


  void dumpPositionInfo();

  void dumpOuccupanceInfo(bool hi=false);

  void dumpOuccupanceIndexInfo(bool hi=false);

#ifdef DEBUG_BIN
  void writeDescriptionAsIV(ofstream &, const int indentLevel, const int pieceNo);

  void writePieceAsIV(ofstream &, const int indentLevel, const int pieceNo);

  void writeRotatedPiecesAsIV(ofstream & out, const int indentLevel, const int pieceNo);

  void writePiecePositionsAsIV(ofstream & out, const int indentLevel, const int pieceNo);

  static void writeAsIV(ofstream &, const int indentLevel, const int pieceNo,
		 const int x, const int y, const int z);
#endif

 private:

  void buildRotations();
  long long addRotation(long long rotatedPiece, int *rotatedBBox);

  void buildPositions(bool skipSymmetry = false, bool specialSymmetry = false);

  void buildOuccupanceInfo();

  int  (*_bbox)[3];
  bool (*_cubes)[PC_Z_DIM][PC_Y_DIM][PC_X_DIM];

  long long piece;
  static long long mask;

  long long positions[MAX_NUM_POS];
  int positionRotation[MAX_NUM_POS];
  long long rotations[MAX_NUM_ROT];
  int **rotatedBBoxes;

  int numPositions;
  int numRotations;

  int occupiedCoordsPositionIndex[GC_Z_DIM*GC_Y_DIM*GC_X_DIM][MAX_NUM_POS];
  int occupiedCoordsIndex[GC_Z_DIM*GC_Y_DIM*GC_X_DIM];
  long long occupiedCoords[GC_Z_DIM*GC_Y_DIM*GC_X_DIM][MAX_NUM_POS];

  int id;

};



#endif
