/*

  File: Puzzle.cpp

  (c) 2003 by Oliver Ehli

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

     See File LICENSE for more Information.

*/
#include <stdlib.h>
#include <iostream.h>
#include "Puzzle.h"




long long Puzzle::translateToOrigin(long long piece)
{

  int xMin=GC_X_DIM;
  for (int k=0; k < GC_Z_DIM; k++)
    for (int j=0; j < GC_Y_DIM; j++) {
      long long mask = (1ll << (k*GC_Y_DIM*GC_X_DIM) << j*GC_X_DIM);
      for (int i=0; i < GC_X_DIM; i++)
	if(piece & (mask<<i) && i < xMin) xMin = i;
    }

  int yMin=GC_Y_DIM;
  for (int k=0; k < GC_Z_DIM; k++)
    for (int j=0; j < GC_Y_DIM; j++) {
      long long mask = (1ll << (k*GC_Y_DIM*GC_X_DIM) << j*GC_X_DIM);
      for (int i=0; i < GC_X_DIM; i++)
	if(piece & (mask<<i) && j < yMin) yMin = j;
    }

  int zMin=GC_Z_DIM;
  for (int k=0; k < GC_Z_DIM; k++)
    for (int j=0; j < GC_Y_DIM; j++) {
      long long mask = (1ll << (k*GC_Y_DIM*GC_X_DIM) << j*GC_X_DIM);
      for (int i=0; i < GC_X_DIM; i++)
	if(piece & (mask<<i) && k < zMin) zMin = k;
    }

  return (((piece >> xMin) >> (yMin*GC_X_DIM)) >> (zMin*GC_X_DIM*GC_Y_DIM)) ;
}


long long Puzzle::rotate(long long piece, int xAngle, int yAngle, int zAngle, int bbox[], int xMask)
{
  if(xAngle) return xRotate(piece, xAngle, yAngle, zAngle, bbox, xMask);
  if(yAngle) return yRotate(piece, xAngle, yAngle, zAngle, bbox, xMask);
  if(zAngle) return zRotate(piece, xAngle, yAngle, zAngle, bbox, xMask);

  return piece;
}





long long Puzzle::zRotate(long long inPiece, int xAngle, int yAngle, int zAngle,
		     int bbox[], int xMask)
{
  long long piece;
  long long result;

  result = inPiece;


  if(zAngle>=2) {
    piece = result;
    result = 0;

    long long mask = 1;
    for (int i = 0; i < GC_Z_DIM; i++) {
      for (int j = 0; j < GC_Y_DIM; j++)
	for (int k = 0; k < GC_X_DIM; k++, mask <<= 1) {
	  if(piece & mask)
	    result |= ((1ll << (i*GC_Y_DIM*GC_X_DIM))
		       << ((GC_Y_DIM-1-j)*GC_X_DIM)) << (GC_X_DIM-1-k);
	}
    }
    result = translateToOrigin(result);
  }


  if(zAngle%2) {
    piece = result;
    result = 0;

    long long mask = 1;
    for (int i = 0; i < GC_Z_DIM; i++) {
      for (int j = 0; j < GC_Y_DIM; j++)
	for (int k = 0; k < GC_X_DIM; k++, mask <<= 1) {
	  if(!(piece & mask) || k >= GC_Y_DIM || j >= GC_X_DIM) continue;
	  result |= ((1ll << (i*GC_Y_DIM*GC_X_DIM))
		     << (k*GC_X_DIM)) << (GC_X_DIM-1-j);
	}
    }
    result = translateToOrigin(result);
  }
  return result;
}



long long Puzzle::yRotate(long long inPiece, int xAngle, int yAngle, int zAngle,
			  int bbox[], int xMask)
{
  long long  piece;
  long long result;

  result = inPiece;

  if(yAngle>=2) {
    piece = result;
    result = 0;

    long long mask = 1;
    for (int i = 0; i < GC_Z_DIM; i++)
      for (int j = 0; j < GC_Y_DIM; j++)
	for (int k = 0; k < GC_X_DIM; k++, mask <<= 1) {
	  if(piece & mask)
	    result |= ((1ll << (GC_X_DIM-1-k)) << (j*GC_X_DIM))
				       << ((GC_Z_DIM-1-i)*GC_Y_DIM*GC_X_DIM);
	}

    result = translateToOrigin(result);
  }

  if(yAngle %2) {
    piece = result;
    result = 0;

    long long mask = 1;
    for (int i = 0; i < GC_Z_DIM; i++)
      for (int j = 0; j < GC_Y_DIM; j++)
	for (int k = 0; k < GC_X_DIM; k++, mask <<= 1) {
	  if(!(piece & mask) ||(k >= GC_Z_DIM || i >= GC_X_DIM)) continue;
	  result |= ((1ll << (GC_X_DIM-1-i)) << (j*GC_X_DIM)) << (k*GC_Y_DIM*GC_X_DIM);
	}

    result = translateToOrigin(result);
  }
  return result;
}






long long Puzzle::xRotate(long long inPiece, int xAngle, int yAngle, int zAngle,
			  int bbox[], int xMask)
{
  long long  piece;
  long long result;

  result = inPiece;

  if(xAngle >= 2) {
    long long mask = 1;
    piece = result;
    result = 0;

    for (int i = 0; i < GC_Z_DIM; i++)
      for (int j = 0; j < GC_Y_DIM; j++)
	for (int k = 0; k < GC_X_DIM; k++, mask <<= 1) {
	  if(!(piece & mask) || ((xAngle%2 && (j >= GC_Z_DIM || i >= GC_Y_DIM)))) continue;
	  result |= ((1ll << k) << ((GC_Y_DIM-1-j)*GC_X_DIM)) << ((GC_Z_DIM-1-i)*GC_Y_DIM*GC_X_DIM);
	}
    result = translateToOrigin(result);
  }

  if(xAngle %2) {
    long long mask = 1;
    piece = result;
    result = 0;

    for (int i = 0; i < GC_Z_DIM; i++)
      for (int j = 0; j < GC_Y_DIM; j++)
	for (int k = 0; k < GC_X_DIM; k++, mask <<= 1) {
	  if(!(piece & mask) || ((xAngle%2 && (j >= GC_Z_DIM || i >= GC_Y_DIM)))) continue;
	  result |= ((1ll << k) << ((i)*GC_X_DIM)) << ((GC_Z_DIM-1-j)*GC_Y_DIM*GC_X_DIM);
	}
    result = translateToOrigin(result);
  }


  return result;
}


// setup puzzle..
#include "PuzzlePieces.h"


Puzzle::PieceDescription Puzzle::piece_desc[] =
    {
     { &piece8_dim,  &piece8_desc },
     { &piece11_dim,  &piece11_desc },
     { &piece4_dim,  &piece4_desc },
     { &piece5_dim,  &piece5_desc },
     { &piece3_dim,  &piece3_desc }, // #define T_PIECE        4
     { &piece10_dim,  &piece10_desc },
     { &piece12_dim,  &piece12_desc },
     { &piece1_dim, &piece1_desc },  // #define C_PIECE        7
     { &piece9_dim,  &piece9_desc },
     { &piece2_dim,  &piece2_desc },
     { &piece6_dim, &piece6_desc },
     { &piece7_dim, &piece7_desc },// #define APO1_PIECE     11
    };


