/*

  File: PuzzlePieces.cpp

  (c) 2003 by Oliver Ehli

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

     See File LICENSE for more Information.

*/
#include "Puzzle.h"

/*
  Piece definitions:
  int array describing 'bounding-box',
  [x][y][z]

  bool array describing unit-cubes:
  [z][y][x]

  { z-areas
    { y-rows
      {
        x-flags 
       }
    }
   }

*/



// Piece 1:
//
//  o o
//  ooo


int piece1_dim[] =
  {
    3, 2, 1
  };

bool piece1_desc[PC_Z_DIM][PC_Y_DIM][PC_X_DIM]  =
{
  {
    { true,  true,  true  },
    { true,  false, true },
    { }
  },
  { }
};


// Piece 2:
//
//  o
//  ooo
//   o


int piece2_dim[] =
  {
    3, 3, 1
  };

bool piece2_desc[PC_Z_DIM][PC_Y_DIM][PC_X_DIM]  =
{
  {
    { false, true,  false  },
    { true,  true,  true  },
    { true,  false, false  }
  }
};




// Piece 3:
//
//  o o
// oooo


int piece3_dim[] =
  {
    4, 2, 1
  };

bool piece3_desc[PC_Z_DIM][PC_Y_DIM][PC_X_DIM]  =
{
  {
    { true,  true,  true,  true  },
    { false, true,  false, true  }
  }
};




// Piece 4:
//
//  o
//  o
// ooo


int piece4_dim[] =
  {
    3, 3, 1
  };

bool piece4_desc[PC_Z_DIM][PC_Y_DIM][PC_X_DIM]  =
{
  {
    { true,  true, true  },
    { false, true, false },
    { false, true, false }
  }
};



// Piece 5:
//
//  oo
//  o
// oo


int piece5_dim[] =
  {
    3, 3, 1
  };

bool piece5_desc[PC_Z_DIM][PC_Y_DIM][PC_X_DIM]  =
{
  {
    { true, true, false },
    { false, true, false },
    { false, true, true }
  }
};




// Piece 6:
//
// oo
// ooo


int piece6_dim[] =
  {
    3, 2, 1
  };

bool piece6_desc[PC_Z_DIM][PC_Y_DIM][PC_X_DIM]  =
{
  {
    { true, true, true  },
    { true, true, false  }
  }
};




// Piece 7:
//
// o    o
// oo   o
// z=0, z=1

int piece7_dim[] =
  {
    2, 2, 2
  };

bool piece7_desc[PC_Z_DIM][PC_Y_DIM][PC_X_DIM]  =
{
  {
    { true, true  },
    { true, false }
  },
  {
    { true, false },
    { true, false }
  }
};




// Piece 8:
//
//  o
// ooo
//  o


int piece8_dim[] =
  {
    3, 3, 1
  };

bool piece8_desc[PC_Z_DIM][PC_Y_DIM][PC_X_DIM]  =
{
  {
    { false, true, false },
    { true,  true, true  },
    { false, true, false }
  }
};



// Piece 9:
//
//       o
// oo    o
// z=0, z=1


int piece9_dim[] =
  {
    2, 2, 2
  };

bool piece9_desc[PC_Z_DIM][PC_Y_DIM][PC_X_DIM]  =
{
  {
    { true, true   },
    { false, false }
  },
  {
    { false, true  },
    { false, true  }
  }
};




// Piece 10:
//
//  o
//  oooo


int piece10_dim[] =
  {
    4, 2, 1
  };

bool piece10_desc[PC_Z_DIM][PC_Y_DIM][PC_X_DIM]  =
{
  {
    { true, true,  true,  true  },
    { true, false, false, false }
  }
};




// Piece 11:
//
// o
// oo
//  oo


int piece11_dim[] =
  {
    3, 3, 1
  };

bool piece11_desc[PC_Z_DIM][PC_Y_DIM][PC_X_DIM]  =
{
  {
    { false, true,  true  },
    { true,  true,  false },
    { true,  false, false }
  }
};




// Piece 12:
//
//
//  o
// oooo


int piece12_dim[] =
  {
    4, 2, 1
  };

bool piece12_desc[PC_Z_DIM][PC_Y_DIM][PC_X_DIM]  =
{
  {
    { true,  true, true,  true  },
    { false, true, false, false }
  }
};



