/*

  File: Solver.cpp

  (c) 2003 by Oliver Ehli

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

     See File LICENSE for more Information.

*/
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "Puzzle.h"
#include "Piece.h"
#include "Solver.h"

#include "InventorWriter.h"



int levelCounter[NUM_PCS] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

int Solver::partMask[] = {
  1, 1<<1, 1<<2, 1<<3, 1<<4, 1<<5, 1<<6, 1<<7, 1<<8, 1<<9, 1<<10, 1<<11
};

int Solver::allParts = (1<<12)-1;

int ***positionsLeft;
int **positionsLeftIndex;

#define MAX_POS 32
#define MAX_POS_32 28

Solver::Solver(Piece **inPieces) : partsUsed(0), currentDepth(0), numSolutions(0)
{
  for (int i=0; i< NUM_PCS; i++)
    currentPositions[i] = 0;
  pieces = inPieces;

}





double counter = 0;
int percent = 0;

static clock_t startTime = clock();


int Solver::solve()
{
  int usePiece = 0;

  for(int i=0; i < 8; i++)
    doSolve(4, 1<<4, (pieces[usePiece]->getPosition(i)) << 4, partMask[usePiece]);

  Piece *tmp = pieces[11];
  pieces[11] = pieces[12];

  for(int i=8; i < pieces[usePiece]->getNumPositions(); i++)
    doSolve(4, 1<<4, (pieces[usePiece]->getPosition(i)) << 4, partMask[usePiece]);


  cout << endl << "done. " << ((double)(clock()-startTime))/1000000.0 << " seconds for "
       << numSolutions << " solutions and " << counter << " tries." << endl;

  return 0;
}






void Solver::placePiece(int pieceNo, int start, long long startMask, long long puzzle, int partsUsed)
{
  long long *positionList = pieces[pieceNo]->getPositions(start);
  for (int j= 0; j < pieces[pieceNo]->getNumPositions(start); j++, positionList++) {
    if(!(puzzle & *positionList))
      doSolve(start+1, startMask<<1, puzzle | *positionList, partsUsed | partMask[pieceNo]);
  }
}


void Solver::doSolve(int start, long long startMask, long long puzzle, int partsUsed)
{

  if(partsUsed == allParts) {
    if(!(++numSolutions%1000)) {cout << "."; fflush(stdout);}
    if(!((numSolutions) % (409963/10))) {
      percent++;
      cout << endl << percent*10 << "% done, " << ((double)(clock()-startTime))/1000000.0
	   << " seconds so far." << endl << " expecting "
	   << ((double)(clock()-startTime))/1000000.0*10.0/((double)percent) << " seconds."
	   << endl;
    }
    return;
  }

  for (;(puzzle & startMask) && start < MAX_POS+MAX_POS_32; start++, startMask <<= 1);

 piece1:
  if (partsUsed & partMask[1]) goto piece2;
  placePiece(1, start, startMask, puzzle, partsUsed);
 piece2:
  if (partsUsed & partMask[2]) goto piece3;
  placePiece(2, start, startMask, puzzle, partsUsed);
 piece3:
  if (partsUsed & partMask[3]) goto piece4;
  placePiece(3, start, startMask, puzzle, partsUsed);
 piece4:
  if (partsUsed & partMask[4]) goto piece5;
  placePiece(4, start, startMask, puzzle, partsUsed);
 piece5:
  if (partsUsed & partMask[5]) goto piece6;
  placePiece(5, start, startMask, puzzle, partsUsed);
 piece6:
  if (partsUsed & partMask[6]) goto piece7;
  placePiece(6, start, startMask, puzzle, partsUsed);
 piece7:
  if (partsUsed & partMask[7]) goto piece8;
  placePiece(7, start, startMask, puzzle, partsUsed);
 piece8:
  if (partsUsed & partMask[8]) goto piece9;
  placePiece(8, start, startMask, puzzle, partsUsed);
 piece9:
  if (partsUsed & partMask[9]) goto piece10;
  placePiece(9, start, startMask, puzzle, partsUsed);
 piece10:
  if (partsUsed & partMask[10]) goto piece11;
  placePiece(10, start, startMask, puzzle, partsUsed);
 piece11:
  if(!(partsUsed & partMask[11]))
    placePiece(11, start, startMask, puzzle, partsUsed);
     


}




