/*

  File: createSolver.c

  (c) 2003 by Oliver Ehli

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

     See File LICENSE for more Information.

*/
#include <stdio.h>
#include <time.h>

#define NUM_PCS 12
#define START_BIT 4
#define NUM_BITS 60
#define NUM_HI_BITS 32
#define NUM_LO_BITS 32
#include "piecePositions.h"
#include "createSolver.h"



int counter = 0;
int leftCounter = 0;
int firstNot = -1;
int group32[24];
long long group[24];
int groupLeft32[24];
long long groupLeft[24];
int partMask[] = {
  1, 1<<1, 1<<2, 1<<3, 1<<4, 1<<5, 1<<6, 1<<7, 1<<8, 1<<9, 1<<10, 1<<11, 1<<11
};


long long numBits(long long  mix, long long *piece, int pieceNo, int limit)
{
  long long result = mix & *piece;
  int num=0;

  if(*piece == 0ll) return mix;

  for (int i=0; i<64; i++) if(1ll<<i & result) num++;

  if(num>=limit) {
    group[counter++] = *piece;
    *piece=0ll;
  }
  else if(firstNot < 0) firstNot = pieceNo;

  return num >= limit ? result : mix;
}


int numBits32(int mix, int *piece, int pieceNo, int limit)
{
  int result = mix & *piece;
  int num=0;

  if(*piece == 0l) return mix;

  for (int i=0; i<32; i++) if(1l<<i & result) num++;

  if(num>=limit) {
    group32[counter++] = *piece;
    *piece=0l;
  }

  else if(firstNot < 0) firstNot = pieceNo;

  return num >= limit ? result : mix;
}



void writeGroupedPieces(int coord, int piece, char *prefix)
{
  int k, l;
  int groupCounter = 0;
  long long bl = -1ll;

  if(positionIndex[piece][coord] > 2) {
    while(1) {
      if(firstNot >= positionIndex[piece][coord]) break;
      for(k=0; k < positionIndex[piece][coord]; k++) {
	bl = 1ll<<coord ^ numBits(bl ^ 1ll<<coord, &positions[piece][coord][k], k, 2);
      }

      if(counter > 2) {
	printf("      if (!(puzzle & 0x%llxll)) {\n", bl);
	int l;
	for(l=0; l < counter; l++)
	  printf("         if(!(puzzle & 0x%llxll))\n           %s%soSolve(%d, 0x%llxll, puzzle | 0x%llxll,\n                   partsUsed | 0x%lxl);\n",
		 group[l], prefix ? prefix : "", prefix ? "D" : "d", coord+1, 1ll<<(coord+1),
		 group[l], partMask[piece]);
	printf("      }\n");
      }
      else {
	for(l=0; l < counter; l++)
	  groupLeft[leftCounter++] = group[l];
      }
      groupCounter += counter;
      counter = 0;
      bl = -1ll;
      firstNot++;
      if(groupCounter == positionIndex[piece][coord]) break;
    }
    firstNot = -1;
    groupCounter = 0;
  }

  for(k=0; k < positionIndex[piece][coord]; k++)
    if(positions[piece][coord][k] != 0ll)
      printf("      if(!(puzzle & 0x%llxll))\n        %s%soSolve(%d, 0x%llxll, puzzle | 0x%llxll,\n                   partsUsed | 0x%lxl);\n",
	     positions[piece][coord][k], prefix ? prefix : "", prefix ? "D" : "d", coord+1,
	     1ll<<(coord+1), positions[piece][coord][k], partMask[piece]);

  for(k=0; k < leftCounter; k++)
      printf("      if(!(puzzle & 0x%llxll))\n        %s%soSolve(%d, 0x%llxll, puzzle | 0x%llxll,\n                   partsUsed | 0x%lxl);\n",
	   groupLeft[k], prefix ? prefix : "", prefix ? "D" : "d", coord+1,
	   1ll<<(coord+1), groupLeft[k], partMask[piece]);

  firstNot = -1;
  counter = 0;
  leftCounter = 0;
}




void writeGroupedPieces32(int coord, int piece, char *prefix)
{
  int k, l;
  int groupCounter = 0;

  int bl32 = -1l;

  if(positionIndex32[piece][coord] > 2) {
    while(1) {
      if(firstNot >= positionIndex32[piece][coord]) break;
      for(k=0; k < positionIndex32[piece][coord]; k++) {
	bl32 = 1l<<coord ^ numBits32(bl32 ^ 1l<<coord, &positions32[piece][coord][k], k, 2);
      }

      if(counter > 2) {
	printf("      if (!(puzzle & 0x%lxl)) {\n", bl32);
	int l;
	for(l=0; l < counter; l++)
	  printf("         if(!(puzzle & 0x%lxl))\n           %s%soSolve32(%d, 0x%lxl, puzzle | 0x%lxl, partsUsed | 0x%lxl);\n",
		 group32[l], prefix ? prefix : "", prefix ? "D" : "d", coord+1, 1l<<(coord+1),
		 group32[l], partMask[piece]);
	printf("      }\n");
      }
      else {
	for(l=0; l < counter; l++)
	  groupLeft32[leftCounter++] = group32[l];
      }
      groupCounter += counter;
      counter = 0;
      bl32 = -1l;
      firstNot++;
      if(groupCounter == positionIndex32[piece][coord]) break;
    }
    firstNot = -1;
    groupCounter = 0;
  }

  for(k=0; k < positionIndex32[piece][coord]; k++)
    if(positions32[piece][coord][k] != 0l)
      printf("      if(!(puzzle & 0x%lxl))\n        %s%soSolve32(%d, 0x%lxl, puzzle | 0x%lxl, partsUsed | 0x%lxl);\n",
	     positions32[piece][coord][k], prefix ? prefix : "", prefix ? "D" : "d", coord+1,
	     1l<<(coord+1), positions32[piece][coord][k], partMask[piece]);

  for(k=0; k < leftCounter; k++)
    printf("      if(!(puzzle & 0x%lxl))\n        %s%soSolve32(%d, 0x%lxl, puzzle | 0x%lxl, partsUsed | 0x%lxl);\n",
	   groupLeft32[k], prefix ? prefix : "", prefix ? "D" : "d", coord+1,
	   1l<<(coord+1), groupLeft32[k], partMask[piece]);


  firstNot = -1;
  counter = 0;
  leftCounter = 0;
}







void writeGroupedAltPieces(int coord, int piece, char *prefix)
{
  int k, l;
  int groupCounter = 0;
  long long bl = -1ll;

  if(positionIndex[piece][coord] > 2) {
    while(1) {
      if(firstNot >= altPositionIndex[piece][coord]) break;
      for(k=0; k < altPositionIndex[piece][coord]; k++) {
	bl = 1ll<<coord ^ numBits(bl ^ 1ll<<coord, &altPositions[piece][coord][k], k, 2);
      }

      if(counter > 2) {
	printf("      if (!(puzzle & 0x%llxll)) {\n", bl);
	int l;
	for(l=0; l < counter; l++)
	  printf("         if(!(puzzle & 0x%llxll))\n           %s%soSolve(%d, 0x%llxll, puzzle | 0x%llxll,\n                   partsUsed | 0x%lxl);\n",
		 group[l], prefix ? prefix : "", prefix ? "D" : "d", coord+1, 1ll<<(coord+1),
		 group[l], partMask[piece]);
	printf("      }\n");
      }
      else {
	for(l=0; l < counter; l++)
	  groupLeft[leftCounter++] = group[l];
      }
      groupCounter += counter;
      counter = 0;
      bl = -1ll;
      firstNot++;
      if(groupCounter == altPositionIndex[piece][coord]) break;
    }
    firstNot = -1;
    groupCounter = 0;
  }

  for(k=0; k < altPositionIndex[piece][coord]; k++)
    if(altPositions[piece][coord][k] != 0ll)
      printf("      if(!(puzzle & 0x%llxll))\n        %s%soSolve(%d, 0x%llxll, puzzle | 0x%llxll,\n                   partsUsed | 0x%lxl);\n",
	     altPositions[piece][coord][k], prefix ? prefix : "", prefix ? "D" : "d", coord+1,
	     1ll<<(coord+1), altPositions[piece][coord][k], partMask[piece]);

  for(k=0; k < leftCounter; k++)
      printf("      if(!(puzzle & 0x%llxll))\n        %s%soSolve(%d, 0x%llxll, puzzle | 0x%llxll,\n                   partsUsed | 0x%lxl);\n",
	   groupLeft[k], prefix ? prefix : "", prefix ? "D" : "d", coord+1,
	   1ll<<(coord+1), groupLeft[k], partMask[piece]);

  firstNot = -1;
  counter = 0;
  leftCounter = 0;
}




void writeGroupedAltPieces32(int coord, int piece, char *prefix)
{
  int k, l;
  int groupCounter = 0;

  int bl32 = -1l;

  if(altPositionIndex32[piece][coord] > 2) {
    while(1) {
      if(firstNot >= altPositionIndex32[piece][coord]) break;
      for(k=0; k < altPositionIndex32[piece][coord]; k++) {
	bl32 = 1l<<coord ^ numBits32(bl32 ^ 1l<<coord, &altPositions32[piece][coord][k], k, 2);
      }

      if(counter > 2) {
	printf("      if (!(puzzle & 0x%lxl)) {\n", bl32);
	int l;
	for(l=0; l < counter; l++)
	  printf("         if(!(puzzle & 0x%lxl))\n           %s%soSolve32(%d, 0x%lxl, puzzle | 0x%lxl, partsUsed | 0x%lxl);\n",
		 group32[l], prefix ? prefix : "", prefix ? "D" : "d", coord+1, 1l<<(coord+1),
		 group32[l], partMask[piece]);
	printf("      }\n");
      }
      else {
	for(l=0; l < counter; l++)
	  groupLeft32[leftCounter++] = group32[l];
      }
      groupCounter += counter;
      counter = 0;
      bl32 = -1l;
      firstNot++;
      if(groupCounter == altPositionIndex32[piece][coord]) break;
    }
    firstNot = -1;
    groupCounter = 0;
  }

  for(k=0; k < altPositionIndex32[piece][coord]; k++)
    if(altPositions32[piece][coord][k] != 0l)
      printf("      if(!(puzzle & 0x%lxl))\n        %s%soSolve32(%d, 0x%lxl, puzzle | 0x%lxl, partsUsed | 0x%lxl);\n",
	     altPositions32[piece][coord][k], prefix ? prefix : "", prefix ? "D" : "d", coord+1,
	     1l<<(coord+1), altPositions32[piece][coord][k], partMask[piece]);

  for(k=0; k < leftCounter; k++)
    printf("      if(!(puzzle & 0x%lxl))\n        %s%soSolve32(%d, 0x%lxl, puzzle | 0x%lxl, partsUsed | 0x%lxl);\n",
	   groupLeft32[k], prefix ? prefix : "", prefix ? "D" : "d", coord+1,
	   1l<<(coord+1), groupLeft32[k], partMask[piece]);


  firstNot = -1;
  counter = 0;
  leftCounter = 0;
}



void writeSolver(char *prefix)
{
  printf("%s%s%s%s%s%s%s", prefix ? "void " : "void ", prefix ? prefix : "", prefix ? "D" : "d", solverStart1,
	 prefix ? prefix : "", prefix ? "D" : "d", solverStart2);

  printf("  switch(start) {\n");

  int i;
  for(i=START_BIT; i<NUM_LO_BITS; i++) {
    int j=0;
    while(++j < NUM_PCS+1 && !positionIndex[j][i]);

    if(j==NUM_PCS+1) continue;
    if(j==NUM_PCS && !prefix) continue;
    if(j==NUM_PCS-1 && prefix && !positionIndex[NUM_PCS][i]) continue;

    printf("   case %d:\n", i);
    for(; j<NUM_PCS+1; j++) {
      if(j==NUM_PCS && !prefix) continue;
      if(j==NUM_PCS-1 && prefix) continue;

      if(positionIndex[j][i]) {
	printf("      if (partsUsed & 0x%lxl) goto %spiece%dat%d;\n",
	       partMask[j], prefix ? prefix : "", j+1, i);
	if(prefix)
	  writeGroupedAltPieces(i, j, prefix);
	else
	  writeGroupedPieces(i, j, prefix);
 	printf("    %spiece%dat%d:\n", prefix ? prefix : "", j+1, i);
      }
    }
    printf("    break;\n");
  }

  printf("  }\n");
  printf("}\n\n\n");
}


void writeSolver32(char *prefix)
{
  printf("%s%s%s%s", prefix ? "void " : "void ", prefix ? prefix : "", prefix ? "D" : "d", solver32Start);

  printf("  switch(start) {\n");

  int i;
  for(i=0; i<NUM_HI_BITS; i++) {
    int j=0;
    while(++j < NUM_PCS+1 && !positionIndex32[j][i]);

    if(j==NUM_PCS+1) continue;
    if(j==NUM_PCS && !prefix) continue;
    if(j==NUM_PCS-1 && prefix && !positionIndex32[NUM_PCS][i]) continue;

    printf("   case %d:\n", i);
    for(; j<NUM_PCS+1; j++) {
      if(j==NUM_PCS && !prefix) continue;
      if(j==NUM_PCS-1 && prefix) continue;

      if(positionIndex32[j][i]) {
	printf("      if (partsUsed & 0x%lxl) goto %spiece%dat%dat32;\n",
	       partMask[j], prefix ? prefix : "", j+1, i);
	if(prefix)
	  writeGroupedAltPieces32(i, j, prefix);
	else
	  writeGroupedPieces32(i, j, prefix);
 	printf("    %spiece%dat%dat32:\n", prefix ? prefix : "", j+1, i);
      }
    }
    printf("    break;\n");
  }

  printf("  }\n");
  printf("}\n\n\n");
}



int main(int argc, char** argv)
{
  int i;

  printf(solver_head);

  printf("\n\nvoid solve()\n{\n");
  for(i=0; i < 8; i++)
    printf("  doSolve(4, 1<<4, 0x%llxll, 0x%lxl);\n",
	   piece0Positions[i] << 4, partMask[0]);
  for(i=8; i < piece0NumPositions; i++)
    printf("  altDoSolve(4, 1<<4, 0x%llxll, 0x%lxl);\n",
	   piece0Positions[i] << 4, partMask[0]);
  printf("\n}\n\n\n");


  writeSolver(0);
  writeSolver32(0);

  writeSolver("alt");
  writeSolver32("alt");
}
