/*

  File: main.cpp

  (c) 2003 by Oliver Ehli

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

     See File LICENSE for more Information.

*/

#include <stdio.h>
#include <fstream.h>

#include "Puzzle.h"
#include "Piece.h"
#include "Solver.h"
#include "InventorWriter.h"



#ifdef DEBUG_BIN

void dumpPieces(Piece *pieces[])
{
  ofstream iv_out;
  iv_out.open("pieces.iv");

  iv_out << InventorWriter::header;

  for(int i=0; i<NUM_PCS; i++) {
    iv_out << InventorWriter::indent;
    iv_out << InventorWriter::beginGroup;

    iv_out << InventorWriter::indent;
    iv_out << InventorWriter::indent;

    InventorWriter::writeTranslation(iv_out, 0.0,
				     i%3 * 5 * InventorWriter::baseUnit,
				     i/3 * 5 * InventorWriter::baseUnit);

//     (pieces[i])->writeDescriptionAsIV(iv_out, 2, i);
    (pieces[i])->writePieceAsIV(iv_out, 2, i);
    iv_out << InventorWriter::indent;
    iv_out << InventorWriter::endGroup;
  }

  iv_out << InventorWriter::footer;
  iv_out.close();

}


void dumpRotatedPieces(Piece *pieces[])
{
  ofstream iv_out;
  char filename[128];

  for(int i=0; i<NUM_PCS; i++) {
    sprintf(filename, "rotatedPieces-%02d.iv", i);
    iv_out.open(filename);

    iv_out << InventorWriter::header;

    (pieces[i])->writeRotatedPiecesAsIV(iv_out, 1, i);

    iv_out << InventorWriter::footer;
    iv_out.close();
  }

}

void dumpPiecePositions(Piece *pieces[])
{
  ofstream iv_out;
  char filename[128];

//   for(int i=0; i<NUM_PCS; i++) {
   int i=0;{
    sprintf(filename, "piecePositions-%02d.iv", i);
    iv_out.open(filename);

    iv_out << InventorWriter::header;

    (pieces[i])->writePiecePositionsAsIV(iv_out, 1, i);

    iv_out << InventorWriter::footer;
    iv_out.close();
  }

}

#endif


int main(int argc, char **argv)
{
  Piece *pieces[NUM_PCS+1];

  pieces[0] = new Piece(0, Puzzle::getPieces()[0].bbox,
			Puzzle::getPieces()[0].positions, true);
  for(int i=1; i<NUM_PCS; i++)
    pieces[i] = new Piece(i, Puzzle::getPieces()[i].bbox,
			  Puzzle::getPieces()[i].positions);

//   pieces[NUM_PCS-1] = new Piece(NUM_PCS-1, Puzzle::getPieces()[NUM_PCS-1].bbox,
// 			      Puzzle::getPieces()[NUM_PCS-1].positions, false, true);

  pieces[NUM_PCS] = new Piece(NUM_PCS-1, Puzzle::getPieces()[NUM_PCS-1].bbox,
			      Puzzle::getPieces()[NUM_PCS-1].positions, false, true);

#ifdef DEBUG_BIN
  dumpPieces(pieces);
  dumpRotatedPieces(pieces);
  dumpPiecePositions(pieces);
#endif

  if(argc>1 && *argv[1] == 'c') {
    pieces[0]->dumpPositionInfo();

    cout << "int positionIndex[NUM_PCS+1][NUM_LO_BITS] = {" << endl;
    for(int i=0; i<NUM_PCS+1; i++)
      pieces[i]->dumpOuccupanceIndexInfo();
    cout << "};" << endl;

    cout << "long long positions[NUM_PCS+1][NUM_LO_BITS][24] = {" << endl;
    for(int i=0; i<NUM_PCS+1; i++)
      pieces[i]->dumpOuccupanceInfo();
    cout << "};" << endl;


    cout << "int positionIndex32[NUM_PCS+1][NUM_HI_BITS] = {" << endl;
    for(int i=0; i<NUM_PCS+1; i++)
      pieces[i]->dumpOuccupanceIndexInfo(true);
    cout << "};" << endl;

    cout << "int positions32[NUM_PCS+1][NUM_HI_BITS][24] = {" << endl;
    for(int i=0; i<NUM_PCS+1; i++)
      pieces[i]->dumpOuccupanceInfo(true);
    cout << "};" << endl;




    cout << "int altPositionIndex[NUM_PCS+1][NUM_LO_BITS] = {" << endl;
    for(int i=0; i<NUM_PCS+1; i++)
      pieces[i]->dumpOuccupanceIndexInfo();
    cout << "};" << endl;

    cout << "long long altPositions[NUM_PCS+1][NUM_LO_BITS][24] = {" << endl;
    for(int i=0; i<NUM_PCS+1; i++)
      pieces[i]->dumpOuccupanceInfo();
    cout << "};" << endl;


    cout << "int altPositionIndex32[NUM_PCS+1][NUM_HI_BITS] = {" << endl;
    for(int i=0; i<NUM_PCS+1; i++)
      pieces[i]->dumpOuccupanceIndexInfo(true);
    cout << "};" << endl;

    cout << "int altPositions32[NUM_PCS+1][NUM_HI_BITS][24] = {" << endl;
    for(int i=0; i<NUM_PCS+1; i++)
      pieces[i]->dumpOuccupanceInfo(true);
    cout << "};" << endl;


    return 0;
  }

  for(int i=0; i<NUM_PCS; i++)
    cout << "piece " << i << ": "
	 << pieces[i]->getNumPositions() << " possible Positions" << endl;

  Solver *solver = new Solver(pieces);

  solver->solve();

}
