/*

  File: Piece.cpp

  (c) 2003 by Oliver Ehli

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

     See File LICENSE for more Information.

*/
#include <stdio.h>
#include <time.h>

#define NUM_PCS 12
#define START 4
#define NUM_BITS 60
#define NUM_HI_BITS 32
#define NUM_LO_BITS 32
#include "piecePositions.h"


int partMask[] = {
  1, 1<<1, 1<<2, 1<<3, 1<<4, 1<<5, 1<<6, 1<<7, 1<<8, 1<<9, 1<<10, 1<<11, 1<<11
};
int allParts = (1<<12)-1;

int partsUsed = 0;
int numSolutions = 0;

int counter = 0;
int percent = 0;

clock_t startTime;


extern void doSolve(int start, long long startMask, long long puzzle, int partsUsed);
extern void altDoSolve(int start, long long startMask, long long puzzle, int partsUsed);

extern void doSolve32(int start, int startMask, int puzzle, int partsUsed);
extern void altDoSolve32(int start, int startMask, int puzzle, int partsUsed);


#define placePiece(pieceNo, start, startMask, puzzle, partsUsed, size)                            \
{                                                                                                 \
  for (j= 0; j < positionIndex##size[pieceNo][start]; j++) {                                  \
    if(!(puzzle & positions##size[pieceNo][start][j]))                                            \
      doSolve##size(start+1, startMask<<1, puzzle | positions##size[pieceNo][start][j],           \
		    partsUsed | partMask[pieceNo]);                                               \
  }                                                                                               \
}



void doSolve(int start, long long startMask, long long puzzle, int partsUsed)
{
  int j;

  for (;(puzzle & startMask) && start < NUM_LO_BITS; start++, startMask <<= 1);

  if(start == NUM_LO_BITS) {
    doSolve32(0, 1, puzzle>>NUM_LO_BITS, partsUsed);
    return;
  }

/*   counter++; */

 piece1:
  if (partsUsed & partMask[1]) goto piece2;
  placePiece(1, start, startMask, puzzle, partsUsed,);
 piece2:
  if (partsUsed & partMask[2]) goto piece3;
  placePiece(2, start, startMask, puzzle, partsUsed,);
 piece3:
  if (partsUsed & partMask[3]) goto piece4;
  placePiece(3, start, startMask, puzzle, partsUsed,);
 piece4:
  if (partsUsed & partMask[4]) goto piece5;
  placePiece(4, start, startMask, puzzle, partsUsed,);
 piece5:
  if (partsUsed & partMask[5]) goto piece6;
  placePiece(5, start, startMask, puzzle, partsUsed,);
 piece6:
  if (partsUsed & partMask[6]) goto piece7;
  placePiece(6, start, startMask, puzzle, partsUsed,);
 piece7:
  if (partsUsed & partMask[7]) goto piece8;
  placePiece(7, start, startMask, puzzle, partsUsed,);
 piece8:
  if (partsUsed & partMask[8]) goto piece9;
  placePiece(8, start, startMask, puzzle, partsUsed,);
 piece9:
  if (partsUsed & partMask[9]) goto piece10;
  placePiece(9, start, startMask, puzzle, partsUsed,);
 piece10:
  if (partsUsed & partMask[10]) goto piece11;
  placePiece(10, start, startMask, puzzle, partsUsed,);
 piece11:
  if(!(partsUsed & partMask[11]))
    placePiece(11, start, startMask, puzzle, partsUsed,);
     
}




void doSolve32(int start, int startMask, int puzzle, int partsUsed)
{
  int j;

  if(partsUsed == allParts) {
    if(!(++numSolutions%4099)) { write(0, ".", 1); }
    if(!((numSolutions) % (409963/10))) {
      percent++;
      printf("\r%d%% done, %ld seconds so far; expecting %ld seconds.\n", 
	     percent*10, (clock()-startTime)/1000000, (clock()-startTime)/100000/percent);
      fflush(stdout);
    }
    return;
  }

  for (;(puzzle & startMask) && start < NUM_BITS; start++, startMask <<= 1);

/*   counter++; */

 piece32_1:
  if (partsUsed & partMask[1]) goto piece32_2;
  placePiece(1, start, startMask, puzzle, partsUsed, 32);
 piece32_2:
  if (partsUsed & partMask[2]) goto piece32_3;
  placePiece(2, start, startMask, puzzle, partsUsed, 32);
 piece32_3:
  if (partsUsed & partMask[3]) goto piece32_4;
  placePiece(3, start, startMask, puzzle, partsUsed, 32);
 piece32_4:
  if (partsUsed & partMask[4]) goto piece32_5;
  placePiece(4, start, startMask, puzzle, partsUsed, 32);
 piece32_5:
  if (partsUsed & partMask[5]) goto piece32_6;
  placePiece(5, start, startMask, puzzle, partsUsed, 32);
 piece32_6:
  if (partsUsed & partMask[6]) goto piece32_7;
  placePiece(6, start, startMask, puzzle, partsUsed, 32);
 piece32_7:
  if (partsUsed & partMask[7]) goto piece32_8;
  placePiece(7, start, startMask, puzzle, partsUsed, 32);
 piece32_8:
  if (partsUsed & partMask[8]) goto piece32_9;
  placePiece(8, start, startMask, puzzle, partsUsed, 32);
 piece32_9:
  if (partsUsed & partMask[9]) goto piece32_10;
  placePiece(9, start, startMask, puzzle, partsUsed, 32);
 piece32_10:
  if (partsUsed & partMask[10]) goto piece32_11;
  placePiece(10, start, startMask, puzzle, partsUsed, 32);
 piece32_11:
  if(!(partsUsed & partMask[11]))
    placePiece(11, start, startMask, puzzle, partsUsed, 32);
     
}



#define placeAltPiece(pieceNo, start, startMask, puzzle, partsUsed, size)                    \
{                                                                                            \
  for (j= 0; j < positionIndex##size[pieceNo][start]; j++) {                                 \
    if(!(puzzle & positions##size[pieceNo][start][j]))                                       \
      altDoSolve##size(start+1, startMask<<1, puzzle | positions##size[pieceNo][start][j],   \
                 partsUsed | partMask[pieceNo]);                                             \
  }                                                                                          \
}



void altDoSolve32(int start, int startMask, int puzzle, int partsUsed)
{
  int j;
  if(partsUsed == allParts) {
    if(!(++numSolutions%4099))  { write(0, ".", 1); }
    if(!((numSolutions) % (409963/10))) {
      percent++;
      printf("\r%d%% done, %ld seconds so far; expecting %ld seconds.\n", 
	     percent*10, (clock()-startTime)/1000000, (clock()-startTime)/100000/percent);
      fflush(stdout);
    }
    return;
  }

  for (;(puzzle & startMask) && start < NUM_BITS; start++, startMask <<= 1);

/*   counter++; */

 altPiece32_1:
  if (partsUsed & partMask[1]) goto altPiece32_2;
  placeAltPiece(1, start, startMask, puzzle, partsUsed, 32);
 altPiece32_2:
  if (partsUsed & partMask[2]) goto altPiece32_3;
  placeAltPiece(2, start, startMask, puzzle, partsUsed, 32);
 altPiece32_3:
  if (partsUsed & partMask[3]) goto altPiece32_4;
  placeAltPiece(3, start, startMask, puzzle, partsUsed, 32);
 altPiece32_4:
  if (partsUsed & partMask[4]) goto altPiece32_5;
  placeAltPiece(4, start, startMask, puzzle, partsUsed, 32);
 altPiece32_5:
  if (partsUsed & partMask[5]) goto altPiece32_6;
  placeAltPiece(5, start, startMask, puzzle, partsUsed, 32);
 altPiece32_6:
  if (partsUsed & partMask[6]) goto altPiece32_7;
  placeAltPiece(6, start, startMask, puzzle, partsUsed, 32);
 altPiece32_7:
  if (partsUsed & partMask[7]) goto altPiece32_8;
  placeAltPiece(7, start, startMask, puzzle, partsUsed, 32);
 altPiece32_8:
  if (partsUsed & partMask[8]) goto altPiece32_9;
  placeAltPiece(8, start, startMask, puzzle, partsUsed, 32);
 altPiece32_9:
  if (partsUsed & partMask[9]) goto altPiece32_10;
  placeAltPiece(9, start, startMask, puzzle, partsUsed, 32);
 altPiece32_10:
  if (partsUsed & partMask[10]) goto altPiece32_12;
  placeAltPiece(10, start, startMask, puzzle, partsUsed, 32);
 altPiece32_12:
  if(!(partsUsed & partMask[12]))
    placeAltPiece(12, start, startMask, puzzle, partsUsed, 32);
     
}



void altDoSolve(int start, long long startMask, long long puzzle, int partsUsed)
{
  int j;

  for (;(puzzle & startMask) && start < NUM_LO_BITS; start++, startMask <<= 1);

  if(start == NUM_LO_BITS){
    altDoSolve32(0, 1, puzzle>>NUM_LO_BITS, partsUsed);
    return;
  }

/*   counter++; */

 altPiece1:
  if (partsUsed & partMask[1]) goto altPiece2;
  placeAltPiece(1, start, startMask, puzzle, partsUsed,);
 altPiece2:
  if (partsUsed & partMask[2]) goto altPiece3;
  placeAltPiece(2, start, startMask, puzzle, partsUsed,);
 altPiece3:
  if (partsUsed & partMask[3]) goto altPiece4;
  placeAltPiece(3, start, startMask, puzzle, partsUsed,);
 altPiece4:
  if (partsUsed & partMask[4]) goto altPiece5;
  placeAltPiece(4, start, startMask, puzzle, partsUsed,);
 altPiece5:
  if (partsUsed & partMask[5]) goto altPiece6;
  placeAltPiece(5, start, startMask, puzzle, partsUsed,);
 altPiece6:
  if (partsUsed & partMask[6]) goto altPiece7;
  placeAltPiece(6, start, startMask, puzzle, partsUsed,);
 altPiece7:
  if (partsUsed & partMask[7]) goto altPiece8;
  placeAltPiece(7, start, startMask, puzzle, partsUsed,);
 altPiece8:
  if (partsUsed & partMask[8]) goto altPiece9;
  placeAltPiece(8, start, startMask, puzzle, partsUsed,);
 altPiece9:
  if (partsUsed & partMask[9]) goto altPiece10;
  placeAltPiece(9, start, startMask, puzzle, partsUsed,);
 altPiece10:
  if (partsUsed & partMask[10]) goto altPiece12;
  placeAltPiece(10, start, startMask, puzzle, partsUsed,);
 altPiece12:
  if(!(partsUsed & partMask[12]))
    placeAltPiece(12, start, startMask, puzzle, partsUsed,);
     
}



int solve()
{
  int usePiece = 0;
  int i;

  for(i=0; i < 8; i++)
    doSolve(4, 1<<4, piece0Positions[i] << 4, partMask[usePiece]);
  
  for(i=8; i < piece0NumPositions; i++)
    altDoSolve(4, 1<<4, piece0Positions[i] << 4, partMask[usePiece]);

  return 0;
}







int main(int argc, char**argv)
{
  clock_t startTime = clock();

  solve();

  printf("\ndone. %ld seconds, %ld solutions.\n", 
	 (clock()-startTime)/1000000, numSolutions);

/*   printf("\ndone. %ld seconds, %ld solutions, %ld tries.\n",  */
/* 	 (clock()-startTime)/1000000, numSolutions, counter); */

}
