#include <stdio.h>
#include <unistd.h>
#include <time.h>

#define NUM_PCS 12
#define START 4
#define NUM_BITS 60
#define NUM_HI_BITS 32
#define NUM_LO_BITS 32


int percent = 0;
int partsUsed = 0;
int numSolutions = 0;
clock_t startTime;
double correction[] = { 1.26, 1.05, 1, 0.94, 0.97, 1.005, 0.985, 0.94, 0.95, 1 };



extern void solve();

extern void doSolve(int start, long long startMask, long long puzzle, int partsUsed);
extern void altDoSolve(int start, long long startMask, long long puzzle, int partsUsed);

extern void doSolve32(int start, int startMask, int puzzle, int partsUsed);
extern void altDoSolve32(int start, int startMask, int puzzle, int partsUsed);


int main(int argc, char**argv)
{
  clock_t startTime = clock();

  solve();

  printf("\ndone. %ld seconds, %ld solutions.\n", 
	 (clock()-startTime)/1000000, numSolutions);
}





void solve()
{
  doSolve(4, 1<<4, 0x80490080ll, 0x1l);
  doSolve(4, 1<<4, 0xba0ll, 0x1l);
  doSolve(4, 1<<4, 0x20070020ll, 0x1l);
  doSolve(4, 1<<4, 0x100920100ll, 0x1l);
  doSolve(4, 1<<4, 0x201240200ll, 0x1l);
  doSolve(4, 1<<4, 0x100380100ll, 0x1l);
  doSolve(4, 1<<4, 0x80490080000ll, 0x1l);
  doSolve(4, 1<<4, 0xba0000ll, 0x1l);
  altDoSolve(4, 1<<4, 0x20070020000ll, 0x1l);
  altDoSolve(4, 1<<4, 0x100920100000ll, 0x1l);
  altDoSolve(4, 1<<4, 0x100380100000ll, 0x1l);
  altDoSolve(4, 1<<4, 0xba0000000ll, 0x1l);

}


void doSolve(int start, long long startMask, long long puzzle, int partsUsed)
{
  for (;(puzzle & startMask) && start < NUM_LO_BITS; start++, startMask <<= 1);

  if(start == NUM_LO_BITS) {
    doSolve32(0, 1, puzzle>>NUM_LO_BITS, partsUsed);
    return;
  }

  switch(start) {
   case 4:
      if (partsUsed & 0x2l) goto piece2at4;
      if(!(puzzle & 0x480090010ll))
        doSolve(5, 0x20ll, puzzle | 0x480090010ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x400480090ll))
        doSolve(5, 0x20ll, puzzle | 0x400480090ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1330ll))
        doSolve(5, 0x20ll, puzzle | 0x1330ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1990ll))
        doSolve(5, 0x20ll, puzzle | 0x1990ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x40060030ll))
        doSolve(5, 0x20ll, puzzle | 0x40060030ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x60030010ll))
        doSolve(5, 0x20ll, puzzle | 0x60030010ll,
                   partsUsed | 0x2l);
    piece2at4:
      if (partsUsed & 0x4l) goto piece3at4;
      if(!(puzzle & 0x10490010ll))
        doSolve(5, 0x20ll, puzzle | 0x10490010ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x10070010ll))
        doSolve(5, 0x20ll, puzzle | 0x10070010ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x80080490ll))
        doSolve(5, 0x20ll, puzzle | 0x80080490ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x790ll))
        doSolve(5, 0x20ll, puzzle | 0x790ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x970ll))
        doSolve(5, 0x20ll, puzzle | 0x970ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x20020070ll))
        doSolve(5, 0x20ll, puzzle | 0x20020070ll,
                   partsUsed | 0x4l);
    piece3at4:
      if (partsUsed & 0x8l) goto piece4at4;
      if(!(puzzle & 0x400490010ll))
        doSolve(5, 0x20ll, puzzle | 0x400490010ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x480080090ll))
        doSolve(5, 0x20ll, puzzle | 0x480080090ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1930ll))
        doSolve(5, 0x20ll, puzzle | 0x1930ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1390ll))
        doSolve(5, 0x20ll, puzzle | 0x1390ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x40070010ll))
        doSolve(5, 0x20ll, puzzle | 0x40070010ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x60020030ll))
        doSolve(5, 0x20ll, puzzle | 0x60020030ll,
                   partsUsed | 0x8l);
    piece4at4:
      if (partsUsed & 0x10l) goto piece5at4;
      if (!(puzzle & 0x2490ll)) {
         if(!(puzzle & 0x6590ll))
           doSolve(5, 0x20ll, puzzle | 0x6590ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x2082490ll))
           doSolve(5, 0x20ll, puzzle | 0x2082490ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x2cb0ll))
           doSolve(5, 0x20ll, puzzle | 0x2cb0ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x90010090010ll))
        doSolve(5, 0x20ll, puzzle | 0x90010090010ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x80090080090ll))
        doSolve(5, 0x20ll, puzzle | 0x80090080090ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x412490ll))
        doSolve(5, 0x20ll, puzzle | 0x412490ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x10090010090ll))
        doSolve(5, 0x20ll, puzzle | 0x10090010090ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x4d30ll))
        doSolve(5, 0x20ll, puzzle | 0x4d30ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x2490410ll))
        doSolve(5, 0x20ll, puzzle | 0x2490410ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x30010030010ll))
        doSolve(5, 0x20ll, puzzle | 0x30010030010ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x10030010030ll))
        doSolve(5, 0x20ll, puzzle | 0x10030010030ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x20030020030ll))
        doSolve(5, 0x20ll, puzzle | 0x20030020030ll,
                   partsUsed | 0x10l);
    piece5at4:
      if (partsUsed & 0x20l) goto piece6at4;
      if (!(puzzle & 0x10010010010ll)) {
         if(!(puzzle & 0x10010010090ll))
           doSolve(5, 0x20ll, puzzle | 0x10010010090ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x90010010010ll))
           doSolve(5, 0x20ll, puzzle | 0x90010010010ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x10010010030ll))
           doSolve(5, 0x20ll, puzzle | 0x10010010030ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x30010010010ll))
           doSolve(5, 0x20ll, puzzle | 0x30010010010ll,
                   partsUsed | 0x20l);
      }
      if (!(puzzle & 0x2490ll)) {
         if(!(puzzle & 0x2002490ll))
           doSolve(5, 0x20ll, puzzle | 0x2002490ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x24b0ll))
           doSolve(5, 0x20ll, puzzle | 0x24b0ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x12490ll))
           doSolve(5, 0x20ll, puzzle | 0x12490ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x6490ll))
           doSolve(5, 0x20ll, puzzle | 0x6490ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x2490010ll))
        doSolve(5, 0x20ll, puzzle | 0x2490010ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x4930ll))
        doSolve(5, 0x20ll, puzzle | 0x4930ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x80080080090ll))
        doSolve(5, 0x20ll, puzzle | 0x80080080090ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x20020020030ll))
        doSolve(5, 0x20ll, puzzle | 0x20020020030ll,
                   partsUsed | 0x20l);
    piece6at4:
      if (partsUsed & 0x40l) goto piece7at4;
      if (!(puzzle & 0x10010010010ll)) {
         if(!(puzzle & 0x10010090010ll))
           doSolve(5, 0x20ll, puzzle | 0x10010090010ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x10090010010ll))
           doSolve(5, 0x20ll, puzzle | 0x10090010010ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x10010030010ll))
           doSolve(5, 0x20ll, puzzle | 0x10010030010ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x10030010010ll))
           doSolve(5, 0x20ll, puzzle | 0x10030010010ll,
                   partsUsed | 0x40l);
      }
      if (!(puzzle & 0x2490ll)) {
         if(!(puzzle & 0x402490ll))
           doSolve(5, 0x20ll, puzzle | 0x402490ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2590ll))
           doSolve(5, 0x20ll, puzzle | 0x2590ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x82490ll))
           doSolve(5, 0x20ll, puzzle | 0x82490ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2c90ll))
           doSolve(5, 0x20ll, puzzle | 0x2c90ll,
                   partsUsed | 0x40l);
      }
    piece7at4:
      if (partsUsed & 0x80l) goto piece8at4;
      if(!(puzzle & 0x90010090ll))
        doSolve(5, 0x20ll, puzzle | 0x90010090ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x90080090ll))
        doSolve(5, 0x20ll, puzzle | 0x90080090ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x490410ll))
        doSolve(5, 0x20ll, puzzle | 0x490410ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x410490ll))
        doSolve(5, 0x20ll, puzzle | 0x410490ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x2f0ll))
        doSolve(5, 0x20ll, puzzle | 0x2f0ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xcb0ll))
        doSolve(5, 0x20ll, puzzle | 0xcb0ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xd30ll))
        doSolve(5, 0x20ll, puzzle | 0xd30ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x3d0ll))
        doSolve(5, 0x20ll, puzzle | 0x3d0ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x30020030ll))
        doSolve(5, 0x20ll, puzzle | 0x30020030ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x30010030ll))
        doSolve(5, 0x20ll, puzzle | 0x30010030ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x70050ll))
        doSolve(5, 0x20ll, puzzle | 0x70050ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x50070ll))
        doSolve(5, 0x20ll, puzzle | 0x50070ll,
                   partsUsed | 0x80l);
    piece8at4:
      if (partsUsed & 0x100l) goto piece9at4;
      if(!(puzzle & 0x100130ll))
        doSolve(5, 0x20ll, puzzle | 0x100130ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x130010ll))
        doSolve(5, 0x20ll, puzzle | 0x130010ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x200b0ll))
        doSolve(5, 0x20ll, puzzle | 0x200b0ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x90030ll))
        doSolve(5, 0x20ll, puzzle | 0x90030ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x180090ll))
        doSolve(5, 0x20ll, puzzle | 0x180090ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x10190ll))
        doSolve(5, 0x20ll, puzzle | 0x10190ll,
                   partsUsed | 0x100l);
    piece9at4:
      if (partsUsed & 0x200l) goto piece10at4;
      if(!(puzzle & 0x80490010ll))
        doSolve(5, 0x20ll, puzzle | 0x80490010ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x80480090ll))
        doSolve(5, 0x20ll, puzzle | 0x80480090ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0xb30ll))
        doSolve(5, 0x20ll, puzzle | 0xb30ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0xb90ll))
        doSolve(5, 0x20ll, puzzle | 0xb90ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x20060030ll))
        doSolve(5, 0x20ll, puzzle | 0x20060030ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x20070010ll))
        doSolve(5, 0x20ll, puzzle | 0x20070010ll,
                   partsUsed | 0x200l);
    piece10at4:
      if (partsUsed & 0x400l) goto piece11at4;
      if (!(puzzle & 0x80090ll)) {
         if(!(puzzle & 0x10090090ll))
           doSolve(5, 0x20ll, puzzle | 0x10090090ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x490090ll))
           doSolve(5, 0x20ll, puzzle | 0x490090ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x480490ll))
           doSolve(5, 0x20ll, puzzle | 0x480490ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x90490ll))
           doSolve(5, 0x20ll, puzzle | 0x90490ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x80090090ll))
           doSolve(5, 0x20ll, puzzle | 0x80090090ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x130ll)) {
         if(!(puzzle & 0x370ll))
           doSolve(5, 0x20ll, puzzle | 0x370ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1f0ll))
           doSolve(5, 0x20ll, puzzle | 0x1f0ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x5b0ll))
           doSolve(5, 0x20ll, puzzle | 0x5b0ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x9b0ll))
           doSolve(5, 0x20ll, puzzle | 0x9b0ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x3b0ll))
           doSolve(5, 0x20ll, puzzle | 0x3b0ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x10010010ll)) {
         if(!(puzzle & 0x90090010ll))
           doSolve(5, 0x20ll, puzzle | 0x90090010ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x10030030ll))
           doSolve(5, 0x20ll, puzzle | 0x10030030ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x30030010ll))
           doSolve(5, 0x20ll, puzzle | 0x30030010ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x20030ll)) {
         if(!(puzzle & 0x20030030ll))
           doSolve(5, 0x20ll, puzzle | 0x20030030ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x70030ll))
           doSolve(5, 0x20ll, puzzle | 0x70030ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x30070ll))
           doSolve(5, 0x20ll, puzzle | 0x30070ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x60070ll))
           doSolve(5, 0x20ll, puzzle | 0x60070ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0xd90ll))
        doSolve(5, 0x20ll, puzzle | 0xd90ll,
                   partsUsed | 0x400l);
    piece11at4:
      if (partsUsed & 0x800l) goto piece12at4;
      if (!(puzzle & 0x10030ll)) {
         if(!(puzzle & 0x101b0ll))
           doSolve(5, 0x20ll, puzzle | 0x101b0ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xb0030ll))
           doSolve(5, 0x20ll, puzzle | 0xb0030ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x300b0ll))
           doSolve(5, 0x20ll, puzzle | 0x300b0ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x30130ll))
           doSolve(5, 0x20ll, puzzle | 0x30130ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x130030ll))
           doSolve(5, 0x20ll, puzzle | 0x130030ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x900b0ll))
           doSolve(5, 0x20ll, puzzle | 0x900b0ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x100110ll)) {
         if(!(puzzle & 0x180190ll))
           doSolve(5, 0x20ll, puzzle | 0x180190ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x120130ll))
           doSolve(5, 0x20ll, puzzle | 0x120130ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1001b0ll))
           doSolve(5, 0x20ll, puzzle | 0x1001b0ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x80090ll)) {
         if(!(puzzle & 0xb0090ll))
           doSolve(5, 0x20ll, puzzle | 0xb0090ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x90190ll))
           doSolve(5, 0x20ll, puzzle | 0x90190ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x801b0ll))
           doSolve(5, 0x20ll, puzzle | 0x801b0ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x190090ll))
           doSolve(5, 0x20ll, puzzle | 0x190090ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0x201b0ll))
        doSolve(5, 0x20ll, puzzle | 0x201b0ll,
                   partsUsed | 0x800l);
      if(!(puzzle & 0x1b0010ll))
        doSolve(5, 0x20ll, puzzle | 0x1b0010ll,
                   partsUsed | 0x800l);
    piece12at4:
    break;
   case 5:
      if (partsUsed & 0x2l) goto piece2at5;
      if(!(puzzle & 0x5e0ll))
        doSolve(6, 0x40ll, puzzle | 0x5e0ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x10030060ll))
        doSolve(6, 0x40ll, puzzle | 0x10030060ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x900120020ll))
        doSolve(6, 0x40ll, puzzle | 0x900120020ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x800900120ll))
        doSolve(6, 0x40ll, puzzle | 0x800900120ll,
                   partsUsed | 0x2l);
    piece2at5:
      if (partsUsed & 0x4l) goto piece3at5;
      if(!(puzzle & 0x1d20ll))
        doSolve(6, 0x40ll, puzzle | 0x1d20ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x100100920ll))
        doSolve(6, 0x40ll, puzzle | 0x100100920ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x70020020ll))
        doSolve(6, 0x40ll, puzzle | 0x70020020ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x20920020ll))
        doSolve(6, 0x40ll, puzzle | 0x20920020ll,
                   partsUsed | 0x4l);
    piece3at5:
      if (partsUsed & 0x8l) goto piece4at5;
      if(!(puzzle & 0xd60ll))
        doSolve(6, 0x40ll, puzzle | 0xd60ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x30020060ll))
        doSolve(6, 0x40ll, puzzle | 0x30020060ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x800920020ll))
        doSolve(6, 0x40ll, puzzle | 0x800920020ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x900100120ll))
        doSolve(6, 0x40ll, puzzle | 0x900100120ll,
                   partsUsed | 0x8l);
    piece4at5:
      if (partsUsed & 0x10l) goto piece5at5;
      if (!(puzzle & 0x4920ll)) {
         if(!(puzzle & 0x69a0ll))
           doSolve(6, 0x40ll, puzzle | 0x69a0ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x824920ll))
           doSolve(6, 0x40ll, puzzle | 0x824920ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0xcb20ll))
           doSolve(6, 0x40ll, puzzle | 0xcb20ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x4104920ll))
           doSolve(6, 0x40ll, puzzle | 0x4104920ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x5960ll))
           doSolve(6, 0x40ll, puzzle | 0x5960ll,
                   partsUsed | 0x10l);
      }
      if (!(puzzle & 0x20020020020ll)) {
         if(!(puzzle & 0x30020030020ll))
           doSolve(6, 0x40ll, puzzle | 0x30020030020ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x120020120020ll))
           doSolve(6, 0x40ll, puzzle | 0x120020120020ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x20120020120ll))
           doSolve(6, 0x40ll, puzzle | 0x20120020120ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x60020060020ll))
           doSolve(6, 0x40ll, puzzle | 0x60020060020ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x20060020060ll))
           doSolve(6, 0x40ll, puzzle | 0x20060020060ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x100120100120ll))
        doSolve(6, 0x40ll, puzzle | 0x100120100120ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x9a60ll))
        doSolve(6, 0x40ll, puzzle | 0x9a60ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x4920820ll))
        doSolve(6, 0x40ll, puzzle | 0x4920820ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x40060040060ll))
        doSolve(6, 0x40ll, puzzle | 0x40060040060ll,
                   partsUsed | 0x10l);
    piece5at5:
      if (partsUsed & 0x20l) goto piece6at5;
      if (!(puzzle & 0x4920ll)) {
         if(!(puzzle & 0x6920ll))
           doSolve(6, 0x40ll, puzzle | 0x6920ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x4004920ll))
           doSolve(6, 0x40ll, puzzle | 0x4004920ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x4960ll))
           doSolve(6, 0x40ll, puzzle | 0x4960ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x24920ll))
           doSolve(6, 0x40ll, puzzle | 0x24920ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0xc920ll))
           doSolve(6, 0x40ll, puzzle | 0xc920ll,
                   partsUsed | 0x20l);
      }
      if (!(puzzle & 0x20020020020ll)) {
         if(!(puzzle & 0x30020020020ll))
           doSolve(6, 0x40ll, puzzle | 0x30020020020ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x20020020120ll))
           doSolve(6, 0x40ll, puzzle | 0x20020020120ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x120020020020ll))
           doSolve(6, 0x40ll, puzzle | 0x120020020020ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x20020020060ll))
           doSolve(6, 0x40ll, puzzle | 0x20020020060ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x60020020020ll))
           doSolve(6, 0x40ll, puzzle | 0x60020020020ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x4920020ll))
        doSolve(6, 0x40ll, puzzle | 0x4920020ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x9260ll))
        doSolve(6, 0x40ll, puzzle | 0x9260ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x100100100120ll))
        doSolve(6, 0x40ll, puzzle | 0x100100100120ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x40040040060ll))
        doSolve(6, 0x40ll, puzzle | 0x40040040060ll,
                   partsUsed | 0x20l);
    piece6at5:
      if (partsUsed & 0x40l) goto piece7at5;
      if (!(puzzle & 0x4920ll)) {
         if(!(puzzle & 0x49a0ll))
           doSolve(6, 0x40ll, puzzle | 0x49a0ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4d20ll))
           doSolve(6, 0x40ll, puzzle | 0x4d20ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x804920ll))
           doSolve(6, 0x40ll, puzzle | 0x804920ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4b20ll))
           doSolve(6, 0x40ll, puzzle | 0x4b20ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x104920ll))
           doSolve(6, 0x40ll, puzzle | 0x104920ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x5920ll))
           doSolve(6, 0x40ll, puzzle | 0x5920ll,
                   partsUsed | 0x40l);
      }
      if (!(puzzle & 0x20020020020ll)) {
         if(!(puzzle & 0x20020030020ll))
           doSolve(6, 0x40ll, puzzle | 0x20020030020ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x20030020020ll))
           doSolve(6, 0x40ll, puzzle | 0x20030020020ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x20020120020ll))
           doSolve(6, 0x40ll, puzzle | 0x20020120020ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x20120020020ll))
           doSolve(6, 0x40ll, puzzle | 0x20120020020ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x20020060020ll))
           doSolve(6, 0x40ll, puzzle | 0x20020060020ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x20060020020ll))
           doSolve(6, 0x40ll, puzzle | 0x20060020020ll,
                   partsUsed | 0x40l);
      }
    piece7at5:
      if (partsUsed & 0x80l) goto piece8at5;
      if(!(puzzle & 0x120020120ll))
        doSolve(6, 0x40ll, puzzle | 0x120020120ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x120100120ll))
        doSolve(6, 0x40ll, puzzle | 0x120100120ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x920820ll))
        doSolve(6, 0x40ll, puzzle | 0x920820ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x820920ll))
        doSolve(6, 0x40ll, puzzle | 0x820920ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1960ll))
        doSolve(6, 0x40ll, puzzle | 0x1960ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1a60ll))
        doSolve(6, 0x40ll, puzzle | 0x1a60ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x60040060ll))
        doSolve(6, 0x40ll, puzzle | 0x60040060ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x60020060ll))
        doSolve(6, 0x40ll, puzzle | 0x60020060ll,
                   partsUsed | 0x80l);
    piece8at5:
      if (partsUsed & 0x100l) goto piece9at5;
      if(!(puzzle & 0x801a0ll))
        doSolve(6, 0x40ll, puzzle | 0x801a0ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x30120ll))
        doSolve(6, 0x40ll, puzzle | 0x30120ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x20320ll))
        doSolve(6, 0x40ll, puzzle | 0x20320ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1a0020ll))
        doSolve(6, 0x40ll, puzzle | 0x1a0020ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x120060ll))
        doSolve(6, 0x40ll, puzzle | 0x120060ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x260020ll))
        doSolve(6, 0x40ll, puzzle | 0x260020ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x40160ll))
        doSolve(6, 0x40ll, puzzle | 0x40160ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x300120ll))
        doSolve(6, 0x40ll, puzzle | 0x300120ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x200260ll))
        doSolve(6, 0x40ll, puzzle | 0x200260ll,
                   partsUsed | 0x100l);
    piece9at5:
      if (partsUsed & 0x200l) goto piece10at5;
      if (!(puzzle & 0x1a0ll)) {
         if(!(puzzle & 0x13a0ll))
           doSolve(6, 0x40ll, puzzle | 0x13a0ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x7a0ll))
           doSolve(6, 0x40ll, puzzle | 0x7a0ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x9e0ll))
           doSolve(6, 0x40ll, puzzle | 0x9e0ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x19a0ll))
           doSolve(6, 0x40ll, puzzle | 0x19a0ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x20020020ll)) {
         if(!(puzzle & 0x20030060ll))
           doSolve(6, 0x40ll, puzzle | 0x20030060ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x60030020ll))
           doSolve(6, 0x40ll, puzzle | 0x60030020ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x30060020ll))
           doSolve(6, 0x40ll, puzzle | 0x30060020ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0xf20ll))
        doSolve(6, 0x40ll, puzzle | 0xf20ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x10070020ll))
        doSolve(6, 0x40ll, puzzle | 0x10070020ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x40070020ll))
        doSolve(6, 0x40ll, puzzle | 0x40070020ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100920020ll))
        doSolve(6, 0x40ll, puzzle | 0x100920020ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100900120ll))
        doSolve(6, 0x40ll, puzzle | 0x100900120ll,
                   partsUsed | 0x200l);
    piece10at5:
      if (partsUsed & 0x400l) goto piece11at5;
      if (!(puzzle & 0x100120ll)) {
         if(!(puzzle & 0x20120120ll))
           doSolve(6, 0x40ll, puzzle | 0x20120120ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x920120ll))
           doSolve(6, 0x40ll, puzzle | 0x920120ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x900920ll))
           doSolve(6, 0x40ll, puzzle | 0x900920ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x120920ll))
           doSolve(6, 0x40ll, puzzle | 0x120920ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x100120120ll))
           doSolve(6, 0x40ll, puzzle | 0x100120120ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x20020020ll)) {
         if(!(puzzle & 0x120120020ll))
           doSolve(6, 0x40ll, puzzle | 0x120120020ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x20060060ll))
           doSolve(6, 0x40ll, puzzle | 0x20060060ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x60060020ll))
           doSolve(6, 0x40ll, puzzle | 0x60060020ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x320ll)) {
         if(!(puzzle & 0xb60ll))
           doSolve(6, 0x40ll, puzzle | 0xb60ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1360ll))
           doSolve(6, 0x40ll, puzzle | 0x1360ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1b20ll))
           doSolve(6, 0x40ll, puzzle | 0x1b20ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x3e0ll))
        doSolve(6, 0x40ll, puzzle | 0x3e0ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0xda0ll))
        doSolve(6, 0x40ll, puzzle | 0xda0ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x70060ll))
        doSolve(6, 0x40ll, puzzle | 0x70060ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x30030020ll))
        doSolve(6, 0x40ll, puzzle | 0x30030020ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x40060060ll))
        doSolve(6, 0x40ll, puzzle | 0x40060060ll,
                   partsUsed | 0x400l);
    piece11at5:
      if (partsUsed & 0x800l) goto piece12at5;
      if (!(puzzle & 0x180020ll)) {
         if(!(puzzle & 0x1b0020ll))
           doSolve(6, 0x40ll, puzzle | 0x1b0020ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1a0120ll))
           doSolve(6, 0x40ll, puzzle | 0x1a0120ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1801a0ll))
           doSolve(6, 0x40ll, puzzle | 0x1801a0ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x20120ll)) {
         if(!(puzzle & 0x130120ll))
           doSolve(6, 0x40ll, puzzle | 0x130120ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1201a0ll))
           doSolve(6, 0x40ll, puzzle | 0x1201a0ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x20360ll))
           doSolve(6, 0x40ll, puzzle | 0x20360ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x160120ll))
           doSolve(6, 0x40ll, puzzle | 0x160120ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x60160ll))
           doSolve(6, 0x40ll, puzzle | 0x60160ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x120320ll))
           doSolve(6, 0x40ll, puzzle | 0x120320ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x320120ll))
           doSolve(6, 0x40ll, puzzle | 0x320120ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x120160ll))
           doSolve(6, 0x40ll, puzzle | 0x120160ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x40060ll)) {
         if(!(puzzle & 0x160060ll))
           doSolve(6, 0x40ll, puzzle | 0x160060ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x240260ll))
           doSolve(6, 0x40ll, puzzle | 0x240260ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x60260ll))
           doSolve(6, 0x40ll, puzzle | 0x60260ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x260060ll))
           doSolve(6, 0x40ll, puzzle | 0x260060ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x40360ll))
           doSolve(6, 0x40ll, puzzle | 0x40360ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x320ll)) {
         if(!(puzzle & 0x300320ll))
           doSolve(6, 0x40ll, puzzle | 0x300320ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x200360ll))
           doSolve(6, 0x40ll, puzzle | 0x200360ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x100360ll))
           doSolve(6, 0x40ll, puzzle | 0x100360ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0x360020ll))
        doSolve(6, 0x40ll, puzzle | 0x360020ll,
                   partsUsed | 0x800l);
    piece12at5:
    break;
   case 6:
      if (partsUsed & 0x2l) goto piece2at6;
      if(!(puzzle & 0xf40ll))
        doSolve(7, 0x80ll, puzzle | 0xf40ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x30060040ll))
        doSolve(7, 0x80ll, puzzle | 0x30060040ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1200240040ll))
        doSolve(7, 0x80ll, puzzle | 0x1200240040ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1001200240ll))
        doSolve(7, 0x80ll, puzzle | 0x1001200240ll,
                   partsUsed | 0x2l);
    piece2at6:
      if (partsUsed & 0x4l) goto piece3at6;
      if(!(puzzle & 0x13c0ll))
        doSolve(7, 0x80ll, puzzle | 0x13c0ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x200201240ll))
        doSolve(7, 0x80ll, puzzle | 0x200201240ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x40070040ll))
        doSolve(7, 0x80ll, puzzle | 0x40070040ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x41240040ll))
        doSolve(7, 0x80ll, puzzle | 0x41240040ll,
                   partsUsed | 0x4l);
    piece3at6:
      if (partsUsed & 0x8l) goto piece4at6;
      if(!(puzzle & 0x7c0ll))
        doSolve(7, 0x80ll, puzzle | 0x7c0ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x10070040ll))
        doSolve(7, 0x80ll, puzzle | 0x10070040ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1001240040ll))
        doSolve(7, 0x80ll, puzzle | 0x1001240040ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1200200240ll))
        doSolve(7, 0x80ll, puzzle | 0x1200200240ll,
                   partsUsed | 0x8l);
    piece4at6:
      if (partsUsed & 0x10l) goto piece5at6;
      if (!(puzzle & 0x9240ll)) {
         if(!(puzzle & 0xd340ll))
           doSolve(7, 0x80ll, puzzle | 0xd340ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x1049240ll))
           doSolve(7, 0x80ll, puzzle | 0x1049240ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x8209240ll))
           doSolve(7, 0x80ll, puzzle | 0x8209240ll,
                   partsUsed | 0x10l);
      }
      if (!(puzzle & 0x40040040040ll)) {
         if(!(puzzle & 0x60040060040ll))
           doSolve(7, 0x80ll, puzzle | 0x60040060040ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x240040240040ll))
           doSolve(7, 0x80ll, puzzle | 0x240040240040ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x40240040240ll))
           doSolve(7, 0x80ll, puzzle | 0x40240040240ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x200240200240ll))
        doSolve(7, 0x80ll, puzzle | 0x200240200240ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x9241040ll))
        doSolve(7, 0x80ll, puzzle | 0x9241040ll,
                   partsUsed | 0x10l);
    piece5at6:
      if (partsUsed & 0x20l) goto piece6at6;
      if (!(puzzle & 0x9240ll)) {
         if(!(puzzle & 0xd240ll))
           doSolve(7, 0x80ll, puzzle | 0xd240ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x8009240ll))
           doSolve(7, 0x80ll, puzzle | 0x8009240ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x49240ll))
           doSolve(7, 0x80ll, puzzle | 0x49240ll,
                   partsUsed | 0x20l);
      }
      if (!(puzzle & 0x40040040040ll)) {
         if(!(puzzle & 0x60040040040ll))
           doSolve(7, 0x80ll, puzzle | 0x60040040040ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x40040040240ll))
           doSolve(7, 0x80ll, puzzle | 0x40040040240ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x240040040040ll))
           doSolve(7, 0x80ll, puzzle | 0x240040040040ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x9240040ll))
        doSolve(7, 0x80ll, puzzle | 0x9240040ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x200200200240ll))
        doSolve(7, 0x80ll, puzzle | 0x200200200240ll,
                   partsUsed | 0x20l);
    piece6at6:
      if (partsUsed & 0x40l) goto piece7at6;
      if (!(puzzle & 0x9240ll)) {
         if(!(puzzle & 0x9340ll))
           doSolve(7, 0x80ll, puzzle | 0x9340ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x9a40ll))
           doSolve(7, 0x80ll, puzzle | 0x9a40ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x1009240ll))
           doSolve(7, 0x80ll, puzzle | 0x1009240ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x209240ll))
           doSolve(7, 0x80ll, puzzle | 0x209240ll,
                   partsUsed | 0x40l);
      }
      if (!(puzzle & 0x40040040040ll)) {
         if(!(puzzle & 0x40040060040ll))
           doSolve(7, 0x80ll, puzzle | 0x40040060040ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x40060040040ll))
           doSolve(7, 0x80ll, puzzle | 0x40060040040ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x40040240040ll))
           doSolve(7, 0x80ll, puzzle | 0x40040240040ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x40240040040ll))
           doSolve(7, 0x80ll, puzzle | 0x40240040040ll,
                   partsUsed | 0x40l);
      }
    piece7at6:
      if (partsUsed & 0x80l) goto piece8at6;
      if(!(puzzle & 0x240040240ll))
        doSolve(7, 0x80ll, puzzle | 0x240040240ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x240200240ll))
        doSolve(7, 0x80ll, puzzle | 0x240200240ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1241040ll))
        doSolve(7, 0x80ll, puzzle | 0x1241040ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1041240ll))
        doSolve(7, 0x80ll, puzzle | 0x1041240ll,
                   partsUsed | 0x80l);
    piece8at6:
      if (partsUsed & 0x100l) goto piece9at6;
      if(!(puzzle & 0x340040ll))
        doSolve(7, 0x80ll, puzzle | 0x340040ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x100340ll))
        doSolve(7, 0x80ll, puzzle | 0x100340ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x60240ll))
        doSolve(7, 0x80ll, puzzle | 0x60240ll,
                   partsUsed | 0x100l);
    piece9at6:
      if (partsUsed & 0x200l) goto piece10at6;
      if(!(puzzle & 0xbc0ll))
        doSolve(7, 0x80ll, puzzle | 0xbc0ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x20070040ll))
        doSolve(7, 0x80ll, puzzle | 0x20070040ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x201240040ll))
        doSolve(7, 0x80ll, puzzle | 0x201240040ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x201200240ll))
        doSolve(7, 0x80ll, puzzle | 0x201200240ll,
                   partsUsed | 0x200l);
    piece10at6:
      if (partsUsed & 0x400l) goto piece11at6;
      if (!(puzzle & 0x1240ll)) {
         if(!(puzzle & 0x1b40ll))
           doSolve(7, 0x80ll, puzzle | 0x1b40ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1201240ll))
           doSolve(7, 0x80ll, puzzle | 0x1201240ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x241240ll))
           doSolve(7, 0x80ll, puzzle | 0x241240ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x40040040ll)) {
         if(!(puzzle & 0x60060040ll))
           doSolve(7, 0x80ll, puzzle | 0x60060040ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x40240240ll))
           doSolve(7, 0x80ll, puzzle | 0x40240240ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x240240040ll))
           doSolve(7, 0x80ll, puzzle | 0x240240040ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x1240240ll))
        doSolve(7, 0x80ll, puzzle | 0x1240240ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x200240240ll))
        doSolve(7, 0x80ll, puzzle | 0x200240240ll,
                   partsUsed | 0x400l);
    piece11at6:
      if (partsUsed & 0x800l) goto piece12at6;
      if (!(puzzle & 0x300040ll)) {
         if(!(puzzle & 0x360040ll))
           doSolve(7, 0x80ll, puzzle | 0x360040ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x340240ll))
           doSolve(7, 0x80ll, puzzle | 0x340240ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x300340ll))
           doSolve(7, 0x80ll, puzzle | 0x300340ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0x260240ll))
        doSolve(7, 0x80ll, puzzle | 0x260240ll,
                   partsUsed | 0x800l);
      if(!(puzzle & 0x240340ll))
        doSolve(7, 0x80ll, puzzle | 0x240340ll,
                   partsUsed | 0x800l);
    piece12at6:
    break;
   case 7:
      if (partsUsed & 0x2l) goto piece2at7;
      if(!(puzzle & 0x10090480ll))
        doSolve(8, 0x100ll, puzzle | 0x10090480ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x2400480080ll))
        doSolve(8, 0x100ll, puzzle | 0x2400480080ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x2002400480ll))
        doSolve(8, 0x100ll, puzzle | 0x2002400480ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x9980ll))
        doSolve(8, 0x100ll, puzzle | 0x9980ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0xcc80ll))
        doSolve(8, 0x100ll, puzzle | 0xcc80ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x200300180ll))
        doSolve(8, 0x100ll, puzzle | 0x200300180ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x300180080ll))
        doSolve(8, 0x100ll, puzzle | 0x300180080ll,
                   partsUsed | 0x2l);
    piece2at7:
      if (partsUsed & 0x4l) goto piece3at7;
      if (!(puzzle & 0x80080080ll)) {
         if(!(puzzle & 0x490080080ll))
           doSolve(8, 0x100ll, puzzle | 0x490080080ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x82480080ll))
           doSolve(8, 0x100ll, puzzle | 0x82480080ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x80380080ll))
           doSolve(8, 0x100ll, puzzle | 0x80380080ll,
                   partsUsed | 0x4l);
      }
      if(!(puzzle & 0x400402480ll))
        doSolve(8, 0x100ll, puzzle | 0x400402480ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x3c80ll))
        doSolve(8, 0x100ll, puzzle | 0x3c80ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x4b80ll))
        doSolve(8, 0x100ll, puzzle | 0x4b80ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x100100380ll))
        doSolve(8, 0x100ll, puzzle | 0x100100380ll,
                   partsUsed | 0x4l);
    piece3at7:
      if (partsUsed & 0x8l) goto piece4at7;
      if(!(puzzle & 0x90080480ll))
        doSolve(8, 0x100ll, puzzle | 0x90080480ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x2002480080ll))
        doSolve(8, 0x100ll, puzzle | 0x2002480080ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x2400400480ll))
        doSolve(8, 0x100ll, puzzle | 0x2400400480ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0xc980ll))
        doSolve(8, 0x100ll, puzzle | 0xc980ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x9c80ll))
        doSolve(8, 0x100ll, puzzle | 0x9c80ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x200380080ll))
        doSolve(8, 0x100ll, puzzle | 0x200380080ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x300100180ll))
        doSolve(8, 0x100ll, puzzle | 0x300100180ll,
                   partsUsed | 0x8l);
    piece4at7:
      if (partsUsed & 0x10l) goto piece5at7;
      if (!(puzzle & 0x80080080080ll)) {
         if(!(puzzle & 0x80480080480ll))
           doSolve(8, 0x100ll, puzzle | 0x80480080480ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x180080180080ll))
           doSolve(8, 0x100ll, puzzle | 0x180080180080ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x80180080180ll))
           doSolve(8, 0x100ll, puzzle | 0x80180080180ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x2492080ll))
        doSolve(8, 0x100ll, puzzle | 0x2492080ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x90080090080ll))
        doSolve(8, 0x100ll, puzzle | 0x90080090080ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x480080480080ll))
        doSolve(8, 0x100ll, puzzle | 0x480080480080ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x400480400480ll))
        doSolve(8, 0x100ll, puzzle | 0x400480400480ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x100180100180ll))
        doSolve(8, 0x100ll, puzzle | 0x100180100180ll,
                   partsUsed | 0x10l);
    piece5at7:
      if (partsUsed & 0x20l) goto piece6at7;
      if (!(puzzle & 0x80080080080ll)) {
         if(!(puzzle & 0x90080080080ll))
           doSolve(8, 0x100ll, puzzle | 0x90080080080ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x80080080480ll))
           doSolve(8, 0x100ll, puzzle | 0x80080080480ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x480080080080ll))
           doSolve(8, 0x100ll, puzzle | 0x480080080080ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x80080080180ll))
           doSolve(8, 0x100ll, puzzle | 0x80080080180ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x180080080080ll))
           doSolve(8, 0x100ll, puzzle | 0x180080080080ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x400400400480ll))
        doSolve(8, 0x100ll, puzzle | 0x400400400480ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x100100100180ll))
        doSolve(8, 0x100ll, puzzle | 0x100100100180ll,
                   partsUsed | 0x20l);
    piece6at7:
      if (partsUsed & 0x40l) goto piece7at7;
      if (!(puzzle & 0x80080080080ll)) {
         if(!(puzzle & 0x80090080080ll))
           doSolve(8, 0x100ll, puzzle | 0x80090080080ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x80080480080ll))
           doSolve(8, 0x100ll, puzzle | 0x80080480080ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x80480080080ll))
           doSolve(8, 0x100ll, puzzle | 0x80480080080ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x80080180080ll))
           doSolve(8, 0x100ll, puzzle | 0x80080180080ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x80180080080ll))
           doSolve(8, 0x100ll, puzzle | 0x80180080080ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x2490080ll))
        doSolve(8, 0x100ll, puzzle | 0x2490080ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x80080090080ll))
        doSolve(8, 0x100ll, puzzle | 0x80080090080ll,
                   partsUsed | 0x40l);
    piece7at7:
      if (partsUsed & 0x80l) goto piece8at7;
      if(!(puzzle & 0x480080480ll))
        doSolve(8, 0x100ll, puzzle | 0x480080480ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x480400480ll))
        doSolve(8, 0x100ll, puzzle | 0x480400480ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x2482080ll))
        doSolve(8, 0x100ll, puzzle | 0x2482080ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x2082480ll))
        doSolve(8, 0x100ll, puzzle | 0x2082480ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1780ll))
        doSolve(8, 0x100ll, puzzle | 0x1780ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x6580ll))
        doSolve(8, 0x100ll, puzzle | 0x6580ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x6980ll))
        doSolve(8, 0x100ll, puzzle | 0x6980ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1e80ll))
        doSolve(8, 0x100ll, puzzle | 0x1e80ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x180100180ll))
        doSolve(8, 0x100ll, puzzle | 0x180100180ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x180080180ll))
        doSolve(8, 0x100ll, puzzle | 0x180080180ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x380280ll))
        doSolve(8, 0x100ll, puzzle | 0x380280ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x280380ll))
        doSolve(8, 0x100ll, puzzle | 0x280380ll,
                   partsUsed | 0x80l);
    piece8at7:
      if (partsUsed & 0x100l) goto piece9at7;
      if(!(puzzle & 0x120180ll))
        doSolve(8, 0x100ll, puzzle | 0x120180ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x100580ll))
        doSolve(8, 0x100ll, puzzle | 0x100580ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xb0080ll))
        doSolve(8, 0x100ll, puzzle | 0xb0080ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x980080ll))
        doSolve(8, 0x100ll, puzzle | 0x980080ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x480180ll))
        doSolve(8, 0x100ll, puzzle | 0x480180ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xc00480ll))
        doSolve(8, 0x100ll, puzzle | 0xc00480ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x80c80ll))
        doSolve(8, 0x100ll, puzzle | 0x80c80ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x800980ll))
        doSolve(8, 0x100ll, puzzle | 0x800980ll,
                   partsUsed | 0x100l);
    piece9at7:
      if (partsUsed & 0x200l) goto piece10at7;
      if (!(puzzle & 0x80080080ll)) {
         if(!(puzzle & 0x80090480ll))
           doSolve(8, 0x100ll, puzzle | 0x80090480ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x90480080ll))
           doSolve(8, 0x100ll, puzzle | 0x90480080ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x480090080ll))
           doSolve(8, 0x100ll, puzzle | 0x480090080ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x480080ll)) {
         if(!(puzzle & 0x400490080ll))
           doSolve(8, 0x100ll, puzzle | 0x400490080ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x10490080ll))
           doSolve(8, 0x100ll, puzzle | 0x10490080ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x402480080ll))
           doSolve(8, 0x100ll, puzzle | 0x402480080ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x5980ll))
        doSolve(8, 0x100ll, puzzle | 0x5980ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x5c80ll))
        doSolve(8, 0x100ll, puzzle | 0x5c80ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x402400480ll))
        doSolve(8, 0x100ll, puzzle | 0x402400480ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100300180ll))
        doSolve(8, 0x100ll, puzzle | 0x100300180ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100380080ll))
        doSolve(8, 0x100ll, puzzle | 0x100380080ll,
                   partsUsed | 0x200l);
    piece10at7:
      if (partsUsed & 0x400l) goto piece11at7;
      if (!(puzzle & 0x400480ll)) {
         if(!(puzzle & 0x80480480ll))
           doSolve(8, 0x100ll, puzzle | 0x80480480ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2480480ll))
           doSolve(8, 0x100ll, puzzle | 0x2480480ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2402480ll))
           doSolve(8, 0x100ll, puzzle | 0x2402480ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x482480ll))
           doSolve(8, 0x100ll, puzzle | 0x482480ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x400480480ll))
           doSolve(8, 0x100ll, puzzle | 0x400480480ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x980ll)) {
         if(!(puzzle & 0x1b80ll))
           doSolve(8, 0x100ll, puzzle | 0x1b80ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xf80ll))
           doSolve(8, 0x100ll, puzzle | 0xf80ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2d80ll))
           doSolve(8, 0x100ll, puzzle | 0x2d80ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x4d80ll))
           doSolve(8, 0x100ll, puzzle | 0x4d80ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1d80ll))
           doSolve(8, 0x100ll, puzzle | 0x1d80ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x80080080ll)) {
         if(!(puzzle & 0x480480080ll))
           doSolve(8, 0x100ll, puzzle | 0x480480080ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x80180180ll))
           doSolve(8, 0x100ll, puzzle | 0x80180180ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x180180080ll))
           doSolve(8, 0x100ll, puzzle | 0x180180080ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x100180ll)) {
         if(!(puzzle & 0x100180180ll))
           doSolve(8, 0x100ll, puzzle | 0x100180180ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x380180ll))
           doSolve(8, 0x100ll, puzzle | 0x380180ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x180380ll))
           doSolve(8, 0x100ll, puzzle | 0x180380ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x300380ll))
           doSolve(8, 0x100ll, puzzle | 0x300380ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x90090080ll))
        doSolve(8, 0x100ll, puzzle | 0x90090080ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x490480ll))
        doSolve(8, 0x100ll, puzzle | 0x490480ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x6c80ll))
        doSolve(8, 0x100ll, puzzle | 0x6c80ll,
                   partsUsed | 0x400l);
    piece11at7:
      if (partsUsed & 0x800l) goto piece12at7;
      if (!(puzzle & 0x180080ll)) {
         if(!(puzzle & 0x1b0080ll))
           doSolve(8, 0x100ll, puzzle | 0x1b0080ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1a0180ll))
           doSolve(8, 0x100ll, puzzle | 0x1a0180ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x190180ll))
           doSolve(8, 0x100ll, puzzle | 0x190180ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x580180ll))
           doSolve(8, 0x100ll, puzzle | 0x580180ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x580480ll))
           doSolve(8, 0x100ll, puzzle | 0x580480ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x180580ll))
           doSolve(8, 0x100ll, puzzle | 0x180580ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x180980ll))
           doSolve(8, 0x100ll, puzzle | 0x180980ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x980180ll))
           doSolve(8, 0x100ll, puzzle | 0x980180ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xd80080ll))
           doSolve(8, 0x100ll, puzzle | 0xd80080ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0xc80ll)) {
         if(!(puzzle & 0x80d80ll))
           doSolve(8, 0x100ll, puzzle | 0x80d80ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xc00c80ll))
           doSolve(8, 0x100ll, puzzle | 0xc00c80ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x480c80ll))
           doSolve(8, 0x100ll, puzzle | 0x480c80ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x800d80ll))
           doSolve(8, 0x100ll, puzzle | 0x800d80ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x100d80ll))
           doSolve(8, 0x100ll, puzzle | 0x100d80ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x400d80ll))
           doSolve(8, 0x100ll, puzzle | 0x400d80ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0x900980ll))
        doSolve(8, 0x100ll, puzzle | 0x900980ll,
                   partsUsed | 0x800l);
      if(!(puzzle & 0xc80480ll))
        doSolve(8, 0x100ll, puzzle | 0xc80480ll,
                   partsUsed | 0x800l);
      if(!(puzzle & 0x480580ll))
        doSolve(8, 0x100ll, puzzle | 0x480580ll,
                   partsUsed | 0x800l);
    piece12at7:
    break;
   case 8:
      if (partsUsed & 0x2l) goto piece2at8;
      if(!(puzzle & 0x20120900ll))
        doSolve(9, 0x200ll, puzzle | 0x20120900ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x2f00ll))
        doSolve(9, 0x200ll, puzzle | 0x2f00ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x80180300ll))
        doSolve(9, 0x200ll, puzzle | 0x80180300ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x4800900100ll))
        doSolve(9, 0x200ll, puzzle | 0x4800900100ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x4004800900ll))
        doSolve(9, 0x200ll, puzzle | 0x4004800900ll,
                   partsUsed | 0x2l);
    piece2at8:
      if (partsUsed & 0x4l) goto piece3at8;
      if (!(puzzle & 0x100100100ll)) {
         if(!(puzzle & 0x920100100ll))
           doSolve(9, 0x200ll, puzzle | 0x920100100ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x380100100ll))
           doSolve(9, 0x200ll, puzzle | 0x380100100ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x104900100ll))
           doSolve(9, 0x200ll, puzzle | 0x104900100ll,
                   partsUsed | 0x4l);
      }
      if(!(puzzle & 0xe900ll))
        doSolve(9, 0x200ll, puzzle | 0xe900ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x800804900ll))
        doSolve(9, 0x200ll, puzzle | 0x800804900ll,
                   partsUsed | 0x4l);
    piece3at8:
      if (partsUsed & 0x8l) goto piece4at8;
      if(!(puzzle & 0x120100900ll))
        doSolve(9, 0x200ll, puzzle | 0x120100900ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x180100300ll))
        doSolve(9, 0x200ll, puzzle | 0x180100300ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x6b00ll))
        doSolve(9, 0x200ll, puzzle | 0x6b00ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x4004900100ll))
        doSolve(9, 0x200ll, puzzle | 0x4004900100ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x4800800900ll))
        doSolve(9, 0x200ll, puzzle | 0x4800800900ll,
                   partsUsed | 0x8l);
    piece4at8:
      if (partsUsed & 0x10l) goto piece5at8;
      if (!(puzzle & 0x100100100100ll)) {
         if(!(puzzle & 0x180100180100ll))
           doSolve(9, 0x200ll, puzzle | 0x180100180100ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x900100900100ll))
           doSolve(9, 0x200ll, puzzle | 0x900100900100ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x100900100900ll))
           doSolve(9, 0x200ll, puzzle | 0x100900100900ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x300100300100ll))
           doSolve(9, 0x200ll, puzzle | 0x300100300100ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x100300100300ll))
           doSolve(9, 0x200ll, puzzle | 0x100300100300ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x4924100ll))
        doSolve(9, 0x200ll, puzzle | 0x4924100ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x120100120100ll))
        doSolve(9, 0x200ll, puzzle | 0x120100120100ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x800900800900ll))
        doSolve(9, 0x200ll, puzzle | 0x800900800900ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x200300200300ll))
        doSolve(9, 0x200ll, puzzle | 0x200300200300ll,
                   partsUsed | 0x10l);
    piece5at8:
      if (partsUsed & 0x20l) goto piece6at8;
      if (!(puzzle & 0x100100100100ll)) {
         if(!(puzzle & 0x120100100100ll))
           doSolve(9, 0x200ll, puzzle | 0x120100100100ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x180100100100ll))
           doSolve(9, 0x200ll, puzzle | 0x180100100100ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x100100100900ll))
           doSolve(9, 0x200ll, puzzle | 0x100100100900ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x900100100100ll))
           doSolve(9, 0x200ll, puzzle | 0x900100100100ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x100100100300ll))
           doSolve(9, 0x200ll, puzzle | 0x100100100300ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x300100100100ll))
           doSolve(9, 0x200ll, puzzle | 0x300100100100ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x800800800900ll))
        doSolve(9, 0x200ll, puzzle | 0x800800800900ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x200200200300ll))
        doSolve(9, 0x200ll, puzzle | 0x200200200300ll,
                   partsUsed | 0x20l);
    piece6at8:
      if (partsUsed & 0x40l) goto piece7at8;
      if (!(puzzle & 0x100100100100ll)) {
         if(!(puzzle & 0x100120100100ll))
           doSolve(9, 0x200ll, puzzle | 0x100120100100ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100100180100ll))
           doSolve(9, 0x200ll, puzzle | 0x100100180100ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100180100100ll))
           doSolve(9, 0x200ll, puzzle | 0x100180100100ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100100900100ll))
           doSolve(9, 0x200ll, puzzle | 0x100100900100ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100900100100ll))
           doSolve(9, 0x200ll, puzzle | 0x100900100100ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100100300100ll))
           doSolve(9, 0x200ll, puzzle | 0x100100300100ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100300100100ll))
           doSolve(9, 0x200ll, puzzle | 0x100300100100ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x4920100ll))
        doSolve(9, 0x200ll, puzzle | 0x4920100ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x100100120100ll))
        doSolve(9, 0x200ll, puzzle | 0x100100120100ll,
                   partsUsed | 0x40l);
    piece7at8:
      if (partsUsed & 0x80l) goto piece8at8;
      if(!(puzzle & 0x900100900ll))
        doSolve(9, 0x200ll, puzzle | 0x900100900ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x900800900ll))
        doSolve(9, 0x200ll, puzzle | 0x900800900ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x4904100ll))
        doSolve(9, 0x200ll, puzzle | 0x4904100ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x4104900ll))
        doSolve(9, 0x200ll, puzzle | 0x4104900ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xcb00ll))
        doSolve(9, 0x200ll, puzzle | 0xcb00ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xd300ll))
        doSolve(9, 0x200ll, puzzle | 0xd300ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x300200300ll))
        doSolve(9, 0x200ll, puzzle | 0x300200300ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x300100300ll))
        doSolve(9, 0x200ll, puzzle | 0x300100300ll,
                   partsUsed | 0x80l);
    piece8at8:
      if (partsUsed & 0x100l) goto piece9at8;
      if(!(puzzle & 0x190100ll))
        doSolve(9, 0x200ll, puzzle | 0x190100ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x180900ll))
        doSolve(9, 0x200ll, puzzle | 0x180900ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x240300ll))
        doSolve(9, 0x200ll, puzzle | 0x240300ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x200b00ll))
        doSolve(9, 0x200ll, puzzle | 0x200b00ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x160100ll))
        doSolve(9, 0x200ll, puzzle | 0x160100ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x400d00ll))
        doSolve(9, 0x200ll, puzzle | 0x400d00ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xd00100ll))
        doSolve(9, 0x200ll, puzzle | 0xd00100ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x900300ll))
        doSolve(9, 0x200ll, puzzle | 0x900300ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1300100ll))
        doSolve(9, 0x200ll, puzzle | 0x1300100ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1800900ll))
        doSolve(9, 0x200ll, puzzle | 0x1800900ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x101900ll))
        doSolve(9, 0x200ll, puzzle | 0x101900ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1001300ll))
        doSolve(9, 0x200ll, puzzle | 0x1001300ll,
                   partsUsed | 0x100l);
    piece9at8:
      if (partsUsed & 0x200l) goto piece10at8;
      if (!(puzzle & 0x100100100ll)) {
         if(!(puzzle & 0x100120900ll))
           doSolve(9, 0x200ll, puzzle | 0x100120900ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x120900100ll))
           doSolve(9, 0x200ll, puzzle | 0x120900100ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x900120100ll))
           doSolve(9, 0x200ll, puzzle | 0x900120100ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x100180300ll))
           doSolve(9, 0x200ll, puzzle | 0x100180300ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x300180100ll))
           doSolve(9, 0x200ll, puzzle | 0x300180100ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x180300100ll))
           doSolve(9, 0x200ll, puzzle | 0x180300100ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x900100ll)) {
         if(!(puzzle & 0x800920100ll))
           doSolve(9, 0x200ll, puzzle | 0x800920100ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x20920100ll))
           doSolve(9, 0x200ll, puzzle | 0x20920100ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x804900100ll))
           doSolve(9, 0x200ll, puzzle | 0x804900100ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0xd00ll)) {
         if(!(puzzle & 0x9d00ll))
           doSolve(9, 0x200ll, puzzle | 0x9d00ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x3d00ll))
           doSolve(9, 0x200ll, puzzle | 0x3d00ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x4f00ll))
           doSolve(9, 0x200ll, puzzle | 0x4f00ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0xcd00ll))
           doSolve(9, 0x200ll, puzzle | 0xcd00ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x7900ll))
        doSolve(9, 0x200ll, puzzle | 0x7900ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x80380100ll))
        doSolve(9, 0x200ll, puzzle | 0x80380100ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x200380100ll))
        doSolve(9, 0x200ll, puzzle | 0x200380100ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x804800900ll))
        doSolve(9, 0x200ll, puzzle | 0x804800900ll,
                   partsUsed | 0x200l);
    piece10at8:
      if (partsUsed & 0x400l) goto piece11at8;
      if (!(puzzle & 0x800900ll)) {
         if(!(puzzle & 0x100900900ll))
           doSolve(9, 0x200ll, puzzle | 0x100900900ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x4900900ll))
           doSolve(9, 0x200ll, puzzle | 0x4900900ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x4804900ll))
           doSolve(9, 0x200ll, puzzle | 0x4804900ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x904900ll))
           doSolve(9, 0x200ll, puzzle | 0x904900ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x800900900ll))
           doSolve(9, 0x200ll, puzzle | 0x800900900ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x100100100ll)) {
         if(!(puzzle & 0x900900100ll))
           doSolve(9, 0x200ll, puzzle | 0x900900100ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x100300300ll))
           doSolve(9, 0x200ll, puzzle | 0x100300300ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x300300100ll))
           doSolve(9, 0x200ll, puzzle | 0x300300100ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x1900ll)) {
         if(!(puzzle & 0x5b00ll))
           doSolve(9, 0x200ll, puzzle | 0x5b00ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x9b00ll))
           doSolve(9, 0x200ll, puzzle | 0x9b00ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xd900ll))
           doSolve(9, 0x200ll, puzzle | 0xd900ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x120120100ll))
        doSolve(9, 0x200ll, puzzle | 0x120120100ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x920900ll))
        doSolve(9, 0x200ll, puzzle | 0x920900ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x1f00ll))
        doSolve(9, 0x200ll, puzzle | 0x1f00ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x6d00ll))
        doSolve(9, 0x200ll, puzzle | 0x6d00ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x380300ll))
        doSolve(9, 0x200ll, puzzle | 0x380300ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x180180100ll))
        doSolve(9, 0x200ll, puzzle | 0x180180100ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x200300300ll))
        doSolve(9, 0x200ll, puzzle | 0x200300300ll,
                   partsUsed | 0x400l);
    piece11at8:
      if (partsUsed & 0x800l) goto piece12at8;
      if (!(puzzle & 0x120100ll)) {
         if(!(puzzle & 0x1b0100ll))
           doSolve(9, 0x200ll, puzzle | 0x1b0100ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x360100ll))
           doSolve(9, 0x200ll, puzzle | 0x360100ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x320300ll))
           doSolve(9, 0x200ll, puzzle | 0x320300ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x100300ll)) {
         if(!(puzzle & 0x340300ll))
           doSolve(9, 0x200ll, puzzle | 0x340300ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x101b00ll))
           doSolve(9, 0x200ll, puzzle | 0x101b00ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xb00300ll))
           doSolve(9, 0x200ll, puzzle | 0xb00300ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x300b00ll))
           doSolve(9, 0x200ll, puzzle | 0x300b00ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x301300ll))
           doSolve(9, 0x200ll, puzzle | 0x301300ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1300300ll))
           doSolve(9, 0x200ll, puzzle | 0x1300300ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x900b00ll))
           doSolve(9, 0x200ll, puzzle | 0x900b00ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0xc00100ll)) {
         if(!(puzzle & 0xd80100ll))
           doSolve(9, 0x200ll, puzzle | 0xd80100ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xd00900ll))
           doSolve(9, 0x200ll, puzzle | 0xd00900ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xc00d00ll))
           doSolve(9, 0x200ll, puzzle | 0xc00d00ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x800900ll)) {
         if(!(puzzle & 0x980900ll))
           doSolve(9, 0x200ll, puzzle | 0x980900ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x900d00ll))
           doSolve(9, 0x200ll, puzzle | 0x900d00ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1801900ll))
           doSolve(9, 0x200ll, puzzle | 0x1801900ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xb00900ll))
           doSolve(9, 0x200ll, puzzle | 0xb00900ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x901900ll))
           doSolve(9, 0x200ll, puzzle | 0x901900ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x801b00ll))
           doSolve(9, 0x200ll, puzzle | 0x801b00ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1900900ll))
           doSolve(9, 0x200ll, puzzle | 0x1900900ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x1300ll)) {
         if(!(puzzle & 0x1201300ll))
           doSolve(9, 0x200ll, puzzle | 0x1201300ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1001b00ll))
           doSolve(9, 0x200ll, puzzle | 0x1001b00ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x201b00ll))
           doSolve(9, 0x200ll, puzzle | 0x201b00ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0x1b00100ll))
        doSolve(9, 0x200ll, puzzle | 0x1b00100ll,
                   partsUsed | 0x800l);
    piece12at8:
    break;
   case 9:
      if (partsUsed & 0x2l) goto piece2at9;
      if(!(puzzle & 0x40241200ll))
        doSolve(10, 0x400ll, puzzle | 0x40241200ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x7a00ll))
        doSolve(10, 0x400ll, puzzle | 0x7a00ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x180300200ll))
        doSolve(10, 0x400ll, puzzle | 0x180300200ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x9001200200ll))
        doSolve(10, 0x400ll, puzzle | 0x9001200200ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x8009001200ll))
        doSolve(10, 0x400ll, puzzle | 0x8009001200ll,
                   partsUsed | 0x2l);
    piece2at9:
      if (partsUsed & 0x4l) goto piece3at9;
      if (!(puzzle & 0x200200200ll)) {
         if(!(puzzle & 0x1240200200ll))
           doSolve(10, 0x400ll, puzzle | 0x1240200200ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x200380200ll))
           doSolve(10, 0x400ll, puzzle | 0x200380200ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x209200200ll))
           doSolve(10, 0x400ll, puzzle | 0x209200200ll,
                   partsUsed | 0x4l);
      }
      if(!(puzzle & 0x9e00ll))
        doSolve(10, 0x400ll, puzzle | 0x9e00ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x1001009200ll))
        doSolve(10, 0x400ll, puzzle | 0x1001009200ll,
                   partsUsed | 0x4l);
    piece3at9:
      if (partsUsed & 0x8l) goto piece4at9;
      if(!(puzzle & 0x240201200ll))
        doSolve(10, 0x400ll, puzzle | 0x240201200ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x3e00ll))
        doSolve(10, 0x400ll, puzzle | 0x3e00ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x80380200ll))
        doSolve(10, 0x400ll, puzzle | 0x80380200ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x8009200200ll))
        doSolve(10, 0x400ll, puzzle | 0x8009200200ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x9001001200ll))
        doSolve(10, 0x400ll, puzzle | 0x9001001200ll,
                   partsUsed | 0x8l);
    piece4at9:
      if (partsUsed & 0x10l) goto piece5at9;
      if (!(puzzle & 0x200200200200ll)) {
         if(!(puzzle & 0x300200300200ll))
           doSolve(10, 0x400ll, puzzle | 0x300200300200ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x1200201200200ll))
           doSolve(10, 0x400ll, puzzle | 0x1200201200200ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x201200201200ll))
           doSolve(10, 0x400ll, puzzle | 0x201200201200ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x9248200ll))
        doSolve(10, 0x400ll, puzzle | 0x9248200ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x240200240200ll))
        doSolve(10, 0x400ll, puzzle | 0x240200240200ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x1001201001200ll))
        doSolve(10, 0x400ll, puzzle | 0x1001201001200ll,
                   partsUsed | 0x10l);
    piece5at9:
      if (partsUsed & 0x20l) goto piece6at9;
      if (!(puzzle & 0x200200200200ll)) {
         if(!(puzzle & 0x240200200200ll))
           doSolve(10, 0x400ll, puzzle | 0x240200200200ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x300200200200ll))
           doSolve(10, 0x400ll, puzzle | 0x300200200200ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x200200201200ll))
           doSolve(10, 0x400ll, puzzle | 0x200200201200ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x1200200200200ll))
           doSolve(10, 0x400ll, puzzle | 0x1200200200200ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x1001001001200ll))
        doSolve(10, 0x400ll, puzzle | 0x1001001001200ll,
                   partsUsed | 0x20l);
    piece6at9:
      if (partsUsed & 0x40l) goto piece7at9;
      if (!(puzzle & 0x200200200200ll)) {
         if(!(puzzle & 0x200240200200ll))
           doSolve(10, 0x400ll, puzzle | 0x200240200200ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x200200300200ll))
           doSolve(10, 0x400ll, puzzle | 0x200200300200ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x200300200200ll))
           doSolve(10, 0x400ll, puzzle | 0x200300200200ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x200201200200ll))
           doSolve(10, 0x400ll, puzzle | 0x200201200200ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x201200200200ll))
           doSolve(10, 0x400ll, puzzle | 0x201200200200ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x9240200ll))
        doSolve(10, 0x400ll, puzzle | 0x9240200ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x200200240200ll))
        doSolve(10, 0x400ll, puzzle | 0x200200240200ll,
                   partsUsed | 0x40l);
    piece7at9:
      if (partsUsed & 0x80l) goto piece8at9;
      if(!(puzzle & 0x1200201200ll))
        doSolve(10, 0x400ll, puzzle | 0x1200201200ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1201001200ll))
        doSolve(10, 0x400ll, puzzle | 0x1201001200ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x9208200ll))
        doSolve(10, 0x400ll, puzzle | 0x9208200ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x8209200ll))
        doSolve(10, 0x400ll, puzzle | 0x8209200ll,
                   partsUsed | 0x80l);
    piece8at9:
      if (partsUsed & 0x100l) goto piece9at9;
      if(!(puzzle & 0x320200ll))
        doSolve(10, 0x400ll, puzzle | 0x320200ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x301200ll))
        doSolve(10, 0x400ll, puzzle | 0x301200ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x801a00ll))
        doSolve(10, 0x400ll, puzzle | 0x801a00ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1a00200ll))
        doSolve(10, 0x400ll, puzzle | 0x1a00200ll,
                   partsUsed | 0x100l);
    piece9at9:
      if (partsUsed & 0x200l) goto piece10at9;
      if (!(puzzle & 0x200200200ll)) {
         if(!(puzzle & 0x200241200ll))
           doSolve(10, 0x400ll, puzzle | 0x200241200ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x241200200ll))
           doSolve(10, 0x400ll, puzzle | 0x241200200ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x1200240200ll))
           doSolve(10, 0x400ll, puzzle | 0x1200240200ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x1200200ll)) {
         if(!(puzzle & 0x1001240200ll))
           doSolve(10, 0x400ll, puzzle | 0x1001240200ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x41240200ll))
           doSolve(10, 0x400ll, puzzle | 0x41240200ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x1009200200ll))
           doSolve(10, 0x400ll, puzzle | 0x1009200200ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x5e00ll))
        doSolve(10, 0x400ll, puzzle | 0x5e00ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100380200ll))
        doSolve(10, 0x400ll, puzzle | 0x100380200ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x1009001200ll))
        doSolve(10, 0x400ll, puzzle | 0x1009001200ll,
                   partsUsed | 0x200l);
    piece10at9:
      if (partsUsed & 0x400l) goto piece11at9;
      if (!(puzzle & 0x9200ll)) {
         if(!(puzzle & 0xda00ll))
           doSolve(10, 0x400ll, puzzle | 0xda00ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x9009200ll))
           doSolve(10, 0x400ll, puzzle | 0x9009200ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1209200ll))
           doSolve(10, 0x400ll, puzzle | 0x1209200ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x200200200ll)) {
         if(!(puzzle & 0x300300200ll))
           doSolve(10, 0x400ll, puzzle | 0x300300200ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x201201200ll))
           doSolve(10, 0x400ll, puzzle | 0x201201200ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1201200200ll))
           doSolve(10, 0x400ll, puzzle | 0x1201200200ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x240240200ll))
        doSolve(10, 0x400ll, puzzle | 0x240240200ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x1241200ll))
        doSolve(10, 0x400ll, puzzle | 0x1241200ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x9201200ll))
        doSolve(10, 0x400ll, puzzle | 0x9201200ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x1001201200ll))
        doSolve(10, 0x400ll, puzzle | 0x1001201200ll,
                   partsUsed | 0x400l);
    piece11at9:
      if (partsUsed & 0x800l) goto piece12at9;
      if (!(puzzle & 0x300200ll)) {
         if(!(puzzle & 0x360200ll))
           doSolve(10, 0x400ll, puzzle | 0x360200ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1b00200ll))
           doSolve(10, 0x400ll, puzzle | 0x1b00200ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1301200ll))
           doSolve(10, 0x400ll, puzzle | 0x1301200ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x1001200ll)) {
         if(!(puzzle & 0x1a01200ll))
           doSolve(10, 0x400ll, puzzle | 0x1a01200ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1801a00ll))
           doSolve(10, 0x400ll, puzzle | 0x1801a00ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1201a00ll))
           doSolve(10, 0x400ll, puzzle | 0x1201a00ll,
                   partsUsed | 0x800l);
      }
    piece12at9:
    break;
   case 10:
      if (partsUsed & 0x2l) goto piece2at10;
      if(!(puzzle & 0x90480400ll))
        doSolve(11, 0x800ll, puzzle | 0x90480400ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x80482400ll))
        doSolve(11, 0x800ll, puzzle | 0x80482400ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1001800c00ll))
        doSolve(11, 0x800ll, puzzle | 0x1001800c00ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1800c00400ll))
        doSolve(11, 0x800ll, puzzle | 0x1800c00400ll,
                   partsUsed | 0x2l);
    piece2at10:
      if (partsUsed & 0x4l) goto piece3at10;
      if (!(puzzle & 0x400400400ll)) {
         if(!(puzzle & 0x400490400ll))
           doSolve(11, 0x800ll, puzzle | 0x400490400ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x2480400400ll))
           doSolve(11, 0x800ll, puzzle | 0x2480400400ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x401c00400ll))
           doSolve(11, 0x800ll, puzzle | 0x401c00400ll,
                   partsUsed | 0x4l);
      }
      if(!(puzzle & 0x800801c00ll))
        doSolve(11, 0x800ll, puzzle | 0x800801c00ll,
                   partsUsed | 0x4l);
    piece3at10:
      if (partsUsed & 0x8l) goto piece4at10;
      if(!(puzzle & 0x10490400ll))
        doSolve(11, 0x800ll, puzzle | 0x10490400ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x480402400ll))
        doSolve(11, 0x800ll, puzzle | 0x480402400ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1001c00400ll))
        doSolve(11, 0x800ll, puzzle | 0x1001c00400ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1800800c00ll))
        doSolve(11, 0x800ll, puzzle | 0x1800800c00ll,
                   partsUsed | 0x8l);
    piece4at10:
      if (partsUsed & 0x10l) goto piece5at10;
      if (!(puzzle & 0x400400400400ll)) {
         if(!(puzzle & 0x480400480400ll))
           doSolve(11, 0x800ll, puzzle | 0x480400480400ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x2400402400400ll))
           doSolve(11, 0x800ll, puzzle | 0x2400402400400ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x402400402400ll))
           doSolve(11, 0x800ll, puzzle | 0x402400402400ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0xc00400c00400ll))
           doSolve(11, 0x800ll, puzzle | 0xc00400c00400ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x400c00400c00ll))
           doSolve(11, 0x800ll, puzzle | 0x400c00400c00ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x2002402002400ll))
        doSolve(11, 0x800ll, puzzle | 0x2002402002400ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x800c00800c00ll))
        doSolve(11, 0x800ll, puzzle | 0x800c00800c00ll,
                   partsUsed | 0x10l);
    piece5at10:
      if (partsUsed & 0x20l) goto piece6at10;
      if (!(puzzle & 0x400400400400ll)) {
         if(!(puzzle & 0x480400400400ll))
           doSolve(11, 0x800ll, puzzle | 0x480400400400ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x400400402400ll))
           doSolve(11, 0x800ll, puzzle | 0x400400402400ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x2400400400400ll))
           doSolve(11, 0x800ll, puzzle | 0x2400400400400ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x400400400c00ll))
           doSolve(11, 0x800ll, puzzle | 0x400400400c00ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0xc00400400400ll))
           doSolve(11, 0x800ll, puzzle | 0xc00400400400ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x2002002002400ll))
        doSolve(11, 0x800ll, puzzle | 0x2002002002400ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x800800800c00ll))
        doSolve(11, 0x800ll, puzzle | 0x800800800c00ll,
                   partsUsed | 0x20l);
    piece6at10:
      if (partsUsed & 0x40l) goto piece7at10;
      if (!(puzzle & 0x400400400400ll)) {
         if(!(puzzle & 0x400480400400ll))
           doSolve(11, 0x800ll, puzzle | 0x400480400400ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x400402400400ll))
           doSolve(11, 0x800ll, puzzle | 0x400402400400ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x402400400400ll))
           doSolve(11, 0x800ll, puzzle | 0x402400400400ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x400400c00400ll))
           doSolve(11, 0x800ll, puzzle | 0x400400c00400ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x400c00400400ll))
           doSolve(11, 0x800ll, puzzle | 0x400c00400400ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x2490400ll))
        doSolve(11, 0x800ll, puzzle | 0x2490400ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x400400480400ll))
        doSolve(11, 0x800ll, puzzle | 0x400400480400ll,
                   partsUsed | 0x40l);
    piece7at10:
      if (partsUsed & 0x80l) goto piece8at10;
      if(!(puzzle & 0x2400402400ll))
        doSolve(11, 0x800ll, puzzle | 0x2400402400ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x2402002400ll))
        doSolve(11, 0x800ll, puzzle | 0x2402002400ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xbc00ll))
        doSolve(11, 0x800ll, puzzle | 0xbc00ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xf400ll))
        doSolve(11, 0x800ll, puzzle | 0xf400ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xc00800c00ll))
        doSolve(11, 0x800ll, puzzle | 0xc00800c00ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xc00400c00ll))
        doSolve(11, 0x800ll, puzzle | 0xc00400c00ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1c01400ll))
        doSolve(11, 0x800ll, puzzle | 0x1c01400ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1401c00ll))
        doSolve(11, 0x800ll, puzzle | 0x1401c00ll,
                   partsUsed | 0x80l);
    piece8at10:
      if (partsUsed & 0x100l) goto piece9at10;
      if(!(puzzle & 0x900c00ll))
        doSolve(11, 0x800ll, puzzle | 0x900c00ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x802c00ll))
        doSolve(11, 0x800ll, puzzle | 0x802c00ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x580400ll))
        doSolve(11, 0x800ll, puzzle | 0x580400ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x4c00400ll))
        doSolve(11, 0x800ll, puzzle | 0x4c00400ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x2400c00ll))
        doSolve(11, 0x800ll, puzzle | 0x2400c00ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x6002400ll))
        doSolve(11, 0x800ll, puzzle | 0x6002400ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x406400ll))
        doSolve(11, 0x800ll, puzzle | 0x406400ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x4004c00ll))
        doSolve(11, 0x800ll, puzzle | 0x4004c00ll,
                   partsUsed | 0x100l);
    piece9at10:
      if (partsUsed & 0x200l) goto piece10at10;
      if (!(puzzle & 0x480400ll)) {
         if(!(puzzle & 0x80490400ll))
           doSolve(11, 0x800ll, puzzle | 0x80490400ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x400482400ll))
           doSolve(11, 0x800ll, puzzle | 0x400482400ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x2002480400ll))
           doSolve(11, 0x800ll, puzzle | 0x2002480400ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x2400480400ll))
           doSolve(11, 0x800ll, puzzle | 0x2400480400ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x82480400ll))
           doSolve(11, 0x800ll, puzzle | 0x82480400ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x482400400ll))
        doSolve(11, 0x800ll, puzzle | 0x482400400ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x801800c00ll))
        doSolve(11, 0x800ll, puzzle | 0x801800c00ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x801c00400ll))
        doSolve(11, 0x800ll, puzzle | 0x801c00400ll,
                   partsUsed | 0x200l);
    piece10at10:
      if (partsUsed & 0x400l) goto piece11at10;
      if (!(puzzle & 0x2400400ll)) {
         if(!(puzzle & 0x402402400ll))
           doSolve(11, 0x800ll, puzzle | 0x402402400ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2402400400ll))
           doSolve(11, 0x800ll, puzzle | 0x2402400400ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2002402400ll))
           doSolve(11, 0x800ll, puzzle | 0x2002402400ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x4c00ll)) {
         if(!(puzzle & 0xdc00ll))
           doSolve(11, 0x800ll, puzzle | 0xdc00ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x7c00ll))
           doSolve(11, 0x800ll, puzzle | 0x7c00ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xec00ll))
           doSolve(11, 0x800ll, puzzle | 0xec00ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0xc00400ll)) {
         if(!(puzzle & 0x800c00c00ll))
           doSolve(11, 0x800ll, puzzle | 0x800c00c00ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x400c00c00ll))
           doSolve(11, 0x800ll, puzzle | 0x400c00c00ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1c00c00ll))
           doSolve(11, 0x800ll, puzzle | 0x1c00c00ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xc00c00400ll))
           doSolve(11, 0x800ll, puzzle | 0xc00c00400ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xc01c00ll))
           doSolve(11, 0x800ll, puzzle | 0xc01c00ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x480480400ll))
        doSolve(11, 0x800ll, puzzle | 0x480480400ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x2482400ll))
        doSolve(11, 0x800ll, puzzle | 0x2482400ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x1801c00ll))
        doSolve(11, 0x800ll, puzzle | 0x1801c00ll,
                   partsUsed | 0x400l);
    piece11at10:
      if (partsUsed & 0x800l) goto piece12at10;
      if (!(puzzle & 0xc00400ll)) {
         if(!(puzzle & 0xd80400ll))
           doSolve(11, 0x800ll, puzzle | 0xd80400ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xd00c00ll))
           doSolve(11, 0x800ll, puzzle | 0xd00c00ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xc80c00ll))
           doSolve(11, 0x800ll, puzzle | 0xc80c00ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x2c00c00ll))
           doSolve(11, 0x800ll, puzzle | 0x2c00c00ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x2c02400ll))
           doSolve(11, 0x800ll, puzzle | 0x2c02400ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xc02c00ll))
           doSolve(11, 0x800ll, puzzle | 0xc02c00ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xc04c00ll))
           doSolve(11, 0x800ll, puzzle | 0xc04c00ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x4c00c00ll))
           doSolve(11, 0x800ll, puzzle | 0x4c00c00ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x6c00400ll))
           doSolve(11, 0x800ll, puzzle | 0x6c00400ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x6400ll)) {
         if(!(puzzle & 0x406c00ll))
           doSolve(11, 0x800ll, puzzle | 0x406c00ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x6006400ll))
           doSolve(11, 0x800ll, puzzle | 0x6006400ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x2406400ll))
           doSolve(11, 0x800ll, puzzle | 0x2406400ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x4006c00ll))
           doSolve(11, 0x800ll, puzzle | 0x4006c00ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x806c00ll))
           doSolve(11, 0x800ll, puzzle | 0x806c00ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x2006c00ll))
           doSolve(11, 0x800ll, puzzle | 0x2006c00ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0x4804c00ll))
        doSolve(11, 0x800ll, puzzle | 0x4804c00ll,
                   partsUsed | 0x800l);
      if(!(puzzle & 0x6402400ll))
        doSolve(11, 0x800ll, puzzle | 0x6402400ll,
                   partsUsed | 0x800l);
      if(!(puzzle & 0x2402c00ll))
        doSolve(11, 0x800ll, puzzle | 0x2402c00ll,
                   partsUsed | 0x800l);
    piece12at10:
    break;
   case 11:
      if (partsUsed & 0x2l) goto piece2at11;
      if(!(puzzle & 0x400c01800ll))
        doSolve(12, 0x1000ll, puzzle | 0x400c01800ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x120900800ll))
        doSolve(12, 0x1000ll, puzzle | 0x120900800ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x100904800ll))
        doSolve(12, 0x1000ll, puzzle | 0x100904800ll,
                   partsUsed | 0x2l);
    piece2at11:
      if (partsUsed & 0x4l) goto piece3at11;
      if (!(puzzle & 0x800800800ll)) {
         if(!(puzzle & 0x800920800ll))
           doSolve(12, 0x1000ll, puzzle | 0x800920800ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x4900800800ll))
           doSolve(12, 0x1000ll, puzzle | 0x4900800800ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x1c00800800ll))
           doSolve(12, 0x1000ll, puzzle | 0x1c00800800ll,
                   partsUsed | 0x4l);
      }
    piece3at11:
      if (partsUsed & 0x8l) goto piece4at11;
      if(!(puzzle & 0x20920800ll))
        doSolve(12, 0x1000ll, puzzle | 0x20920800ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x900804800ll))
        doSolve(12, 0x1000ll, puzzle | 0x900804800ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0xc00801800ll))
        doSolve(12, 0x1000ll, puzzle | 0xc00801800ll,
                   partsUsed | 0x8l);
    piece4at11:
      if (partsUsed & 0x10l) goto piece5at11;
      if (!(puzzle & 0x800800800800ll)) {
         if(!(puzzle & 0x900800900800ll))
           doSolve(12, 0x1000ll, puzzle | 0x900800900800ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0xc00800c00800ll))
           doSolve(12, 0x1000ll, puzzle | 0xc00800c00800ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x4800804800800ll))
           doSolve(12, 0x1000ll, puzzle | 0x4800804800800ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x804800804800ll))
           doSolve(12, 0x1000ll, puzzle | 0x804800804800ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x1800801800800ll))
           doSolve(12, 0x1000ll, puzzle | 0x1800801800800ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x801800801800ll))
           doSolve(12, 0x1000ll, puzzle | 0x801800801800ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x4004804004800ll))
        doSolve(12, 0x1000ll, puzzle | 0x4004804004800ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x1001801001800ll))
        doSolve(12, 0x1000ll, puzzle | 0x1001801001800ll,
                   partsUsed | 0x10l);
    piece5at11:
      if (partsUsed & 0x20l) goto piece6at11;
      if (!(puzzle & 0x800800800800ll)) {
         if(!(puzzle & 0x900800800800ll))
           doSolve(12, 0x1000ll, puzzle | 0x900800800800ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0xc00800800800ll))
           doSolve(12, 0x1000ll, puzzle | 0xc00800800800ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x800800804800ll))
           doSolve(12, 0x1000ll, puzzle | 0x800800804800ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x4800800800800ll))
           doSolve(12, 0x1000ll, puzzle | 0x4800800800800ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x800800801800ll))
           doSolve(12, 0x1000ll, puzzle | 0x800800801800ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x1800800800800ll))
           doSolve(12, 0x1000ll, puzzle | 0x1800800800800ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x4004004004800ll))
        doSolve(12, 0x1000ll, puzzle | 0x4004004004800ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x1001001001800ll))
        doSolve(12, 0x1000ll, puzzle | 0x1001001001800ll,
                   partsUsed | 0x20l);
    piece6at11:
      if (partsUsed & 0x40l) goto piece7at11;
      if (!(puzzle & 0x800800800800ll)) {
         if(!(puzzle & 0x800900800800ll))
           doSolve(12, 0x1000ll, puzzle | 0x800900800800ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800800c00800ll))
           doSolve(12, 0x1000ll, puzzle | 0x800800c00800ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800c00800800ll))
           doSolve(12, 0x1000ll, puzzle | 0x800c00800800ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800804800800ll))
           doSolve(12, 0x1000ll, puzzle | 0x800804800800ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x804800800800ll))
           doSolve(12, 0x1000ll, puzzle | 0x804800800800ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800801800800ll))
           doSolve(12, 0x1000ll, puzzle | 0x800801800800ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x801800800800ll))
           doSolve(12, 0x1000ll, puzzle | 0x801800800800ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x4920800ll))
        doSolve(12, 0x1000ll, puzzle | 0x4920800ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x800800900800ll))
        doSolve(12, 0x1000ll, puzzle | 0x800800900800ll,
                   partsUsed | 0x40l);
    piece7at11:
      if (partsUsed & 0x80l) goto piece8at11;
      if(!(puzzle & 0x4800804800ll))
        doSolve(12, 0x1000ll, puzzle | 0x4800804800ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x4804004800ll))
        doSolve(12, 0x1000ll, puzzle | 0x4804004800ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1801001800ll))
        doSolve(12, 0x1000ll, puzzle | 0x1801001800ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1800801800ll))
        doSolve(12, 0x1000ll, puzzle | 0x1800801800ll,
                   partsUsed | 0x80l);
    piece8at11:
      if (partsUsed & 0x100l) goto piece9at11;
      if(!(puzzle & 0xc80800ll))
        doSolve(12, 0x1000ll, puzzle | 0xc80800ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xc04800ll))
        doSolve(12, 0x1000ll, puzzle | 0xc04800ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1201800ll))
        doSolve(12, 0x1000ll, puzzle | 0x1201800ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1005800ll))
        doSolve(12, 0x1000ll, puzzle | 0x1005800ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xb00800ll))
        doSolve(12, 0x1000ll, puzzle | 0xb00800ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x2006800ll))
        doSolve(12, 0x1000ll, puzzle | 0x2006800ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x6800800ll))
        doSolve(12, 0x1000ll, puzzle | 0x6800800ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x4801800ll))
        doSolve(12, 0x1000ll, puzzle | 0x4801800ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x9800800ll))
        doSolve(12, 0x1000ll, puzzle | 0x9800800ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xc004800ll))
        doSolve(12, 0x1000ll, puzzle | 0xc004800ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x80c800ll))
        doSolve(12, 0x1000ll, puzzle | 0x80c800ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x8009800ll))
        doSolve(12, 0x1000ll, puzzle | 0x8009800ll,
                   partsUsed | 0x100l);
    piece9at11:
      if (partsUsed & 0x200l) goto piece10at11;
      if (!(puzzle & 0x900800ll)) {
         if(!(puzzle & 0x100920800ll))
           doSolve(12, 0x1000ll, puzzle | 0x100920800ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x800904800ll))
           doSolve(12, 0x1000ll, puzzle | 0x800904800ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x4004900800ll))
           doSolve(12, 0x1000ll, puzzle | 0x4004900800ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x4800900800ll))
           doSolve(12, 0x1000ll, puzzle | 0x4800900800ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x104900800ll))
           doSolve(12, 0x1000ll, puzzle | 0x104900800ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x800800800ll)) {
         if(!(puzzle & 0x904800800ll))
           doSolve(12, 0x1000ll, puzzle | 0x904800800ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x800c01800ll))
           doSolve(12, 0x1000ll, puzzle | 0x800c01800ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x1800c00800ll))
           doSolve(12, 0x1000ll, puzzle | 0x1800c00800ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0xc01800800ll))
           doSolve(12, 0x1000ll, puzzle | 0xc01800800ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x401c00800ll))
        doSolve(12, 0x1000ll, puzzle | 0x401c00800ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x1001c00800ll))
        doSolve(12, 0x1000ll, puzzle | 0x1001c00800ll,
                   partsUsed | 0x200l);
    piece10at11:
      if (partsUsed & 0x400l) goto piece11at11;
      if (!(puzzle & 0x4800800ll)) {
         if(!(puzzle & 0x804804800ll))
           doSolve(12, 0x1000ll, puzzle | 0x804804800ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x4804800800ll))
           doSolve(12, 0x1000ll, puzzle | 0x4804800800ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x4004804800ll))
           doSolve(12, 0x1000ll, puzzle | 0x4004804800ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x1800800ll)) {
         if(!(puzzle & 0x1001801800ll))
           doSolve(12, 0x1000ll, puzzle | 0x1001801800ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x801801800ll))
           doSolve(12, 0x1000ll, puzzle | 0x801801800ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1801800800ll))
           doSolve(12, 0x1000ll, puzzle | 0x1801800800ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x900900800ll))
        doSolve(12, 0x1000ll, puzzle | 0x900900800ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x4904800ll))
        doSolve(12, 0x1000ll, puzzle | 0x4904800ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0xf800ll))
        doSolve(12, 0x1000ll, puzzle | 0xf800ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x1c01800ll))
        doSolve(12, 0x1000ll, puzzle | 0x1c01800ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0xc00c00800ll))
        doSolve(12, 0x1000ll, puzzle | 0xc00c00800ll,
                   partsUsed | 0x400l);
    piece11at11:
      if (partsUsed & 0x800l) goto piece12at11;
      if (!(puzzle & 0x900800ll)) {
         if(!(puzzle & 0xd80800ll))
           doSolve(12, 0x1000ll, puzzle | 0xd80800ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1b00800ll))
           doSolve(12, 0x1000ll, puzzle | 0x1b00800ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1901800ll))
           doSolve(12, 0x1000ll, puzzle | 0x1901800ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x801800ll)) {
         if(!(puzzle & 0x1a01800ll))
           doSolve(12, 0x1000ll, puzzle | 0x1a01800ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x80d800ll))
           doSolve(12, 0x1000ll, puzzle | 0x80d800ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x5801800ll))
           doSolve(12, 0x1000ll, puzzle | 0x5801800ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1805800ll))
           doSolve(12, 0x1000ll, puzzle | 0x1805800ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1809800ll))
           doSolve(12, 0x1000ll, puzzle | 0x1809800ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x9801800ll))
           doSolve(12, 0x1000ll, puzzle | 0x9801800ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x4805800ll))
           doSolve(12, 0x1000ll, puzzle | 0x4805800ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x6000800ll)) {
         if(!(puzzle & 0x6c00800ll))
           doSolve(12, 0x1000ll, puzzle | 0x6c00800ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x6804800ll))
           doSolve(12, 0x1000ll, puzzle | 0x6804800ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x6006800ll))
           doSolve(12, 0x1000ll, puzzle | 0x6006800ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x4004800ll)) {
         if(!(puzzle & 0x4c04800ll))
           doSolve(12, 0x1000ll, puzzle | 0x4c04800ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x4806800ll))
           doSolve(12, 0x1000ll, puzzle | 0x4806800ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xc00c800ll))
           doSolve(12, 0x1000ll, puzzle | 0xc00c800ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x5804800ll))
           doSolve(12, 0x1000ll, puzzle | 0x5804800ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x480c800ll))
           doSolve(12, 0x1000ll, puzzle | 0x480c800ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x400d800ll))
           doSolve(12, 0x1000ll, puzzle | 0x400d800ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xc804800ll))
           doSolve(12, 0x1000ll, puzzle | 0xc804800ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x9800ll)) {
         if(!(puzzle & 0x9009800ll))
           doSolve(12, 0x1000ll, puzzle | 0x9009800ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x800d800ll))
           doSolve(12, 0x1000ll, puzzle | 0x800d800ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x100d800ll))
           doSolve(12, 0x1000ll, puzzle | 0x100d800ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0xd800800ll))
        doSolve(12, 0x1000ll, puzzle | 0xd800800ll,
                   partsUsed | 0x800l);
    piece12at11:
    break;
   case 12:
      if (partsUsed & 0x2l) goto piece2at12;
      if(!(puzzle & 0xc01801000ll))
        doSolve(13, 0x2000ll, puzzle | 0xc01801000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x241201000ll))
        doSolve(13, 0x2000ll, puzzle | 0x241201000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x201209000ll))
        doSolve(13, 0x2000ll, puzzle | 0x201209000ll,
                   partsUsed | 0x2l);
    piece2at12:
      if (partsUsed & 0x4l) goto piece3at12;
      if (!(puzzle & 0x1001001000ll)) {
         if(!(puzzle & 0x1001241000ll))
           doSolve(13, 0x2000ll, puzzle | 0x1001241000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x9201001000ll))
           doSolve(13, 0x2000ll, puzzle | 0x9201001000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x1001c01000ll))
           doSolve(13, 0x2000ll, puzzle | 0x1001c01000ll,
                   partsUsed | 0x4l);
      }
    piece3at12:
      if (partsUsed & 0x8l) goto piece4at12;
      if(!(puzzle & 0x401c01000ll))
        doSolve(13, 0x2000ll, puzzle | 0x401c01000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x41241000ll))
        doSolve(13, 0x2000ll, puzzle | 0x41241000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1201009000ll))
        doSolve(13, 0x2000ll, puzzle | 0x1201009000ll,
                   partsUsed | 0x8l);
    piece4at12:
      if (partsUsed & 0x10l) goto piece5at12;
      if (!(puzzle & 0x1001001001000ll)) {
         if(!(puzzle & 0x1201001201000ll))
           doSolve(13, 0x2000ll, puzzle | 0x1201001201000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x1801001801000ll))
           doSolve(13, 0x2000ll, puzzle | 0x1801001801000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x9001009001000ll))
           doSolve(13, 0x2000ll, puzzle | 0x9001009001000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x1009001009000ll))
           doSolve(13, 0x2000ll, puzzle | 0x1009001009000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x8009008009000ll))
        doSolve(13, 0x2000ll, puzzle | 0x8009008009000ll,
                   partsUsed | 0x10l);
    piece5at12:
      if (partsUsed & 0x20l) goto piece6at12;
      if (!(puzzle & 0x1001001001000ll)) {
         if(!(puzzle & 0x1201001001000ll))
           doSolve(13, 0x2000ll, puzzle | 0x1201001001000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x1801001001000ll))
           doSolve(13, 0x2000ll, puzzle | 0x1801001001000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x1001001009000ll))
           doSolve(13, 0x2000ll, puzzle | 0x1001001009000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x9001001001000ll))
           doSolve(13, 0x2000ll, puzzle | 0x9001001001000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x8008008009000ll))
        doSolve(13, 0x2000ll, puzzle | 0x8008008009000ll,
                   partsUsed | 0x20l);
    piece6at12:
      if (partsUsed & 0x40l) goto piece7at12;
      if (!(puzzle & 0x1001001001000ll)) {
         if(!(puzzle & 0x1001201001000ll))
           doSolve(13, 0x2000ll, puzzle | 0x1001201001000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x1001001801000ll))
           doSolve(13, 0x2000ll, puzzle | 0x1001001801000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x1001801001000ll))
           doSolve(13, 0x2000ll, puzzle | 0x1001801001000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x1001009001000ll))
           doSolve(13, 0x2000ll, puzzle | 0x1001009001000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x1009001001000ll))
           doSolve(13, 0x2000ll, puzzle | 0x1009001001000ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x9241000ll))
        doSolve(13, 0x2000ll, puzzle | 0x9241000ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x1001001201000ll))
        doSolve(13, 0x2000ll, puzzle | 0x1001001201000ll,
                   partsUsed | 0x40l);
    piece7at12:
      if (partsUsed & 0x80l) goto piece8at12;
      if(!(puzzle & 0x9001009000ll))
        doSolve(13, 0x2000ll, puzzle | 0x9001009000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x9008009000ll))
        doSolve(13, 0x2000ll, puzzle | 0x9008009000ll,
                   partsUsed | 0x80l);
    piece8at12:
      if (partsUsed & 0x100l) goto piece9at12;
      if(!(puzzle & 0x1901000ll))
        doSolve(13, 0x2000ll, puzzle | 0x1901000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1809000ll))
        doSolve(13, 0x2000ll, puzzle | 0x1809000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x400d000ll))
        doSolve(13, 0x2000ll, puzzle | 0x400d000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xd001000ll))
        doSolve(13, 0x2000ll, puzzle | 0xd001000ll,
                   partsUsed | 0x100l);
    piece9at12:
      if (partsUsed & 0x200l) goto piece10at12;
      if (!(puzzle & 0x1201000ll)) {
         if(!(puzzle & 0x201241000ll))
           doSolve(13, 0x2000ll, puzzle | 0x201241000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x1001209000ll))
           doSolve(13, 0x2000ll, puzzle | 0x1001209000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x8009201000ll))
           doSolve(13, 0x2000ll, puzzle | 0x8009201000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x9001201000ll))
           doSolve(13, 0x2000ll, puzzle | 0x9001201000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x209201000ll))
           doSolve(13, 0x2000ll, puzzle | 0x209201000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x1209001000ll))
        doSolve(13, 0x2000ll, puzzle | 0x1209001000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x801c01000ll))
        doSolve(13, 0x2000ll, puzzle | 0x801c01000ll,
                   partsUsed | 0x200l);
    piece10at12:
      if (partsUsed & 0x400l) goto piece11at12;
      if (!(puzzle & 0x1001001000ll)) {
         if(!(puzzle & 0x1801801000ll))
           doSolve(13, 0x2000ll, puzzle | 0x1801801000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1009009000ll))
           doSolve(13, 0x2000ll, puzzle | 0x1009009000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x9009001000ll))
           doSolve(13, 0x2000ll, puzzle | 0x9009001000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x1201201000ll))
        doSolve(13, 0x2000ll, puzzle | 0x1201201000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x9209000ll))
        doSolve(13, 0x2000ll, puzzle | 0x9209000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x8009009000ll))
        doSolve(13, 0x2000ll, puzzle | 0x8009009000ll,
                   partsUsed | 0x400l);
    piece11at12:
      if (partsUsed & 0x800l) goto piece12at12;
      if (!(puzzle & 0x1801000ll)) {
         if(!(puzzle & 0x1b01000ll))
           doSolve(13, 0x2000ll, puzzle | 0x1b01000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xd801000ll))
           doSolve(13, 0x2000ll, puzzle | 0xd801000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x9809000ll))
           doSolve(13, 0x2000ll, puzzle | 0x9809000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x8009000ll)) {
         if(!(puzzle & 0xd009000ll))
           doSolve(13, 0x2000ll, puzzle | 0xd009000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xc00d000ll))
           doSolve(13, 0x2000ll, puzzle | 0xc00d000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x900d000ll))
           doSolve(13, 0x2000ll, puzzle | 0x900d000ll,
                   partsUsed | 0x800l);
      }
    piece12at12:
    break;
   case 13:
      if (partsUsed & 0x2l) goto piece2at13;
      if(!(puzzle & 0x482402000ll))
        doSolve(14, 0x4000ll, puzzle | 0x482402000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x800c006000ll))
        doSolve(14, 0x4000ll, puzzle | 0x800c006000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0xc006002000ll))
        doSolve(14, 0x4000ll, puzzle | 0xc006002000ll,
                   partsUsed | 0x2l);
    piece2at13:
      if (partsUsed & 0x4l) goto piece3at13;
      if(!(puzzle & 0x400400e000ll))
        doSolve(14, 0x4000ll, puzzle | 0x400400e000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x2002482000ll))
        doSolve(14, 0x4000ll, puzzle | 0x2002482000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x200e002000ll))
        doSolve(14, 0x4000ll, puzzle | 0x200e002000ll,
                   partsUsed | 0x4l);
    piece3at13:
      if (partsUsed & 0x8l) goto piece4at13;
      if(!(puzzle & 0x82482000ll))
        doSolve(14, 0x4000ll, puzzle | 0x82482000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x800e002000ll))
        doSolve(14, 0x4000ll, puzzle | 0x800e002000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0xc004006000ll))
        doSolve(14, 0x4000ll, puzzle | 0xc004006000ll,
                   partsUsed | 0x8l);
    piece4at13:
      if (partsUsed & 0x10l) goto piece5at13;
      if (!(puzzle & 0x2002002002000ll)) {
         if(!(puzzle & 0x2402002402000ll))
           doSolve(14, 0x4000ll, puzzle | 0x2402002402000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x6002006002000ll))
           doSolve(14, 0x4000ll, puzzle | 0x6002006002000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x2006002006000ll))
           doSolve(14, 0x4000ll, puzzle | 0x2006002006000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x4006004006000ll))
        doSolve(14, 0x4000ll, puzzle | 0x4006004006000ll,
                   partsUsed | 0x10l);
    piece5at13:
      if (partsUsed & 0x20l) goto piece6at13;
      if (!(puzzle & 0x2002002002000ll)) {
         if(!(puzzle & 0x2402002002000ll))
           doSolve(14, 0x4000ll, puzzle | 0x2402002002000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x2002002006000ll))
           doSolve(14, 0x4000ll, puzzle | 0x2002002006000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x6002002002000ll))
           doSolve(14, 0x4000ll, puzzle | 0x6002002002000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x2492000ll))
        doSolve(14, 0x4000ll, puzzle | 0x2492000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x4004004006000ll))
        doSolve(14, 0x4000ll, puzzle | 0x4004004006000ll,
                   partsUsed | 0x20l);
    piece6at13:
      if (partsUsed & 0x40l) goto piece7at13;
      if (!(puzzle & 0x2002002002000ll)) {
         if(!(puzzle & 0x2002402002000ll))
           doSolve(14, 0x4000ll, puzzle | 0x2002402002000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2002002402000ll))
           doSolve(14, 0x4000ll, puzzle | 0x2002002402000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2002006002000ll))
           doSolve(14, 0x4000ll, puzzle | 0x2002006002000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2006002002000ll))
           doSolve(14, 0x4000ll, puzzle | 0x2006002002000ll,
                   partsUsed | 0x40l);
      }
    piece7at13:
      if (partsUsed & 0x80l) goto piece8at13;
      if(!(puzzle & 0x6004006000ll))
        doSolve(14, 0x4000ll, puzzle | 0x6004006000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x6002006000ll))
        doSolve(14, 0x4000ll, puzzle | 0x6002006000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xe00a000ll))
        doSolve(14, 0x4000ll, puzzle | 0xe00a000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xa00e000ll))
        doSolve(14, 0x4000ll, puzzle | 0xa00e000ll,
                   partsUsed | 0x80l);
    piece8at13:
      if (partsUsed & 0x100l) goto piece9at13;
      if(!(puzzle & 0x4806000ll))
        doSolve(14, 0x4000ll, puzzle | 0x4806000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x2c02000ll))
        doSolve(14, 0x4000ll, puzzle | 0x2c02000ll,
                   partsUsed | 0x100l);
    piece9at13:
      if (partsUsed & 0x200l) goto piece10at13;
      if(!(puzzle & 0x402482000ll))
        doSolve(14, 0x4000ll, puzzle | 0x402482000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x400c006000ll))
        doSolve(14, 0x4000ll, puzzle | 0x400c006000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x400e002000ll))
        doSolve(14, 0x4000ll, puzzle | 0x400e002000ll,
                   partsUsed | 0x200l);
    piece10at13:
      if (partsUsed & 0x400l) goto piece11at13;
      if (!(puzzle & 0x2002002000ll)) {
         if(!(puzzle & 0x2402402000ll))
           doSolve(14, 0x4000ll, puzzle | 0x2402402000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2006006000ll))
           doSolve(14, 0x4000ll, puzzle | 0x2006006000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x6006002000ll))
           doSolve(14, 0x4000ll, puzzle | 0x6006002000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x4006000ll)) {
         if(!(puzzle & 0x4006006000ll))
           doSolve(14, 0x4000ll, puzzle | 0x4006006000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xe006000ll))
           doSolve(14, 0x4000ll, puzzle | 0xe006000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x600e000ll))
           doSolve(14, 0x4000ll, puzzle | 0x600e000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xc00e000ll))
           doSolve(14, 0x4000ll, puzzle | 0xc00e000ll,
                   partsUsed | 0x400l);
      }
    piece11at13:
      if (partsUsed & 0x800l) goto piece12at13;
      if (!(puzzle & 0x6002000ll)) {
         if(!(puzzle & 0x6c02000ll))
           doSolve(14, 0x4000ll, puzzle | 0x6c02000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x6806000ll))
           doSolve(14, 0x4000ll, puzzle | 0x6806000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x6406000ll))
           doSolve(14, 0x4000ll, puzzle | 0x6406000ll,
                   partsUsed | 0x800l);
      }
    piece12at13:
    break;
   case 14:
      if (partsUsed & 0x2l) goto piece2at14;
      if(!(puzzle & 0x904804000ll))
        doSolve(15, 0x8000ll, puzzle | 0x904804000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x200600c000ll))
        doSolve(15, 0x8000ll, puzzle | 0x200600c000ll,
                   partsUsed | 0x2l);
    piece2at14:
      if (partsUsed & 0x4l) goto piece3at14;
      if(!(puzzle & 0x4004904000ll))
        doSolve(15, 0x8000ll, puzzle | 0x4004904000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0xe004004000ll))
        doSolve(15, 0x8000ll, puzzle | 0xe004004000ll,
                   partsUsed | 0x4l);
    piece3at14:
      if (partsUsed & 0x8l) goto piece4at14;
      if(!(puzzle & 0x104904000ll))
        doSolve(15, 0x8000ll, puzzle | 0x104904000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x600400c000ll))
        doSolve(15, 0x8000ll, puzzle | 0x600400c000ll,
                   partsUsed | 0x8l);
    piece4at14:
      if (partsUsed & 0x10l) goto piece5at14;
      if (!(puzzle & 0x4004004004000ll)) {
         if(!(puzzle & 0x4804004804000ll))
           doSolve(15, 0x8000ll, puzzle | 0x4804004804000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x6004006004000ll))
           doSolve(15, 0x8000ll, puzzle | 0x6004006004000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0xc00400c004000ll))
           doSolve(15, 0x8000ll, puzzle | 0xc00400c004000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x400c00400c000ll))
           doSolve(15, 0x8000ll, puzzle | 0x400c00400c000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x800c00800c000ll))
        doSolve(15, 0x8000ll, puzzle | 0x800c00800c000ll,
                   partsUsed | 0x10l);
    piece5at14:
      if (partsUsed & 0x20l) goto piece6at14;
      if (!(puzzle & 0x4004004004000ll)) {
         if(!(puzzle & 0x4804004004000ll))
           doSolve(15, 0x8000ll, puzzle | 0x4804004004000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x6004004004000ll))
           doSolve(15, 0x8000ll, puzzle | 0x6004004004000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x400400400c000ll))
           doSolve(15, 0x8000ll, puzzle | 0x400400400c000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0xc004004004000ll))
           doSolve(15, 0x8000ll, puzzle | 0xc004004004000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x4924000ll))
        doSolve(15, 0x8000ll, puzzle | 0x4924000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x800800800c000ll))
        doSolve(15, 0x8000ll, puzzle | 0x800800800c000ll,
                   partsUsed | 0x20l);
    piece6at14:
      if (partsUsed & 0x40l) goto piece7at14;
      if (!(puzzle & 0x4004004004000ll)) {
         if(!(puzzle & 0x4004804004000ll))
           doSolve(15, 0x8000ll, puzzle | 0x4004804004000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4004004804000ll))
           doSolve(15, 0x8000ll, puzzle | 0x4004004804000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4004006004000ll))
           doSolve(15, 0x8000ll, puzzle | 0x4004006004000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4006004004000ll))
           doSolve(15, 0x8000ll, puzzle | 0x4006004004000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x400400c004000ll))
           doSolve(15, 0x8000ll, puzzle | 0x400400c004000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x400c004004000ll))
           doSolve(15, 0x8000ll, puzzle | 0x400c004004000ll,
                   partsUsed | 0x40l);
      }
    piece7at14:
      if (partsUsed & 0x80l) goto piece8at14;
      if(!(puzzle & 0xc00800c000ll))
        doSolve(15, 0x8000ll, puzzle | 0xc00800c000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xc00400c000ll))
        doSolve(15, 0x8000ll, puzzle | 0xc00400c000ll,
                   partsUsed | 0x80l);
    piece8at14:
      if (partsUsed & 0x100l) goto piece9at14;
      if(!(puzzle & 0x5804000ll))
        doSolve(15, 0x8000ll, puzzle | 0x5804000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x6404000ll))
        doSolve(15, 0x8000ll, puzzle | 0x6404000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x900c000ll))
        doSolve(15, 0x8000ll, puzzle | 0x900c000ll,
                   partsUsed | 0x100l);
    piece9at14:
      if (partsUsed & 0x200l) goto piece10at14;
      if (!(puzzle & 0x4004004000ll)) {
         if(!(puzzle & 0x400600c000ll))
           doSolve(15, 0x8000ll, puzzle | 0x400600c000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0xc006004000ll))
           doSolve(15, 0x8000ll, puzzle | 0xc006004000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x600c004000ll))
           doSolve(15, 0x8000ll, puzzle | 0x600c004000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x804904000ll))
        doSolve(15, 0x8000ll, puzzle | 0x804904000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x200e004000ll))
        doSolve(15, 0x8000ll, puzzle | 0x200e004000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x800e004000ll))
        doSolve(15, 0x8000ll, puzzle | 0x800e004000ll,
                   partsUsed | 0x200l);
    piece10at14:
      if (partsUsed & 0x400l) goto piece11at14;
      if (!(puzzle & 0x4004004000ll)) {
         if(!(puzzle & 0x4804804000ll))
           doSolve(15, 0x8000ll, puzzle | 0x4804804000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x6006004000ll))
           doSolve(15, 0x8000ll, puzzle | 0x6006004000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x400c00c000ll))
           doSolve(15, 0x8000ll, puzzle | 0x400c00c000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xc00c004000ll))
           doSolve(15, 0x8000ll, puzzle | 0xc00c004000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0xe00c000ll))
        doSolve(15, 0x8000ll, puzzle | 0xe00c000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x800c00c000ll))
        doSolve(15, 0x8000ll, puzzle | 0x800c00c000ll,
                   partsUsed | 0x400l);
    piece11at14:
      if (partsUsed & 0x800l) goto piece12at14;
      if (!(puzzle & 0x4804000ll)) {
         if(!(puzzle & 0x6c04000ll))
           doSolve(15, 0x8000ll, puzzle | 0x6c04000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xd804000ll))
           doSolve(15, 0x8000ll, puzzle | 0xd804000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xc80c000ll))
           doSolve(15, 0x8000ll, puzzle | 0xc80c000ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0xd00c000ll))
        doSolve(15, 0x8000ll, puzzle | 0xd00c000ll,
                   partsUsed | 0x800l);
    piece12at14:
    break;
   case 15:
      if (partsUsed & 0x2l) goto piece2at15;
      if(!(puzzle & 0x1209008000ll))
        doSolve(16, 0x10000ll, puzzle | 0x1209008000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x600c008000ll))
        doSolve(16, 0x10000ll, puzzle | 0x600c008000ll,
                   partsUsed | 0x2l);
    piece2at15:
      if (partsUsed & 0x4l) goto piece3at15;
      if(!(puzzle & 0x8009208000ll))
        doSolve(16, 0x10000ll, puzzle | 0x8009208000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x800e008000ll))
        doSolve(16, 0x10000ll, puzzle | 0x800e008000ll,
                   partsUsed | 0x4l);
    piece3at15:
      if (partsUsed & 0x8l) goto piece4at15;
      if(!(puzzle & 0x209208000ll))
        doSolve(16, 0x10000ll, puzzle | 0x209208000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x200e008000ll))
        doSolve(16, 0x10000ll, puzzle | 0x200e008000ll,
                   partsUsed | 0x8l);
    piece4at15:
      if (partsUsed & 0x10l) goto piece5at15;
      if(!(puzzle & 0x9008009008000ll))
        doSolve(16, 0x10000ll, puzzle | 0x9008009008000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0xc00800c008000ll))
        doSolve(16, 0x10000ll, puzzle | 0xc00800c008000ll,
                   partsUsed | 0x10l);
    piece5at15:
      if (partsUsed & 0x20l) goto piece6at15;
      if(!(puzzle & 0x9248000ll))
        doSolve(16, 0x10000ll, puzzle | 0x9248000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x9008008008000ll))
        doSolve(16, 0x10000ll, puzzle | 0x9008008008000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0xc008008008000ll))
        doSolve(16, 0x10000ll, puzzle | 0xc008008008000ll,
                   partsUsed | 0x20l);
    piece6at15:
      if (partsUsed & 0x40l) goto piece7at15;
      if (!(puzzle & 0x8008008008000ll)) {
         if(!(puzzle & 0x8009008008000ll))
           doSolve(16, 0x10000ll, puzzle | 0x8009008008000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x8008009008000ll))
           doSolve(16, 0x10000ll, puzzle | 0x8008009008000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800800c008000ll))
           doSolve(16, 0x10000ll, puzzle | 0x800800c008000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800c008008000ll))
           doSolve(16, 0x10000ll, puzzle | 0x800c008008000ll,
                   partsUsed | 0x40l);
      }
    piece7at15:
      if (partsUsed & 0x100l) goto piece9at15;
      if(!(puzzle & 0xc808000ll))
        doSolve(16, 0x10000ll, puzzle | 0xc808000ll,
                   partsUsed | 0x100l);
    piece9at15:
      if (partsUsed & 0x200l) goto piece10at15;
      if(!(puzzle & 0x1009208000ll))
        doSolve(16, 0x10000ll, puzzle | 0x1009208000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x400e008000ll))
        doSolve(16, 0x10000ll, puzzle | 0x400e008000ll,
                   partsUsed | 0x200l);
    piece10at15:
      if (partsUsed & 0x400l) goto piece11at15;
      if(!(puzzle & 0x9009008000ll))
        doSolve(16, 0x10000ll, puzzle | 0x9009008000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0xc00c008000ll))
        doSolve(16, 0x10000ll, puzzle | 0xc00c008000ll,
                   partsUsed | 0x400l);
    piece11at15:
      if (partsUsed & 0x800l) goto piece12at15;
      if(!(puzzle & 0xd808000ll))
        doSolve(16, 0x10000ll, puzzle | 0xd808000ll,
                   partsUsed | 0x800l);
    piece12at15:
    break;
   case 16:
      if (partsUsed & 0x2l) goto piece2at16;
      if(!(puzzle & 0x480090010000ll))
        doSolve(17, 0x20000ll, puzzle | 0x480090010000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x400480090000ll))
        doSolve(17, 0x20000ll, puzzle | 0x400480090000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1330000ll))
        doSolve(17, 0x20000ll, puzzle | 0x1330000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1990000ll))
        doSolve(17, 0x20000ll, puzzle | 0x1990000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x40060030000ll))
        doSolve(17, 0x20000ll, puzzle | 0x40060030000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x60030010000ll))
        doSolve(17, 0x20000ll, puzzle | 0x60030010000ll,
                   partsUsed | 0x2l);
    piece2at16:
      if (partsUsed & 0x4l) goto piece3at16;
      if(!(puzzle & 0x10490010000ll))
        doSolve(17, 0x20000ll, puzzle | 0x10490010000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x10070010000ll))
        doSolve(17, 0x20000ll, puzzle | 0x10070010000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x80080490000ll))
        doSolve(17, 0x20000ll, puzzle | 0x80080490000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x790000ll))
        doSolve(17, 0x20000ll, puzzle | 0x790000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x970000ll))
        doSolve(17, 0x20000ll, puzzle | 0x970000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x20020070000ll))
        doSolve(17, 0x20000ll, puzzle | 0x20020070000ll,
                   partsUsed | 0x4l);
    piece3at16:
      if (partsUsed & 0x8l) goto piece4at16;
      if(!(puzzle & 0x400490010000ll))
        doSolve(17, 0x20000ll, puzzle | 0x400490010000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x480080090000ll))
        doSolve(17, 0x20000ll, puzzle | 0x480080090000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1930000ll))
        doSolve(17, 0x20000ll, puzzle | 0x1930000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1390000ll))
        doSolve(17, 0x20000ll, puzzle | 0x1390000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x40070010000ll))
        doSolve(17, 0x20000ll, puzzle | 0x40070010000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x60020030000ll))
        doSolve(17, 0x20000ll, puzzle | 0x60020030000ll,
                   partsUsed | 0x8l);
    piece4at16:
      if (partsUsed & 0x10l) goto piece5at16;
      if (!(puzzle & 0x2490000ll)) {
         if(!(puzzle & 0x6590000ll))
           doSolve(17, 0x20000ll, puzzle | 0x6590000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x2082490000ll))
           doSolve(17, 0x20000ll, puzzle | 0x2082490000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x2cb0000ll))
           doSolve(17, 0x20000ll, puzzle | 0x2cb0000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x90010090010000ll))
        doSolve(17, 0x20000ll, puzzle | 0x90010090010000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x80090080090000ll))
        doSolve(17, 0x20000ll, puzzle | 0x80090080090000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x412490000ll))
        doSolve(17, 0x20000ll, puzzle | 0x412490000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x10090010090000ll))
        doSolve(17, 0x20000ll, puzzle | 0x10090010090000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x4d30000ll))
        doSolve(17, 0x20000ll, puzzle | 0x4d30000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x2490410000ll))
        doSolve(17, 0x20000ll, puzzle | 0x2490410000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x30010030010000ll))
        doSolve(17, 0x20000ll, puzzle | 0x30010030010000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x10030010030000ll))
        doSolve(17, 0x20000ll, puzzle | 0x10030010030000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x20030020030000ll))
        doSolve(17, 0x20000ll, puzzle | 0x20030020030000ll,
                   partsUsed | 0x10l);
    piece5at16:
      if (partsUsed & 0x20l) goto piece6at16;
      if (!(puzzle & 0x10010010010000ll)) {
         if(!(puzzle & 0x10010010090000ll))
           doSolve(17, 0x20000ll, puzzle | 0x10010010090000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x90010010010000ll))
           doSolve(17, 0x20000ll, puzzle | 0x90010010010000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x10010010030000ll))
           doSolve(17, 0x20000ll, puzzle | 0x10010010030000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x30010010010000ll))
           doSolve(17, 0x20000ll, puzzle | 0x30010010010000ll,
                   partsUsed | 0x20l);
      }
      if (!(puzzle & 0x2490000ll)) {
         if(!(puzzle & 0x2002490000ll))
           doSolve(17, 0x20000ll, puzzle | 0x2002490000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x24b0000ll))
           doSolve(17, 0x20000ll, puzzle | 0x24b0000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x12490000ll))
           doSolve(17, 0x20000ll, puzzle | 0x12490000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x6490000ll))
           doSolve(17, 0x20000ll, puzzle | 0x6490000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x2490010000ll))
        doSolve(17, 0x20000ll, puzzle | 0x2490010000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x4930000ll))
        doSolve(17, 0x20000ll, puzzle | 0x4930000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x80080080090000ll))
        doSolve(17, 0x20000ll, puzzle | 0x80080080090000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x20020020030000ll))
        doSolve(17, 0x20000ll, puzzle | 0x20020020030000ll,
                   partsUsed | 0x20l);
    piece6at16:
      if (partsUsed & 0x40l) goto piece7at16;
      if (!(puzzle & 0x10010010010000ll)) {
         if(!(puzzle & 0x10010090010000ll))
           doSolve(17, 0x20000ll, puzzle | 0x10010090010000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x10090010010000ll))
           doSolve(17, 0x20000ll, puzzle | 0x10090010010000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x10010030010000ll))
           doSolve(17, 0x20000ll, puzzle | 0x10010030010000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x10030010010000ll))
           doSolve(17, 0x20000ll, puzzle | 0x10030010010000ll,
                   partsUsed | 0x40l);
      }
      if (!(puzzle & 0x2490000ll)) {
         if(!(puzzle & 0x402490000ll))
           doSolve(17, 0x20000ll, puzzle | 0x402490000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2590000ll))
           doSolve(17, 0x20000ll, puzzle | 0x2590000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x82490000ll))
           doSolve(17, 0x20000ll, puzzle | 0x82490000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2c90000ll))
           doSolve(17, 0x20000ll, puzzle | 0x2c90000ll,
                   partsUsed | 0x40l);
      }
    piece7at16:
      if (partsUsed & 0x80l) goto piece8at16;
      if(!(puzzle & 0x90010090000ll))
        doSolve(17, 0x20000ll, puzzle | 0x90010090000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x90080090000ll))
        doSolve(17, 0x20000ll, puzzle | 0x90080090000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x490410000ll))
        doSolve(17, 0x20000ll, puzzle | 0x490410000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x410490000ll))
        doSolve(17, 0x20000ll, puzzle | 0x410490000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x2f0000ll))
        doSolve(17, 0x20000ll, puzzle | 0x2f0000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xcb0000ll))
        doSolve(17, 0x20000ll, puzzle | 0xcb0000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xd30000ll))
        doSolve(17, 0x20000ll, puzzle | 0xd30000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x3d0000ll))
        doSolve(17, 0x20000ll, puzzle | 0x3d0000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x30020030000ll))
        doSolve(17, 0x20000ll, puzzle | 0x30020030000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x30010030000ll))
        doSolve(17, 0x20000ll, puzzle | 0x30010030000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x70050000ll))
        doSolve(17, 0x20000ll, puzzle | 0x70050000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x50070000ll))
        doSolve(17, 0x20000ll, puzzle | 0x50070000ll,
                   partsUsed | 0x80l);
    piece8at16:
      if (partsUsed & 0x100l) goto piece9at16;
      if(!(puzzle & 0x100130000ll))
        doSolve(17, 0x20000ll, puzzle | 0x100130000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x130010000ll))
        doSolve(17, 0x20000ll, puzzle | 0x130010000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x200b0000ll))
        doSolve(17, 0x20000ll, puzzle | 0x200b0000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x90030000ll))
        doSolve(17, 0x20000ll, puzzle | 0x90030000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x180090000ll))
        doSolve(17, 0x20000ll, puzzle | 0x180090000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x10190000ll))
        doSolve(17, 0x20000ll, puzzle | 0x10190000ll,
                   partsUsed | 0x100l);
    piece9at16:
      if (partsUsed & 0x200l) goto piece10at16;
      if(!(puzzle & 0x80490010000ll))
        doSolve(17, 0x20000ll, puzzle | 0x80490010000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x80480090000ll))
        doSolve(17, 0x20000ll, puzzle | 0x80480090000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0xb30000ll))
        doSolve(17, 0x20000ll, puzzle | 0xb30000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0xb90000ll))
        doSolve(17, 0x20000ll, puzzle | 0xb90000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x20060030000ll))
        doSolve(17, 0x20000ll, puzzle | 0x20060030000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x20070010000ll))
        doSolve(17, 0x20000ll, puzzle | 0x20070010000ll,
                   partsUsed | 0x200l);
    piece10at16:
      if (partsUsed & 0x400l) goto piece11at16;
      if (!(puzzle & 0x80090000ll)) {
         if(!(puzzle & 0x10090090000ll))
           doSolve(17, 0x20000ll, puzzle | 0x10090090000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x490090000ll))
           doSolve(17, 0x20000ll, puzzle | 0x490090000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x480490000ll))
           doSolve(17, 0x20000ll, puzzle | 0x480490000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x90490000ll))
           doSolve(17, 0x20000ll, puzzle | 0x90490000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x80090090000ll))
           doSolve(17, 0x20000ll, puzzle | 0x80090090000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x130000ll)) {
         if(!(puzzle & 0x370000ll))
           doSolve(17, 0x20000ll, puzzle | 0x370000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1f0000ll))
           doSolve(17, 0x20000ll, puzzle | 0x1f0000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x5b0000ll))
           doSolve(17, 0x20000ll, puzzle | 0x5b0000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x9b0000ll))
           doSolve(17, 0x20000ll, puzzle | 0x9b0000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x3b0000ll))
           doSolve(17, 0x20000ll, puzzle | 0x3b0000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x10010010000ll)) {
         if(!(puzzle & 0x90090010000ll))
           doSolve(17, 0x20000ll, puzzle | 0x90090010000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x10030030000ll))
           doSolve(17, 0x20000ll, puzzle | 0x10030030000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x30030010000ll))
           doSolve(17, 0x20000ll, puzzle | 0x30030010000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x20030000ll)) {
         if(!(puzzle & 0x20030030000ll))
           doSolve(17, 0x20000ll, puzzle | 0x20030030000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x70030000ll))
           doSolve(17, 0x20000ll, puzzle | 0x70030000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x30070000ll))
           doSolve(17, 0x20000ll, puzzle | 0x30070000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x60070000ll))
           doSolve(17, 0x20000ll, puzzle | 0x60070000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0xd90000ll))
        doSolve(17, 0x20000ll, puzzle | 0xd90000ll,
                   partsUsed | 0x400l);
    piece11at16:
      if (partsUsed & 0x800l) goto piece12at16;
      if (!(puzzle & 0x10030000ll)) {
         if(!(puzzle & 0x101b0000ll))
           doSolve(17, 0x20000ll, puzzle | 0x101b0000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xb0030000ll))
           doSolve(17, 0x20000ll, puzzle | 0xb0030000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x300b0000ll))
           doSolve(17, 0x20000ll, puzzle | 0x300b0000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x30130000ll))
           doSolve(17, 0x20000ll, puzzle | 0x30130000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x130030000ll))
           doSolve(17, 0x20000ll, puzzle | 0x130030000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x900b0000ll))
           doSolve(17, 0x20000ll, puzzle | 0x900b0000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x100110000ll)) {
         if(!(puzzle & 0x180190000ll))
           doSolve(17, 0x20000ll, puzzle | 0x180190000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x120130000ll))
           doSolve(17, 0x20000ll, puzzle | 0x120130000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1001b0000ll))
           doSolve(17, 0x20000ll, puzzle | 0x1001b0000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x80090000ll)) {
         if(!(puzzle & 0xb0090000ll))
           doSolve(17, 0x20000ll, puzzle | 0xb0090000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x90190000ll))
           doSolve(17, 0x20000ll, puzzle | 0x90190000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x801b0000ll))
           doSolve(17, 0x20000ll, puzzle | 0x801b0000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x190090000ll))
           doSolve(17, 0x20000ll, puzzle | 0x190090000ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0x201b0000ll))
        doSolve(17, 0x20000ll, puzzle | 0x201b0000ll,
                   partsUsed | 0x800l);
      if(!(puzzle & 0x1b0010000ll))
        doSolve(17, 0x20000ll, puzzle | 0x1b0010000ll,
                   partsUsed | 0x800l);
    piece12at16:
    break;
   case 17:
      if (partsUsed & 0x2l) goto piece2at17;
      if(!(puzzle & 0x5e0000ll))
        doSolve(18, 0x40000ll, puzzle | 0x5e0000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x10030060000ll))
        doSolve(18, 0x40000ll, puzzle | 0x10030060000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x900120020000ll))
        doSolve(18, 0x40000ll, puzzle | 0x900120020000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x800900120000ll))
        doSolve(18, 0x40000ll, puzzle | 0x800900120000ll,
                   partsUsed | 0x2l);
    piece2at17:
      if (partsUsed & 0x4l) goto piece3at17;
      if(!(puzzle & 0x1d20000ll))
        doSolve(18, 0x40000ll, puzzle | 0x1d20000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x100100920000ll))
        doSolve(18, 0x40000ll, puzzle | 0x100100920000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x70020020000ll))
        doSolve(18, 0x40000ll, puzzle | 0x70020020000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x20920020000ll))
        doSolve(18, 0x40000ll, puzzle | 0x20920020000ll,
                   partsUsed | 0x4l);
    piece3at17:
      if (partsUsed & 0x8l) goto piece4at17;
      if(!(puzzle & 0xd60000ll))
        doSolve(18, 0x40000ll, puzzle | 0xd60000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x30020060000ll))
        doSolve(18, 0x40000ll, puzzle | 0x30020060000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x800920020000ll))
        doSolve(18, 0x40000ll, puzzle | 0x800920020000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x900100120000ll))
        doSolve(18, 0x40000ll, puzzle | 0x900100120000ll,
                   partsUsed | 0x8l);
    piece4at17:
      if (partsUsed & 0x10l) goto piece5at17;
      if (!(puzzle & 0x4920000ll)) {
         if(!(puzzle & 0x69a0000ll))
           doSolve(18, 0x40000ll, puzzle | 0x69a0000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x824920000ll))
           doSolve(18, 0x40000ll, puzzle | 0x824920000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0xcb20000ll))
           doSolve(18, 0x40000ll, puzzle | 0xcb20000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x4104920000ll))
           doSolve(18, 0x40000ll, puzzle | 0x4104920000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x5960000ll))
           doSolve(18, 0x40000ll, puzzle | 0x5960000ll,
                   partsUsed | 0x10l);
      }
      if (!(puzzle & 0x20020020020000ll)) {
         if(!(puzzle & 0x30020030020000ll))
           doSolve(18, 0x40000ll, puzzle | 0x30020030020000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x120020120020000ll))
           doSolve(18, 0x40000ll, puzzle | 0x120020120020000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x20120020120000ll))
           doSolve(18, 0x40000ll, puzzle | 0x20120020120000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x60020060020000ll))
           doSolve(18, 0x40000ll, puzzle | 0x60020060020000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x20060020060000ll))
           doSolve(18, 0x40000ll, puzzle | 0x20060020060000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x100120100120000ll))
        doSolve(18, 0x40000ll, puzzle | 0x100120100120000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x9a60000ll))
        doSolve(18, 0x40000ll, puzzle | 0x9a60000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x4920820000ll))
        doSolve(18, 0x40000ll, puzzle | 0x4920820000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x40060040060000ll))
        doSolve(18, 0x40000ll, puzzle | 0x40060040060000ll,
                   partsUsed | 0x10l);
    piece5at17:
      if (partsUsed & 0x20l) goto piece6at17;
      if (!(puzzle & 0x4920000ll)) {
         if(!(puzzle & 0x6920000ll))
           doSolve(18, 0x40000ll, puzzle | 0x6920000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x4004920000ll))
           doSolve(18, 0x40000ll, puzzle | 0x4004920000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x4960000ll))
           doSolve(18, 0x40000ll, puzzle | 0x4960000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x24920000ll))
           doSolve(18, 0x40000ll, puzzle | 0x24920000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0xc920000ll))
           doSolve(18, 0x40000ll, puzzle | 0xc920000ll,
                   partsUsed | 0x20l);
      }
      if (!(puzzle & 0x20020020020000ll)) {
         if(!(puzzle & 0x30020020020000ll))
           doSolve(18, 0x40000ll, puzzle | 0x30020020020000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x20020020120000ll))
           doSolve(18, 0x40000ll, puzzle | 0x20020020120000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x120020020020000ll))
           doSolve(18, 0x40000ll, puzzle | 0x120020020020000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x20020020060000ll))
           doSolve(18, 0x40000ll, puzzle | 0x20020020060000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x60020020020000ll))
           doSolve(18, 0x40000ll, puzzle | 0x60020020020000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x4920020000ll))
        doSolve(18, 0x40000ll, puzzle | 0x4920020000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x9260000ll))
        doSolve(18, 0x40000ll, puzzle | 0x9260000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x100100100120000ll))
        doSolve(18, 0x40000ll, puzzle | 0x100100100120000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x40040040060000ll))
        doSolve(18, 0x40000ll, puzzle | 0x40040040060000ll,
                   partsUsed | 0x20l);
    piece6at17:
      if (partsUsed & 0x40l) goto piece7at17;
      if (!(puzzle & 0x4920000ll)) {
         if(!(puzzle & 0x49a0000ll))
           doSolve(18, 0x40000ll, puzzle | 0x49a0000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4d20000ll))
           doSolve(18, 0x40000ll, puzzle | 0x4d20000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x804920000ll))
           doSolve(18, 0x40000ll, puzzle | 0x804920000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4b20000ll))
           doSolve(18, 0x40000ll, puzzle | 0x4b20000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x104920000ll))
           doSolve(18, 0x40000ll, puzzle | 0x104920000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x5920000ll))
           doSolve(18, 0x40000ll, puzzle | 0x5920000ll,
                   partsUsed | 0x40l);
      }
      if (!(puzzle & 0x20020020020000ll)) {
         if(!(puzzle & 0x20020030020000ll))
           doSolve(18, 0x40000ll, puzzle | 0x20020030020000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x20030020020000ll))
           doSolve(18, 0x40000ll, puzzle | 0x20030020020000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x20020120020000ll))
           doSolve(18, 0x40000ll, puzzle | 0x20020120020000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x20120020020000ll))
           doSolve(18, 0x40000ll, puzzle | 0x20120020020000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x20020060020000ll))
           doSolve(18, 0x40000ll, puzzle | 0x20020060020000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x20060020020000ll))
           doSolve(18, 0x40000ll, puzzle | 0x20060020020000ll,
                   partsUsed | 0x40l);
      }
    piece7at17:
      if (partsUsed & 0x80l) goto piece8at17;
      if(!(puzzle & 0x120020120000ll))
        doSolve(18, 0x40000ll, puzzle | 0x120020120000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x120100120000ll))
        doSolve(18, 0x40000ll, puzzle | 0x120100120000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x920820000ll))
        doSolve(18, 0x40000ll, puzzle | 0x920820000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x820920000ll))
        doSolve(18, 0x40000ll, puzzle | 0x820920000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1960000ll))
        doSolve(18, 0x40000ll, puzzle | 0x1960000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1a60000ll))
        doSolve(18, 0x40000ll, puzzle | 0x1a60000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x60040060000ll))
        doSolve(18, 0x40000ll, puzzle | 0x60040060000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x60020060000ll))
        doSolve(18, 0x40000ll, puzzle | 0x60020060000ll,
                   partsUsed | 0x80l);
    piece8at17:
      if (partsUsed & 0x100l) goto piece9at17;
      if(!(puzzle & 0x801a0000ll))
        doSolve(18, 0x40000ll, puzzle | 0x801a0000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x30120000ll))
        doSolve(18, 0x40000ll, puzzle | 0x30120000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x20320000ll))
        doSolve(18, 0x40000ll, puzzle | 0x20320000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1a0020000ll))
        doSolve(18, 0x40000ll, puzzle | 0x1a0020000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x120060000ll))
        doSolve(18, 0x40000ll, puzzle | 0x120060000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x260020000ll))
        doSolve(18, 0x40000ll, puzzle | 0x260020000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x40160000ll))
        doSolve(18, 0x40000ll, puzzle | 0x40160000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x300120000ll))
        doSolve(18, 0x40000ll, puzzle | 0x300120000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x200260000ll))
        doSolve(18, 0x40000ll, puzzle | 0x200260000ll,
                   partsUsed | 0x100l);
    piece9at17:
      if (partsUsed & 0x200l) goto piece10at17;
      if (!(puzzle & 0x1a0000ll)) {
         if(!(puzzle & 0x13a0000ll))
           doSolve(18, 0x40000ll, puzzle | 0x13a0000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x7a0000ll))
           doSolve(18, 0x40000ll, puzzle | 0x7a0000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x9e0000ll))
           doSolve(18, 0x40000ll, puzzle | 0x9e0000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x19a0000ll))
           doSolve(18, 0x40000ll, puzzle | 0x19a0000ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x20020020000ll)) {
         if(!(puzzle & 0x20030060000ll))
           doSolve(18, 0x40000ll, puzzle | 0x20030060000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x60030020000ll))
           doSolve(18, 0x40000ll, puzzle | 0x60030020000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x30060020000ll))
           doSolve(18, 0x40000ll, puzzle | 0x30060020000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0xf20000ll))
        doSolve(18, 0x40000ll, puzzle | 0xf20000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x10070020000ll))
        doSolve(18, 0x40000ll, puzzle | 0x10070020000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x40070020000ll))
        doSolve(18, 0x40000ll, puzzle | 0x40070020000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100920020000ll))
        doSolve(18, 0x40000ll, puzzle | 0x100920020000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100900120000ll))
        doSolve(18, 0x40000ll, puzzle | 0x100900120000ll,
                   partsUsed | 0x200l);
    piece10at17:
      if (partsUsed & 0x400l) goto piece11at17;
      if (!(puzzle & 0x100120000ll)) {
         if(!(puzzle & 0x20120120000ll))
           doSolve(18, 0x40000ll, puzzle | 0x20120120000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x920120000ll))
           doSolve(18, 0x40000ll, puzzle | 0x920120000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x900920000ll))
           doSolve(18, 0x40000ll, puzzle | 0x900920000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x120920000ll))
           doSolve(18, 0x40000ll, puzzle | 0x120920000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x100120120000ll))
           doSolve(18, 0x40000ll, puzzle | 0x100120120000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x20020020000ll)) {
         if(!(puzzle & 0x120120020000ll))
           doSolve(18, 0x40000ll, puzzle | 0x120120020000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x20060060000ll))
           doSolve(18, 0x40000ll, puzzle | 0x20060060000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x60060020000ll))
           doSolve(18, 0x40000ll, puzzle | 0x60060020000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x320000ll)) {
         if(!(puzzle & 0xb60000ll))
           doSolve(18, 0x40000ll, puzzle | 0xb60000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1360000ll))
           doSolve(18, 0x40000ll, puzzle | 0x1360000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1b20000ll))
           doSolve(18, 0x40000ll, puzzle | 0x1b20000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x3e0000ll))
        doSolve(18, 0x40000ll, puzzle | 0x3e0000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0xda0000ll))
        doSolve(18, 0x40000ll, puzzle | 0xda0000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x70060000ll))
        doSolve(18, 0x40000ll, puzzle | 0x70060000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x30030020000ll))
        doSolve(18, 0x40000ll, puzzle | 0x30030020000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x40060060000ll))
        doSolve(18, 0x40000ll, puzzle | 0x40060060000ll,
                   partsUsed | 0x400l);
    piece11at17:
      if (partsUsed & 0x800l) goto piece12at17;
      if (!(puzzle & 0x180020000ll)) {
         if(!(puzzle & 0x1b0020000ll))
           doSolve(18, 0x40000ll, puzzle | 0x1b0020000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1a0120000ll))
           doSolve(18, 0x40000ll, puzzle | 0x1a0120000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1801a0000ll))
           doSolve(18, 0x40000ll, puzzle | 0x1801a0000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x20120000ll)) {
         if(!(puzzle & 0x130120000ll))
           doSolve(18, 0x40000ll, puzzle | 0x130120000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1201a0000ll))
           doSolve(18, 0x40000ll, puzzle | 0x1201a0000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x20360000ll))
           doSolve(18, 0x40000ll, puzzle | 0x20360000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x160120000ll))
           doSolve(18, 0x40000ll, puzzle | 0x160120000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x60160000ll))
           doSolve(18, 0x40000ll, puzzle | 0x60160000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x120320000ll))
           doSolve(18, 0x40000ll, puzzle | 0x120320000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x320120000ll))
           doSolve(18, 0x40000ll, puzzle | 0x320120000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x120160000ll))
           doSolve(18, 0x40000ll, puzzle | 0x120160000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x40060000ll)) {
         if(!(puzzle & 0x160060000ll))
           doSolve(18, 0x40000ll, puzzle | 0x160060000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x240260000ll))
           doSolve(18, 0x40000ll, puzzle | 0x240260000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x60260000ll))
           doSolve(18, 0x40000ll, puzzle | 0x60260000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x260060000ll))
           doSolve(18, 0x40000ll, puzzle | 0x260060000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x40360000ll))
           doSolve(18, 0x40000ll, puzzle | 0x40360000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x320000ll)) {
         if(!(puzzle & 0x300320000ll))
           doSolve(18, 0x40000ll, puzzle | 0x300320000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x200360000ll))
           doSolve(18, 0x40000ll, puzzle | 0x200360000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x100360000ll))
           doSolve(18, 0x40000ll, puzzle | 0x100360000ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0x360020000ll))
        doSolve(18, 0x40000ll, puzzle | 0x360020000ll,
                   partsUsed | 0x800l);
    piece12at17:
    break;
   case 18:
      if (partsUsed & 0x2l) goto piece2at18;
      if(!(puzzle & 0xf40000ll))
        doSolve(19, 0x80000ll, puzzle | 0xf40000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x30060040000ll))
        doSolve(19, 0x80000ll, puzzle | 0x30060040000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1200240040000ll))
        doSolve(19, 0x80000ll, puzzle | 0x1200240040000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1001200240000ll))
        doSolve(19, 0x80000ll, puzzle | 0x1001200240000ll,
                   partsUsed | 0x2l);
    piece2at18:
      if (partsUsed & 0x4l) goto piece3at18;
      if(!(puzzle & 0x13c0000ll))
        doSolve(19, 0x80000ll, puzzle | 0x13c0000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x200201240000ll))
        doSolve(19, 0x80000ll, puzzle | 0x200201240000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x40070040000ll))
        doSolve(19, 0x80000ll, puzzle | 0x40070040000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x41240040000ll))
        doSolve(19, 0x80000ll, puzzle | 0x41240040000ll,
                   partsUsed | 0x4l);
    piece3at18:
      if (partsUsed & 0x8l) goto piece4at18;
      if(!(puzzle & 0x7c0000ll))
        doSolve(19, 0x80000ll, puzzle | 0x7c0000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x10070040000ll))
        doSolve(19, 0x80000ll, puzzle | 0x10070040000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1001240040000ll))
        doSolve(19, 0x80000ll, puzzle | 0x1001240040000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1200200240000ll))
        doSolve(19, 0x80000ll, puzzle | 0x1200200240000ll,
                   partsUsed | 0x8l);
    piece4at18:
      if (partsUsed & 0x10l) goto piece5at18;
      if (!(puzzle & 0x9240000ll)) {
         if(!(puzzle & 0xd340000ll))
           doSolve(19, 0x80000ll, puzzle | 0xd340000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x1049240000ll))
           doSolve(19, 0x80000ll, puzzle | 0x1049240000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x8209240000ll))
           doSolve(19, 0x80000ll, puzzle | 0x8209240000ll,
                   partsUsed | 0x10l);
      }
      if (!(puzzle & 0x40040040040000ll)) {
         if(!(puzzle & 0x60040060040000ll))
           doSolve(19, 0x80000ll, puzzle | 0x60040060040000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x240040240040000ll))
           doSolve(19, 0x80000ll, puzzle | 0x240040240040000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x40240040240000ll))
           doSolve(19, 0x80000ll, puzzle | 0x40240040240000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x200240200240000ll))
        doSolve(19, 0x80000ll, puzzle | 0x200240200240000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x9241040000ll))
        doSolve(19, 0x80000ll, puzzle | 0x9241040000ll,
                   partsUsed | 0x10l);
    piece5at18:
      if (partsUsed & 0x20l) goto piece6at18;
      if (!(puzzle & 0x9240000ll)) {
         if(!(puzzle & 0xd240000ll))
           doSolve(19, 0x80000ll, puzzle | 0xd240000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x8009240000ll))
           doSolve(19, 0x80000ll, puzzle | 0x8009240000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x49240000ll))
           doSolve(19, 0x80000ll, puzzle | 0x49240000ll,
                   partsUsed | 0x20l);
      }
      if (!(puzzle & 0x40040040040000ll)) {
         if(!(puzzle & 0x60040040040000ll))
           doSolve(19, 0x80000ll, puzzle | 0x60040040040000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x40040040240000ll))
           doSolve(19, 0x80000ll, puzzle | 0x40040040240000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x240040040040000ll))
           doSolve(19, 0x80000ll, puzzle | 0x240040040040000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x9240040000ll))
        doSolve(19, 0x80000ll, puzzle | 0x9240040000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x200200200240000ll))
        doSolve(19, 0x80000ll, puzzle | 0x200200200240000ll,
                   partsUsed | 0x20l);
    piece6at18:
      if (partsUsed & 0x40l) goto piece7at18;
      if (!(puzzle & 0x9240000ll)) {
         if(!(puzzle & 0x9340000ll))
           doSolve(19, 0x80000ll, puzzle | 0x9340000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x9a40000ll))
           doSolve(19, 0x80000ll, puzzle | 0x9a40000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x1009240000ll))
           doSolve(19, 0x80000ll, puzzle | 0x1009240000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x209240000ll))
           doSolve(19, 0x80000ll, puzzle | 0x209240000ll,
                   partsUsed | 0x40l);
      }
      if (!(puzzle & 0x40040040040000ll)) {
         if(!(puzzle & 0x40040060040000ll))
           doSolve(19, 0x80000ll, puzzle | 0x40040060040000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x40060040040000ll))
           doSolve(19, 0x80000ll, puzzle | 0x40060040040000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x40040240040000ll))
           doSolve(19, 0x80000ll, puzzle | 0x40040240040000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x40240040040000ll))
           doSolve(19, 0x80000ll, puzzle | 0x40240040040000ll,
                   partsUsed | 0x40l);
      }
    piece7at18:
      if (partsUsed & 0x80l) goto piece8at18;
      if(!(puzzle & 0x240040240000ll))
        doSolve(19, 0x80000ll, puzzle | 0x240040240000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x240200240000ll))
        doSolve(19, 0x80000ll, puzzle | 0x240200240000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1241040000ll))
        doSolve(19, 0x80000ll, puzzle | 0x1241040000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1041240000ll))
        doSolve(19, 0x80000ll, puzzle | 0x1041240000ll,
                   partsUsed | 0x80l);
    piece8at18:
      if (partsUsed & 0x100l) goto piece9at18;
      if(!(puzzle & 0x340040000ll))
        doSolve(19, 0x80000ll, puzzle | 0x340040000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x100340000ll))
        doSolve(19, 0x80000ll, puzzle | 0x100340000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x60240000ll))
        doSolve(19, 0x80000ll, puzzle | 0x60240000ll,
                   partsUsed | 0x100l);
    piece9at18:
      if (partsUsed & 0x200l) goto piece10at18;
      if(!(puzzle & 0xbc0000ll))
        doSolve(19, 0x80000ll, puzzle | 0xbc0000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x20070040000ll))
        doSolve(19, 0x80000ll, puzzle | 0x20070040000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x201240040000ll))
        doSolve(19, 0x80000ll, puzzle | 0x201240040000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x201200240000ll))
        doSolve(19, 0x80000ll, puzzle | 0x201200240000ll,
                   partsUsed | 0x200l);
    piece10at18:
      if (partsUsed & 0x400l) goto piece11at18;
      if (!(puzzle & 0x1240000ll)) {
         if(!(puzzle & 0x1b40000ll))
           doSolve(19, 0x80000ll, puzzle | 0x1b40000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1201240000ll))
           doSolve(19, 0x80000ll, puzzle | 0x1201240000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x241240000ll))
           doSolve(19, 0x80000ll, puzzle | 0x241240000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x40040040000ll)) {
         if(!(puzzle & 0x60060040000ll))
           doSolve(19, 0x80000ll, puzzle | 0x60060040000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x40240240000ll))
           doSolve(19, 0x80000ll, puzzle | 0x40240240000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x240240040000ll))
           doSolve(19, 0x80000ll, puzzle | 0x240240040000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x1240240000ll))
        doSolve(19, 0x80000ll, puzzle | 0x1240240000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x200240240000ll))
        doSolve(19, 0x80000ll, puzzle | 0x200240240000ll,
                   partsUsed | 0x400l);
    piece11at18:
      if (partsUsed & 0x800l) goto piece12at18;
      if (!(puzzle & 0x300040000ll)) {
         if(!(puzzle & 0x360040000ll))
           doSolve(19, 0x80000ll, puzzle | 0x360040000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x340240000ll))
           doSolve(19, 0x80000ll, puzzle | 0x340240000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x300340000ll))
           doSolve(19, 0x80000ll, puzzle | 0x300340000ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0x260240000ll))
        doSolve(19, 0x80000ll, puzzle | 0x260240000ll,
                   partsUsed | 0x800l);
      if(!(puzzle & 0x240340000ll))
        doSolve(19, 0x80000ll, puzzle | 0x240340000ll,
                   partsUsed | 0x800l);
    piece12at18:
    break;
   case 19:
      if (partsUsed & 0x2l) goto piece2at19;
      if(!(puzzle & 0x10090480000ll))
        doSolve(20, 0x100000ll, puzzle | 0x10090480000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x2400480080000ll))
        doSolve(20, 0x100000ll, puzzle | 0x2400480080000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x2002400480000ll))
        doSolve(20, 0x100000ll, puzzle | 0x2002400480000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x9980000ll))
        doSolve(20, 0x100000ll, puzzle | 0x9980000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0xcc80000ll))
        doSolve(20, 0x100000ll, puzzle | 0xcc80000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x200300180000ll))
        doSolve(20, 0x100000ll, puzzle | 0x200300180000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x300180080000ll))
        doSolve(20, 0x100000ll, puzzle | 0x300180080000ll,
                   partsUsed | 0x2l);
    piece2at19:
      if (partsUsed & 0x4l) goto piece3at19;
      if (!(puzzle & 0x80080080000ll)) {
         if(!(puzzle & 0x490080080000ll))
           doSolve(20, 0x100000ll, puzzle | 0x490080080000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x82480080000ll))
           doSolve(20, 0x100000ll, puzzle | 0x82480080000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x80380080000ll))
           doSolve(20, 0x100000ll, puzzle | 0x80380080000ll,
                   partsUsed | 0x4l);
      }
      if(!(puzzle & 0x400402480000ll))
        doSolve(20, 0x100000ll, puzzle | 0x400402480000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x3c80000ll))
        doSolve(20, 0x100000ll, puzzle | 0x3c80000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x4b80000ll))
        doSolve(20, 0x100000ll, puzzle | 0x4b80000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x100100380000ll))
        doSolve(20, 0x100000ll, puzzle | 0x100100380000ll,
                   partsUsed | 0x4l);
    piece3at19:
      if (partsUsed & 0x8l) goto piece4at19;
      if(!(puzzle & 0x90080480000ll))
        doSolve(20, 0x100000ll, puzzle | 0x90080480000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x2002480080000ll))
        doSolve(20, 0x100000ll, puzzle | 0x2002480080000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x2400400480000ll))
        doSolve(20, 0x100000ll, puzzle | 0x2400400480000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0xc980000ll))
        doSolve(20, 0x100000ll, puzzle | 0xc980000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x9c80000ll))
        doSolve(20, 0x100000ll, puzzle | 0x9c80000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x200380080000ll))
        doSolve(20, 0x100000ll, puzzle | 0x200380080000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x300100180000ll))
        doSolve(20, 0x100000ll, puzzle | 0x300100180000ll,
                   partsUsed | 0x8l);
    piece4at19:
      if (partsUsed & 0x10l) goto piece5at19;
      if (!(puzzle & 0x80080080080000ll)) {
         if(!(puzzle & 0x80480080480000ll))
           doSolve(20, 0x100000ll, puzzle | 0x80480080480000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x180080180080000ll))
           doSolve(20, 0x100000ll, puzzle | 0x180080180080000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x80180080180000ll))
           doSolve(20, 0x100000ll, puzzle | 0x80180080180000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x2492080000ll))
        doSolve(20, 0x100000ll, puzzle | 0x2492080000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x90080090080000ll))
        doSolve(20, 0x100000ll, puzzle | 0x90080090080000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x480080480080000ll))
        doSolve(20, 0x100000ll, puzzle | 0x480080480080000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x400480400480000ll))
        doSolve(20, 0x100000ll, puzzle | 0x400480400480000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x100180100180000ll))
        doSolve(20, 0x100000ll, puzzle | 0x100180100180000ll,
                   partsUsed | 0x10l);
    piece5at19:
      if (partsUsed & 0x20l) goto piece6at19;
      if (!(puzzle & 0x80080080080000ll)) {
         if(!(puzzle & 0x90080080080000ll))
           doSolve(20, 0x100000ll, puzzle | 0x90080080080000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x80080080480000ll))
           doSolve(20, 0x100000ll, puzzle | 0x80080080480000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x480080080080000ll))
           doSolve(20, 0x100000ll, puzzle | 0x480080080080000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x80080080180000ll))
           doSolve(20, 0x100000ll, puzzle | 0x80080080180000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x180080080080000ll))
           doSolve(20, 0x100000ll, puzzle | 0x180080080080000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x400400400480000ll))
        doSolve(20, 0x100000ll, puzzle | 0x400400400480000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x100100100180000ll))
        doSolve(20, 0x100000ll, puzzle | 0x100100100180000ll,
                   partsUsed | 0x20l);
    piece6at19:
      if (partsUsed & 0x40l) goto piece7at19;
      if (!(puzzle & 0x80080080080000ll)) {
         if(!(puzzle & 0x80090080080000ll))
           doSolve(20, 0x100000ll, puzzle | 0x80090080080000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x80080480080000ll))
           doSolve(20, 0x100000ll, puzzle | 0x80080480080000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x80480080080000ll))
           doSolve(20, 0x100000ll, puzzle | 0x80480080080000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x80080180080000ll))
           doSolve(20, 0x100000ll, puzzle | 0x80080180080000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x80180080080000ll))
           doSolve(20, 0x100000ll, puzzle | 0x80180080080000ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x2490080000ll))
        doSolve(20, 0x100000ll, puzzle | 0x2490080000ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x80080090080000ll))
        doSolve(20, 0x100000ll, puzzle | 0x80080090080000ll,
                   partsUsed | 0x40l);
    piece7at19:
      if (partsUsed & 0x80l) goto piece8at19;
      if(!(puzzle & 0x480080480000ll))
        doSolve(20, 0x100000ll, puzzle | 0x480080480000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x480400480000ll))
        doSolve(20, 0x100000ll, puzzle | 0x480400480000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x2482080000ll))
        doSolve(20, 0x100000ll, puzzle | 0x2482080000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x2082480000ll))
        doSolve(20, 0x100000ll, puzzle | 0x2082480000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1780000ll))
        doSolve(20, 0x100000ll, puzzle | 0x1780000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x6580000ll))
        doSolve(20, 0x100000ll, puzzle | 0x6580000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x6980000ll))
        doSolve(20, 0x100000ll, puzzle | 0x6980000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1e80000ll))
        doSolve(20, 0x100000ll, puzzle | 0x1e80000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x180100180000ll))
        doSolve(20, 0x100000ll, puzzle | 0x180100180000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x180080180000ll))
        doSolve(20, 0x100000ll, puzzle | 0x180080180000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x380280000ll))
        doSolve(20, 0x100000ll, puzzle | 0x380280000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x280380000ll))
        doSolve(20, 0x100000ll, puzzle | 0x280380000ll,
                   partsUsed | 0x80l);
    piece8at19:
      if (partsUsed & 0x100l) goto piece9at19;
      if(!(puzzle & 0x120180000ll))
        doSolve(20, 0x100000ll, puzzle | 0x120180000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x100580000ll))
        doSolve(20, 0x100000ll, puzzle | 0x100580000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xb0080000ll))
        doSolve(20, 0x100000ll, puzzle | 0xb0080000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x980080000ll))
        doSolve(20, 0x100000ll, puzzle | 0x980080000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x480180000ll))
        doSolve(20, 0x100000ll, puzzle | 0x480180000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xc00480000ll))
        doSolve(20, 0x100000ll, puzzle | 0xc00480000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x80c80000ll))
        doSolve(20, 0x100000ll, puzzle | 0x80c80000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x800980000ll))
        doSolve(20, 0x100000ll, puzzle | 0x800980000ll,
                   partsUsed | 0x100l);
    piece9at19:
      if (partsUsed & 0x200l) goto piece10at19;
      if (!(puzzle & 0x80080080000ll)) {
         if(!(puzzle & 0x80090480000ll))
           doSolve(20, 0x100000ll, puzzle | 0x80090480000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x90480080000ll))
           doSolve(20, 0x100000ll, puzzle | 0x90480080000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x480090080000ll))
           doSolve(20, 0x100000ll, puzzle | 0x480090080000ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x480080000ll)) {
         if(!(puzzle & 0x400490080000ll))
           doSolve(20, 0x100000ll, puzzle | 0x400490080000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x10490080000ll))
           doSolve(20, 0x100000ll, puzzle | 0x10490080000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x402480080000ll))
           doSolve(20, 0x100000ll, puzzle | 0x402480080000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x5980000ll))
        doSolve(20, 0x100000ll, puzzle | 0x5980000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x5c80000ll))
        doSolve(20, 0x100000ll, puzzle | 0x5c80000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x402400480000ll))
        doSolve(20, 0x100000ll, puzzle | 0x402400480000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100300180000ll))
        doSolve(20, 0x100000ll, puzzle | 0x100300180000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100380080000ll))
        doSolve(20, 0x100000ll, puzzle | 0x100380080000ll,
                   partsUsed | 0x200l);
    piece10at19:
      if (partsUsed & 0x400l) goto piece11at19;
      if (!(puzzle & 0x400480000ll)) {
         if(!(puzzle & 0x80480480000ll))
           doSolve(20, 0x100000ll, puzzle | 0x80480480000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2480480000ll))
           doSolve(20, 0x100000ll, puzzle | 0x2480480000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2402480000ll))
           doSolve(20, 0x100000ll, puzzle | 0x2402480000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x482480000ll))
           doSolve(20, 0x100000ll, puzzle | 0x482480000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x400480480000ll))
           doSolve(20, 0x100000ll, puzzle | 0x400480480000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x980000ll)) {
         if(!(puzzle & 0x1b80000ll))
           doSolve(20, 0x100000ll, puzzle | 0x1b80000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xf80000ll))
           doSolve(20, 0x100000ll, puzzle | 0xf80000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2d80000ll))
           doSolve(20, 0x100000ll, puzzle | 0x2d80000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x4d80000ll))
           doSolve(20, 0x100000ll, puzzle | 0x4d80000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1d80000ll))
           doSolve(20, 0x100000ll, puzzle | 0x1d80000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x80080080000ll)) {
         if(!(puzzle & 0x480480080000ll))
           doSolve(20, 0x100000ll, puzzle | 0x480480080000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x80180180000ll))
           doSolve(20, 0x100000ll, puzzle | 0x80180180000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x180180080000ll))
           doSolve(20, 0x100000ll, puzzle | 0x180180080000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x100180000ll)) {
         if(!(puzzle & 0x100180180000ll))
           doSolve(20, 0x100000ll, puzzle | 0x100180180000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x380180000ll))
           doSolve(20, 0x100000ll, puzzle | 0x380180000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x180380000ll))
           doSolve(20, 0x100000ll, puzzle | 0x180380000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x300380000ll))
           doSolve(20, 0x100000ll, puzzle | 0x300380000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x90090080000ll))
        doSolve(20, 0x100000ll, puzzle | 0x90090080000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x490480000ll))
        doSolve(20, 0x100000ll, puzzle | 0x490480000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x6c80000ll))
        doSolve(20, 0x100000ll, puzzle | 0x6c80000ll,
                   partsUsed | 0x400l);
    piece11at19:
      if (partsUsed & 0x800l) goto piece12at19;
      if (!(puzzle & 0x180080000ll)) {
         if(!(puzzle & 0x1b0080000ll))
           doSolve(20, 0x100000ll, puzzle | 0x1b0080000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1a0180000ll))
           doSolve(20, 0x100000ll, puzzle | 0x1a0180000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x190180000ll))
           doSolve(20, 0x100000ll, puzzle | 0x190180000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x580180000ll))
           doSolve(20, 0x100000ll, puzzle | 0x580180000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x580480000ll))
           doSolve(20, 0x100000ll, puzzle | 0x580480000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x180580000ll))
           doSolve(20, 0x100000ll, puzzle | 0x180580000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x180980000ll))
           doSolve(20, 0x100000ll, puzzle | 0x180980000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x980180000ll))
           doSolve(20, 0x100000ll, puzzle | 0x980180000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xd80080000ll))
           doSolve(20, 0x100000ll, puzzle | 0xd80080000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0xc80000ll)) {
         if(!(puzzle & 0x80d80000ll))
           doSolve(20, 0x100000ll, puzzle | 0x80d80000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xc00c80000ll))
           doSolve(20, 0x100000ll, puzzle | 0xc00c80000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x480c80000ll))
           doSolve(20, 0x100000ll, puzzle | 0x480c80000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x800d80000ll))
           doSolve(20, 0x100000ll, puzzle | 0x800d80000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x100d80000ll))
           doSolve(20, 0x100000ll, puzzle | 0x100d80000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x400d80000ll))
           doSolve(20, 0x100000ll, puzzle | 0x400d80000ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0x900980000ll))
        doSolve(20, 0x100000ll, puzzle | 0x900980000ll,
                   partsUsed | 0x800l);
      if(!(puzzle & 0xc80480000ll))
        doSolve(20, 0x100000ll, puzzle | 0xc80480000ll,
                   partsUsed | 0x800l);
      if(!(puzzle & 0x480580000ll))
        doSolve(20, 0x100000ll, puzzle | 0x480580000ll,
                   partsUsed | 0x800l);
    piece12at19:
    break;
   case 20:
      if (partsUsed & 0x2l) goto piece2at20;
      if(!(puzzle & 0x20120900000ll))
        doSolve(21, 0x200000ll, puzzle | 0x20120900000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x2f00000ll))
        doSolve(21, 0x200000ll, puzzle | 0x2f00000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x80180300000ll))
        doSolve(21, 0x200000ll, puzzle | 0x80180300000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x4800900100000ll))
        doSolve(21, 0x200000ll, puzzle | 0x4800900100000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x4004800900000ll))
        doSolve(21, 0x200000ll, puzzle | 0x4004800900000ll,
                   partsUsed | 0x2l);
    piece2at20:
      if (partsUsed & 0x4l) goto piece3at20;
      if (!(puzzle & 0x100100100000ll)) {
         if(!(puzzle & 0x920100100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x920100100000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x380100100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x380100100000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x104900100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x104900100000ll,
                   partsUsed | 0x4l);
      }
      if(!(puzzle & 0xe900000ll))
        doSolve(21, 0x200000ll, puzzle | 0xe900000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x800804900000ll))
        doSolve(21, 0x200000ll, puzzle | 0x800804900000ll,
                   partsUsed | 0x4l);
    piece3at20:
      if (partsUsed & 0x8l) goto piece4at20;
      if(!(puzzle & 0x120100900000ll))
        doSolve(21, 0x200000ll, puzzle | 0x120100900000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x180100300000ll))
        doSolve(21, 0x200000ll, puzzle | 0x180100300000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x6b00000ll))
        doSolve(21, 0x200000ll, puzzle | 0x6b00000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x4004900100000ll))
        doSolve(21, 0x200000ll, puzzle | 0x4004900100000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x4800800900000ll))
        doSolve(21, 0x200000ll, puzzle | 0x4800800900000ll,
                   partsUsed | 0x8l);
    piece4at20:
      if (partsUsed & 0x10l) goto piece5at20;
      if (!(puzzle & 0x100100100100000ll)) {
         if(!(puzzle & 0x180100180100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x180100180100000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x900100900100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x900100900100000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x100900100900000ll))
           doSolve(21, 0x200000ll, puzzle | 0x100900100900000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x300100300100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x300100300100000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x100300100300000ll))
           doSolve(21, 0x200000ll, puzzle | 0x100300100300000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x4924100000ll))
        doSolve(21, 0x200000ll, puzzle | 0x4924100000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x120100120100000ll))
        doSolve(21, 0x200000ll, puzzle | 0x120100120100000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x800900800900000ll))
        doSolve(21, 0x200000ll, puzzle | 0x800900800900000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x200300200300000ll))
        doSolve(21, 0x200000ll, puzzle | 0x200300200300000ll,
                   partsUsed | 0x10l);
    piece5at20:
      if (partsUsed & 0x20l) goto piece6at20;
      if (!(puzzle & 0x100100100100000ll)) {
         if(!(puzzle & 0x120100100100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x120100100100000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x180100100100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x180100100100000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x100100100900000ll))
           doSolve(21, 0x200000ll, puzzle | 0x100100100900000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x900100100100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x900100100100000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x100100100300000ll))
           doSolve(21, 0x200000ll, puzzle | 0x100100100300000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x300100100100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x300100100100000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x800800800900000ll))
        doSolve(21, 0x200000ll, puzzle | 0x800800800900000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x200200200300000ll))
        doSolve(21, 0x200000ll, puzzle | 0x200200200300000ll,
                   partsUsed | 0x20l);
    piece6at20:
      if (partsUsed & 0x40l) goto piece7at20;
      if (!(puzzle & 0x100100100100000ll)) {
         if(!(puzzle & 0x100120100100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x100120100100000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100100180100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x100100180100000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100180100100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x100180100100000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100100900100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x100100900100000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100900100100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x100900100100000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100100300100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x100100300100000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100300100100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x100300100100000ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x4920100000ll))
        doSolve(21, 0x200000ll, puzzle | 0x4920100000ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x100100120100000ll))
        doSolve(21, 0x200000ll, puzzle | 0x100100120100000ll,
                   partsUsed | 0x40l);
    piece7at20:
      if (partsUsed & 0x80l) goto piece8at20;
      if(!(puzzle & 0x900100900000ll))
        doSolve(21, 0x200000ll, puzzle | 0x900100900000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x900800900000ll))
        doSolve(21, 0x200000ll, puzzle | 0x900800900000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x4904100000ll))
        doSolve(21, 0x200000ll, puzzle | 0x4904100000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x4104900000ll))
        doSolve(21, 0x200000ll, puzzle | 0x4104900000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xcb00000ll))
        doSolve(21, 0x200000ll, puzzle | 0xcb00000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xd300000ll))
        doSolve(21, 0x200000ll, puzzle | 0xd300000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x300200300000ll))
        doSolve(21, 0x200000ll, puzzle | 0x300200300000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x300100300000ll))
        doSolve(21, 0x200000ll, puzzle | 0x300100300000ll,
                   partsUsed | 0x80l);
    piece8at20:
      if (partsUsed & 0x100l) goto piece9at20;
      if(!(puzzle & 0x190100000ll))
        doSolve(21, 0x200000ll, puzzle | 0x190100000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x180900000ll))
        doSolve(21, 0x200000ll, puzzle | 0x180900000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x240300000ll))
        doSolve(21, 0x200000ll, puzzle | 0x240300000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x200b00000ll))
        doSolve(21, 0x200000ll, puzzle | 0x200b00000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x160100000ll))
        doSolve(21, 0x200000ll, puzzle | 0x160100000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x400d00000ll))
        doSolve(21, 0x200000ll, puzzle | 0x400d00000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xd00100000ll))
        doSolve(21, 0x200000ll, puzzle | 0xd00100000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x900300000ll))
        doSolve(21, 0x200000ll, puzzle | 0x900300000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1300100000ll))
        doSolve(21, 0x200000ll, puzzle | 0x1300100000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1800900000ll))
        doSolve(21, 0x200000ll, puzzle | 0x1800900000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x101900000ll))
        doSolve(21, 0x200000ll, puzzle | 0x101900000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1001300000ll))
        doSolve(21, 0x200000ll, puzzle | 0x1001300000ll,
                   partsUsed | 0x100l);
    piece9at20:
      if (partsUsed & 0x200l) goto piece10at20;
      if (!(puzzle & 0x100100100000ll)) {
         if(!(puzzle & 0x100120900000ll))
           doSolve(21, 0x200000ll, puzzle | 0x100120900000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x120900100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x120900100000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x900120100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x900120100000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x100180300000ll))
           doSolve(21, 0x200000ll, puzzle | 0x100180300000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x300180100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x300180100000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x180300100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x180300100000ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x900100000ll)) {
         if(!(puzzle & 0x800920100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x800920100000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x20920100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x20920100000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x804900100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x804900100000ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0xd00000ll)) {
         if(!(puzzle & 0x9d00000ll))
           doSolve(21, 0x200000ll, puzzle | 0x9d00000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x3d00000ll))
           doSolve(21, 0x200000ll, puzzle | 0x3d00000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x4f00000ll))
           doSolve(21, 0x200000ll, puzzle | 0x4f00000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0xcd00000ll))
           doSolve(21, 0x200000ll, puzzle | 0xcd00000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x7900000ll))
        doSolve(21, 0x200000ll, puzzle | 0x7900000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x80380100000ll))
        doSolve(21, 0x200000ll, puzzle | 0x80380100000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x200380100000ll))
        doSolve(21, 0x200000ll, puzzle | 0x200380100000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x804800900000ll))
        doSolve(21, 0x200000ll, puzzle | 0x804800900000ll,
                   partsUsed | 0x200l);
    piece10at20:
      if (partsUsed & 0x400l) goto piece11at20;
      if (!(puzzle & 0x800900000ll)) {
         if(!(puzzle & 0x100900900000ll))
           doSolve(21, 0x200000ll, puzzle | 0x100900900000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x4900900000ll))
           doSolve(21, 0x200000ll, puzzle | 0x4900900000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x4804900000ll))
           doSolve(21, 0x200000ll, puzzle | 0x4804900000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x904900000ll))
           doSolve(21, 0x200000ll, puzzle | 0x904900000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x800900900000ll))
           doSolve(21, 0x200000ll, puzzle | 0x800900900000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x100100100000ll)) {
         if(!(puzzle & 0x900900100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x900900100000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x100300300000ll))
           doSolve(21, 0x200000ll, puzzle | 0x100300300000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x300300100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x300300100000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x1900000ll)) {
         if(!(puzzle & 0x5b00000ll))
           doSolve(21, 0x200000ll, puzzle | 0x5b00000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x9b00000ll))
           doSolve(21, 0x200000ll, puzzle | 0x9b00000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xd900000ll))
           doSolve(21, 0x200000ll, puzzle | 0xd900000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x120120100000ll))
        doSolve(21, 0x200000ll, puzzle | 0x120120100000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x920900000ll))
        doSolve(21, 0x200000ll, puzzle | 0x920900000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x1f00000ll))
        doSolve(21, 0x200000ll, puzzle | 0x1f00000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x6d00000ll))
        doSolve(21, 0x200000ll, puzzle | 0x6d00000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x380300000ll))
        doSolve(21, 0x200000ll, puzzle | 0x380300000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x180180100000ll))
        doSolve(21, 0x200000ll, puzzle | 0x180180100000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x200300300000ll))
        doSolve(21, 0x200000ll, puzzle | 0x200300300000ll,
                   partsUsed | 0x400l);
    piece11at20:
      if (partsUsed & 0x800l) goto piece12at20;
      if (!(puzzle & 0x120100000ll)) {
         if(!(puzzle & 0x1b0100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x1b0100000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x360100000ll))
           doSolve(21, 0x200000ll, puzzle | 0x360100000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x320300000ll))
           doSolve(21, 0x200000ll, puzzle | 0x320300000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x100300000ll)) {
         if(!(puzzle & 0x340300000ll))
           doSolve(21, 0x200000ll, puzzle | 0x340300000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x101b00000ll))
           doSolve(21, 0x200000ll, puzzle | 0x101b00000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xb00300000ll))
           doSolve(21, 0x200000ll, puzzle | 0xb00300000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x300b00000ll))
           doSolve(21, 0x200000ll, puzzle | 0x300b00000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x301300000ll))
           doSolve(21, 0x200000ll, puzzle | 0x301300000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1300300000ll))
           doSolve(21, 0x200000ll, puzzle | 0x1300300000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x900b00000ll))
           doSolve(21, 0x200000ll, puzzle | 0x900b00000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0xc00100000ll)) {
         if(!(puzzle & 0xd80100000ll))
           doSolve(21, 0x200000ll, puzzle | 0xd80100000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xd00900000ll))
           doSolve(21, 0x200000ll, puzzle | 0xd00900000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xc00d00000ll))
           doSolve(21, 0x200000ll, puzzle | 0xc00d00000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x800900000ll)) {
         if(!(puzzle & 0x980900000ll))
           doSolve(21, 0x200000ll, puzzle | 0x980900000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x900d00000ll))
           doSolve(21, 0x200000ll, puzzle | 0x900d00000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1801900000ll))
           doSolve(21, 0x200000ll, puzzle | 0x1801900000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xb00900000ll))
           doSolve(21, 0x200000ll, puzzle | 0xb00900000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x901900000ll))
           doSolve(21, 0x200000ll, puzzle | 0x901900000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x801b00000ll))
           doSolve(21, 0x200000ll, puzzle | 0x801b00000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1900900000ll))
           doSolve(21, 0x200000ll, puzzle | 0x1900900000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x1300000ll)) {
         if(!(puzzle & 0x1201300000ll))
           doSolve(21, 0x200000ll, puzzle | 0x1201300000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1001b00000ll))
           doSolve(21, 0x200000ll, puzzle | 0x1001b00000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x201b00000ll))
           doSolve(21, 0x200000ll, puzzle | 0x201b00000ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0x1b00100000ll))
        doSolve(21, 0x200000ll, puzzle | 0x1b00100000ll,
                   partsUsed | 0x800l);
    piece12at20:
    break;
   case 21:
      if (partsUsed & 0x2l) goto piece2at21;
      if(!(puzzle & 0x40241200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x40241200000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x7a00000ll))
        doSolve(22, 0x400000ll, puzzle | 0x7a00000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x180300200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x180300200000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x9001200200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x9001200200000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x8009001200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x8009001200000ll,
                   partsUsed | 0x2l);
    piece2at21:
      if (partsUsed & 0x4l) goto piece3at21;
      if (!(puzzle & 0x200200200000ll)) {
         if(!(puzzle & 0x1240200200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x1240200200000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x200380200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x200380200000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x209200200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x209200200000ll,
                   partsUsed | 0x4l);
      }
      if(!(puzzle & 0x9e00000ll))
        doSolve(22, 0x400000ll, puzzle | 0x9e00000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x1001009200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x1001009200000ll,
                   partsUsed | 0x4l);
    piece3at21:
      if (partsUsed & 0x8l) goto piece4at21;
      if(!(puzzle & 0x240201200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x240201200000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x3e00000ll))
        doSolve(22, 0x400000ll, puzzle | 0x3e00000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x80380200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x80380200000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x8009200200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x8009200200000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x9001001200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x9001001200000ll,
                   partsUsed | 0x8l);
    piece4at21:
      if (partsUsed & 0x10l) goto piece5at21;
      if (!(puzzle & 0x200200200200000ll)) {
         if(!(puzzle & 0x300200300200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x300200300200000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x1200201200200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x1200201200200000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x201200201200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x201200201200000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x9248200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x9248200000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x240200240200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x240200240200000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x1001201001200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x1001201001200000ll,
                   partsUsed | 0x10l);
    piece5at21:
      if (partsUsed & 0x20l) goto piece6at21;
      if (!(puzzle & 0x200200200200000ll)) {
         if(!(puzzle & 0x240200200200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x240200200200000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x300200200200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x300200200200000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x200200201200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x200200201200000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x1200200200200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x1200200200200000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x1001001001200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x1001001001200000ll,
                   partsUsed | 0x20l);
    piece6at21:
      if (partsUsed & 0x40l) goto piece7at21;
      if (!(puzzle & 0x200200200200000ll)) {
         if(!(puzzle & 0x200240200200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x200240200200000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x200200300200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x200200300200000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x200300200200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x200300200200000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x200201200200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x200201200200000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x201200200200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x201200200200000ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x9240200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x9240200000ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x200200240200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x200200240200000ll,
                   partsUsed | 0x40l);
    piece7at21:
      if (partsUsed & 0x80l) goto piece8at21;
      if(!(puzzle & 0x1200201200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x1200201200000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1201001200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x1201001200000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x9208200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x9208200000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x8209200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x8209200000ll,
                   partsUsed | 0x80l);
    piece8at21:
      if (partsUsed & 0x100l) goto piece9at21;
      if(!(puzzle & 0x320200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x320200000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x301200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x301200000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x801a00000ll))
        doSolve(22, 0x400000ll, puzzle | 0x801a00000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1a00200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x1a00200000ll,
                   partsUsed | 0x100l);
    piece9at21:
      if (partsUsed & 0x200l) goto piece10at21;
      if (!(puzzle & 0x200200200000ll)) {
         if(!(puzzle & 0x200241200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x200241200000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x241200200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x241200200000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x1200240200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x1200240200000ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x1200200000ll)) {
         if(!(puzzle & 0x1001240200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x1001240200000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x41240200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x41240200000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x1009200200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x1009200200000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x5e00000ll))
        doSolve(22, 0x400000ll, puzzle | 0x5e00000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100380200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x100380200000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x1009001200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x1009001200000ll,
                   partsUsed | 0x200l);
    piece10at21:
      if (partsUsed & 0x400l) goto piece11at21;
      if (!(puzzle & 0x9200000ll)) {
         if(!(puzzle & 0xda00000ll))
           doSolve(22, 0x400000ll, puzzle | 0xda00000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x9009200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x9009200000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1209200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x1209200000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x200200200000ll)) {
         if(!(puzzle & 0x300300200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x300300200000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x201201200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x201201200000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1201200200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x1201200200000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x240240200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x240240200000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x1241200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x1241200000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x9201200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x9201200000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x1001201200000ll))
        doSolve(22, 0x400000ll, puzzle | 0x1001201200000ll,
                   partsUsed | 0x400l);
    piece11at21:
      if (partsUsed & 0x800l) goto piece12at21;
      if (!(puzzle & 0x300200000ll)) {
         if(!(puzzle & 0x360200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x360200000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1b00200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x1b00200000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1301200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x1301200000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x1001200000ll)) {
         if(!(puzzle & 0x1a01200000ll))
           doSolve(22, 0x400000ll, puzzle | 0x1a01200000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1801a00000ll))
           doSolve(22, 0x400000ll, puzzle | 0x1801a00000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1201a00000ll))
           doSolve(22, 0x400000ll, puzzle | 0x1201a00000ll,
                   partsUsed | 0x800l);
      }
    piece12at21:
    break;
   case 22:
      if (partsUsed & 0x2l) goto piece2at22;
      if(!(puzzle & 0x90480400000ll))
        doSolve(23, 0x800000ll, puzzle | 0x90480400000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x80482400000ll))
        doSolve(23, 0x800000ll, puzzle | 0x80482400000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1001800c00000ll))
        doSolve(23, 0x800000ll, puzzle | 0x1001800c00000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1800c00400000ll))
        doSolve(23, 0x800000ll, puzzle | 0x1800c00400000ll,
                   partsUsed | 0x2l);
    piece2at22:
      if (partsUsed & 0x4l) goto piece3at22;
      if (!(puzzle & 0x400400400000ll)) {
         if(!(puzzle & 0x400490400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x400490400000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x2480400400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x2480400400000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x401c00400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x401c00400000ll,
                   partsUsed | 0x4l);
      }
      if(!(puzzle & 0x800801c00000ll))
        doSolve(23, 0x800000ll, puzzle | 0x800801c00000ll,
                   partsUsed | 0x4l);
    piece3at22:
      if (partsUsed & 0x8l) goto piece4at22;
      if(!(puzzle & 0x10490400000ll))
        doSolve(23, 0x800000ll, puzzle | 0x10490400000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x480402400000ll))
        doSolve(23, 0x800000ll, puzzle | 0x480402400000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1001c00400000ll))
        doSolve(23, 0x800000ll, puzzle | 0x1001c00400000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1800800c00000ll))
        doSolve(23, 0x800000ll, puzzle | 0x1800800c00000ll,
                   partsUsed | 0x8l);
    piece4at22:
      if (partsUsed & 0x10l) goto piece5at22;
      if (!(puzzle & 0x400400400400000ll)) {
         if(!(puzzle & 0x480400480400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x480400480400000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x2400402400400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x2400402400400000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x402400402400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x402400402400000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0xc00400c00400000ll))
           doSolve(23, 0x800000ll, puzzle | 0xc00400c00400000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x400c00400c00000ll))
           doSolve(23, 0x800000ll, puzzle | 0x400c00400c00000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x2002402002400000ll))
        doSolve(23, 0x800000ll, puzzle | 0x2002402002400000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x800c00800c00000ll))
        doSolve(23, 0x800000ll, puzzle | 0x800c00800c00000ll,
                   partsUsed | 0x10l);
    piece5at22:
      if (partsUsed & 0x20l) goto piece6at22;
      if (!(puzzle & 0x400400400400000ll)) {
         if(!(puzzle & 0x480400400400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x480400400400000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x400400402400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x400400402400000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x2400400400400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x2400400400400000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x400400400c00000ll))
           doSolve(23, 0x800000ll, puzzle | 0x400400400c00000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0xc00400400400000ll))
           doSolve(23, 0x800000ll, puzzle | 0xc00400400400000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x2002002002400000ll))
        doSolve(23, 0x800000ll, puzzle | 0x2002002002400000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x800800800c00000ll))
        doSolve(23, 0x800000ll, puzzle | 0x800800800c00000ll,
                   partsUsed | 0x20l);
    piece6at22:
      if (partsUsed & 0x40l) goto piece7at22;
      if (!(puzzle & 0x400400400400000ll)) {
         if(!(puzzle & 0x400480400400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x400480400400000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x400402400400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x400402400400000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x402400400400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x402400400400000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x400400c00400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x400400c00400000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x400c00400400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x400c00400400000ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x2490400000ll))
        doSolve(23, 0x800000ll, puzzle | 0x2490400000ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x400400480400000ll))
        doSolve(23, 0x800000ll, puzzle | 0x400400480400000ll,
                   partsUsed | 0x40l);
    piece7at22:
      if (partsUsed & 0x80l) goto piece8at22;
      if(!(puzzle & 0x2400402400000ll))
        doSolve(23, 0x800000ll, puzzle | 0x2400402400000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x2402002400000ll))
        doSolve(23, 0x800000ll, puzzle | 0x2402002400000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xbc00000ll))
        doSolve(23, 0x800000ll, puzzle | 0xbc00000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xf400000ll))
        doSolve(23, 0x800000ll, puzzle | 0xf400000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xc00800c00000ll))
        doSolve(23, 0x800000ll, puzzle | 0xc00800c00000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xc00400c00000ll))
        doSolve(23, 0x800000ll, puzzle | 0xc00400c00000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1c01400000ll))
        doSolve(23, 0x800000ll, puzzle | 0x1c01400000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1401c00000ll))
        doSolve(23, 0x800000ll, puzzle | 0x1401c00000ll,
                   partsUsed | 0x80l);
    piece8at22:
      if (partsUsed & 0x100l) goto piece9at22;
      if(!(puzzle & 0x900c00000ll))
        doSolve(23, 0x800000ll, puzzle | 0x900c00000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x802c00000ll))
        doSolve(23, 0x800000ll, puzzle | 0x802c00000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x580400000ll))
        doSolve(23, 0x800000ll, puzzle | 0x580400000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x4c00400000ll))
        doSolve(23, 0x800000ll, puzzle | 0x4c00400000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x2400c00000ll))
        doSolve(23, 0x800000ll, puzzle | 0x2400c00000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x6002400000ll))
        doSolve(23, 0x800000ll, puzzle | 0x6002400000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x406400000ll))
        doSolve(23, 0x800000ll, puzzle | 0x406400000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x4004c00000ll))
        doSolve(23, 0x800000ll, puzzle | 0x4004c00000ll,
                   partsUsed | 0x100l);
    piece9at22:
      if (partsUsed & 0x200l) goto piece10at22;
      if (!(puzzle & 0x480400000ll)) {
         if(!(puzzle & 0x80490400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x80490400000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x400482400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x400482400000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x2002480400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x2002480400000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x2400480400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x2400480400000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x82480400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x82480400000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x482400400000ll))
        doSolve(23, 0x800000ll, puzzle | 0x482400400000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x801800c00000ll))
        doSolve(23, 0x800000ll, puzzle | 0x801800c00000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x801c00400000ll))
        doSolve(23, 0x800000ll, puzzle | 0x801c00400000ll,
                   partsUsed | 0x200l);
    piece10at22:
      if (partsUsed & 0x400l) goto piece11at22;
      if (!(puzzle & 0x2400400000ll)) {
         if(!(puzzle & 0x402402400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x402402400000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2402400400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x2402400400000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2002402400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x2002402400000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x4c00000ll)) {
         if(!(puzzle & 0xdc00000ll))
           doSolve(23, 0x800000ll, puzzle | 0xdc00000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x7c00000ll))
           doSolve(23, 0x800000ll, puzzle | 0x7c00000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xec00000ll))
           doSolve(23, 0x800000ll, puzzle | 0xec00000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0xc00400000ll)) {
         if(!(puzzle & 0x800c00c00000ll))
           doSolve(23, 0x800000ll, puzzle | 0x800c00c00000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x400c00c00000ll))
           doSolve(23, 0x800000ll, puzzle | 0x400c00c00000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1c00c00000ll))
           doSolve(23, 0x800000ll, puzzle | 0x1c00c00000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xc00c00400000ll))
           doSolve(23, 0x800000ll, puzzle | 0xc00c00400000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xc01c00000ll))
           doSolve(23, 0x800000ll, puzzle | 0xc01c00000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x480480400000ll))
        doSolve(23, 0x800000ll, puzzle | 0x480480400000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x2482400000ll))
        doSolve(23, 0x800000ll, puzzle | 0x2482400000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x1801c00000ll))
        doSolve(23, 0x800000ll, puzzle | 0x1801c00000ll,
                   partsUsed | 0x400l);
    piece11at22:
      if (partsUsed & 0x800l) goto piece12at22;
      if (!(puzzle & 0xc00400000ll)) {
         if(!(puzzle & 0xd80400000ll))
           doSolve(23, 0x800000ll, puzzle | 0xd80400000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xd00c00000ll))
           doSolve(23, 0x800000ll, puzzle | 0xd00c00000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xc80c00000ll))
           doSolve(23, 0x800000ll, puzzle | 0xc80c00000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x2c00c00000ll))
           doSolve(23, 0x800000ll, puzzle | 0x2c00c00000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x2c02400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x2c02400000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xc02c00000ll))
           doSolve(23, 0x800000ll, puzzle | 0xc02c00000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xc04c00000ll))
           doSolve(23, 0x800000ll, puzzle | 0xc04c00000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x4c00c00000ll))
           doSolve(23, 0x800000ll, puzzle | 0x4c00c00000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x6c00400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x6c00400000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x6400000ll)) {
         if(!(puzzle & 0x406c00000ll))
           doSolve(23, 0x800000ll, puzzle | 0x406c00000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x6006400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x6006400000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x2406400000ll))
           doSolve(23, 0x800000ll, puzzle | 0x2406400000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x4006c00000ll))
           doSolve(23, 0x800000ll, puzzle | 0x4006c00000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x806c00000ll))
           doSolve(23, 0x800000ll, puzzle | 0x806c00000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x2006c00000ll))
           doSolve(23, 0x800000ll, puzzle | 0x2006c00000ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0x4804c00000ll))
        doSolve(23, 0x800000ll, puzzle | 0x4804c00000ll,
                   partsUsed | 0x800l);
      if(!(puzzle & 0x6402400000ll))
        doSolve(23, 0x800000ll, puzzle | 0x6402400000ll,
                   partsUsed | 0x800l);
      if(!(puzzle & 0x2402c00000ll))
        doSolve(23, 0x800000ll, puzzle | 0x2402c00000ll,
                   partsUsed | 0x800l);
    piece12at22:
    break;
   case 23:
      if (partsUsed & 0x2l) goto piece2at23;
      if(!(puzzle & 0x400c01800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x400c01800000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x120900800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x120900800000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x100904800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x100904800000ll,
                   partsUsed | 0x2l);
    piece2at23:
      if (partsUsed & 0x4l) goto piece3at23;
      if (!(puzzle & 0x800800800000ll)) {
         if(!(puzzle & 0x800920800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x800920800000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x4900800800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x4900800800000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x1c00800800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x1c00800800000ll,
                   partsUsed | 0x4l);
      }
    piece3at23:
      if (partsUsed & 0x8l) goto piece4at23;
      if(!(puzzle & 0x20920800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x20920800000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x900804800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x900804800000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0xc00801800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0xc00801800000ll,
                   partsUsed | 0x8l);
    piece4at23:
      if (partsUsed & 0x10l) goto piece5at23;
      if (!(puzzle & 0x800800800800000ll)) {
         if(!(puzzle & 0x900800900800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x900800900800000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0xc00800c00800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0xc00800c00800000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x4800804800800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x4800804800800000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x804800804800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x804800804800000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x1800801800800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x1800801800800000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x801800801800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x801800801800000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x4004804004800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x4004804004800000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x1001801001800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x1001801001800000ll,
                   partsUsed | 0x10l);
    piece5at23:
      if (partsUsed & 0x20l) goto piece6at23;
      if (!(puzzle & 0x800800800800000ll)) {
         if(!(puzzle & 0x900800800800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x900800800800000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0xc00800800800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0xc00800800800000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x800800804800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x800800804800000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x4800800800800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x4800800800800000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x800800801800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x800800801800000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x1800800800800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x1800800800800000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x4004004004800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x4004004004800000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x1001001001800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x1001001001800000ll,
                   partsUsed | 0x20l);
    piece6at23:
      if (partsUsed & 0x40l) goto piece7at23;
      if (!(puzzle & 0x800800800800000ll)) {
         if(!(puzzle & 0x800900800800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x800900800800000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800800c00800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x800800c00800000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800c00800800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x800c00800800000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800804800800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x800804800800000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x804800800800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x804800800800000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800801800800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x800801800800000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x801800800800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x801800800800000ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x4920800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x4920800000ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x800800900800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x800800900800000ll,
                   partsUsed | 0x40l);
    piece7at23:
      if (partsUsed & 0x80l) goto piece8at23;
      if(!(puzzle & 0x4800804800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x4800804800000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x4804004800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x4804004800000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1801001800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x1801001800000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1800801800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x1800801800000ll,
                   partsUsed | 0x80l);
    piece8at23:
      if (partsUsed & 0x100l) goto piece9at23;
      if(!(puzzle & 0xc80800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0xc80800000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xc04800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0xc04800000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1201800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x1201800000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1005800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x1005800000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xb00800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0xb00800000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x2006800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x2006800000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x6800800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x6800800000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x4801800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x4801800000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x9800800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x9800800000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xc004800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0xc004800000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x80c800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x80c800000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x8009800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x8009800000ll,
                   partsUsed | 0x100l);
    piece9at23:
      if (partsUsed & 0x200l) goto piece10at23;
      if (!(puzzle & 0x900800000ll)) {
         if(!(puzzle & 0x100920800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x100920800000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x800904800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x800904800000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x4004900800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x4004900800000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x4800900800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x4800900800000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x104900800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x104900800000ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x800800800000ll)) {
         if(!(puzzle & 0x904800800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x904800800000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x800c01800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x800c01800000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x1800c00800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x1800c00800000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0xc01800800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0xc01800800000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x401c00800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x401c00800000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x1001c00800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x1001c00800000ll,
                   partsUsed | 0x200l);
    piece10at23:
      if (partsUsed & 0x400l) goto piece11at23;
      if (!(puzzle & 0x4800800000ll)) {
         if(!(puzzle & 0x804804800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x804804800000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x4804800800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x4804800800000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x4004804800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x4004804800000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x1800800000ll)) {
         if(!(puzzle & 0x1001801800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x1001801800000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x801801800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x801801800000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1801800800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x1801800800000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x900900800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x900900800000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x4904800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x4904800000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0xf800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0xf800000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x1c01800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0x1c01800000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0xc00c00800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0xc00c00800000ll,
                   partsUsed | 0x400l);
    piece11at23:
      if (partsUsed & 0x800l) goto piece12at23;
      if (!(puzzle & 0x900800000ll)) {
         if(!(puzzle & 0xd80800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0xd80800000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1b00800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x1b00800000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1901800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x1901800000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x801800000ll)) {
         if(!(puzzle & 0x1a01800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x1a01800000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x80d800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x80d800000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x5801800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x5801800000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1805800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x1805800000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1809800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x1809800000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x9801800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x9801800000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x4805800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x4805800000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x6000800000ll)) {
         if(!(puzzle & 0x6c00800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x6c00800000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x6804800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x6804800000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x6006800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x6006800000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x4004800000ll)) {
         if(!(puzzle & 0x4c04800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x4c04800000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x4806800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x4806800000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xc00c800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0xc00c800000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x5804800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x5804800000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x480c800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x480c800000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x400d800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x400d800000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xc804800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0xc804800000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x9800000ll)) {
         if(!(puzzle & 0x9009800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x9009800000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x800d800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x800d800000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x100d800000ll))
           doSolve(24, 0x1000000ll, puzzle | 0x100d800000ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0xd800800000ll))
        doSolve(24, 0x1000000ll, puzzle | 0xd800800000ll,
                   partsUsed | 0x800l);
    piece12at23:
    break;
   case 24:
      if (partsUsed & 0x2l) goto piece2at24;
      if(!(puzzle & 0xc01801000000ll))
        doSolve(25, 0x2000000ll, puzzle | 0xc01801000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x241201000000ll))
        doSolve(25, 0x2000000ll, puzzle | 0x241201000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x201209000000ll))
        doSolve(25, 0x2000000ll, puzzle | 0x201209000000ll,
                   partsUsed | 0x2l);
    piece2at24:
      if (partsUsed & 0x4l) goto piece3at24;
      if (!(puzzle & 0x1001001000000ll)) {
         if(!(puzzle & 0x1001241000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x1001241000000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x9201001000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x9201001000000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x1001c01000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x1001c01000000ll,
                   partsUsed | 0x4l);
      }
    piece3at24:
      if (partsUsed & 0x8l) goto piece4at24;
      if(!(puzzle & 0x401c01000000ll))
        doSolve(25, 0x2000000ll, puzzle | 0x401c01000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x41241000000ll))
        doSolve(25, 0x2000000ll, puzzle | 0x41241000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1201009000000ll))
        doSolve(25, 0x2000000ll, puzzle | 0x1201009000000ll,
                   partsUsed | 0x8l);
    piece4at24:
      if (partsUsed & 0x10l) goto piece5at24;
      if (!(puzzle & 0x1001001001000000ll)) {
         if(!(puzzle & 0x1201001201000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x1201001201000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x1801001801000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x1801001801000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x9001009001000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x9001009001000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x1009001009000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x1009001009000000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x8009008009000000ll))
        doSolve(25, 0x2000000ll, puzzle | 0x8009008009000000ll,
                   partsUsed | 0x10l);
    piece5at24:
      if (partsUsed & 0x20l) goto piece6at24;
      if (!(puzzle & 0x1001001001000000ll)) {
         if(!(puzzle & 0x1201001001000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x1201001001000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x1801001001000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x1801001001000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x1001001009000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x1001001009000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x9001001001000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x9001001001000000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x8008008009000000ll))
        doSolve(25, 0x2000000ll, puzzle | 0x8008008009000000ll,
                   partsUsed | 0x20l);
    piece6at24:
      if (partsUsed & 0x40l) goto piece7at24;
      if (!(puzzle & 0x1001001001000000ll)) {
         if(!(puzzle & 0x1001201001000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x1001201001000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x1001001801000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x1001001801000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x1001801001000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x1001801001000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x1001009001000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x1001009001000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x1009001001000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x1009001001000000ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x9241000000ll))
        doSolve(25, 0x2000000ll, puzzle | 0x9241000000ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x1001001201000000ll))
        doSolve(25, 0x2000000ll, puzzle | 0x1001001201000000ll,
                   partsUsed | 0x40l);
    piece7at24:
      if (partsUsed & 0x80l) goto piece8at24;
      if(!(puzzle & 0x9001009000000ll))
        doSolve(25, 0x2000000ll, puzzle | 0x9001009000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x9008009000000ll))
        doSolve(25, 0x2000000ll, puzzle | 0x9008009000000ll,
                   partsUsed | 0x80l);
    piece8at24:
      if (partsUsed & 0x100l) goto piece9at24;
      if(!(puzzle & 0x1901000000ll))
        doSolve(25, 0x2000000ll, puzzle | 0x1901000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1809000000ll))
        doSolve(25, 0x2000000ll, puzzle | 0x1809000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x400d000000ll))
        doSolve(25, 0x2000000ll, puzzle | 0x400d000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xd001000000ll))
        doSolve(25, 0x2000000ll, puzzle | 0xd001000000ll,
                   partsUsed | 0x100l);
    piece9at24:
      if (partsUsed & 0x200l) goto piece10at24;
      if (!(puzzle & 0x1201000000ll)) {
         if(!(puzzle & 0x201241000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x201241000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x1001209000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x1001209000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x8009201000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x8009201000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x9001201000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x9001201000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x209201000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x209201000000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x1209001000000ll))
        doSolve(25, 0x2000000ll, puzzle | 0x1209001000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x801c01000000ll))
        doSolve(25, 0x2000000ll, puzzle | 0x801c01000000ll,
                   partsUsed | 0x200l);
    piece10at24:
      if (partsUsed & 0x400l) goto piece11at24;
      if (!(puzzle & 0x1001001000000ll)) {
         if(!(puzzle & 0x1801801000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x1801801000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1009009000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x1009009000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x9009001000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x9009001000000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x1201201000000ll))
        doSolve(25, 0x2000000ll, puzzle | 0x1201201000000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x9209000000ll))
        doSolve(25, 0x2000000ll, puzzle | 0x9209000000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x8009009000000ll))
        doSolve(25, 0x2000000ll, puzzle | 0x8009009000000ll,
                   partsUsed | 0x400l);
    piece11at24:
      if (partsUsed & 0x800l) goto piece12at24;
      if (!(puzzle & 0x1801000000ll)) {
         if(!(puzzle & 0x1b01000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x1b01000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xd801000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0xd801000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x9809000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x9809000000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x8009000000ll)) {
         if(!(puzzle & 0xd009000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0xd009000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xc00d000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0xc00d000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x900d000000ll))
           doSolve(25, 0x2000000ll, puzzle | 0x900d000000ll,
                   partsUsed | 0x800l);
      }
    piece12at24:
    break;
   case 25:
      if (partsUsed & 0x2l) goto piece2at25;
      if(!(puzzle & 0x482402000000ll))
        doSolve(26, 0x4000000ll, puzzle | 0x482402000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x800c006000000ll))
        doSolve(26, 0x4000000ll, puzzle | 0x800c006000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0xc006002000000ll))
        doSolve(26, 0x4000000ll, puzzle | 0xc006002000000ll,
                   partsUsed | 0x2l);
    piece2at25:
      if (partsUsed & 0x4l) goto piece3at25;
      if(!(puzzle & 0x400400e000000ll))
        doSolve(26, 0x4000000ll, puzzle | 0x400400e000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x2002482000000ll))
        doSolve(26, 0x4000000ll, puzzle | 0x2002482000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x200e002000000ll))
        doSolve(26, 0x4000000ll, puzzle | 0x200e002000000ll,
                   partsUsed | 0x4l);
    piece3at25:
      if (partsUsed & 0x8l) goto piece4at25;
      if(!(puzzle & 0x82482000000ll))
        doSolve(26, 0x4000000ll, puzzle | 0x82482000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x800e002000000ll))
        doSolve(26, 0x4000000ll, puzzle | 0x800e002000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0xc004006000000ll))
        doSolve(26, 0x4000000ll, puzzle | 0xc004006000000ll,
                   partsUsed | 0x8l);
    piece4at25:
      if (partsUsed & 0x10l) goto piece5at25;
      if (!(puzzle & 0x2002002002000000ll)) {
         if(!(puzzle & 0x2402002402000000ll))
           doSolve(26, 0x4000000ll, puzzle | 0x2402002402000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x6002006002000000ll))
           doSolve(26, 0x4000000ll, puzzle | 0x6002006002000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x2006002006000000ll))
           doSolve(26, 0x4000000ll, puzzle | 0x2006002006000000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x4006004006000000ll))
        doSolve(26, 0x4000000ll, puzzle | 0x4006004006000000ll,
                   partsUsed | 0x10l);
    piece5at25:
      if (partsUsed & 0x20l) goto piece6at25;
      if (!(puzzle & 0x2002002002000000ll)) {
         if(!(puzzle & 0x2402002002000000ll))
           doSolve(26, 0x4000000ll, puzzle | 0x2402002002000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x2002002006000000ll))
           doSolve(26, 0x4000000ll, puzzle | 0x2002002006000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x6002002002000000ll))
           doSolve(26, 0x4000000ll, puzzle | 0x6002002002000000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x2492000000ll))
        doSolve(26, 0x4000000ll, puzzle | 0x2492000000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x4004004006000000ll))
        doSolve(26, 0x4000000ll, puzzle | 0x4004004006000000ll,
                   partsUsed | 0x20l);
    piece6at25:
      if (partsUsed & 0x40l) goto piece7at25;
      if (!(puzzle & 0x2002002002000000ll)) {
         if(!(puzzle & 0x2002402002000000ll))
           doSolve(26, 0x4000000ll, puzzle | 0x2002402002000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2002002402000000ll))
           doSolve(26, 0x4000000ll, puzzle | 0x2002002402000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2002006002000000ll))
           doSolve(26, 0x4000000ll, puzzle | 0x2002006002000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2006002002000000ll))
           doSolve(26, 0x4000000ll, puzzle | 0x2006002002000000ll,
                   partsUsed | 0x40l);
      }
    piece7at25:
      if (partsUsed & 0x80l) goto piece8at25;
      if(!(puzzle & 0x6004006000000ll))
        doSolve(26, 0x4000000ll, puzzle | 0x6004006000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x6002006000000ll))
        doSolve(26, 0x4000000ll, puzzle | 0x6002006000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xe00a000000ll))
        doSolve(26, 0x4000000ll, puzzle | 0xe00a000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xa00e000000ll))
        doSolve(26, 0x4000000ll, puzzle | 0xa00e000000ll,
                   partsUsed | 0x80l);
    piece8at25:
      if (partsUsed & 0x100l) goto piece9at25;
      if(!(puzzle & 0x4806000000ll))
        doSolve(26, 0x4000000ll, puzzle | 0x4806000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x2c02000000ll))
        doSolve(26, 0x4000000ll, puzzle | 0x2c02000000ll,
                   partsUsed | 0x100l);
    piece9at25:
      if (partsUsed & 0x200l) goto piece10at25;
      if(!(puzzle & 0x402482000000ll))
        doSolve(26, 0x4000000ll, puzzle | 0x402482000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x400c006000000ll))
        doSolve(26, 0x4000000ll, puzzle | 0x400c006000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x400e002000000ll))
        doSolve(26, 0x4000000ll, puzzle | 0x400e002000000ll,
                   partsUsed | 0x200l);
    piece10at25:
      if (partsUsed & 0x400l) goto piece11at25;
      if (!(puzzle & 0x2002002000000ll)) {
         if(!(puzzle & 0x2402402000000ll))
           doSolve(26, 0x4000000ll, puzzle | 0x2402402000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2006006000000ll))
           doSolve(26, 0x4000000ll, puzzle | 0x2006006000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x6006002000000ll))
           doSolve(26, 0x4000000ll, puzzle | 0x6006002000000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x4006000000ll)) {
         if(!(puzzle & 0x4006006000000ll))
           doSolve(26, 0x4000000ll, puzzle | 0x4006006000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xe006000000ll))
           doSolve(26, 0x4000000ll, puzzle | 0xe006000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x600e000000ll))
           doSolve(26, 0x4000000ll, puzzle | 0x600e000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xc00e000000ll))
           doSolve(26, 0x4000000ll, puzzle | 0xc00e000000ll,
                   partsUsed | 0x400l);
      }
    piece11at25:
      if (partsUsed & 0x800l) goto piece12at25;
      if (!(puzzle & 0x6002000000ll)) {
         if(!(puzzle & 0x6c02000000ll))
           doSolve(26, 0x4000000ll, puzzle | 0x6c02000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x6806000000ll))
           doSolve(26, 0x4000000ll, puzzle | 0x6806000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x6406000000ll))
           doSolve(26, 0x4000000ll, puzzle | 0x6406000000ll,
                   partsUsed | 0x800l);
      }
    piece12at25:
    break;
   case 26:
      if (partsUsed & 0x2l) goto piece2at26;
      if(!(puzzle & 0x904804000000ll))
        doSolve(27, 0x8000000ll, puzzle | 0x904804000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x200600c000000ll))
        doSolve(27, 0x8000000ll, puzzle | 0x200600c000000ll,
                   partsUsed | 0x2l);
    piece2at26:
      if (partsUsed & 0x4l) goto piece3at26;
      if(!(puzzle & 0x4004904000000ll))
        doSolve(27, 0x8000000ll, puzzle | 0x4004904000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0xe004004000000ll))
        doSolve(27, 0x8000000ll, puzzle | 0xe004004000000ll,
                   partsUsed | 0x4l);
    piece3at26:
      if (partsUsed & 0x8l) goto piece4at26;
      if(!(puzzle & 0x104904000000ll))
        doSolve(27, 0x8000000ll, puzzle | 0x104904000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x600400c000000ll))
        doSolve(27, 0x8000000ll, puzzle | 0x600400c000000ll,
                   partsUsed | 0x8l);
    piece4at26:
      if (partsUsed & 0x10l) goto piece5at26;
      if (!(puzzle & 0x4004004004000000ll)) {
         if(!(puzzle & 0x4804004804000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0x4804004804000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x6004006004000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0x6004006004000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0xc00400c004000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0xc00400c004000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x400c00400c000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0x400c00400c000000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x800c00800c000000ll))
        doSolve(27, 0x8000000ll, puzzle | 0x800c00800c000000ll,
                   partsUsed | 0x10l);
    piece5at26:
      if (partsUsed & 0x20l) goto piece6at26;
      if (!(puzzle & 0x4004004004000000ll)) {
         if(!(puzzle & 0x4804004004000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0x4804004004000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x6004004004000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0x6004004004000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x400400400c000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0x400400400c000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0xc004004004000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0xc004004004000000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x4924000000ll))
        doSolve(27, 0x8000000ll, puzzle | 0x4924000000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x800800800c000000ll))
        doSolve(27, 0x8000000ll, puzzle | 0x800800800c000000ll,
                   partsUsed | 0x20l);
    piece6at26:
      if (partsUsed & 0x40l) goto piece7at26;
      if (!(puzzle & 0x4004004004000000ll)) {
         if(!(puzzle & 0x4004804004000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0x4004804004000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4004004804000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0x4004004804000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4004006004000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0x4004006004000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4006004004000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0x4006004004000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x400400c004000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0x400400c004000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x400c004004000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0x400c004004000000ll,
                   partsUsed | 0x40l);
      }
    piece7at26:
      if (partsUsed & 0x80l) goto piece8at26;
      if(!(puzzle & 0xc00800c000000ll))
        doSolve(27, 0x8000000ll, puzzle | 0xc00800c000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xc00400c000000ll))
        doSolve(27, 0x8000000ll, puzzle | 0xc00400c000000ll,
                   partsUsed | 0x80l);
    piece8at26:
      if (partsUsed & 0x100l) goto piece9at26;
      if(!(puzzle & 0x5804000000ll))
        doSolve(27, 0x8000000ll, puzzle | 0x5804000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x6404000000ll))
        doSolve(27, 0x8000000ll, puzzle | 0x6404000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x900c000000ll))
        doSolve(27, 0x8000000ll, puzzle | 0x900c000000ll,
                   partsUsed | 0x100l);
    piece9at26:
      if (partsUsed & 0x200l) goto piece10at26;
      if (!(puzzle & 0x4004004000000ll)) {
         if(!(puzzle & 0x400600c000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0x400600c000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0xc006004000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0xc006004000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x600c004000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0x600c004000000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x804904000000ll))
        doSolve(27, 0x8000000ll, puzzle | 0x804904000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x200e004000000ll))
        doSolve(27, 0x8000000ll, puzzle | 0x200e004000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x800e004000000ll))
        doSolve(27, 0x8000000ll, puzzle | 0x800e004000000ll,
                   partsUsed | 0x200l);
    piece10at26:
      if (partsUsed & 0x400l) goto piece11at26;
      if (!(puzzle & 0x4004004000000ll)) {
         if(!(puzzle & 0x4804804000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0x4804804000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x6006004000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0x6006004000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x400c00c000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0x400c00c000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xc00c004000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0xc00c004000000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0xe00c000000ll))
        doSolve(27, 0x8000000ll, puzzle | 0xe00c000000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x800c00c000000ll))
        doSolve(27, 0x8000000ll, puzzle | 0x800c00c000000ll,
                   partsUsed | 0x400l);
    piece11at26:
      if (partsUsed & 0x800l) goto piece12at26;
      if (!(puzzle & 0x4804000000ll)) {
         if(!(puzzle & 0x6c04000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0x6c04000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xd804000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0xd804000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xc80c000000ll))
           doSolve(27, 0x8000000ll, puzzle | 0xc80c000000ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0xd00c000000ll))
        doSolve(27, 0x8000000ll, puzzle | 0xd00c000000ll,
                   partsUsed | 0x800l);
    piece12at26:
    break;
   case 27:
      if (partsUsed & 0x2l) goto piece2at27;
      if(!(puzzle & 0x1209008000000ll))
        doSolve(28, 0x10000000ll, puzzle | 0x1209008000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x600c008000000ll))
        doSolve(28, 0x10000000ll, puzzle | 0x600c008000000ll,
                   partsUsed | 0x2l);
    piece2at27:
      if (partsUsed & 0x4l) goto piece3at27;
      if(!(puzzle & 0x8009208000000ll))
        doSolve(28, 0x10000000ll, puzzle | 0x8009208000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x800e008000000ll))
        doSolve(28, 0x10000000ll, puzzle | 0x800e008000000ll,
                   partsUsed | 0x4l);
    piece3at27:
      if (partsUsed & 0x8l) goto piece4at27;
      if(!(puzzle & 0x209208000000ll))
        doSolve(28, 0x10000000ll, puzzle | 0x209208000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x200e008000000ll))
        doSolve(28, 0x10000000ll, puzzle | 0x200e008000000ll,
                   partsUsed | 0x8l);
    piece4at27:
      if (partsUsed & 0x10l) goto piece5at27;
      if(!(puzzle & 0x9008009008000000ll))
        doSolve(28, 0x10000000ll, puzzle | 0x9008009008000000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0xc00800c008000000ll))
        doSolve(28, 0x10000000ll, puzzle | 0xc00800c008000000ll,
                   partsUsed | 0x10l);
    piece5at27:
      if (partsUsed & 0x20l) goto piece6at27;
      if(!(puzzle & 0x9248000000ll))
        doSolve(28, 0x10000000ll, puzzle | 0x9248000000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x9008008008000000ll))
        doSolve(28, 0x10000000ll, puzzle | 0x9008008008000000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0xc008008008000000ll))
        doSolve(28, 0x10000000ll, puzzle | 0xc008008008000000ll,
                   partsUsed | 0x20l);
    piece6at27:
      if (partsUsed & 0x40l) goto piece7at27;
      if (!(puzzle & 0x8008008008000000ll)) {
         if(!(puzzle & 0x8009008008000000ll))
           doSolve(28, 0x10000000ll, puzzle | 0x8009008008000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x8008009008000000ll))
           doSolve(28, 0x10000000ll, puzzle | 0x8008009008000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800800c008000000ll))
           doSolve(28, 0x10000000ll, puzzle | 0x800800c008000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800c008008000000ll))
           doSolve(28, 0x10000000ll, puzzle | 0x800c008008000000ll,
                   partsUsed | 0x40l);
      }
    piece7at27:
      if (partsUsed & 0x100l) goto piece9at27;
      if(!(puzzle & 0xc808000000ll))
        doSolve(28, 0x10000000ll, puzzle | 0xc808000000ll,
                   partsUsed | 0x100l);
    piece9at27:
      if (partsUsed & 0x200l) goto piece10at27;
      if(!(puzzle & 0x1009208000000ll))
        doSolve(28, 0x10000000ll, puzzle | 0x1009208000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x400e008000000ll))
        doSolve(28, 0x10000000ll, puzzle | 0x400e008000000ll,
                   partsUsed | 0x200l);
    piece10at27:
      if (partsUsed & 0x400l) goto piece11at27;
      if(!(puzzle & 0x9009008000000ll))
        doSolve(28, 0x10000000ll, puzzle | 0x9009008000000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0xc00c008000000ll))
        doSolve(28, 0x10000000ll, puzzle | 0xc00c008000000ll,
                   partsUsed | 0x400l);
    piece11at27:
      if (partsUsed & 0x800l) goto piece12at27;
      if(!(puzzle & 0xd808000000ll))
        doSolve(28, 0x10000000ll, puzzle | 0xd808000000ll,
                   partsUsed | 0x800l);
    piece12at27:
    break;
   case 28:
      if (partsUsed & 0x2l) goto piece2at28;
      if(!(puzzle & 0x480090010000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x480090010000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x400480090000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x400480090000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1330000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x1330000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1990000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x1990000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x40060030000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x40060030000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x60030010000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x60030010000000ll,
                   partsUsed | 0x2l);
    piece2at28:
      if (partsUsed & 0x4l) goto piece3at28;
      if(!(puzzle & 0x10490010000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x10490010000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x10070010000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x10070010000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x80080490000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x80080490000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x790000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x790000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x970000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x970000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x20020070000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x20020070000000ll,
                   partsUsed | 0x4l);
    piece3at28:
      if (partsUsed & 0x8l) goto piece4at28;
      if(!(puzzle & 0x400490010000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x400490010000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x480080090000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x480080090000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1930000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x1930000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1390000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x1390000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x40070010000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x40070010000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x60020030000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x60020030000000ll,
                   partsUsed | 0x8l);
    piece4at28:
      if (partsUsed & 0x10l) goto piece5at28;
      if (!(puzzle & 0x2490000000ll)) {
         if(!(puzzle & 0x412490000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x412490000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x6590000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x6590000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x2082490000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x2082490000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x2cb0000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x2cb0000000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x4d30000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x4d30000000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x2490410000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x2490410000000ll,
                   partsUsed | 0x10l);
    piece5at28:
      if (partsUsed & 0x20l) goto piece6at28;
      if (!(puzzle & 0x2490000000ll)) {
         if(!(puzzle & 0x2002490000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x2002490000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x24b0000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x24b0000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x12490000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x12490000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x6490000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x6490000000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x2490010000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x2490010000000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x4930000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x4930000000ll,
                   partsUsed | 0x20l);
    piece6at28:
      if (partsUsed & 0x40l) goto piece7at28;
      if (!(puzzle & 0x2490000000ll)) {
         if(!(puzzle & 0x402490000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x402490000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2590000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x2590000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x82490000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x82490000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2c90000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x2c90000000ll,
                   partsUsed | 0x40l);
      }
    piece7at28:
      if (partsUsed & 0x80l) goto piece8at28;
      if(!(puzzle & 0x90010090000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x90010090000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x90080090000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x90080090000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x490410000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x490410000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x410490000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x410490000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x2f0000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x2f0000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xcb0000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0xcb0000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xd30000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0xd30000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x3d0000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x3d0000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x30020030000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x30020030000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x30010030000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x30010030000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x70050000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x70050000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x50070000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x50070000000ll,
                   partsUsed | 0x80l);
    piece8at28:
      if (partsUsed & 0x100l) goto piece9at28;
      if(!(puzzle & 0x100130000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x100130000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x130010000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x130010000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x200b0000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x200b0000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x90030000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x90030000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x180090000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x180090000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x10190000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x10190000000ll,
                   partsUsed | 0x100l);
    piece9at28:
      if (partsUsed & 0x200l) goto piece10at28;
      if(!(puzzle & 0x80490010000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x80490010000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x80480090000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x80480090000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0xb30000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0xb30000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0xb90000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0xb90000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x20060030000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x20060030000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x20070010000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x20070010000000ll,
                   partsUsed | 0x200l);
    piece10at28:
      if (partsUsed & 0x400l) goto piece11at28;
      if (!(puzzle & 0x80090000000ll)) {
         if(!(puzzle & 0x10090090000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x10090090000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x490090000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x490090000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x480490000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x480490000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x90490000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x90490000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x80090090000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x80090090000000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x130000000ll)) {
         if(!(puzzle & 0x370000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x370000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1f0000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x1f0000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x5b0000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x5b0000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x9b0000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x9b0000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x3b0000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x3b0000000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x10010010000000ll)) {
         if(!(puzzle & 0x90090010000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x90090010000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x10030030000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x10030030000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x30030010000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x30030010000000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x20030000000ll)) {
         if(!(puzzle & 0x20030030000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x20030030000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x70030000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x70030000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x30070000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x30070000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x60070000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x60070000000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0xd90000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0xd90000000ll,
                   partsUsed | 0x400l);
    piece11at28:
      if (partsUsed & 0x800l) goto piece12at28;
      if (!(puzzle & 0x10030000000ll)) {
         if(!(puzzle & 0x101b0000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x101b0000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xb0030000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0xb0030000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x300b0000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x300b0000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x30130000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x30130000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x130030000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x130030000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x900b0000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x900b0000000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x100110000000ll)) {
         if(!(puzzle & 0x180190000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x180190000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x120130000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x120130000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1001b0000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x1001b0000000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x80090000000ll)) {
         if(!(puzzle & 0xb0090000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0xb0090000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x90190000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x90190000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x801b0000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x801b0000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x190090000000ll))
           doSolve(29, 0x20000000ll, puzzle | 0x190090000000ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0x201b0000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x201b0000000ll,
                   partsUsed | 0x800l);
      if(!(puzzle & 0x1b0010000000ll))
        doSolve(29, 0x20000000ll, puzzle | 0x1b0010000000ll,
                   partsUsed | 0x800l);
    piece12at28:
    break;
   case 29:
      if (partsUsed & 0x2l) goto piece2at29;
      if(!(puzzle & 0x5e0000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x5e0000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x10030060000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x10030060000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x900120020000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x900120020000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x800900120000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x800900120000000ll,
                   partsUsed | 0x2l);
    piece2at29:
      if (partsUsed & 0x4l) goto piece3at29;
      if(!(puzzle & 0x1d20000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x1d20000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x100100920000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x100100920000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x70020020000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x70020020000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x20920020000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x20920020000000ll,
                   partsUsed | 0x4l);
    piece3at29:
      if (partsUsed & 0x8l) goto piece4at29;
      if(!(puzzle & 0xd60000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0xd60000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x30020060000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x30020060000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x800920020000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x800920020000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x900100120000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x900100120000000ll,
                   partsUsed | 0x8l);
    piece4at29:
      if (partsUsed & 0x10l) goto piece5at29;
      if (!(puzzle & 0x4920000000ll)) {
         if(!(puzzle & 0x69a0000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x69a0000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x824920000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x824920000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0xcb20000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0xcb20000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x4104920000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x4104920000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x5960000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x5960000000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x9a60000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x9a60000000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x4920820000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x4920820000000ll,
                   partsUsed | 0x10l);
    piece5at29:
      if (partsUsed & 0x20l) goto piece6at29;
      if (!(puzzle & 0x4920000000ll)) {
         if(!(puzzle & 0x6920000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x6920000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x4004920000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x4004920000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x4960000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x4960000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x24920000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x24920000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0xc920000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0xc920000000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x4920020000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x4920020000000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x9260000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x9260000000ll,
                   partsUsed | 0x20l);
    piece6at29:
      if (partsUsed & 0x40l) goto piece7at29;
      if (!(puzzle & 0x4920000000ll)) {
         if(!(puzzle & 0x49a0000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x49a0000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4d20000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x4d20000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x804920000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x804920000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4b20000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x4b20000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x104920000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x104920000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x5920000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x5920000000ll,
                   partsUsed | 0x40l);
      }
    piece7at29:
      if (partsUsed & 0x80l) goto piece8at29;
      if(!(puzzle & 0x120020120000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x120020120000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x120100120000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x120100120000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x920820000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x920820000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x820920000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x820920000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1960000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x1960000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1a60000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x1a60000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x60040060000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x60040060000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x60020060000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x60020060000000ll,
                   partsUsed | 0x80l);
    piece8at29:
      if (partsUsed & 0x100l) goto piece9at29;
      if(!(puzzle & 0x801a0000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x801a0000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x30120000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x30120000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x20320000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x20320000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1a0020000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x1a0020000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x120060000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x120060000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x260020000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x260020000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x40160000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x40160000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x300120000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x300120000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x200260000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x200260000000ll,
                   partsUsed | 0x100l);
    piece9at29:
      if (partsUsed & 0x200l) goto piece10at29;
      if (!(puzzle & 0x1a0000000ll)) {
         if(!(puzzle & 0x13a0000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x13a0000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x7a0000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x7a0000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x9e0000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x9e0000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x19a0000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x19a0000000ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x20020020000000ll)) {
         if(!(puzzle & 0x20030060000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x20030060000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x60030020000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x60030020000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x30060020000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x30060020000000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0xf20000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0xf20000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x10070020000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x10070020000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x40070020000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x40070020000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100920020000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x100920020000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100900120000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x100900120000000ll,
                   partsUsed | 0x200l);
    piece10at29:
      if (partsUsed & 0x400l) goto piece11at29;
      if (!(puzzle & 0x100120000000ll)) {
         if(!(puzzle & 0x20120120000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x20120120000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x920120000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x920120000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x900920000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x900920000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x120920000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x120920000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x100120120000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x100120120000000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x20020020000000ll)) {
         if(!(puzzle & 0x120120020000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x120120020000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x20060060000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x20060060000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x60060020000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x60060020000000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x320000000ll)) {
         if(!(puzzle & 0xb60000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0xb60000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1360000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x1360000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1b20000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x1b20000000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x3e0000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x3e0000000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0xda0000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0xda0000000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x70060000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x70060000000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x30030020000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x30030020000000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x40060060000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x40060060000000ll,
                   partsUsed | 0x400l);
    piece11at29:
      if (partsUsed & 0x800l) goto piece12at29;
      if (!(puzzle & 0x180020000000ll)) {
         if(!(puzzle & 0x1b0020000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x1b0020000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1a0120000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x1a0120000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1801a0000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x1801a0000000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x20120000000ll)) {
         if(!(puzzle & 0x130120000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x130120000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1201a0000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x1201a0000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x20360000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x20360000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x160120000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x160120000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x60160000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x60160000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x120320000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x120320000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x320120000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x320120000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x120160000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x120160000000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x40060000000ll)) {
         if(!(puzzle & 0x160060000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x160060000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x240260000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x240260000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x60260000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x60260000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x260060000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x260060000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x40360000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x40360000000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x320000000ll)) {
         if(!(puzzle & 0x300320000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x300320000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x200360000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x200360000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x100360000000ll))
           doSolve(30, 0x40000000ll, puzzle | 0x100360000000ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0x360020000000ll))
        doSolve(30, 0x40000000ll, puzzle | 0x360020000000ll,
                   partsUsed | 0x800l);
    piece12at29:
    break;
   case 30:
      if (partsUsed & 0x2l) goto piece2at30;
      if(!(puzzle & 0xf40000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0xf40000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x30060040000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x30060040000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1200240040000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x1200240040000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1001200240000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x1001200240000000ll,
                   partsUsed | 0x2l);
    piece2at30:
      if (partsUsed & 0x4l) goto piece3at30;
      if(!(puzzle & 0x13c0000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x13c0000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x200201240000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x200201240000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x40070040000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x40070040000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x41240040000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x41240040000000ll,
                   partsUsed | 0x4l);
    piece3at30:
      if (partsUsed & 0x8l) goto piece4at30;
      if(!(puzzle & 0x7c0000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x7c0000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x10070040000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x10070040000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1001240040000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x1001240040000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1200200240000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x1200200240000000ll,
                   partsUsed | 0x8l);
    piece4at30:
      if (partsUsed & 0x10l) goto piece5at30;
      if (!(puzzle & 0x9240000000ll)) {
         if(!(puzzle & 0xd340000000ll))
           doSolve(31, 0x80000000ll, puzzle | 0xd340000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x1049240000000ll))
           doSolve(31, 0x80000000ll, puzzle | 0x1049240000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x8209240000000ll))
           doSolve(31, 0x80000000ll, puzzle | 0x8209240000000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x9241040000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x9241040000000ll,
                   partsUsed | 0x10l);
    piece5at30:
      if (partsUsed & 0x20l) goto piece6at30;
      if (!(puzzle & 0x9240000000ll)) {
         if(!(puzzle & 0xd240000000ll))
           doSolve(31, 0x80000000ll, puzzle | 0xd240000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x8009240000000ll))
           doSolve(31, 0x80000000ll, puzzle | 0x8009240000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x49240000000ll))
           doSolve(31, 0x80000000ll, puzzle | 0x49240000000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x9240040000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x9240040000000ll,
                   partsUsed | 0x20l);
    piece6at30:
      if (partsUsed & 0x40l) goto piece7at30;
      if (!(puzzle & 0x9240000000ll)) {
         if(!(puzzle & 0x9340000000ll))
           doSolve(31, 0x80000000ll, puzzle | 0x9340000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x9a40000000ll))
           doSolve(31, 0x80000000ll, puzzle | 0x9a40000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x1009240000000ll))
           doSolve(31, 0x80000000ll, puzzle | 0x1009240000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x209240000000ll))
           doSolve(31, 0x80000000ll, puzzle | 0x209240000000ll,
                   partsUsed | 0x40l);
      }
    piece7at30:
      if (partsUsed & 0x80l) goto piece8at30;
      if(!(puzzle & 0x240040240000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x240040240000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x240200240000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x240200240000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1241040000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x1241040000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1041240000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x1041240000000ll,
                   partsUsed | 0x80l);
    piece8at30:
      if (partsUsed & 0x100l) goto piece9at30;
      if(!(puzzle & 0x340040000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x340040000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x100340000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x100340000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x60240000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x60240000000ll,
                   partsUsed | 0x100l);
    piece9at30:
      if (partsUsed & 0x200l) goto piece10at30;
      if(!(puzzle & 0xbc0000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0xbc0000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x20070040000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x20070040000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x201240040000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x201240040000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x201200240000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x201200240000000ll,
                   partsUsed | 0x200l);
    piece10at30:
      if (partsUsed & 0x400l) goto piece11at30;
      if (!(puzzle & 0x1240000000ll)) {
         if(!(puzzle & 0x1b40000000ll))
           doSolve(31, 0x80000000ll, puzzle | 0x1b40000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1201240000000ll))
           doSolve(31, 0x80000000ll, puzzle | 0x1201240000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x241240000000ll))
           doSolve(31, 0x80000000ll, puzzle | 0x241240000000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x40040040000000ll)) {
         if(!(puzzle & 0x60060040000000ll))
           doSolve(31, 0x80000000ll, puzzle | 0x60060040000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x40240240000000ll))
           doSolve(31, 0x80000000ll, puzzle | 0x40240240000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x240240040000000ll))
           doSolve(31, 0x80000000ll, puzzle | 0x240240040000000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x1240240000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x1240240000000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x200240240000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x200240240000000ll,
                   partsUsed | 0x400l);
    piece11at30:
      if (partsUsed & 0x800l) goto piece12at30;
      if (!(puzzle & 0x300040000000ll)) {
         if(!(puzzle & 0x360040000000ll))
           doSolve(31, 0x80000000ll, puzzle | 0x360040000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x340240000000ll))
           doSolve(31, 0x80000000ll, puzzle | 0x340240000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x300340000000ll))
           doSolve(31, 0x80000000ll, puzzle | 0x300340000000ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0x260240000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x260240000000ll,
                   partsUsed | 0x800l);
      if(!(puzzle & 0x240340000000ll))
        doSolve(31, 0x80000000ll, puzzle | 0x240340000000ll,
                   partsUsed | 0x800l);
    piece12at30:
    break;
   case 31:
      if (partsUsed & 0x2l) goto piece2at31;
      if(!(puzzle & 0x10090480000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x10090480000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x2400480080000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x2400480080000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x2002400480000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x2002400480000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x9980000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x9980000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0xcc80000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0xcc80000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x200300180000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x200300180000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x300180080000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x300180080000000ll,
                   partsUsed | 0x2l);
    piece2at31:
      if (partsUsed & 0x4l) goto piece3at31;
      if (!(puzzle & 0x80080080000000ll)) {
         if(!(puzzle & 0x490080080000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x490080080000000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x82480080000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x82480080000000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x80380080000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x80380080000000ll,
                   partsUsed | 0x4l);
      }
      if(!(puzzle & 0x400402480000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x400402480000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x3c80000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x3c80000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x4b80000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x4b80000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x100100380000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x100100380000000ll,
                   partsUsed | 0x4l);
    piece3at31:
      if (partsUsed & 0x8l) goto piece4at31;
      if(!(puzzle & 0x90080480000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x90080480000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x2002480080000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x2002480080000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x2400400480000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x2400400480000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0xc980000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0xc980000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x9c80000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x9c80000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x200380080000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x200380080000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x300100180000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x300100180000000ll,
                   partsUsed | 0x8l);
    piece4at31:
      if (partsUsed & 0x10l) goto piece5at31;
      if(!(puzzle & 0x2492080000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x2492080000000ll,
                   partsUsed | 0x10l);
    piece5at31:
      if (partsUsed & 0x40l) goto piece7at31;
      if(!(puzzle & 0x2490080000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x2490080000000ll,
                   partsUsed | 0x40l);
    piece7at31:
      if (partsUsed & 0x80l) goto piece8at31;
      if(!(puzzle & 0x480080480000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x480080480000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x480400480000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x480400480000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x2482080000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x2482080000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x2082480000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x2082480000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1780000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x1780000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x6580000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x6580000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x6980000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x6980000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1e80000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x1e80000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x180100180000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x180100180000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x180080180000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x180080180000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x380280000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x380280000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x280380000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x280380000000ll,
                   partsUsed | 0x80l);
    piece8at31:
      if (partsUsed & 0x100l) goto piece9at31;
      if(!(puzzle & 0x120180000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x120180000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x100580000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x100580000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xb0080000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0xb0080000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x980080000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x980080000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x480180000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x480180000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xc00480000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0xc00480000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x80c80000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x80c80000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x800980000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x800980000000ll,
                   partsUsed | 0x100l);
    piece9at31:
      if (partsUsed & 0x200l) goto piece10at31;
      if (!(puzzle & 0x80080080000000ll)) {
         if(!(puzzle & 0x80090480000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x80090480000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x90480080000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x90480080000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x480090080000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x480090080000000ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x480080000000ll)) {
         if(!(puzzle & 0x400490080000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x400490080000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x10490080000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x10490080000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x402480080000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x402480080000000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x5980000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x5980000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x5c80000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x5c80000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x402400480000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x402400480000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100300180000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x100300180000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100380080000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x100380080000000ll,
                   partsUsed | 0x200l);
    piece10at31:
      if (partsUsed & 0x400l) goto piece11at31;
      if (!(puzzle & 0x400480000000ll)) {
         if(!(puzzle & 0x80480480000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x80480480000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2480480000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x2480480000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2402480000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x2402480000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x482480000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x482480000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x400480480000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x400480480000000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x980000000ll)) {
         if(!(puzzle & 0x1b80000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x1b80000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xf80000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0xf80000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2d80000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x2d80000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x4d80000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x4d80000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1d80000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x1d80000000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x80080080000000ll)) {
         if(!(puzzle & 0x480480080000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x480480080000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x80180180000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x80180180000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x180180080000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x180180080000000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x100180000000ll)) {
         if(!(puzzle & 0x100180180000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x100180180000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x380180000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x380180000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x180380000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x180380000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x300380000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x300380000000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x90090080000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x90090080000000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x490480000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x490480000000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x6c80000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x6c80000000ll,
                   partsUsed | 0x400l);
    piece11at31:
      if (partsUsed & 0x800l) goto piece12at31;
      if (!(puzzle & 0x180080000000ll)) {
         if(!(puzzle & 0x1b0080000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x1b0080000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1a0180000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x1a0180000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x190180000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x190180000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x580180000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x580180000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x580480000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x580480000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x180580000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x180580000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x180980000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x180980000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x980180000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x980180000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xd80080000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0xd80080000000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0xc80000000ll)) {
         if(!(puzzle & 0x80d80000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x80d80000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xc00c80000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0xc00c80000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x480c80000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x480c80000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x800d80000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x800d80000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x100d80000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x100d80000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x400d80000000ll))
           doSolve(32, 0x100000000ll, puzzle | 0x400d80000000ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0x900980000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x900980000000ll,
                   partsUsed | 0x800l);
      if(!(puzzle & 0xc80480000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0xc80480000000ll,
                   partsUsed | 0x800l);
      if(!(puzzle & 0x480580000000ll))
        doSolve(32, 0x100000000ll, puzzle | 0x480580000000ll,
                   partsUsed | 0x800l);
    piece12at31:
    break;
  }
}


void doSolve32(int start, int startMask, int puzzle, int partsUsed)
{
  for (;(puzzle & startMask) && start < NUM_HI_BITS; start++, startMask <<= 1);
  if(partsUsed == (1<<12)-1) {
    if(!(++numSolutions%4099)) { write(0, ".", 1); }
    if(!((numSolutions) % (409963/10))) {
      percent++;
      printf("\r%d%% done, %ld seconds so far; expecting %.0lf seconds.\n", 
	     percent*10, (clock()-startTime)/1000000, ((double)(clock()-startTime)/100000/percent)*correction[percent-1]);
      fflush(stdout);
    }
    return;
  }
  switch(start) {
   case 0:
      if (partsUsed & 0x2l) goto piece2at0at32;
      if(!(puzzle & 0x201209l))
        doSolve32(1, 0x2l, puzzle | 0x201209l, partsUsed | 0x2l);
      if(!(puzzle & 0x2fl))
        doSolve32(1, 0x2l, puzzle | 0x2fl, partsUsed | 0x2l);
      if(!(puzzle & 0x801803l))
        doSolve32(1, 0x2l, puzzle | 0x801803l, partsUsed | 0x2l);
      if(!(puzzle & 0x48009001l))
        doSolve32(1, 0x2l, puzzle | 0x48009001l, partsUsed | 0x2l);
      if(!(puzzle & 0x40048009l))
        doSolve32(1, 0x2l, puzzle | 0x40048009l, partsUsed | 0x2l);
    piece2at0at32:
      if (partsUsed & 0x4l) goto piece3at0at32;
      if (!(puzzle & 0x1001001l)) {
         if(!(puzzle & 0x9201001l))
           doSolve32(1, 0x2l, puzzle | 0x9201001l, partsUsed | 0x4l);
         if(!(puzzle & 0x3801001l))
           doSolve32(1, 0x2l, puzzle | 0x3801001l, partsUsed | 0x4l);
         if(!(puzzle & 0x1049001l))
           doSolve32(1, 0x2l, puzzle | 0x1049001l, partsUsed | 0x4l);
      }
      if(!(puzzle & 0xe9l))
        doSolve32(1, 0x2l, puzzle | 0xe9l, partsUsed | 0x4l);
      if(!(puzzle & 0x8008049l))
        doSolve32(1, 0x2l, puzzle | 0x8008049l, partsUsed | 0x4l);
    piece3at0at32:
      if (partsUsed & 0x8l) goto piece4at0at32;
      if(!(puzzle & 0x1201009l))
        doSolve32(1, 0x2l, puzzle | 0x1201009l, partsUsed | 0x8l);
      if(!(puzzle & 0x1801003l))
        doSolve32(1, 0x2l, puzzle | 0x1801003l, partsUsed | 0x8l);
      if(!(puzzle & 0x6bl))
        doSolve32(1, 0x2l, puzzle | 0x6bl, partsUsed | 0x8l);
      if(!(puzzle & 0x40049001l))
        doSolve32(1, 0x2l, puzzle | 0x40049001l, partsUsed | 0x8l);
      if(!(puzzle & 0x48008009l))
        doSolve32(1, 0x2l, puzzle | 0x48008009l, partsUsed | 0x8l);
    piece4at0at32:
      if (partsUsed & 0x10l) goto piece5at0at32;
      if(!(puzzle & 0x49241l))
        doSolve32(1, 0x2l, puzzle | 0x49241l, partsUsed | 0x10l);
    piece5at0at32:
      if (partsUsed & 0x40l) goto piece7at0at32;
      if(!(puzzle & 0x49201l))
        doSolve32(1, 0x2l, puzzle | 0x49201l, partsUsed | 0x40l);
    piece7at0at32:
      if (partsUsed & 0x80l) goto piece8at0at32;
      if(!(puzzle & 0x9001009l))
        doSolve32(1, 0x2l, puzzle | 0x9001009l, partsUsed | 0x80l);
      if(!(puzzle & 0x9008009l))
        doSolve32(1, 0x2l, puzzle | 0x9008009l, partsUsed | 0x80l);
      if(!(puzzle & 0x49041l))
        doSolve32(1, 0x2l, puzzle | 0x49041l, partsUsed | 0x80l);
      if(!(puzzle & 0x41049l))
        doSolve32(1, 0x2l, puzzle | 0x41049l, partsUsed | 0x80l);
      if(!(puzzle & 0xcbl))
        doSolve32(1, 0x2l, puzzle | 0xcbl, partsUsed | 0x80l);
      if(!(puzzle & 0xd3l))
        doSolve32(1, 0x2l, puzzle | 0xd3l, partsUsed | 0x80l);
      if(!(puzzle & 0x3002003l))
        doSolve32(1, 0x2l, puzzle | 0x3002003l, partsUsed | 0x80l);
      if(!(puzzle & 0x3001003l))
        doSolve32(1, 0x2l, puzzle | 0x3001003l, partsUsed | 0x80l);
    piece8at0at32:
      if (partsUsed & 0x100l) goto piece9at0at32;
      if(!(puzzle & 0x1901l))
        doSolve32(1, 0x2l, puzzle | 0x1901l, partsUsed | 0x100l);
      if(!(puzzle & 0x1809l))
        doSolve32(1, 0x2l, puzzle | 0x1809l, partsUsed | 0x100l);
      if(!(puzzle & 0x2403l))
        doSolve32(1, 0x2l, puzzle | 0x2403l, partsUsed | 0x100l);
      if(!(puzzle & 0x200bl))
        doSolve32(1, 0x2l, puzzle | 0x200bl, partsUsed | 0x100l);
      if(!(puzzle & 0x1601l))
        doSolve32(1, 0x2l, puzzle | 0x1601l, partsUsed | 0x100l);
      if(!(puzzle & 0x400dl))
        doSolve32(1, 0x2l, puzzle | 0x400dl, partsUsed | 0x100l);
      if(!(puzzle & 0xd001l))
        doSolve32(1, 0x2l, puzzle | 0xd001l, partsUsed | 0x100l);
      if(!(puzzle & 0x9003l))
        doSolve32(1, 0x2l, puzzle | 0x9003l, partsUsed | 0x100l);
      if(!(puzzle & 0x13001l))
        doSolve32(1, 0x2l, puzzle | 0x13001l, partsUsed | 0x100l);
      if(!(puzzle & 0x18009l))
        doSolve32(1, 0x2l, puzzle | 0x18009l, partsUsed | 0x100l);
      if(!(puzzle & 0x1019l))
        doSolve32(1, 0x2l, puzzle | 0x1019l, partsUsed | 0x100l);
      if(!(puzzle & 0x10013l))
        doSolve32(1, 0x2l, puzzle | 0x10013l, partsUsed | 0x100l);
    piece9at0at32:
      if (partsUsed & 0x200l) goto piece10at0at32;
      if (!(puzzle & 0x1001001l)) {
         if(!(puzzle & 0x1001209l))
           doSolve32(1, 0x2l, puzzle | 0x1001209l, partsUsed | 0x200l);
         if(!(puzzle & 0x1209001l))
           doSolve32(1, 0x2l, puzzle | 0x1209001l, partsUsed | 0x200l);
         if(!(puzzle & 0x9001201l))
           doSolve32(1, 0x2l, puzzle | 0x9001201l, partsUsed | 0x200l);
         if(!(puzzle & 0x1001803l))
           doSolve32(1, 0x2l, puzzle | 0x1001803l, partsUsed | 0x200l);
         if(!(puzzle & 0x3001801l))
           doSolve32(1, 0x2l, puzzle | 0x3001801l, partsUsed | 0x200l);
         if(!(puzzle & 0x1803001l))
           doSolve32(1, 0x2l, puzzle | 0x1803001l, partsUsed | 0x200l);
      }
      if (!(puzzle & 0x9001l)) {
         if(!(puzzle & 0x8009201l))
           doSolve32(1, 0x2l, puzzle | 0x8009201l, partsUsed | 0x200l);
         if(!(puzzle & 0x209201l))
           doSolve32(1, 0x2l, puzzle | 0x209201l, partsUsed | 0x200l);
         if(!(puzzle & 0x8049001l))
           doSolve32(1, 0x2l, puzzle | 0x8049001l, partsUsed | 0x200l);
      }
      if (!(puzzle & 0xdl)) {
         if(!(puzzle & 0x9dl))
           doSolve32(1, 0x2l, puzzle | 0x9dl, partsUsed | 0x200l);
         if(!(puzzle & 0x3dl))
           doSolve32(1, 0x2l, puzzle | 0x3dl, partsUsed | 0x200l);
         if(!(puzzle & 0x4fl))
           doSolve32(1, 0x2l, puzzle | 0x4fl, partsUsed | 0x200l);
         if(!(puzzle & 0xcdl))
           doSolve32(1, 0x2l, puzzle | 0xcdl, partsUsed | 0x200l);
      }
      if(!(puzzle & 0x79l))
        doSolve32(1, 0x2l, puzzle | 0x79l, partsUsed | 0x200l);
      if(!(puzzle & 0x803801l))
        doSolve32(1, 0x2l, puzzle | 0x803801l, partsUsed | 0x200l);
      if(!(puzzle & 0x2003801l))
        doSolve32(1, 0x2l, puzzle | 0x2003801l, partsUsed | 0x200l);
      if(!(puzzle & 0x8048009l))
        doSolve32(1, 0x2l, puzzle | 0x8048009l, partsUsed | 0x200l);
    piece10at0at32:
      if (partsUsed & 0x400l) goto piece11at0at32;
      if (!(puzzle & 0x8009l)) {
         if(!(puzzle & 0x1009009l))
           doSolve32(1, 0x2l, puzzle | 0x1009009l, partsUsed | 0x400l);
         if(!(puzzle & 0x49009l))
           doSolve32(1, 0x2l, puzzle | 0x49009l, partsUsed | 0x400l);
         if(!(puzzle & 0x48049l))
           doSolve32(1, 0x2l, puzzle | 0x48049l, partsUsed | 0x400l);
         if(!(puzzle & 0x9049l))
           doSolve32(1, 0x2l, puzzle | 0x9049l, partsUsed | 0x400l);
         if(!(puzzle & 0x8009009l))
           doSolve32(1, 0x2l, puzzle | 0x8009009l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x1001001l)) {
         if(!(puzzle & 0x9009001l))
           doSolve32(1, 0x2l, puzzle | 0x9009001l, partsUsed | 0x400l);
         if(!(puzzle & 0x1003003l))
           doSolve32(1, 0x2l, puzzle | 0x1003003l, partsUsed | 0x400l);
         if(!(puzzle & 0x3003001l))
           doSolve32(1, 0x2l, puzzle | 0x3003001l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x19l)) {
         if(!(puzzle & 0x5bl))
           doSolve32(1, 0x2l, puzzle | 0x5bl, partsUsed | 0x400l);
         if(!(puzzle & 0x9bl))
           doSolve32(1, 0x2l, puzzle | 0x9bl, partsUsed | 0x400l);
         if(!(puzzle & 0xd9l))
           doSolve32(1, 0x2l, puzzle | 0xd9l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x1201201l))
        doSolve32(1, 0x2l, puzzle | 0x1201201l, partsUsed | 0x400l);
      if(!(puzzle & 0x9209l))
        doSolve32(1, 0x2l, puzzle | 0x9209l, partsUsed | 0x400l);
      if(!(puzzle & 0x1fl))
        doSolve32(1, 0x2l, puzzle | 0x1fl, partsUsed | 0x400l);
      if(!(puzzle & 0x6dl))
        doSolve32(1, 0x2l, puzzle | 0x6dl, partsUsed | 0x400l);
      if(!(puzzle & 0x3803l))
        doSolve32(1, 0x2l, puzzle | 0x3803l, partsUsed | 0x400l);
      if(!(puzzle & 0x1801801l))
        doSolve32(1, 0x2l, puzzle | 0x1801801l, partsUsed | 0x400l);
      if(!(puzzle & 0x2003003l))
        doSolve32(1, 0x2l, puzzle | 0x2003003l, partsUsed | 0x400l);
    piece11at0at32:
      if (partsUsed & 0x800l) goto piece12at0at32;
      if (!(puzzle & 0x1201l)) {
         if(!(puzzle & 0x1b01l))
           doSolve32(1, 0x2l, puzzle | 0x1b01l, partsUsed | 0x800l);
         if(!(puzzle & 0x3601l))
           doSolve32(1, 0x2l, puzzle | 0x3601l, partsUsed | 0x800l);
         if(!(puzzle & 0x3203l))
           doSolve32(1, 0x2l, puzzle | 0x3203l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x1003l)) {
         if(!(puzzle & 0x3403l))
           doSolve32(1, 0x2l, puzzle | 0x3403l, partsUsed | 0x800l);
         if(!(puzzle & 0x101bl))
           doSolve32(1, 0x2l, puzzle | 0x101bl, partsUsed | 0x800l);
         if(!(puzzle & 0xb003l))
           doSolve32(1, 0x2l, puzzle | 0xb003l, partsUsed | 0x800l);
         if(!(puzzle & 0x300bl))
           doSolve32(1, 0x2l, puzzle | 0x300bl, partsUsed | 0x800l);
         if(!(puzzle & 0x3013l))
           doSolve32(1, 0x2l, puzzle | 0x3013l, partsUsed | 0x800l);
         if(!(puzzle & 0x13003l))
           doSolve32(1, 0x2l, puzzle | 0x13003l, partsUsed | 0x800l);
         if(!(puzzle & 0x900bl))
           doSolve32(1, 0x2l, puzzle | 0x900bl, partsUsed | 0x800l);
      }
      if (!(puzzle & 0xc001l)) {
         if(!(puzzle & 0xd801l))
           doSolve32(1, 0x2l, puzzle | 0xd801l, partsUsed | 0x800l);
         if(!(puzzle & 0xd009l))
           doSolve32(1, 0x2l, puzzle | 0xd009l, partsUsed | 0x800l);
         if(!(puzzle & 0xc00dl))
           doSolve32(1, 0x2l, puzzle | 0xc00dl, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x8009l)) {
         if(!(puzzle & 0x9809l))
           doSolve32(1, 0x2l, puzzle | 0x9809l, partsUsed | 0x800l);
         if(!(puzzle & 0x900dl))
           doSolve32(1, 0x2l, puzzle | 0x900dl, partsUsed | 0x800l);
         if(!(puzzle & 0x18019l))
           doSolve32(1, 0x2l, puzzle | 0x18019l, partsUsed | 0x800l);
         if(!(puzzle & 0xb009l))
           doSolve32(1, 0x2l, puzzle | 0xb009l, partsUsed | 0x800l);
         if(!(puzzle & 0x9019l))
           doSolve32(1, 0x2l, puzzle | 0x9019l, partsUsed | 0x800l);
         if(!(puzzle & 0x801bl))
           doSolve32(1, 0x2l, puzzle | 0x801bl, partsUsed | 0x800l);
         if(!(puzzle & 0x19009l))
           doSolve32(1, 0x2l, puzzle | 0x19009l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x13l)) {
         if(!(puzzle & 0x12013l))
           doSolve32(1, 0x2l, puzzle | 0x12013l, partsUsed | 0x800l);
         if(!(puzzle & 0x1001bl))
           doSolve32(1, 0x2l, puzzle | 0x1001bl, partsUsed | 0x800l);
         if(!(puzzle & 0x201bl))
           doSolve32(1, 0x2l, puzzle | 0x201bl, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x1b001l))
        doSolve32(1, 0x2l, puzzle | 0x1b001l, partsUsed | 0x800l);
    piece12at0at32:
    break;
   case 1:
      if (partsUsed & 0x2l) goto piece2at1at32;
      if(!(puzzle & 0x402412l))
        doSolve32(2, 0x4l, puzzle | 0x402412l, partsUsed | 0x2l);
      if(!(puzzle & 0x7al))
        doSolve32(2, 0x4l, puzzle | 0x7al, partsUsed | 0x2l);
      if(!(puzzle & 0x1803002l))
        doSolve32(2, 0x4l, puzzle | 0x1803002l, partsUsed | 0x2l);
      if(!(puzzle & 0x90012002l))
        doSolve32(2, 0x4l, puzzle | 0x90012002l, partsUsed | 0x2l);
      if(!(puzzle & 0x80090012l))
        doSolve32(2, 0x4l, puzzle | 0x80090012l, partsUsed | 0x2l);
    piece2at1at32:
      if (partsUsed & 0x4l) goto piece3at1at32;
      if (!(puzzle & 0x2002002l)) {
         if(!(puzzle & 0x12402002l))
           doSolve32(2, 0x4l, puzzle | 0x12402002l, partsUsed | 0x4l);
         if(!(puzzle & 0x2003802l))
           doSolve32(2, 0x4l, puzzle | 0x2003802l, partsUsed | 0x4l);
         if(!(puzzle & 0x2092002l))
           doSolve32(2, 0x4l, puzzle | 0x2092002l, partsUsed | 0x4l);
      }
      if(!(puzzle & 0x9el))
        doSolve32(2, 0x4l, puzzle | 0x9el, partsUsed | 0x4l);
      if(!(puzzle & 0x10010092l))
        doSolve32(2, 0x4l, puzzle | 0x10010092l, partsUsed | 0x4l);
    piece3at1at32:
      if (partsUsed & 0x8l) goto piece4at1at32;
      if(!(puzzle & 0x2402012l))
        doSolve32(2, 0x4l, puzzle | 0x2402012l, partsUsed | 0x8l);
      if(!(puzzle & 0x3el))
        doSolve32(2, 0x4l, puzzle | 0x3el, partsUsed | 0x8l);
      if(!(puzzle & 0x803802l))
        doSolve32(2, 0x4l, puzzle | 0x803802l, partsUsed | 0x8l);
      if(!(puzzle & 0x80092002l))
        doSolve32(2, 0x4l, puzzle | 0x80092002l, partsUsed | 0x8l);
      if(!(puzzle & 0x90010012l))
        doSolve32(2, 0x4l, puzzle | 0x90010012l, partsUsed | 0x8l);
    piece4at1at32:
      if (partsUsed & 0x10l) goto piece5at1at32;
      if(!(puzzle & 0x92482l))
        doSolve32(2, 0x4l, puzzle | 0x92482l, partsUsed | 0x10l);
    piece5at1at32:
      if (partsUsed & 0x40l) goto piece7at1at32;
      if(!(puzzle & 0x92402l))
        doSolve32(2, 0x4l, puzzle | 0x92402l, partsUsed | 0x40l);
    piece7at1at32:
      if (partsUsed & 0x80l) goto piece8at1at32;
      if(!(puzzle & 0x12002012l))
        doSolve32(2, 0x4l, puzzle | 0x12002012l, partsUsed | 0x80l);
      if(!(puzzle & 0x12010012l))
        doSolve32(2, 0x4l, puzzle | 0x12010012l, partsUsed | 0x80l);
      if(!(puzzle & 0x92082l))
        doSolve32(2, 0x4l, puzzle | 0x92082l, partsUsed | 0x80l);
      if(!(puzzle & 0x82092l))
        doSolve32(2, 0x4l, puzzle | 0x82092l, partsUsed | 0x80l);
    piece8at1at32:
      if (partsUsed & 0x100l) goto piece9at1at32;
      if(!(puzzle & 0x3202l))
        doSolve32(2, 0x4l, puzzle | 0x3202l, partsUsed | 0x100l);
      if(!(puzzle & 0x3012l))
        doSolve32(2, 0x4l, puzzle | 0x3012l, partsUsed | 0x100l);
      if(!(puzzle & 0x801al))
        doSolve32(2, 0x4l, puzzle | 0x801al, partsUsed | 0x100l);
      if(!(puzzle & 0x1a002l))
        doSolve32(2, 0x4l, puzzle | 0x1a002l, partsUsed | 0x100l);
    piece9at1at32:
      if (partsUsed & 0x200l) goto piece10at1at32;
      if (!(puzzle & 0x2002002l)) {
         if(!(puzzle & 0x2002412l))
           doSolve32(2, 0x4l, puzzle | 0x2002412l, partsUsed | 0x200l);
         if(!(puzzle & 0x2412002l))
           doSolve32(2, 0x4l, puzzle | 0x2412002l, partsUsed | 0x200l);
         if(!(puzzle & 0x12002402l))
           doSolve32(2, 0x4l, puzzle | 0x12002402l, partsUsed | 0x200l);
      }
      if (!(puzzle & 0x12002l)) {
         if(!(puzzle & 0x10012402l))
           doSolve32(2, 0x4l, puzzle | 0x10012402l, partsUsed | 0x200l);
         if(!(puzzle & 0x412402l))
           doSolve32(2, 0x4l, puzzle | 0x412402l, partsUsed | 0x200l);
         if(!(puzzle & 0x10092002l))
           doSolve32(2, 0x4l, puzzle | 0x10092002l, partsUsed | 0x200l);
      }
      if(!(puzzle & 0x5el))
        doSolve32(2, 0x4l, puzzle | 0x5el, partsUsed | 0x200l);
      if(!(puzzle & 0x1003802l))
        doSolve32(2, 0x4l, puzzle | 0x1003802l, partsUsed | 0x200l);
      if(!(puzzle & 0x10090012l))
        doSolve32(2, 0x4l, puzzle | 0x10090012l, partsUsed | 0x200l);
    piece10at1at32:
      if (partsUsed & 0x400l) goto piece11at1at32;
      if (!(puzzle & 0x92l)) {
         if(!(puzzle & 0xdal))
           doSolve32(2, 0x4l, puzzle | 0xdal, partsUsed | 0x400l);
         if(!(puzzle & 0x90092l))
           doSolve32(2, 0x4l, puzzle | 0x90092l, partsUsed | 0x400l);
         if(!(puzzle & 0x12092l))
           doSolve32(2, 0x4l, puzzle | 0x12092l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x2002002l)) {
         if(!(puzzle & 0x3003002l))
           doSolve32(2, 0x4l, puzzle | 0x3003002l, partsUsed | 0x400l);
         if(!(puzzle & 0x2012012l))
           doSolve32(2, 0x4l, puzzle | 0x2012012l, partsUsed | 0x400l);
         if(!(puzzle & 0x12012002l))
           doSolve32(2, 0x4l, puzzle | 0x12012002l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x2402402l))
        doSolve32(2, 0x4l, puzzle | 0x2402402l, partsUsed | 0x400l);
      if(!(puzzle & 0x12412l))
        doSolve32(2, 0x4l, puzzle | 0x12412l, partsUsed | 0x400l);
      if(!(puzzle & 0x92012l))
        doSolve32(2, 0x4l, puzzle | 0x92012l, partsUsed | 0x400l);
      if(!(puzzle & 0x10012012l))
        doSolve32(2, 0x4l, puzzle | 0x10012012l, partsUsed | 0x400l);
    piece11at1at32:
      if (partsUsed & 0x800l) goto piece12at1at32;
      if (!(puzzle & 0x3002l)) {
         if(!(puzzle & 0x3602l))
           doSolve32(2, 0x4l, puzzle | 0x3602l, partsUsed | 0x800l);
         if(!(puzzle & 0x1b002l))
           doSolve32(2, 0x4l, puzzle | 0x1b002l, partsUsed | 0x800l);
         if(!(puzzle & 0x13012l))
           doSolve32(2, 0x4l, puzzle | 0x13012l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x10012l)) {
         if(!(puzzle & 0x1a012l))
           doSolve32(2, 0x4l, puzzle | 0x1a012l, partsUsed | 0x800l);
         if(!(puzzle & 0x1801al))
           doSolve32(2, 0x4l, puzzle | 0x1801al, partsUsed | 0x800l);
         if(!(puzzle & 0x1201al))
           doSolve32(2, 0x4l, puzzle | 0x1201al, partsUsed | 0x800l);
      }
    piece12at1at32:
    break;
   case 2:
      if (partsUsed & 0x2l) goto piece2at2at32;
      if(!(puzzle & 0x904804l))
        doSolve32(3, 0x8l, puzzle | 0x904804l, partsUsed | 0x2l);
      if(!(puzzle & 0x804824l))
        doSolve32(3, 0x8l, puzzle | 0x804824l, partsUsed | 0x2l);
      if(!(puzzle & 0x1001800cl))
        doSolve32(3, 0x8l, puzzle | 0x1001800cl, partsUsed | 0x2l);
      if(!(puzzle & 0x1800c004l))
        doSolve32(3, 0x8l, puzzle | 0x1800c004l, partsUsed | 0x2l);
    piece2at2at32:
      if (partsUsed & 0x4l) goto piece3at2at32;
      if (!(puzzle & 0x4004004l)) {
         if(!(puzzle & 0x4004904l))
           doSolve32(3, 0x8l, puzzle | 0x4004904l, partsUsed | 0x4l);
         if(!(puzzle & 0x24804004l))
           doSolve32(3, 0x8l, puzzle | 0x24804004l, partsUsed | 0x4l);
         if(!(puzzle & 0x401c004l))
           doSolve32(3, 0x8l, puzzle | 0x401c004l, partsUsed | 0x4l);
      }
      if(!(puzzle & 0x800801cl))
        doSolve32(3, 0x8l, puzzle | 0x800801cl, partsUsed | 0x4l);
    piece3at2at32:
      if (partsUsed & 0x8l) goto piece4at2at32;
      if(!(puzzle & 0x104904l))
        doSolve32(3, 0x8l, puzzle | 0x104904l, partsUsed | 0x8l);
      if(!(puzzle & 0x4804024l))
        doSolve32(3, 0x8l, puzzle | 0x4804024l, partsUsed | 0x8l);
      if(!(puzzle & 0x1001c004l))
        doSolve32(3, 0x8l, puzzle | 0x1001c004l, partsUsed | 0x8l);
      if(!(puzzle & 0x1800800cl))
        doSolve32(3, 0x8l, puzzle | 0x1800800cl, partsUsed | 0x8l);
    piece4at2at32:
      if (partsUsed & 0x40l) goto piece7at2at32;
      if(!(puzzle & 0x24904l))
        doSolve32(3, 0x8l, puzzle | 0x24904l, partsUsed | 0x40l);
    piece7at2at32:
      if (partsUsed & 0x80l) goto piece8at2at32;
      if(!(puzzle & 0x24004024l))
        doSolve32(3, 0x8l, puzzle | 0x24004024l, partsUsed | 0x80l);
      if(!(puzzle & 0x24020024l))
        doSolve32(3, 0x8l, puzzle | 0x24020024l, partsUsed | 0x80l);
      if(!(puzzle & 0xbcl))
        doSolve32(3, 0x8l, puzzle | 0xbcl, partsUsed | 0x80l);
      if(!(puzzle & 0xf4l))
        doSolve32(3, 0x8l, puzzle | 0xf4l, partsUsed | 0x80l);
      if(!(puzzle & 0xc00800cl))
        doSolve32(3, 0x8l, puzzle | 0xc00800cl, partsUsed | 0x80l);
      if(!(puzzle & 0xc00400cl))
        doSolve32(3, 0x8l, puzzle | 0xc00400cl, partsUsed | 0x80l);
      if(!(puzzle & 0x1c014l))
        doSolve32(3, 0x8l, puzzle | 0x1c014l, partsUsed | 0x80l);
      if(!(puzzle & 0x1401cl))
        doSolve32(3, 0x8l, puzzle | 0x1401cl, partsUsed | 0x80l);
    piece8at2at32:
      if (partsUsed & 0x100l) goto piece9at2at32;
      if(!(puzzle & 0x900cl))
        doSolve32(3, 0x8l, puzzle | 0x900cl, partsUsed | 0x100l);
      if(!(puzzle & 0x802cl))
        doSolve32(3, 0x8l, puzzle | 0x802cl, partsUsed | 0x100l);
      if(!(puzzle & 0x5804l))
        doSolve32(3, 0x8l, puzzle | 0x5804l, partsUsed | 0x100l);
      if(!(puzzle & 0x4c004l))
        doSolve32(3, 0x8l, puzzle | 0x4c004l, partsUsed | 0x100l);
      if(!(puzzle & 0x2400cl))
        doSolve32(3, 0x8l, puzzle | 0x2400cl, partsUsed | 0x100l);
      if(!(puzzle & 0x60024l))
        doSolve32(3, 0x8l, puzzle | 0x60024l, partsUsed | 0x100l);
      if(!(puzzle & 0x4064l))
        doSolve32(3, 0x8l, puzzle | 0x4064l, partsUsed | 0x100l);
      if(!(puzzle & 0x4004cl))
        doSolve32(3, 0x8l, puzzle | 0x4004cl, partsUsed | 0x100l);
    piece9at2at32:
      if (partsUsed & 0x200l) goto piece10at2at32;
      if (!(puzzle & 0x4804l)) {
         if(!(puzzle & 0x804904l))
           doSolve32(3, 0x8l, puzzle | 0x804904l, partsUsed | 0x200l);
         if(!(puzzle & 0x4004824l))
           doSolve32(3, 0x8l, puzzle | 0x4004824l, partsUsed | 0x200l);
         if(!(puzzle & 0x20024804l))
           doSolve32(3, 0x8l, puzzle | 0x20024804l, partsUsed | 0x200l);
         if(!(puzzle & 0x24004804l))
           doSolve32(3, 0x8l, puzzle | 0x24004804l, partsUsed | 0x200l);
         if(!(puzzle & 0x824804l))
           doSolve32(3, 0x8l, puzzle | 0x824804l, partsUsed | 0x200l);
      }
      if(!(puzzle & 0x4824004l))
        doSolve32(3, 0x8l, puzzle | 0x4824004l, partsUsed | 0x200l);
      if(!(puzzle & 0x801800cl))
        doSolve32(3, 0x8l, puzzle | 0x801800cl, partsUsed | 0x200l);
      if(!(puzzle & 0x801c004l))
        doSolve32(3, 0x8l, puzzle | 0x801c004l, partsUsed | 0x200l);
    piece10at2at32:
      if (partsUsed & 0x400l) goto piece11at2at32;
      if (!(puzzle & 0x24004l)) {
         if(!(puzzle & 0x4024024l))
           doSolve32(3, 0x8l, puzzle | 0x4024024l, partsUsed | 0x400l);
         if(!(puzzle & 0x24024004l))
           doSolve32(3, 0x8l, puzzle | 0x24024004l, partsUsed | 0x400l);
         if(!(puzzle & 0x20024024l))
           doSolve32(3, 0x8l, puzzle | 0x20024024l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x4cl)) {
         if(!(puzzle & 0xdcl))
           doSolve32(3, 0x8l, puzzle | 0xdcl, partsUsed | 0x400l);
         if(!(puzzle & 0x7cl))
           doSolve32(3, 0x8l, puzzle | 0x7cl, partsUsed | 0x400l);
         if(!(puzzle & 0xecl))
           doSolve32(3, 0x8l, puzzle | 0xecl, partsUsed | 0x400l);
      }
      if (!(puzzle & 0xc004l)) {
         if(!(puzzle & 0x800c00cl))
           doSolve32(3, 0x8l, puzzle | 0x800c00cl, partsUsed | 0x400l);
         if(!(puzzle & 0x400c00cl))
           doSolve32(3, 0x8l, puzzle | 0x400c00cl, partsUsed | 0x400l);
         if(!(puzzle & 0x1c00cl))
           doSolve32(3, 0x8l, puzzle | 0x1c00cl, partsUsed | 0x400l);
         if(!(puzzle & 0xc00c004l))
           doSolve32(3, 0x8l, puzzle | 0xc00c004l, partsUsed | 0x400l);
         if(!(puzzle & 0xc01cl))
           doSolve32(3, 0x8l, puzzle | 0xc01cl, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x4804804l))
        doSolve32(3, 0x8l, puzzle | 0x4804804l, partsUsed | 0x400l);
      if(!(puzzle & 0x24824l))
        doSolve32(3, 0x8l, puzzle | 0x24824l, partsUsed | 0x400l);
      if(!(puzzle & 0x1801cl))
        doSolve32(3, 0x8l, puzzle | 0x1801cl, partsUsed | 0x400l);
    piece11at2at32:
      if (partsUsed & 0x800l) goto piece12at2at32;
      if (!(puzzle & 0xc004l)) {
         if(!(puzzle & 0xd804l))
           doSolve32(3, 0x8l, puzzle | 0xd804l, partsUsed | 0x800l);
         if(!(puzzle & 0xd00cl))
           doSolve32(3, 0x8l, puzzle | 0xd00cl, partsUsed | 0x800l);
         if(!(puzzle & 0xc80cl))
           doSolve32(3, 0x8l, puzzle | 0xc80cl, partsUsed | 0x800l);
         if(!(puzzle & 0x2c00cl))
           doSolve32(3, 0x8l, puzzle | 0x2c00cl, partsUsed | 0x800l);
         if(!(puzzle & 0x2c024l))
           doSolve32(3, 0x8l, puzzle | 0x2c024l, partsUsed | 0x800l);
         if(!(puzzle & 0xc02cl))
           doSolve32(3, 0x8l, puzzle | 0xc02cl, partsUsed | 0x800l);
         if(!(puzzle & 0xc04cl))
           doSolve32(3, 0x8l, puzzle | 0xc04cl, partsUsed | 0x800l);
         if(!(puzzle & 0x4c00cl))
           doSolve32(3, 0x8l, puzzle | 0x4c00cl, partsUsed | 0x800l);
         if(!(puzzle & 0x6c004l))
           doSolve32(3, 0x8l, puzzle | 0x6c004l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x64l)) {
         if(!(puzzle & 0x406cl))
           doSolve32(3, 0x8l, puzzle | 0x406cl, partsUsed | 0x800l);
         if(!(puzzle & 0x60064l))
           doSolve32(3, 0x8l, puzzle | 0x60064l, partsUsed | 0x800l);
         if(!(puzzle & 0x24064l))
           doSolve32(3, 0x8l, puzzle | 0x24064l, partsUsed | 0x800l);
         if(!(puzzle & 0x4006cl))
           doSolve32(3, 0x8l, puzzle | 0x4006cl, partsUsed | 0x800l);
         if(!(puzzle & 0x806cl))
           doSolve32(3, 0x8l, puzzle | 0x806cl, partsUsed | 0x800l);
         if(!(puzzle & 0x2006cl))
           doSolve32(3, 0x8l, puzzle | 0x2006cl, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x4804cl))
        doSolve32(3, 0x8l, puzzle | 0x4804cl, partsUsed | 0x800l);
      if(!(puzzle & 0x64024l))
        doSolve32(3, 0x8l, puzzle | 0x64024l, partsUsed | 0x800l);
      if(!(puzzle & 0x2402cl))
        doSolve32(3, 0x8l, puzzle | 0x2402cl, partsUsed | 0x800l);
    piece12at2at32:
    break;
   case 3:
      if (partsUsed & 0x2l) goto piece2at3at32;
      if(!(puzzle & 0x400c018l))
        doSolve32(4, 0x10l, puzzle | 0x400c018l, partsUsed | 0x2l);
      if(!(puzzle & 0x1209008l))
        doSolve32(4, 0x10l, puzzle | 0x1209008l, partsUsed | 0x2l);
      if(!(puzzle & 0x1009048l))
        doSolve32(4, 0x10l, puzzle | 0x1009048l, partsUsed | 0x2l);
    piece2at3at32:
      if (partsUsed & 0x4l) goto piece3at3at32;
      if (!(puzzle & 0x8008008l)) {
         if(!(puzzle & 0x8009208l))
           doSolve32(4, 0x10l, puzzle | 0x8009208l, partsUsed | 0x4l);
         if(!(puzzle & 0x49008008l))
           doSolve32(4, 0x10l, puzzle | 0x49008008l, partsUsed | 0x4l);
         if(!(puzzle & 0x1c008008l))
           doSolve32(4, 0x10l, puzzle | 0x1c008008l, partsUsed | 0x4l);
      }
    piece3at3at32:
      if (partsUsed & 0x8l) goto piece4at3at32;
      if(!(puzzle & 0x209208l))
        doSolve32(4, 0x10l, puzzle | 0x209208l, partsUsed | 0x8l);
      if(!(puzzle & 0x9008048l))
        doSolve32(4, 0x10l, puzzle | 0x9008048l, partsUsed | 0x8l);
      if(!(puzzle & 0xc008018l))
        doSolve32(4, 0x10l, puzzle | 0xc008018l, partsUsed | 0x8l);
    piece4at3at32:
      if (partsUsed & 0x40l) goto piece7at3at32;
      if(!(puzzle & 0x49208l))
        doSolve32(4, 0x10l, puzzle | 0x49208l, partsUsed | 0x40l);
    piece7at3at32:
      if (partsUsed & 0x80l) goto piece8at3at32;
      if(!(puzzle & 0x48008048l))
        doSolve32(4, 0x10l, puzzle | 0x48008048l, partsUsed | 0x80l);
      if(!(puzzle & 0x48040048l))
        doSolve32(4, 0x10l, puzzle | 0x48040048l, partsUsed | 0x80l);
      if(!(puzzle & 0x18010018l))
        doSolve32(4, 0x10l, puzzle | 0x18010018l, partsUsed | 0x80l);
      if(!(puzzle & 0x18008018l))
        doSolve32(4, 0x10l, puzzle | 0x18008018l, partsUsed | 0x80l);
    piece8at3at32:
      if (partsUsed & 0x100l) goto piece9at3at32;
      if(!(puzzle & 0xc808l))
        doSolve32(4, 0x10l, puzzle | 0xc808l, partsUsed | 0x100l);
      if(!(puzzle & 0xc048l))
        doSolve32(4, 0x10l, puzzle | 0xc048l, partsUsed | 0x100l);
      if(!(puzzle & 0x12018l))
        doSolve32(4, 0x10l, puzzle | 0x12018l, partsUsed | 0x100l);
      if(!(puzzle & 0x10058l))
        doSolve32(4, 0x10l, puzzle | 0x10058l, partsUsed | 0x100l);
      if(!(puzzle & 0xb008l))
        doSolve32(4, 0x10l, puzzle | 0xb008l, partsUsed | 0x100l);
      if(!(puzzle & 0x20068l))
        doSolve32(4, 0x10l, puzzle | 0x20068l, partsUsed | 0x100l);
      if(!(puzzle & 0x68008l))
        doSolve32(4, 0x10l, puzzle | 0x68008l, partsUsed | 0x100l);
      if(!(puzzle & 0x48018l))
        doSolve32(4, 0x10l, puzzle | 0x48018l, partsUsed | 0x100l);
      if(!(puzzle & 0x98008l))
        doSolve32(4, 0x10l, puzzle | 0x98008l, partsUsed | 0x100l);
      if(!(puzzle & 0xc0048l))
        doSolve32(4, 0x10l, puzzle | 0xc0048l, partsUsed | 0x100l);
      if(!(puzzle & 0x80c8l))
        doSolve32(4, 0x10l, puzzle | 0x80c8l, partsUsed | 0x100l);
      if(!(puzzle & 0x80098l))
        doSolve32(4, 0x10l, puzzle | 0x80098l, partsUsed | 0x100l);
    piece9at3at32:
      if (partsUsed & 0x200l) goto piece10at3at32;
      if (!(puzzle & 0x9008l)) {
         if(!(puzzle & 0x1009208l))
           doSolve32(4, 0x10l, puzzle | 0x1009208l, partsUsed | 0x200l);
         if(!(puzzle & 0x8009048l))
           doSolve32(4, 0x10l, puzzle | 0x8009048l, partsUsed | 0x200l);
         if(!(puzzle & 0x40049008l))
           doSolve32(4, 0x10l, puzzle | 0x40049008l, partsUsed | 0x200l);
         if(!(puzzle & 0x48009008l))
           doSolve32(4, 0x10l, puzzle | 0x48009008l, partsUsed | 0x200l);
         if(!(puzzle & 0x1049008l))
           doSolve32(4, 0x10l, puzzle | 0x1049008l, partsUsed | 0x200l);
      }
      if (!(puzzle & 0x8008008l)) {
         if(!(puzzle & 0x9048008l))
           doSolve32(4, 0x10l, puzzle | 0x9048008l, partsUsed | 0x200l);
         if(!(puzzle & 0x800c018l))
           doSolve32(4, 0x10l, puzzle | 0x800c018l, partsUsed | 0x200l);
         if(!(puzzle & 0x1800c008l))
           doSolve32(4, 0x10l, puzzle | 0x1800c008l, partsUsed | 0x200l);
         if(!(puzzle & 0xc018008l))
           doSolve32(4, 0x10l, puzzle | 0xc018008l, partsUsed | 0x200l);
      }
      if(!(puzzle & 0x401c008l))
        doSolve32(4, 0x10l, puzzle | 0x401c008l, partsUsed | 0x200l);
      if(!(puzzle & 0x1001c008l))
        doSolve32(4, 0x10l, puzzle | 0x1001c008l, partsUsed | 0x200l);
    piece10at3at32:
      if (partsUsed & 0x400l) goto piece11at3at32;
      if (!(puzzle & 0x48008l)) {
         if(!(puzzle & 0x8048048l))
           doSolve32(4, 0x10l, puzzle | 0x8048048l, partsUsed | 0x400l);
         if(!(puzzle & 0x48048008l))
           doSolve32(4, 0x10l, puzzle | 0x48048008l, partsUsed | 0x400l);
         if(!(puzzle & 0x40048048l))
           doSolve32(4, 0x10l, puzzle | 0x40048048l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x18008l)) {
         if(!(puzzle & 0x10018018l))
           doSolve32(4, 0x10l, puzzle | 0x10018018l, partsUsed | 0x400l);
         if(!(puzzle & 0x8018018l))
           doSolve32(4, 0x10l, puzzle | 0x8018018l, partsUsed | 0x400l);
         if(!(puzzle & 0x18018008l))
           doSolve32(4, 0x10l, puzzle | 0x18018008l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x9009008l))
        doSolve32(4, 0x10l, puzzle | 0x9009008l, partsUsed | 0x400l);
      if(!(puzzle & 0x49048l))
        doSolve32(4, 0x10l, puzzle | 0x49048l, partsUsed | 0x400l);
      if(!(puzzle & 0xf8l))
        doSolve32(4, 0x10l, puzzle | 0xf8l, partsUsed | 0x400l);
      if(!(puzzle & 0x1c018l))
        doSolve32(4, 0x10l, puzzle | 0x1c018l, partsUsed | 0x400l);
      if(!(puzzle & 0xc00c008l))
        doSolve32(4, 0x10l, puzzle | 0xc00c008l, partsUsed | 0x400l);
    piece11at3at32:
      if (partsUsed & 0x800l) goto piece12at3at32;
      if (!(puzzle & 0x9008l)) {
         if(!(puzzle & 0xd808l))
           doSolve32(4, 0x10l, puzzle | 0xd808l, partsUsed | 0x800l);
         if(!(puzzle & 0x1b008l))
           doSolve32(4, 0x10l, puzzle | 0x1b008l, partsUsed | 0x800l);
         if(!(puzzle & 0x19018l))
           doSolve32(4, 0x10l, puzzle | 0x19018l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x8018l)) {
         if(!(puzzle & 0x1a018l))
           doSolve32(4, 0x10l, puzzle | 0x1a018l, partsUsed | 0x800l);
         if(!(puzzle & 0x80d8l))
           doSolve32(4, 0x10l, puzzle | 0x80d8l, partsUsed | 0x800l);
         if(!(puzzle & 0x58018l))
           doSolve32(4, 0x10l, puzzle | 0x58018l, partsUsed | 0x800l);
         if(!(puzzle & 0x18058l))
           doSolve32(4, 0x10l, puzzle | 0x18058l, partsUsed | 0x800l);
         if(!(puzzle & 0x18098l))
           doSolve32(4, 0x10l, puzzle | 0x18098l, partsUsed | 0x800l);
         if(!(puzzle & 0x98018l))
           doSolve32(4, 0x10l, puzzle | 0x98018l, partsUsed | 0x800l);
         if(!(puzzle & 0x48058l))
           doSolve32(4, 0x10l, puzzle | 0x48058l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x60008l)) {
         if(!(puzzle & 0x6c008l))
           doSolve32(4, 0x10l, puzzle | 0x6c008l, partsUsed | 0x800l);
         if(!(puzzle & 0x68048l))
           doSolve32(4, 0x10l, puzzle | 0x68048l, partsUsed | 0x800l);
         if(!(puzzle & 0x60068l))
           doSolve32(4, 0x10l, puzzle | 0x60068l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x40048l)) {
         if(!(puzzle & 0x4c048l))
           doSolve32(4, 0x10l, puzzle | 0x4c048l, partsUsed | 0x800l);
         if(!(puzzle & 0x48068l))
           doSolve32(4, 0x10l, puzzle | 0x48068l, partsUsed | 0x800l);
         if(!(puzzle & 0xc00c8l))
           doSolve32(4, 0x10l, puzzle | 0xc00c8l, partsUsed | 0x800l);
         if(!(puzzle & 0x58048l))
           doSolve32(4, 0x10l, puzzle | 0x58048l, partsUsed | 0x800l);
         if(!(puzzle & 0x480c8l))
           doSolve32(4, 0x10l, puzzle | 0x480c8l, partsUsed | 0x800l);
         if(!(puzzle & 0x400d8l))
           doSolve32(4, 0x10l, puzzle | 0x400d8l, partsUsed | 0x800l);
         if(!(puzzle & 0xc8048l))
           doSolve32(4, 0x10l, puzzle | 0xc8048l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x98l)) {
         if(!(puzzle & 0x90098l))
           doSolve32(4, 0x10l, puzzle | 0x90098l, partsUsed | 0x800l);
         if(!(puzzle & 0x800d8l))
           doSolve32(4, 0x10l, puzzle | 0x800d8l, partsUsed | 0x800l);
         if(!(puzzle & 0x100d8l))
           doSolve32(4, 0x10l, puzzle | 0x100d8l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0xd8008l))
        doSolve32(4, 0x10l, puzzle | 0xd8008l, partsUsed | 0x800l);
    piece12at3at32:
    break;
   case 4:
      if (partsUsed & 0x2l) goto piece2at4at32;
      if(!(puzzle & 0xc018010l))
        doSolve32(5, 0x20l, puzzle | 0xc018010l, partsUsed | 0x2l);
      if(!(puzzle & 0x2412010l))
        doSolve32(5, 0x20l, puzzle | 0x2412010l, partsUsed | 0x2l);
      if(!(puzzle & 0x2012090l))
        doSolve32(5, 0x20l, puzzle | 0x2012090l, partsUsed | 0x2l);
    piece2at4at32:
      if (partsUsed & 0x4l) goto piece3at4at32;
      if (!(puzzle & 0x10010010l)) {
         if(!(puzzle & 0x10012410l))
           doSolve32(5, 0x20l, puzzle | 0x10012410l, partsUsed | 0x4l);
         if(!(puzzle & 0x92010010l))
           doSolve32(5, 0x20l, puzzle | 0x92010010l, partsUsed | 0x4l);
         if(!(puzzle & 0x1001c010l))
           doSolve32(5, 0x20l, puzzle | 0x1001c010l, partsUsed | 0x4l);
      }
    piece3at4at32:
      if (partsUsed & 0x8l) goto piece4at4at32;
      if(!(puzzle & 0x401c010l))
        doSolve32(5, 0x20l, puzzle | 0x401c010l, partsUsed | 0x8l);
      if(!(puzzle & 0x412410l))
        doSolve32(5, 0x20l, puzzle | 0x412410l, partsUsed | 0x8l);
      if(!(puzzle & 0x12010090l))
        doSolve32(5, 0x20l, puzzle | 0x12010090l, partsUsed | 0x8l);
    piece4at4at32:
      if (partsUsed & 0x40l) goto piece7at4at32;
      if(!(puzzle & 0x92410l))
        doSolve32(5, 0x20l, puzzle | 0x92410l, partsUsed | 0x40l);
    piece7at4at32:
      if (partsUsed & 0x80l) goto piece8at4at32;
      if(!(puzzle & 0x90010090l))
        doSolve32(5, 0x20l, puzzle | 0x90010090l, partsUsed | 0x80l);
      if(!(puzzle & 0x90080090l))
        doSolve32(5, 0x20l, puzzle | 0x90080090l, partsUsed | 0x80l);
    piece8at4at32:
      if (partsUsed & 0x100l) goto piece9at4at32;
      if(!(puzzle & 0x19010l))
        doSolve32(5, 0x20l, puzzle | 0x19010l, partsUsed | 0x100l);
      if(!(puzzle & 0x18090l))
        doSolve32(5, 0x20l, puzzle | 0x18090l, partsUsed | 0x100l);
      if(!(puzzle & 0x400d0l))
        doSolve32(5, 0x20l, puzzle | 0x400d0l, partsUsed | 0x100l);
      if(!(puzzle & 0xd0010l))
        doSolve32(5, 0x20l, puzzle | 0xd0010l, partsUsed | 0x100l);
    piece9at4at32:
      if (partsUsed & 0x200l) goto piece10at4at32;
      if (!(puzzle & 0x12010l)) {
         if(!(puzzle & 0x2012410l))
           doSolve32(5, 0x20l, puzzle | 0x2012410l, partsUsed | 0x200l);
         if(!(puzzle & 0x10012090l))
           doSolve32(5, 0x20l, puzzle | 0x10012090l, partsUsed | 0x200l);
         if(!(puzzle & 0x80092010l))
           doSolve32(5, 0x20l, puzzle | 0x80092010l, partsUsed | 0x200l);
         if(!(puzzle & 0x90012010l))
           doSolve32(5, 0x20l, puzzle | 0x90012010l, partsUsed | 0x200l);
         if(!(puzzle & 0x2092010l))
           doSolve32(5, 0x20l, puzzle | 0x2092010l, partsUsed | 0x200l);
      }
      if(!(puzzle & 0x12090010l))
        doSolve32(5, 0x20l, puzzle | 0x12090010l, partsUsed | 0x200l);
      if(!(puzzle & 0x801c010l))
        doSolve32(5, 0x20l, puzzle | 0x801c010l, partsUsed | 0x200l);
    piece10at4at32:
      if (partsUsed & 0x400l) goto piece11at4at32;
      if (!(puzzle & 0x10010010l)) {
         if(!(puzzle & 0x18018010l))
           doSolve32(5, 0x20l, puzzle | 0x18018010l, partsUsed | 0x400l);
         if(!(puzzle & 0x10090090l))
           doSolve32(5, 0x20l, puzzle | 0x10090090l, partsUsed | 0x400l);
         if(!(puzzle & 0x90090010l))
           doSolve32(5, 0x20l, puzzle | 0x90090010l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x12012010l))
        doSolve32(5, 0x20l, puzzle | 0x12012010l, partsUsed | 0x400l);
      if(!(puzzle & 0x92090l))
        doSolve32(5, 0x20l, puzzle | 0x92090l, partsUsed | 0x400l);
      if(!(puzzle & 0x80090090l))
        doSolve32(5, 0x20l, puzzle | 0x80090090l, partsUsed | 0x400l);
    piece11at4at32:
      if (partsUsed & 0x800l) goto piece12at4at32;
      if (!(puzzle & 0x18010l)) {
         if(!(puzzle & 0x1b010l))
           doSolve32(5, 0x20l, puzzle | 0x1b010l, partsUsed | 0x800l);
         if(!(puzzle & 0xd8010l))
           doSolve32(5, 0x20l, puzzle | 0xd8010l, partsUsed | 0x800l);
         if(!(puzzle & 0x98090l))
           doSolve32(5, 0x20l, puzzle | 0x98090l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x80090l)) {
         if(!(puzzle & 0xd0090l))
           doSolve32(5, 0x20l, puzzle | 0xd0090l, partsUsed | 0x800l);
         if(!(puzzle & 0xc00d0l))
           doSolve32(5, 0x20l, puzzle | 0xc00d0l, partsUsed | 0x800l);
         if(!(puzzle & 0x900d0l))
           doSolve32(5, 0x20l, puzzle | 0x900d0l, partsUsed | 0x800l);
      }
    piece12at4at32:
    break;
   case 5:
      if (partsUsed & 0x2l) goto piece2at5at32;
      if(!(puzzle & 0x4824020l))
        doSolve32(6, 0x40l, puzzle | 0x4824020l, partsUsed | 0x2l);
      if(!(puzzle & 0x800c0060l))
        doSolve32(6, 0x40l, puzzle | 0x800c0060l, partsUsed | 0x2l);
      if(!(puzzle & 0xc0060020l))
        doSolve32(6, 0x40l, puzzle | 0xc0060020l, partsUsed | 0x2l);
    piece2at5at32:
      if (partsUsed & 0x4l) goto piece3at5at32;
      if(!(puzzle & 0x400400e0l))
        doSolve32(6, 0x40l, puzzle | 0x400400e0l, partsUsed | 0x4l);
      if(!(puzzle & 0x20024820l))
        doSolve32(6, 0x40l, puzzle | 0x20024820l, partsUsed | 0x4l);
      if(!(puzzle & 0x200e0020l))
        doSolve32(6, 0x40l, puzzle | 0x200e0020l, partsUsed | 0x4l);
    piece3at5at32:
      if (partsUsed & 0x8l) goto piece4at5at32;
      if(!(puzzle & 0x824820l))
        doSolve32(6, 0x40l, puzzle | 0x824820l, partsUsed | 0x8l);
      if(!(puzzle & 0x800e0020l))
        doSolve32(6, 0x40l, puzzle | 0x800e0020l, partsUsed | 0x8l);
      if(!(puzzle & 0xc0040060l))
        doSolve32(6, 0x40l, puzzle | 0xc0040060l, partsUsed | 0x8l);
    piece4at5at32:
      if (partsUsed & 0x20l) goto piece6at5at32;
      if(!(puzzle & 0x24920l))
        doSolve32(6, 0x40l, puzzle | 0x24920l, partsUsed | 0x20l);
    piece6at5at32:
      if (partsUsed & 0x80l) goto piece8at5at32;
      if(!(puzzle & 0x60040060l))
        doSolve32(6, 0x40l, puzzle | 0x60040060l, partsUsed | 0x80l);
      if(!(puzzle & 0x60020060l))
        doSolve32(6, 0x40l, puzzle | 0x60020060l, partsUsed | 0x80l);
      if(!(puzzle & 0xe00a0l))
        doSolve32(6, 0x40l, puzzle | 0xe00a0l, partsUsed | 0x80l);
      if(!(puzzle & 0xa00e0l))
        doSolve32(6, 0x40l, puzzle | 0xa00e0l, partsUsed | 0x80l);
    piece8at5at32:
      if (partsUsed & 0x100l) goto piece9at5at32;
      if(!(puzzle & 0x48060l))
        doSolve32(6, 0x40l, puzzle | 0x48060l, partsUsed | 0x100l);
      if(!(puzzle & 0x2c020l))
        doSolve32(6, 0x40l, puzzle | 0x2c020l, partsUsed | 0x100l);
    piece9at5at32:
      if (partsUsed & 0x200l) goto piece10at5at32;
      if(!(puzzle & 0x4024820l))
        doSolve32(6, 0x40l, puzzle | 0x4024820l, partsUsed | 0x200l);
      if(!(puzzle & 0x400c0060l))
        doSolve32(6, 0x40l, puzzle | 0x400c0060l, partsUsed | 0x200l);
      if(!(puzzle & 0x400e0020l))
        doSolve32(6, 0x40l, puzzle | 0x400e0020l, partsUsed | 0x200l);
    piece10at5at32:
      if (partsUsed & 0x400l) goto piece11at5at32;
      if (!(puzzle & 0x20020020l)) {
         if(!(puzzle & 0x24024020l))
           doSolve32(6, 0x40l, puzzle | 0x24024020l, partsUsed | 0x400l);
         if(!(puzzle & 0x20060060l))
           doSolve32(6, 0x40l, puzzle | 0x20060060l, partsUsed | 0x400l);
         if(!(puzzle & 0x60060020l))
           doSolve32(6, 0x40l, puzzle | 0x60060020l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x40060l)) {
         if(!(puzzle & 0x40060060l))
           doSolve32(6, 0x40l, puzzle | 0x40060060l, partsUsed | 0x400l);
         if(!(puzzle & 0xe0060l))
           doSolve32(6, 0x40l, puzzle | 0xe0060l, partsUsed | 0x400l);
         if(!(puzzle & 0x600e0l))
           doSolve32(6, 0x40l, puzzle | 0x600e0l, partsUsed | 0x400l);
         if(!(puzzle & 0xc00e0l))
           doSolve32(6, 0x40l, puzzle | 0xc00e0l, partsUsed | 0x400l);
      }
    piece11at5at32:
      if (partsUsed & 0x800l) goto piece12at5at32;
      if (!(puzzle & 0x60020l)) {
         if(!(puzzle & 0x6c020l))
           doSolve32(6, 0x40l, puzzle | 0x6c020l, partsUsed | 0x800l);
         if(!(puzzle & 0x68060l))
           doSolve32(6, 0x40l, puzzle | 0x68060l, partsUsed | 0x800l);
         if(!(puzzle & 0x64060l))
           doSolve32(6, 0x40l, puzzle | 0x64060l, partsUsed | 0x800l);
      }
    piece12at5at32:
    break;
   case 6:
      if (partsUsed & 0x2l) goto piece2at6at32;
      if(!(puzzle & 0x9048040l))
        doSolve32(7, 0x80l, puzzle | 0x9048040l, partsUsed | 0x2l);
      if(!(puzzle & 0x200600c0l))
        doSolve32(7, 0x80l, puzzle | 0x200600c0l, partsUsed | 0x2l);
    piece2at6at32:
      if (partsUsed & 0x4l) goto piece3at6at32;
      if(!(puzzle & 0x40049040l))
        doSolve32(7, 0x80l, puzzle | 0x40049040l, partsUsed | 0x4l);
      if(!(puzzle & 0xe0040040l))
        doSolve32(7, 0x80l, puzzle | 0xe0040040l, partsUsed | 0x4l);
    piece3at6at32:
      if (partsUsed & 0x8l) goto piece4at6at32;
      if(!(puzzle & 0x1049040l))
        doSolve32(7, 0x80l, puzzle | 0x1049040l, partsUsed | 0x8l);
      if(!(puzzle & 0x600400c0l))
        doSolve32(7, 0x80l, puzzle | 0x600400c0l, partsUsed | 0x8l);
    piece4at6at32:
      if (partsUsed & 0x20l) goto piece6at6at32;
      if(!(puzzle & 0x49240l))
        doSolve32(7, 0x80l, puzzle | 0x49240l, partsUsed | 0x20l);
    piece6at6at32:
      if (partsUsed & 0x80l) goto piece8at6at32;
      if(!(puzzle & 0xc00800c0l))
        doSolve32(7, 0x80l, puzzle | 0xc00800c0l, partsUsed | 0x80l);
      if(!(puzzle & 0xc00400c0l))
        doSolve32(7, 0x80l, puzzle | 0xc00400c0l, partsUsed | 0x80l);
    piece8at6at32:
      if (partsUsed & 0x100l) goto piece9at6at32;
      if(!(puzzle & 0x58040l))
        doSolve32(7, 0x80l, puzzle | 0x58040l, partsUsed | 0x100l);
      if(!(puzzle & 0x64040l))
        doSolve32(7, 0x80l, puzzle | 0x64040l, partsUsed | 0x100l);
      if(!(puzzle & 0x900c0l))
        doSolve32(7, 0x80l, puzzle | 0x900c0l, partsUsed | 0x100l);
    piece9at6at32:
      if (partsUsed & 0x200l) goto piece10at6at32;
      if (!(puzzle & 0x40040040l)) {
         if(!(puzzle & 0x400600c0l))
           doSolve32(7, 0x80l, puzzle | 0x400600c0l, partsUsed | 0x200l);
         if(!(puzzle & 0xc0060040l))
           doSolve32(7, 0x80l, puzzle | 0xc0060040l, partsUsed | 0x200l);
         if(!(puzzle & 0x600c0040l))
           doSolve32(7, 0x80l, puzzle | 0x600c0040l, partsUsed | 0x200l);
      }
      if(!(puzzle & 0x8049040l))
        doSolve32(7, 0x80l, puzzle | 0x8049040l, partsUsed | 0x200l);
      if(!(puzzle & 0x200e0040l))
        doSolve32(7, 0x80l, puzzle | 0x200e0040l, partsUsed | 0x200l);
      if(!(puzzle & 0x800e0040l))
        doSolve32(7, 0x80l, puzzle | 0x800e0040l, partsUsed | 0x200l);
    piece10at6at32:
      if (partsUsed & 0x400l) goto piece11at6at32;
      if (!(puzzle & 0x40040040l)) {
         if(!(puzzle & 0x48048040l))
           doSolve32(7, 0x80l, puzzle | 0x48048040l, partsUsed | 0x400l);
         if(!(puzzle & 0x60060040l))
           doSolve32(7, 0x80l, puzzle | 0x60060040l, partsUsed | 0x400l);
         if(!(puzzle & 0x400c00c0l))
           doSolve32(7, 0x80l, puzzle | 0x400c00c0l, partsUsed | 0x400l);
         if(!(puzzle & 0xc00c0040l))
           doSolve32(7, 0x80l, puzzle | 0xc00c0040l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0xe00c0l))
        doSolve32(7, 0x80l, puzzle | 0xe00c0l, partsUsed | 0x400l);
      if(!(puzzle & 0x800c00c0l))
        doSolve32(7, 0x80l, puzzle | 0x800c00c0l, partsUsed | 0x400l);
    piece11at6at32:
      if (partsUsed & 0x800l) goto piece12at6at32;
      if (!(puzzle & 0x48040l)) {
         if(!(puzzle & 0x6c040l))
           doSolve32(7, 0x80l, puzzle | 0x6c040l, partsUsed | 0x800l);
         if(!(puzzle & 0xd8040l))
           doSolve32(7, 0x80l, puzzle | 0xd8040l, partsUsed | 0x800l);
         if(!(puzzle & 0xc80c0l))
           doSolve32(7, 0x80l, puzzle | 0xc80c0l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0xd00c0l))
        doSolve32(7, 0x80l, puzzle | 0xd00c0l, partsUsed | 0x800l);
    piece12at6at32:
    break;
   case 7:
      if (partsUsed & 0x2l) goto piece2at7at32;
      if(!(puzzle & 0x12090080l))
        doSolve32(8, 0x100l, puzzle | 0x12090080l, partsUsed | 0x2l);
      if(!(puzzle & 0x600c0080l))
        doSolve32(8, 0x100l, puzzle | 0x600c0080l, partsUsed | 0x2l);
    piece2at7at32:
      if (partsUsed & 0x4l) goto piece3at7at32;
      if(!(puzzle & 0x80092080l))
        doSolve32(8, 0x100l, puzzle | 0x80092080l, partsUsed | 0x4l);
      if(!(puzzle & 0x800e0080l))
        doSolve32(8, 0x100l, puzzle | 0x800e0080l, partsUsed | 0x4l);
    piece3at7at32:
      if (partsUsed & 0x8l) goto piece4at7at32;
      if(!(puzzle & 0x2092080l))
        doSolve32(8, 0x100l, puzzle | 0x2092080l, partsUsed | 0x8l);
      if(!(puzzle & 0x200e0080l))
        doSolve32(8, 0x100l, puzzle | 0x200e0080l, partsUsed | 0x8l);
    piece4at7at32:
      if (partsUsed & 0x20l) goto piece6at7at32;
      if(!(puzzle & 0x92480l))
        doSolve32(8, 0x100l, puzzle | 0x92480l, partsUsed | 0x20l);
    piece6at7at32:
      if (partsUsed & 0x100l) goto piece9at7at32;
      if(!(puzzle & 0xc8080l))
        doSolve32(8, 0x100l, puzzle | 0xc8080l, partsUsed | 0x100l);
    piece9at7at32:
      if (partsUsed & 0x200l) goto piece10at7at32;
      if(!(puzzle & 0x10092080l))
        doSolve32(8, 0x100l, puzzle | 0x10092080l, partsUsed | 0x200l);
      if(!(puzzle & 0x400e0080l))
        doSolve32(8, 0x100l, puzzle | 0x400e0080l, partsUsed | 0x200l);
    piece10at7at32:
      if (partsUsed & 0x400l) goto piece11at7at32;
      if(!(puzzle & 0x90090080l))
        doSolve32(8, 0x100l, puzzle | 0x90090080l, partsUsed | 0x400l);
      if(!(puzzle & 0xc00c0080l))
        doSolve32(8, 0x100l, puzzle | 0xc00c0080l, partsUsed | 0x400l);
    piece11at7at32:
      if (partsUsed & 0x800l) goto piece12at7at32;
      if(!(puzzle & 0xd8080l))
        doSolve32(8, 0x100l, puzzle | 0xd8080l, partsUsed | 0x800l);
    piece12at7at32:
    break;
   case 8:
      if (partsUsed & 0x2l) goto piece2at8at32;
      if(!(puzzle & 0x13300l))
        doSolve32(9, 0x200l, puzzle | 0x13300l, partsUsed | 0x2l);
      if(!(puzzle & 0x19900l))
        doSolve32(9, 0x200l, puzzle | 0x19900l, partsUsed | 0x2l);
    piece2at8at32:
      if (partsUsed & 0x4l) goto piece3at8at32;
      if(!(puzzle & 0x9700l))
        doSolve32(9, 0x200l, puzzle | 0x9700l, partsUsed | 0x4l);
      if(!(puzzle & 0x7900l))
        doSolve32(9, 0x200l, puzzle | 0x7900l, partsUsed | 0x4l);
    piece3at8at32:
      if (partsUsed & 0x8l) goto piece4at8at32;
      if(!(puzzle & 0x19300l))
        doSolve32(9, 0x200l, puzzle | 0x19300l, partsUsed | 0x8l);
      if(!(puzzle & 0x13900l))
        doSolve32(9, 0x200l, puzzle | 0x13900l, partsUsed | 0x8l);
    piece4at8at32:
      if (partsUsed & 0x10l) goto piece5at8at32;
      if (!(puzzle & 0x24900l)) {
         if(!(puzzle & 0x4124900l))
           doSolve32(9, 0x200l, puzzle | 0x4124900l, partsUsed | 0x10l);
         if(!(puzzle & 0x65900l))
           doSolve32(9, 0x200l, puzzle | 0x65900l, partsUsed | 0x10l);
         if(!(puzzle & 0x20824900l))
           doSolve32(9, 0x200l, puzzle | 0x20824900l, partsUsed | 0x10l);
         if(!(puzzle & 0x2cb00l))
           doSolve32(9, 0x200l, puzzle | 0x2cb00l, partsUsed | 0x10l);
      }
      if(!(puzzle & 0x4d300l))
        doSolve32(9, 0x200l, puzzle | 0x4d300l, partsUsed | 0x10l);
      if(!(puzzle & 0x24904100l))
        doSolve32(9, 0x200l, puzzle | 0x24904100l, partsUsed | 0x10l);
    piece5at8at32:
      if (partsUsed & 0x20l) goto piece6at8at32;
      if (!(puzzle & 0x24900l)) {
         if(!(puzzle & 0x20024900l))
           doSolve32(9, 0x200l, puzzle | 0x20024900l, partsUsed | 0x20l);
         if(!(puzzle & 0x24b00l))
           doSolve32(9, 0x200l, puzzle | 0x24b00l, partsUsed | 0x20l);
         if(!(puzzle & 0x124900l))
           doSolve32(9, 0x200l, puzzle | 0x124900l, partsUsed | 0x20l);
         if(!(puzzle & 0x64900l))
           doSolve32(9, 0x200l, puzzle | 0x64900l, partsUsed | 0x20l);
      }
      if(!(puzzle & 0x24900100l))
        doSolve32(9, 0x200l, puzzle | 0x24900100l, partsUsed | 0x20l);
      if(!(puzzle & 0x49300l))
        doSolve32(9, 0x200l, puzzle | 0x49300l, partsUsed | 0x20l);
    piece6at8at32:
      if (partsUsed & 0x40l) goto piece7at8at32;
      if (!(puzzle & 0x24900l)) {
         if(!(puzzle & 0x4024900l))
           doSolve32(9, 0x200l, puzzle | 0x4024900l, partsUsed | 0x40l);
         if(!(puzzle & 0x25900l))
           doSolve32(9, 0x200l, puzzle | 0x25900l, partsUsed | 0x40l);
         if(!(puzzle & 0x824900l))
           doSolve32(9, 0x200l, puzzle | 0x824900l, partsUsed | 0x40l);
         if(!(puzzle & 0x2c900l))
           doSolve32(9, 0x200l, puzzle | 0x2c900l, partsUsed | 0x40l);
      }
    piece7at8at32:
      if (partsUsed & 0x80l) goto piece8at8at32;
      if(!(puzzle & 0x4904100l))
        doSolve32(9, 0x200l, puzzle | 0x4904100l, partsUsed | 0x80l);
      if(!(puzzle & 0x4104900l))
        doSolve32(9, 0x200l, puzzle | 0x4104900l, partsUsed | 0x80l);
      if(!(puzzle & 0x2f00l))
        doSolve32(9, 0x200l, puzzle | 0x2f00l, partsUsed | 0x80l);
      if(!(puzzle & 0xcb00l))
        doSolve32(9, 0x200l, puzzle | 0xcb00l, partsUsed | 0x80l);
      if(!(puzzle & 0xd300l))
        doSolve32(9, 0x200l, puzzle | 0xd300l, partsUsed | 0x80l);
      if(!(puzzle & 0x3d00l))
        doSolve32(9, 0x200l, puzzle | 0x3d00l, partsUsed | 0x80l);
      if(!(puzzle & 0x700500l))
        doSolve32(9, 0x200l, puzzle | 0x700500l, partsUsed | 0x80l);
      if(!(puzzle & 0x500700l))
        doSolve32(9, 0x200l, puzzle | 0x500700l, partsUsed | 0x80l);
    piece8at8at32:
      if (partsUsed & 0x100l) goto piece9at8at32;
      if(!(puzzle & 0x1001300l))
        doSolve32(9, 0x200l, puzzle | 0x1001300l, partsUsed | 0x100l);
      if(!(puzzle & 0x1300100l))
        doSolve32(9, 0x200l, puzzle | 0x1300100l, partsUsed | 0x100l);
      if(!(puzzle & 0x200b00l))
        doSolve32(9, 0x200l, puzzle | 0x200b00l, partsUsed | 0x100l);
      if(!(puzzle & 0x900300l))
        doSolve32(9, 0x200l, puzzle | 0x900300l, partsUsed | 0x100l);
      if(!(puzzle & 0x1800900l))
        doSolve32(9, 0x200l, puzzle | 0x1800900l, partsUsed | 0x100l);
      if(!(puzzle & 0x101900l))
        doSolve32(9, 0x200l, puzzle | 0x101900l, partsUsed | 0x100l);
    piece9at8at32:
      if (partsUsed & 0x200l) goto piece10at8at32;
      if(!(puzzle & 0xb300l))
        doSolve32(9, 0x200l, puzzle | 0xb300l, partsUsed | 0x200l);
      if(!(puzzle & 0xb900l))
        doSolve32(9, 0x200l, puzzle | 0xb900l, partsUsed | 0x200l);
    piece10at8at32:
      if (partsUsed & 0x400l) goto piece11at8at32;
      if (!(puzzle & 0x800900l)) {
         if(!(puzzle & 0x4900900l))
           doSolve32(9, 0x200l, puzzle | 0x4900900l, partsUsed | 0x400l);
         if(!(puzzle & 0x4804900l))
           doSolve32(9, 0x200l, puzzle | 0x4804900l, partsUsed | 0x400l);
         if(!(puzzle & 0x904900l))
           doSolve32(9, 0x200l, puzzle | 0x904900l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x1300l)) {
         if(!(puzzle & 0x3700l))
           doSolve32(9, 0x200l, puzzle | 0x3700l, partsUsed | 0x400l);
         if(!(puzzle & 0x1f00l))
           doSolve32(9, 0x200l, puzzle | 0x1f00l, partsUsed | 0x400l);
         if(!(puzzle & 0x5b00l))
           doSolve32(9, 0x200l, puzzle | 0x5b00l, partsUsed | 0x400l);
         if(!(puzzle & 0x9b00l))
           doSolve32(9, 0x200l, puzzle | 0x9b00l, partsUsed | 0x400l);
         if(!(puzzle & 0x3b00l))
           doSolve32(9, 0x200l, puzzle | 0x3b00l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x200300l)) {
         if(!(puzzle & 0x700300l))
           doSolve32(9, 0x200l, puzzle | 0x700300l, partsUsed | 0x400l);
         if(!(puzzle & 0x300700l))
           doSolve32(9, 0x200l, puzzle | 0x300700l, partsUsed | 0x400l);
         if(!(puzzle & 0x600700l))
           doSolve32(9, 0x200l, puzzle | 0x600700l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0xd900l))
        doSolve32(9, 0x200l, puzzle | 0xd900l, partsUsed | 0x400l);
    piece11at8at32:
      if (partsUsed & 0x800l) goto piece12at8at32;
      if (!(puzzle & 0x100300l)) {
         if(!(puzzle & 0x101b00l))
           doSolve32(9, 0x200l, puzzle | 0x101b00l, partsUsed | 0x800l);
         if(!(puzzle & 0xb00300l))
           doSolve32(9, 0x200l, puzzle | 0xb00300l, partsUsed | 0x800l);
         if(!(puzzle & 0x300b00l))
           doSolve32(9, 0x200l, puzzle | 0x300b00l, partsUsed | 0x800l);
         if(!(puzzle & 0x301300l))
           doSolve32(9, 0x200l, puzzle | 0x301300l, partsUsed | 0x800l);
         if(!(puzzle & 0x1300300l))
           doSolve32(9, 0x200l, puzzle | 0x1300300l, partsUsed | 0x800l);
         if(!(puzzle & 0x900b00l))
           doSolve32(9, 0x200l, puzzle | 0x900b00l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x1001100l)) {
         if(!(puzzle & 0x1801900l))
           doSolve32(9, 0x200l, puzzle | 0x1801900l, partsUsed | 0x800l);
         if(!(puzzle & 0x1201300l))
           doSolve32(9, 0x200l, puzzle | 0x1201300l, partsUsed | 0x800l);
         if(!(puzzle & 0x1001b00l))
           doSolve32(9, 0x200l, puzzle | 0x1001b00l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x800900l)) {
         if(!(puzzle & 0xb00900l))
           doSolve32(9, 0x200l, puzzle | 0xb00900l, partsUsed | 0x800l);
         if(!(puzzle & 0x901900l))
           doSolve32(9, 0x200l, puzzle | 0x901900l, partsUsed | 0x800l);
         if(!(puzzle & 0x801b00l))
           doSolve32(9, 0x200l, puzzle | 0x801b00l, partsUsed | 0x800l);
         if(!(puzzle & 0x1900900l))
           doSolve32(9, 0x200l, puzzle | 0x1900900l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x201b00l))
        doSolve32(9, 0x200l, puzzle | 0x201b00l, partsUsed | 0x800l);
      if(!(puzzle & 0x1b00100l))
        doSolve32(9, 0x200l, puzzle | 0x1b00100l, partsUsed | 0x800l);
    piece12at8at32:
    break;
   case 9:
      if (partsUsed & 0x2l) goto piece2at9at32;
      if(!(puzzle & 0x5e00l))
        doSolve32(10, 0x400l, puzzle | 0x5e00l, partsUsed | 0x2l);
    piece2at9at32:
      if (partsUsed & 0x4l) goto piece3at9at32;
      if(!(puzzle & 0x1d200l))
        doSolve32(10, 0x400l, puzzle | 0x1d200l, partsUsed | 0x4l);
    piece3at9at32:
      if (partsUsed & 0x8l) goto piece4at9at32;
      if(!(puzzle & 0xd600l))
        doSolve32(10, 0x400l, puzzle | 0xd600l, partsUsed | 0x8l);
    piece4at9at32:
      if (partsUsed & 0x10l) goto piece5at9at32;
      if (!(puzzle & 0x49200l)) {
         if(!(puzzle & 0x69a00l))
           doSolve32(10, 0x400l, puzzle | 0x69a00l, partsUsed | 0x10l);
         if(!(puzzle & 0x8249200l))
           doSolve32(10, 0x400l, puzzle | 0x8249200l, partsUsed | 0x10l);
         if(!(puzzle & 0xcb200l))
           doSolve32(10, 0x400l, puzzle | 0xcb200l, partsUsed | 0x10l);
         if(!(puzzle & 0x41049200l))
           doSolve32(10, 0x400l, puzzle | 0x41049200l, partsUsed | 0x10l);
         if(!(puzzle & 0x59600l))
           doSolve32(10, 0x400l, puzzle | 0x59600l, partsUsed | 0x10l);
      }
      if(!(puzzle & 0x9a600l))
        doSolve32(10, 0x400l, puzzle | 0x9a600l, partsUsed | 0x10l);
      if(!(puzzle & 0x49208200l))
        doSolve32(10, 0x400l, puzzle | 0x49208200l, partsUsed | 0x10l);
    piece5at9at32:
      if (partsUsed & 0x20l) goto piece6at9at32;
      if (!(puzzle & 0x49200l)) {
         if(!(puzzle & 0x69200l))
           doSolve32(10, 0x400l, puzzle | 0x69200l, partsUsed | 0x20l);
         if(!(puzzle & 0x40049200l))
           doSolve32(10, 0x400l, puzzle | 0x40049200l, partsUsed | 0x20l);
         if(!(puzzle & 0x49600l))
           doSolve32(10, 0x400l, puzzle | 0x49600l, partsUsed | 0x20l);
         if(!(puzzle & 0x249200l))
           doSolve32(10, 0x400l, puzzle | 0x249200l, partsUsed | 0x20l);
         if(!(puzzle & 0xc9200l))
           doSolve32(10, 0x400l, puzzle | 0xc9200l, partsUsed | 0x20l);
      }
      if(!(puzzle & 0x49200200l))
        doSolve32(10, 0x400l, puzzle | 0x49200200l, partsUsed | 0x20l);
      if(!(puzzle & 0x92600l))
        doSolve32(10, 0x400l, puzzle | 0x92600l, partsUsed | 0x20l);
    piece6at9at32:
      if (partsUsed & 0x40l) goto piece7at9at32;
      if (!(puzzle & 0x49200l)) {
         if(!(puzzle & 0x49a00l))
           doSolve32(10, 0x400l, puzzle | 0x49a00l, partsUsed | 0x40l);
         if(!(puzzle & 0x4d200l))
           doSolve32(10, 0x400l, puzzle | 0x4d200l, partsUsed | 0x40l);
         if(!(puzzle & 0x8049200l))
           doSolve32(10, 0x400l, puzzle | 0x8049200l, partsUsed | 0x40l);
         if(!(puzzle & 0x4b200l))
           doSolve32(10, 0x400l, puzzle | 0x4b200l, partsUsed | 0x40l);
         if(!(puzzle & 0x1049200l))
           doSolve32(10, 0x400l, puzzle | 0x1049200l, partsUsed | 0x40l);
         if(!(puzzle & 0x59200l))
           doSolve32(10, 0x400l, puzzle | 0x59200l, partsUsed | 0x40l);
      }
    piece7at9at32:
      if (partsUsed & 0x80l) goto piece8at9at32;
      if(!(puzzle & 0x9208200l))
        doSolve32(10, 0x400l, puzzle | 0x9208200l, partsUsed | 0x80l);
      if(!(puzzle & 0x8209200l))
        doSolve32(10, 0x400l, puzzle | 0x8209200l, partsUsed | 0x80l);
      if(!(puzzle & 0x19600l))
        doSolve32(10, 0x400l, puzzle | 0x19600l, partsUsed | 0x80l);
      if(!(puzzle & 0x1a600l))
        doSolve32(10, 0x400l, puzzle | 0x1a600l, partsUsed | 0x80l);
    piece8at9at32:
      if (partsUsed & 0x100l) goto piece9at9at32;
      if(!(puzzle & 0x801a00l))
        doSolve32(10, 0x400l, puzzle | 0x801a00l, partsUsed | 0x100l);
      if(!(puzzle & 0x301200l))
        doSolve32(10, 0x400l, puzzle | 0x301200l, partsUsed | 0x100l);
      if(!(puzzle & 0x203200l))
        doSolve32(10, 0x400l, puzzle | 0x203200l, partsUsed | 0x100l);
      if(!(puzzle & 0x1a00200l))
        doSolve32(10, 0x400l, puzzle | 0x1a00200l, partsUsed | 0x100l);
      if(!(puzzle & 0x1200600l))
        doSolve32(10, 0x400l, puzzle | 0x1200600l, partsUsed | 0x100l);
      if(!(puzzle & 0x2600200l))
        doSolve32(10, 0x400l, puzzle | 0x2600200l, partsUsed | 0x100l);
      if(!(puzzle & 0x401600l))
        doSolve32(10, 0x400l, puzzle | 0x401600l, partsUsed | 0x100l);
      if(!(puzzle & 0x3001200l))
        doSolve32(10, 0x400l, puzzle | 0x3001200l, partsUsed | 0x100l);
      if(!(puzzle & 0x2002600l))
        doSolve32(10, 0x400l, puzzle | 0x2002600l, partsUsed | 0x100l);
    piece9at9at32:
      if (partsUsed & 0x200l) goto piece10at9at32;
      if (!(puzzle & 0x1a00l)) {
         if(!(puzzle & 0x13a00l))
           doSolve32(10, 0x400l, puzzle | 0x13a00l, partsUsed | 0x200l);
         if(!(puzzle & 0x7a00l))
           doSolve32(10, 0x400l, puzzle | 0x7a00l, partsUsed | 0x200l);
         if(!(puzzle & 0x9e00l))
           doSolve32(10, 0x400l, puzzle | 0x9e00l, partsUsed | 0x200l);
         if(!(puzzle & 0x19a00l))
           doSolve32(10, 0x400l, puzzle | 0x19a00l, partsUsed | 0x200l);
      }
      if(!(puzzle & 0xf200l))
        doSolve32(10, 0x400l, puzzle | 0xf200l, partsUsed | 0x200l);
    piece10at9at32:
      if (partsUsed & 0x400l) goto piece11at9at32;
      if (!(puzzle & 0x1001200l)) {
         if(!(puzzle & 0x9201200l))
           doSolve32(10, 0x400l, puzzle | 0x9201200l, partsUsed | 0x400l);
         if(!(puzzle & 0x9009200l))
           doSolve32(10, 0x400l, puzzle | 0x9009200l, partsUsed | 0x400l);
         if(!(puzzle & 0x1209200l))
           doSolve32(10, 0x400l, puzzle | 0x1209200l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x3200l)) {
         if(!(puzzle & 0xb600l))
           doSolve32(10, 0x400l, puzzle | 0xb600l, partsUsed | 0x400l);
         if(!(puzzle & 0x13600l))
           doSolve32(10, 0x400l, puzzle | 0x13600l, partsUsed | 0x400l);
         if(!(puzzle & 0x1b200l))
           doSolve32(10, 0x400l, puzzle | 0x1b200l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x3e00l))
        doSolve32(10, 0x400l, puzzle | 0x3e00l, partsUsed | 0x400l);
      if(!(puzzle & 0xda00l))
        doSolve32(10, 0x400l, puzzle | 0xda00l, partsUsed | 0x400l);
      if(!(puzzle & 0x700600l))
        doSolve32(10, 0x400l, puzzle | 0x700600l, partsUsed | 0x400l);
    piece11at9at32:
      if (partsUsed & 0x800l) goto piece12at9at32;
      if (!(puzzle & 0x1800200l)) {
         if(!(puzzle & 0x1b00200l))
           doSolve32(10, 0x400l, puzzle | 0x1b00200l, partsUsed | 0x800l);
         if(!(puzzle & 0x1a01200l))
           doSolve32(10, 0x400l, puzzle | 0x1a01200l, partsUsed | 0x800l);
         if(!(puzzle & 0x1801a00l))
           doSolve32(10, 0x400l, puzzle | 0x1801a00l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x201200l)) {
         if(!(puzzle & 0x1301200l))
           doSolve32(10, 0x400l, puzzle | 0x1301200l, partsUsed | 0x800l);
         if(!(puzzle & 0x1201a00l))
           doSolve32(10, 0x400l, puzzle | 0x1201a00l, partsUsed | 0x800l);
         if(!(puzzle & 0x203600l))
           doSolve32(10, 0x400l, puzzle | 0x203600l, partsUsed | 0x800l);
         if(!(puzzle & 0x1601200l))
           doSolve32(10, 0x400l, puzzle | 0x1601200l, partsUsed | 0x800l);
         if(!(puzzle & 0x601600l))
           doSolve32(10, 0x400l, puzzle | 0x601600l, partsUsed | 0x800l);
         if(!(puzzle & 0x1203200l))
           doSolve32(10, 0x400l, puzzle | 0x1203200l, partsUsed | 0x800l);
         if(!(puzzle & 0x3201200l))
           doSolve32(10, 0x400l, puzzle | 0x3201200l, partsUsed | 0x800l);
         if(!(puzzle & 0x1201600l))
           doSolve32(10, 0x400l, puzzle | 0x1201600l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x400600l)) {
         if(!(puzzle & 0x1600600l))
           doSolve32(10, 0x400l, puzzle | 0x1600600l, partsUsed | 0x800l);
         if(!(puzzle & 0x2402600l))
           doSolve32(10, 0x400l, puzzle | 0x2402600l, partsUsed | 0x800l);
         if(!(puzzle & 0x602600l))
           doSolve32(10, 0x400l, puzzle | 0x602600l, partsUsed | 0x800l);
         if(!(puzzle & 0x2600600l))
           doSolve32(10, 0x400l, puzzle | 0x2600600l, partsUsed | 0x800l);
         if(!(puzzle & 0x403600l))
           doSolve32(10, 0x400l, puzzle | 0x403600l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x3200l)) {
         if(!(puzzle & 0x3003200l))
           doSolve32(10, 0x400l, puzzle | 0x3003200l, partsUsed | 0x800l);
         if(!(puzzle & 0x2003600l))
           doSolve32(10, 0x400l, puzzle | 0x2003600l, partsUsed | 0x800l);
         if(!(puzzle & 0x1003600l))
           doSolve32(10, 0x400l, puzzle | 0x1003600l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x3600200l))
        doSolve32(10, 0x400l, puzzle | 0x3600200l, partsUsed | 0x800l);
    piece12at9at32:
    break;
   case 10:
      if (partsUsed & 0x2l) goto piece2at10at32;
      if(!(puzzle & 0xf400l))
        doSolve32(11, 0x800l, puzzle | 0xf400l, partsUsed | 0x2l);
    piece2at10at32:
      if (partsUsed & 0x4l) goto piece3at10at32;
      if(!(puzzle & 0x13c00l))
        doSolve32(11, 0x800l, puzzle | 0x13c00l, partsUsed | 0x4l);
    piece3at10at32:
      if (partsUsed & 0x8l) goto piece4at10at32;
      if(!(puzzle & 0x7c00l))
        doSolve32(11, 0x800l, puzzle | 0x7c00l, partsUsed | 0x8l);
    piece4at10at32:
      if (partsUsed & 0x10l) goto piece5at10at32;
      if (!(puzzle & 0x92400l)) {
         if(!(puzzle & 0xd3400l))
           doSolve32(11, 0x800l, puzzle | 0xd3400l, partsUsed | 0x10l);
         if(!(puzzle & 0x10492400l))
           doSolve32(11, 0x800l, puzzle | 0x10492400l, partsUsed | 0x10l);
         if(!(puzzle & 0x82092400l))
           doSolve32(11, 0x800l, puzzle | 0x82092400l, partsUsed | 0x10l);
      }
      if(!(puzzle & 0x92410400l))
        doSolve32(11, 0x800l, puzzle | 0x92410400l, partsUsed | 0x10l);
    piece5at10at32:
      if (partsUsed & 0x20l) goto piece6at10at32;
      if (!(puzzle & 0x92400l)) {
         if(!(puzzle & 0xd2400l))
           doSolve32(11, 0x800l, puzzle | 0xd2400l, partsUsed | 0x20l);
         if(!(puzzle & 0x80092400l))
           doSolve32(11, 0x800l, puzzle | 0x80092400l, partsUsed | 0x20l);
         if(!(puzzle & 0x492400l))
           doSolve32(11, 0x800l, puzzle | 0x492400l, partsUsed | 0x20l);
      }
      if(!(puzzle & 0x92400400l))
        doSolve32(11, 0x800l, puzzle | 0x92400400l, partsUsed | 0x20l);
    piece6at10at32:
      if (partsUsed & 0x40l) goto piece7at10at32;
      if (!(puzzle & 0x92400l)) {
         if(!(puzzle & 0x93400l))
           doSolve32(11, 0x800l, puzzle | 0x93400l, partsUsed | 0x40l);
         if(!(puzzle & 0x9a400l))
           doSolve32(11, 0x800l, puzzle | 0x9a400l, partsUsed | 0x40l);
         if(!(puzzle & 0x10092400l))
           doSolve32(11, 0x800l, puzzle | 0x10092400l, partsUsed | 0x40l);
         if(!(puzzle & 0x2092400l))
           doSolve32(11, 0x800l, puzzle | 0x2092400l, partsUsed | 0x40l);
      }
    piece7at10at32:
      if (partsUsed & 0x80l) goto piece8at10at32;
      if(!(puzzle & 0x12410400l))
        doSolve32(11, 0x800l, puzzle | 0x12410400l, partsUsed | 0x80l);
      if(!(puzzle & 0x10412400l))
        doSolve32(11, 0x800l, puzzle | 0x10412400l, partsUsed | 0x80l);
    piece8at10at32:
      if (partsUsed & 0x100l) goto piece9at10at32;
      if(!(puzzle & 0x3400400l))
        doSolve32(11, 0x800l, puzzle | 0x3400400l, partsUsed | 0x100l);
      if(!(puzzle & 0x1003400l))
        doSolve32(11, 0x800l, puzzle | 0x1003400l, partsUsed | 0x100l);
      if(!(puzzle & 0x602400l))
        doSolve32(11, 0x800l, puzzle | 0x602400l, partsUsed | 0x100l);
    piece9at10at32:
      if (partsUsed & 0x200l) goto piece10at10at32;
      if(!(puzzle & 0xbc00l))
        doSolve32(11, 0x800l, puzzle | 0xbc00l, partsUsed | 0x200l);
    piece10at10at32:
      if (partsUsed & 0x400l) goto piece11at10at32;
      if (!(puzzle & 0x12400l)) {
         if(!(puzzle & 0x1b400l))
           doSolve32(11, 0x800l, puzzle | 0x1b400l, partsUsed | 0x400l);
         if(!(puzzle & 0x12012400l))
           doSolve32(11, 0x800l, puzzle | 0x12012400l, partsUsed | 0x400l);
         if(!(puzzle & 0x2412400l))
           doSolve32(11, 0x800l, puzzle | 0x2412400l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x12402400l))
        doSolve32(11, 0x800l, puzzle | 0x12402400l, partsUsed | 0x400l);
    piece11at10at32:
      if (partsUsed & 0x800l) goto piece12at10at32;
      if (!(puzzle & 0x3000400l)) {
         if(!(puzzle & 0x3600400l))
           doSolve32(11, 0x800l, puzzle | 0x3600400l, partsUsed | 0x800l);
         if(!(puzzle & 0x3402400l))
           doSolve32(11, 0x800l, puzzle | 0x3402400l, partsUsed | 0x800l);
         if(!(puzzle & 0x3003400l))
           doSolve32(11, 0x800l, puzzle | 0x3003400l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x2602400l))
        doSolve32(11, 0x800l, puzzle | 0x2602400l, partsUsed | 0x800l);
      if(!(puzzle & 0x2403400l))
        doSolve32(11, 0x800l, puzzle | 0x2403400l, partsUsed | 0x800l);
    piece12at10at32:
    break;
   case 11:
      if (partsUsed & 0x2l) goto piece2at11at32;
      if(!(puzzle & 0x99800l))
        doSolve32(12, 0x1000l, puzzle | 0x99800l, partsUsed | 0x2l);
      if(!(puzzle & 0xcc800l))
        doSolve32(12, 0x1000l, puzzle | 0xcc800l, partsUsed | 0x2l);
    piece2at11at32:
      if (partsUsed & 0x4l) goto piece3at11at32;
      if(!(puzzle & 0x4b800l))
        doSolve32(12, 0x1000l, puzzle | 0x4b800l, partsUsed | 0x4l);
      if(!(puzzle & 0x3c800l))
        doSolve32(12, 0x1000l, puzzle | 0x3c800l, partsUsed | 0x4l);
    piece3at11at32:
      if (partsUsed & 0x8l) goto piece4at11at32;
      if(!(puzzle & 0xc9800l))
        doSolve32(12, 0x1000l, puzzle | 0xc9800l, partsUsed | 0x8l);
      if(!(puzzle & 0x9c800l))
        doSolve32(12, 0x1000l, puzzle | 0x9c800l, partsUsed | 0x8l);
    piece4at11at32:
      if (partsUsed & 0x10l) goto piece5at11at32;
      if(!(puzzle & 0x24920800l))
        doSolve32(12, 0x1000l, puzzle | 0x24920800l, partsUsed | 0x10l);
    piece5at11at32:
      if (partsUsed & 0x40l) goto piece7at11at32;
      if(!(puzzle & 0x24900800l))
        doSolve32(12, 0x1000l, puzzle | 0x24900800l, partsUsed | 0x40l);
    piece7at11at32:
      if (partsUsed & 0x80l) goto piece8at11at32;
      if(!(puzzle & 0x24820800l))
        doSolve32(12, 0x1000l, puzzle | 0x24820800l, partsUsed | 0x80l);
      if(!(puzzle & 0x20824800l))
        doSolve32(12, 0x1000l, puzzle | 0x20824800l, partsUsed | 0x80l);
      if(!(puzzle & 0x17800l))
        doSolve32(12, 0x1000l, puzzle | 0x17800l, partsUsed | 0x80l);
      if(!(puzzle & 0x65800l))
        doSolve32(12, 0x1000l, puzzle | 0x65800l, partsUsed | 0x80l);
      if(!(puzzle & 0x69800l))
        doSolve32(12, 0x1000l, puzzle | 0x69800l, partsUsed | 0x80l);
      if(!(puzzle & 0x1e800l))
        doSolve32(12, 0x1000l, puzzle | 0x1e800l, partsUsed | 0x80l);
      if(!(puzzle & 0x3802800l))
        doSolve32(12, 0x1000l, puzzle | 0x3802800l, partsUsed | 0x80l);
      if(!(puzzle & 0x2803800l))
        doSolve32(12, 0x1000l, puzzle | 0x2803800l, partsUsed | 0x80l);
    piece8at11at32:
      if (partsUsed & 0x100l) goto piece9at11at32;
      if(!(puzzle & 0x1201800l))
        doSolve32(12, 0x1000l, puzzle | 0x1201800l, partsUsed | 0x100l);
      if(!(puzzle & 0x1005800l))
        doSolve32(12, 0x1000l, puzzle | 0x1005800l, partsUsed | 0x100l);
      if(!(puzzle & 0xb00800l))
        doSolve32(12, 0x1000l, puzzle | 0xb00800l, partsUsed | 0x100l);
      if(!(puzzle & 0x9800800l))
        doSolve32(12, 0x1000l, puzzle | 0x9800800l, partsUsed | 0x100l);
      if(!(puzzle & 0x4801800l))
        doSolve32(12, 0x1000l, puzzle | 0x4801800l, partsUsed | 0x100l);
      if(!(puzzle & 0xc004800l))
        doSolve32(12, 0x1000l, puzzle | 0xc004800l, partsUsed | 0x100l);
      if(!(puzzle & 0x80c800l))
        doSolve32(12, 0x1000l, puzzle | 0x80c800l, partsUsed | 0x100l);
      if(!(puzzle & 0x8009800l))
        doSolve32(12, 0x1000l, puzzle | 0x8009800l, partsUsed | 0x100l);
    piece9at11at32:
      if (partsUsed & 0x200l) goto piece10at11at32;
      if(!(puzzle & 0x59800l))
        doSolve32(12, 0x1000l, puzzle | 0x59800l, partsUsed | 0x200l);
      if(!(puzzle & 0x5c800l))
        doSolve32(12, 0x1000l, puzzle | 0x5c800l, partsUsed | 0x200l);
    piece10at11at32:
      if (partsUsed & 0x400l) goto piece11at11at32;
      if (!(puzzle & 0x4004800l)) {
         if(!(puzzle & 0x4904800l))
           doSolve32(12, 0x1000l, puzzle | 0x4904800l, partsUsed | 0x400l);
         if(!(puzzle & 0x24804800l))
           doSolve32(12, 0x1000l, puzzle | 0x24804800l, partsUsed | 0x400l);
         if(!(puzzle & 0x24024800l))
           doSolve32(12, 0x1000l, puzzle | 0x24024800l, partsUsed | 0x400l);
         if(!(puzzle & 0x4824800l))
           doSolve32(12, 0x1000l, puzzle | 0x4824800l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x9800l)) {
         if(!(puzzle & 0x1b800l))
           doSolve32(12, 0x1000l, puzzle | 0x1b800l, partsUsed | 0x400l);
         if(!(puzzle & 0xf800l))
           doSolve32(12, 0x1000l, puzzle | 0xf800l, partsUsed | 0x400l);
         if(!(puzzle & 0x2d800l))
           doSolve32(12, 0x1000l, puzzle | 0x2d800l, partsUsed | 0x400l);
         if(!(puzzle & 0x4d800l))
           doSolve32(12, 0x1000l, puzzle | 0x4d800l, partsUsed | 0x400l);
         if(!(puzzle & 0x1d800l))
           doSolve32(12, 0x1000l, puzzle | 0x1d800l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x1001800l)) {
         if(!(puzzle & 0x3801800l))
           doSolve32(12, 0x1000l, puzzle | 0x3801800l, partsUsed | 0x400l);
         if(!(puzzle & 0x1803800l))
           doSolve32(12, 0x1000l, puzzle | 0x1803800l, partsUsed | 0x400l);
         if(!(puzzle & 0x3003800l))
           doSolve32(12, 0x1000l, puzzle | 0x3003800l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x6c800l))
        doSolve32(12, 0x1000l, puzzle | 0x6c800l, partsUsed | 0x400l);
    piece11at11at32:
      if (partsUsed & 0x800l) goto piece12at11at32;
      if (!(puzzle & 0x1800800l)) {
         if(!(puzzle & 0x1b00800l))
           doSolve32(12, 0x1000l, puzzle | 0x1b00800l, partsUsed | 0x800l);
         if(!(puzzle & 0x1a01800l))
           doSolve32(12, 0x1000l, puzzle | 0x1a01800l, partsUsed | 0x800l);
         if(!(puzzle & 0x1901800l))
           doSolve32(12, 0x1000l, puzzle | 0x1901800l, partsUsed | 0x800l);
         if(!(puzzle & 0x5801800l))
           doSolve32(12, 0x1000l, puzzle | 0x5801800l, partsUsed | 0x800l);
         if(!(puzzle & 0x5804800l))
           doSolve32(12, 0x1000l, puzzle | 0x5804800l, partsUsed | 0x800l);
         if(!(puzzle & 0x1805800l))
           doSolve32(12, 0x1000l, puzzle | 0x1805800l, partsUsed | 0x800l);
         if(!(puzzle & 0x1809800l))
           doSolve32(12, 0x1000l, puzzle | 0x1809800l, partsUsed | 0x800l);
         if(!(puzzle & 0x9801800l))
           doSolve32(12, 0x1000l, puzzle | 0x9801800l, partsUsed | 0x800l);
         if(!(puzzle & 0xd800800l))
           doSolve32(12, 0x1000l, puzzle | 0xd800800l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0xc800l)) {
         if(!(puzzle & 0x80d800l))
           doSolve32(12, 0x1000l, puzzle | 0x80d800l, partsUsed | 0x800l);
         if(!(puzzle & 0xc00c800l))
           doSolve32(12, 0x1000l, puzzle | 0xc00c800l, partsUsed | 0x800l);
         if(!(puzzle & 0x480c800l))
           doSolve32(12, 0x1000l, puzzle | 0x480c800l, partsUsed | 0x800l);
         if(!(puzzle & 0x800d800l))
           doSolve32(12, 0x1000l, puzzle | 0x800d800l, partsUsed | 0x800l);
         if(!(puzzle & 0x100d800l))
           doSolve32(12, 0x1000l, puzzle | 0x100d800l, partsUsed | 0x800l);
         if(!(puzzle & 0x400d800l))
           doSolve32(12, 0x1000l, puzzle | 0x400d800l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x9009800l))
        doSolve32(12, 0x1000l, puzzle | 0x9009800l, partsUsed | 0x800l);
      if(!(puzzle & 0xc804800l))
        doSolve32(12, 0x1000l, puzzle | 0xc804800l, partsUsed | 0x800l);
      if(!(puzzle & 0x4805800l))
        doSolve32(12, 0x1000l, puzzle | 0x4805800l, partsUsed | 0x800l);
    piece12at11at32:
    break;
   case 12:
      if (partsUsed & 0x2l) goto piece2at12at32;
      if(!(puzzle & 0x2f000l))
        doSolve32(13, 0x2000l, puzzle | 0x2f000l, partsUsed | 0x2l);
    piece2at12at32:
      if (partsUsed & 0x4l) goto piece3at12at32;
      if(!(puzzle & 0xe9000l))
        doSolve32(13, 0x2000l, puzzle | 0xe9000l, partsUsed | 0x4l);
    piece3at12at32:
      if (partsUsed & 0x8l) goto piece4at12at32;
      if(!(puzzle & 0x6b000l))
        doSolve32(13, 0x2000l, puzzle | 0x6b000l, partsUsed | 0x8l);
    piece4at12at32:
      if (partsUsed & 0x10l) goto piece5at12at32;
      if(!(puzzle & 0x49241000l))
        doSolve32(13, 0x2000l, puzzle | 0x49241000l, partsUsed | 0x10l);
    piece5at12at32:
      if (partsUsed & 0x40l) goto piece7at12at32;
      if(!(puzzle & 0x49201000l))
        doSolve32(13, 0x2000l, puzzle | 0x49201000l, partsUsed | 0x40l);
    piece7at12at32:
      if (partsUsed & 0x80l) goto piece8at12at32;
      if(!(puzzle & 0x49041000l))
        doSolve32(13, 0x2000l, puzzle | 0x49041000l, partsUsed | 0x80l);
      if(!(puzzle & 0x41049000l))
        doSolve32(13, 0x2000l, puzzle | 0x41049000l, partsUsed | 0x80l);
      if(!(puzzle & 0xcb000l))
        doSolve32(13, 0x2000l, puzzle | 0xcb000l, partsUsed | 0x80l);
      if(!(puzzle & 0xd3000l))
        doSolve32(13, 0x2000l, puzzle | 0xd3000l, partsUsed | 0x80l);
    piece8at12at32:
      if (partsUsed & 0x100l) goto piece9at12at32;
      if(!(puzzle & 0x1901000l))
        doSolve32(13, 0x2000l, puzzle | 0x1901000l, partsUsed | 0x100l);
      if(!(puzzle & 0x1809000l))
        doSolve32(13, 0x2000l, puzzle | 0x1809000l, partsUsed | 0x100l);
      if(!(puzzle & 0x2403000l))
        doSolve32(13, 0x2000l, puzzle | 0x2403000l, partsUsed | 0x100l);
      if(!(puzzle & 0x200b000l))
        doSolve32(13, 0x2000l, puzzle | 0x200b000l, partsUsed | 0x100l);
      if(!(puzzle & 0x1601000l))
        doSolve32(13, 0x2000l, puzzle | 0x1601000l, partsUsed | 0x100l);
      if(!(puzzle & 0x400d000l))
        doSolve32(13, 0x2000l, puzzle | 0x400d000l, partsUsed | 0x100l);
      if(!(puzzle & 0xd001000l))
        doSolve32(13, 0x2000l, puzzle | 0xd001000l, partsUsed | 0x100l);
      if(!(puzzle & 0x9003000l))
        doSolve32(13, 0x2000l, puzzle | 0x9003000l, partsUsed | 0x100l);
      if(!(puzzle & 0x13001000l))
        doSolve32(13, 0x2000l, puzzle | 0x13001000l, partsUsed | 0x100l);
      if(!(puzzle & 0x18009000l))
        doSolve32(13, 0x2000l, puzzle | 0x18009000l, partsUsed | 0x100l);
      if(!(puzzle & 0x1019000l))
        doSolve32(13, 0x2000l, puzzle | 0x1019000l, partsUsed | 0x100l);
      if(!(puzzle & 0x10013000l))
        doSolve32(13, 0x2000l, puzzle | 0x10013000l, partsUsed | 0x100l);
    piece9at12at32:
      if (partsUsed & 0x200l) goto piece10at12at32;
      if (!(puzzle & 0xd000l)) {
         if(!(puzzle & 0x9d000l))
           doSolve32(13, 0x2000l, puzzle | 0x9d000l, partsUsed | 0x200l);
         if(!(puzzle & 0x3d000l))
           doSolve32(13, 0x2000l, puzzle | 0x3d000l, partsUsed | 0x200l);
         if(!(puzzle & 0x4f000l))
           doSolve32(13, 0x2000l, puzzle | 0x4f000l, partsUsed | 0x200l);
         if(!(puzzle & 0xcd000l))
           doSolve32(13, 0x2000l, puzzle | 0xcd000l, partsUsed | 0x200l);
      }
      if(!(puzzle & 0x79000l))
        doSolve32(13, 0x2000l, puzzle | 0x79000l, partsUsed | 0x200l);
    piece10at12at32:
      if (partsUsed & 0x400l) goto piece11at12at32;
      if (!(puzzle & 0x8009000l)) {
         if(!(puzzle & 0x9209000l))
           doSolve32(13, 0x2000l, puzzle | 0x9209000l, partsUsed | 0x400l);
         if(!(puzzle & 0x49009000l))
           doSolve32(13, 0x2000l, puzzle | 0x49009000l, partsUsed | 0x400l);
         if(!(puzzle & 0x48049000l))
           doSolve32(13, 0x2000l, puzzle | 0x48049000l, partsUsed | 0x400l);
         if(!(puzzle & 0x9049000l))
           doSolve32(13, 0x2000l, puzzle | 0x9049000l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x19000l)) {
         if(!(puzzle & 0x5b000l))
           doSolve32(13, 0x2000l, puzzle | 0x5b000l, partsUsed | 0x400l);
         if(!(puzzle & 0x9b000l))
           doSolve32(13, 0x2000l, puzzle | 0x9b000l, partsUsed | 0x400l);
         if(!(puzzle & 0xd9000l))
           doSolve32(13, 0x2000l, puzzle | 0xd9000l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x1f000l))
        doSolve32(13, 0x2000l, puzzle | 0x1f000l, partsUsed | 0x400l);
      if(!(puzzle & 0x6d000l))
        doSolve32(13, 0x2000l, puzzle | 0x6d000l, partsUsed | 0x400l);
      if(!(puzzle & 0x3803000l))
        doSolve32(13, 0x2000l, puzzle | 0x3803000l, partsUsed | 0x400l);
    piece11at12at32:
      if (partsUsed & 0x800l) goto piece12at12at32;
      if (!(puzzle & 0x1201000l)) {
         if(!(puzzle & 0x1b01000l))
           doSolve32(13, 0x2000l, puzzle | 0x1b01000l, partsUsed | 0x800l);
         if(!(puzzle & 0x3601000l))
           doSolve32(13, 0x2000l, puzzle | 0x3601000l, partsUsed | 0x800l);
         if(!(puzzle & 0x3203000l))
           doSolve32(13, 0x2000l, puzzle | 0x3203000l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x1003000l)) {
         if(!(puzzle & 0x3403000l))
           doSolve32(13, 0x2000l, puzzle | 0x3403000l, partsUsed | 0x800l);
         if(!(puzzle & 0x101b000l))
           doSolve32(13, 0x2000l, puzzle | 0x101b000l, partsUsed | 0x800l);
         if(!(puzzle & 0xb003000l))
           doSolve32(13, 0x2000l, puzzle | 0xb003000l, partsUsed | 0x800l);
         if(!(puzzle & 0x300b000l))
           doSolve32(13, 0x2000l, puzzle | 0x300b000l, partsUsed | 0x800l);
         if(!(puzzle & 0x3013000l))
           doSolve32(13, 0x2000l, puzzle | 0x3013000l, partsUsed | 0x800l);
         if(!(puzzle & 0x13003000l))
           doSolve32(13, 0x2000l, puzzle | 0x13003000l, partsUsed | 0x800l);
         if(!(puzzle & 0x900b000l))
           doSolve32(13, 0x2000l, puzzle | 0x900b000l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0xc001000l)) {
         if(!(puzzle & 0xd801000l))
           doSolve32(13, 0x2000l, puzzle | 0xd801000l, partsUsed | 0x800l);
         if(!(puzzle & 0xd009000l))
           doSolve32(13, 0x2000l, puzzle | 0xd009000l, partsUsed | 0x800l);
         if(!(puzzle & 0xc00d000l))
           doSolve32(13, 0x2000l, puzzle | 0xc00d000l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x8009000l)) {
         if(!(puzzle & 0x9809000l))
           doSolve32(13, 0x2000l, puzzle | 0x9809000l, partsUsed | 0x800l);
         if(!(puzzle & 0x900d000l))
           doSolve32(13, 0x2000l, puzzle | 0x900d000l, partsUsed | 0x800l);
         if(!(puzzle & 0x18019000l))
           doSolve32(13, 0x2000l, puzzle | 0x18019000l, partsUsed | 0x800l);
         if(!(puzzle & 0xb009000l))
           doSolve32(13, 0x2000l, puzzle | 0xb009000l, partsUsed | 0x800l);
         if(!(puzzle & 0x9019000l))
           doSolve32(13, 0x2000l, puzzle | 0x9019000l, partsUsed | 0x800l);
         if(!(puzzle & 0x801b000l))
           doSolve32(13, 0x2000l, puzzle | 0x801b000l, partsUsed | 0x800l);
         if(!(puzzle & 0x19009000l))
           doSolve32(13, 0x2000l, puzzle | 0x19009000l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x13000l)) {
         if(!(puzzle & 0x12013000l))
           doSolve32(13, 0x2000l, puzzle | 0x12013000l, partsUsed | 0x800l);
         if(!(puzzle & 0x1001b000l))
           doSolve32(13, 0x2000l, puzzle | 0x1001b000l, partsUsed | 0x800l);
         if(!(puzzle & 0x201b000l))
           doSolve32(13, 0x2000l, puzzle | 0x201b000l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x1b001000l))
        doSolve32(13, 0x2000l, puzzle | 0x1b001000l, partsUsed | 0x800l);
    piece12at12at32:
    break;
   case 13:
      if (partsUsed & 0x2l) goto piece2at13at32;
      if(!(puzzle & 0x7a000l))
        doSolve32(14, 0x4000l, puzzle | 0x7a000l, partsUsed | 0x2l);
    piece2at13at32:
      if (partsUsed & 0x4l) goto piece3at13at32;
      if(!(puzzle & 0x9e000l))
        doSolve32(14, 0x4000l, puzzle | 0x9e000l, partsUsed | 0x4l);
    piece3at13at32:
      if (partsUsed & 0x8l) goto piece4at13at32;
      if(!(puzzle & 0x3e000l))
        doSolve32(14, 0x4000l, puzzle | 0x3e000l, partsUsed | 0x8l);
    piece4at13at32:
      if (partsUsed & 0x10l) goto piece5at13at32;
      if(!(puzzle & 0x92482000l))
        doSolve32(14, 0x4000l, puzzle | 0x92482000l, partsUsed | 0x10l);
    piece5at13at32:
      if (partsUsed & 0x40l) goto piece7at13at32;
      if(!(puzzle & 0x92402000l))
        doSolve32(14, 0x4000l, puzzle | 0x92402000l, partsUsed | 0x40l);
    piece7at13at32:
      if (partsUsed & 0x80l) goto piece8at13at32;
      if(!(puzzle & 0x92082000l))
        doSolve32(14, 0x4000l, puzzle | 0x92082000l, partsUsed | 0x80l);
      if(!(puzzle & 0x82092000l))
        doSolve32(14, 0x4000l, puzzle | 0x82092000l, partsUsed | 0x80l);
    piece8at13at32:
      if (partsUsed & 0x100l) goto piece9at13at32;
      if(!(puzzle & 0x3202000l))
        doSolve32(14, 0x4000l, puzzle | 0x3202000l, partsUsed | 0x100l);
      if(!(puzzle & 0x3012000l))
        doSolve32(14, 0x4000l, puzzle | 0x3012000l, partsUsed | 0x100l);
      if(!(puzzle & 0x801a000l))
        doSolve32(14, 0x4000l, puzzle | 0x801a000l, partsUsed | 0x100l);
      if(!(puzzle & 0x1a002000l))
        doSolve32(14, 0x4000l, puzzle | 0x1a002000l, partsUsed | 0x100l);
    piece9at13at32:
      if (partsUsed & 0x200l) goto piece10at13at32;
      if(!(puzzle & 0x5e000l))
        doSolve32(14, 0x4000l, puzzle | 0x5e000l, partsUsed | 0x200l);
    piece10at13at32:
      if (partsUsed & 0x400l) goto piece11at13at32;
      if (!(puzzle & 0x10012000l)) {
         if(!(puzzle & 0x12412000l))
           doSolve32(14, 0x4000l, puzzle | 0x12412000l, partsUsed | 0x400l);
         if(!(puzzle & 0x92012000l))
           doSolve32(14, 0x4000l, puzzle | 0x92012000l, partsUsed | 0x400l);
         if(!(puzzle & 0x90092000l))
           doSolve32(14, 0x4000l, puzzle | 0x90092000l, partsUsed | 0x400l);
         if(!(puzzle & 0x12092000l))
           doSolve32(14, 0x4000l, puzzle | 0x12092000l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0xda000l))
        doSolve32(14, 0x4000l, puzzle | 0xda000l, partsUsed | 0x400l);
    piece11at13at32:
      if (partsUsed & 0x800l) goto piece12at13at32;
      if (!(puzzle & 0x3002000l)) {
         if(!(puzzle & 0x3602000l))
           doSolve32(14, 0x4000l, puzzle | 0x3602000l, partsUsed | 0x800l);
         if(!(puzzle & 0x1b002000l))
           doSolve32(14, 0x4000l, puzzle | 0x1b002000l, partsUsed | 0x800l);
         if(!(puzzle & 0x13012000l))
           doSolve32(14, 0x4000l, puzzle | 0x13012000l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x10012000l)) {
         if(!(puzzle & 0x1a012000l))
           doSolve32(14, 0x4000l, puzzle | 0x1a012000l, partsUsed | 0x800l);
         if(!(puzzle & 0x1801a000l))
           doSolve32(14, 0x4000l, puzzle | 0x1801a000l, partsUsed | 0x800l);
         if(!(puzzle & 0x1201a000l))
           doSolve32(14, 0x4000l, puzzle | 0x1201a000l, partsUsed | 0x800l);
      }
    piece12at13at32:
    break;
   case 14:
      if (partsUsed & 0x40l) goto piece7at14at32;
      if(!(puzzle & 0x24904000l))
        doSolve32(15, 0x8000l, puzzle | 0x24904000l, partsUsed | 0x40l);
    piece7at14at32:
      if (partsUsed & 0x80l) goto piece8at14at32;
      if(!(puzzle & 0xbc000l))
        doSolve32(15, 0x8000l, puzzle | 0xbc000l, partsUsed | 0x80l);
      if(!(puzzle & 0xf4000l))
        doSolve32(15, 0x8000l, puzzle | 0xf4000l, partsUsed | 0x80l);
      if(!(puzzle & 0x1c014000l))
        doSolve32(15, 0x8000l, puzzle | 0x1c014000l, partsUsed | 0x80l);
      if(!(puzzle & 0x1401c000l))
        doSolve32(15, 0x8000l, puzzle | 0x1401c000l, partsUsed | 0x80l);
    piece8at14at32:
      if (partsUsed & 0x100l) goto piece9at14at32;
      if(!(puzzle & 0x900c000l))
        doSolve32(15, 0x8000l, puzzle | 0x900c000l, partsUsed | 0x100l);
      if(!(puzzle & 0x802c000l))
        doSolve32(15, 0x8000l, puzzle | 0x802c000l, partsUsed | 0x100l);
      if(!(puzzle & 0x5804000l))
        doSolve32(15, 0x8000l, puzzle | 0x5804000l, partsUsed | 0x100l);
      if(!(puzzle & 0x4c004000l))
        doSolve32(15, 0x8000l, puzzle | 0x4c004000l, partsUsed | 0x100l);
      if(!(puzzle & 0x2400c000l))
        doSolve32(15, 0x8000l, puzzle | 0x2400c000l, partsUsed | 0x100l);
      if(!(puzzle & 0x60024000l))
        doSolve32(15, 0x8000l, puzzle | 0x60024000l, partsUsed | 0x100l);
      if(!(puzzle & 0x4064000l))
        doSolve32(15, 0x8000l, puzzle | 0x4064000l, partsUsed | 0x100l);
      if(!(puzzle & 0x4004c000l))
        doSolve32(15, 0x8000l, puzzle | 0x4004c000l, partsUsed | 0x100l);
    piece9at14at32:
      if (partsUsed & 0x400l) goto piece11at14at32;
      if (!(puzzle & 0x4c000l)) {
         if(!(puzzle & 0xdc000l))
           doSolve32(15, 0x8000l, puzzle | 0xdc000l, partsUsed | 0x400l);
         if(!(puzzle & 0x7c000l))
           doSolve32(15, 0x8000l, puzzle | 0x7c000l, partsUsed | 0x400l);
         if(!(puzzle & 0xec000l))
           doSolve32(15, 0x8000l, puzzle | 0xec000l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x800c000l)) {
         if(!(puzzle & 0x1c00c000l))
           doSolve32(15, 0x8000l, puzzle | 0x1c00c000l, partsUsed | 0x400l);
         if(!(puzzle & 0xc01c000l))
           doSolve32(15, 0x8000l, puzzle | 0xc01c000l, partsUsed | 0x400l);
         if(!(puzzle & 0x1801c000l))
           doSolve32(15, 0x8000l, puzzle | 0x1801c000l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x24824000l))
        doSolve32(15, 0x8000l, puzzle | 0x24824000l, partsUsed | 0x400l);
    piece11at14at32:
      if (partsUsed & 0x800l) goto piece12at14at32;
      if (!(puzzle & 0xc004000l)) {
         if(!(puzzle & 0xd804000l))
           doSolve32(15, 0x8000l, puzzle | 0xd804000l, partsUsed | 0x800l);
         if(!(puzzle & 0xd00c000l))
           doSolve32(15, 0x8000l, puzzle | 0xd00c000l, partsUsed | 0x800l);
         if(!(puzzle & 0xc80c000l))
           doSolve32(15, 0x8000l, puzzle | 0xc80c000l, partsUsed | 0x800l);
         if(!(puzzle & 0x2c00c000l))
           doSolve32(15, 0x8000l, puzzle | 0x2c00c000l, partsUsed | 0x800l);
         if(!(puzzle & 0x2c024000l))
           doSolve32(15, 0x8000l, puzzle | 0x2c024000l, partsUsed | 0x800l);
         if(!(puzzle & 0xc02c000l))
           doSolve32(15, 0x8000l, puzzle | 0xc02c000l, partsUsed | 0x800l);
         if(!(puzzle & 0xc04c000l))
           doSolve32(15, 0x8000l, puzzle | 0xc04c000l, partsUsed | 0x800l);
         if(!(puzzle & 0x4c00c000l))
           doSolve32(15, 0x8000l, puzzle | 0x4c00c000l, partsUsed | 0x800l);
         if(!(puzzle & 0x6c004000l))
           doSolve32(15, 0x8000l, puzzle | 0x6c004000l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x64000l)) {
         if(!(puzzle & 0x406c000l))
           doSolve32(15, 0x8000l, puzzle | 0x406c000l, partsUsed | 0x800l);
         if(!(puzzle & 0x60064000l))
           doSolve32(15, 0x8000l, puzzle | 0x60064000l, partsUsed | 0x800l);
         if(!(puzzle & 0x24064000l))
           doSolve32(15, 0x8000l, puzzle | 0x24064000l, partsUsed | 0x800l);
         if(!(puzzle & 0x4006c000l))
           doSolve32(15, 0x8000l, puzzle | 0x4006c000l, partsUsed | 0x800l);
         if(!(puzzle & 0x806c000l))
           doSolve32(15, 0x8000l, puzzle | 0x806c000l, partsUsed | 0x800l);
         if(!(puzzle & 0x2006c000l))
           doSolve32(15, 0x8000l, puzzle | 0x2006c000l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x4804c000l))
        doSolve32(15, 0x8000l, puzzle | 0x4804c000l, partsUsed | 0x800l);
      if(!(puzzle & 0x64024000l))
        doSolve32(15, 0x8000l, puzzle | 0x64024000l, partsUsed | 0x800l);
      if(!(puzzle & 0x2402c000l))
        doSolve32(15, 0x8000l, puzzle | 0x2402c000l, partsUsed | 0x800l);
    piece12at14at32:
    break;
   case 15:
      if (partsUsed & 0x40l) goto piece7at15at32;
      if(!(puzzle & 0x49208000l))
        doSolve32(16, 0x10000l, puzzle | 0x49208000l, partsUsed | 0x40l);
    piece7at15at32:
      if (partsUsed & 0x100l) goto piece9at15at32;
      if(!(puzzle & 0xc808000l))
        doSolve32(16, 0x10000l, puzzle | 0xc808000l, partsUsed | 0x100l);
      if(!(puzzle & 0xc048000l))
        doSolve32(16, 0x10000l, puzzle | 0xc048000l, partsUsed | 0x100l);
      if(!(puzzle & 0x12018000l))
        doSolve32(16, 0x10000l, puzzle | 0x12018000l, partsUsed | 0x100l);
      if(!(puzzle & 0x10058000l))
        doSolve32(16, 0x10000l, puzzle | 0x10058000l, partsUsed | 0x100l);
      if(!(puzzle & 0xb008000l))
        doSolve32(16, 0x10000l, puzzle | 0xb008000l, partsUsed | 0x100l);
      if(!(puzzle & 0x20068000l))
        doSolve32(16, 0x10000l, puzzle | 0x20068000l, partsUsed | 0x100l);
      if(!(puzzle & 0x68008000l))
        doSolve32(16, 0x10000l, puzzle | 0x68008000l, partsUsed | 0x100l);
      if(!(puzzle & 0x48018000l))
        doSolve32(16, 0x10000l, puzzle | 0x48018000l, partsUsed | 0x100l);
      if(!(puzzle & 0x98008000l))
        doSolve32(16, 0x10000l, puzzle | 0x98008000l, partsUsed | 0x100l);
      if(!(puzzle & 0xc0048000l))
        doSolve32(16, 0x10000l, puzzle | 0xc0048000l, partsUsed | 0x100l);
      if(!(puzzle & 0x80c8000l))
        doSolve32(16, 0x10000l, puzzle | 0x80c8000l, partsUsed | 0x100l);
      if(!(puzzle & 0x80098000l))
        doSolve32(16, 0x10000l, puzzle | 0x80098000l, partsUsed | 0x100l);
    piece9at15at32:
      if (partsUsed & 0x400l) goto piece11at15at32;
      if(!(puzzle & 0x1c018000l))
        doSolve32(16, 0x10000l, puzzle | 0x1c018000l, partsUsed | 0x400l);
      if(!(puzzle & 0x49048000l))
        doSolve32(16, 0x10000l, puzzle | 0x49048000l, partsUsed | 0x400l);
      if(!(puzzle & 0xf8000l))
        doSolve32(16, 0x10000l, puzzle | 0xf8000l, partsUsed | 0x400l);
    piece11at15at32:
      if (partsUsed & 0x800l) goto piece12at15at32;
      if (!(puzzle & 0x9008000l)) {
         if(!(puzzle & 0xd808000l))
           doSolve32(16, 0x10000l, puzzle | 0xd808000l, partsUsed | 0x800l);
         if(!(puzzle & 0x1b008000l))
           doSolve32(16, 0x10000l, puzzle | 0x1b008000l, partsUsed | 0x800l);
         if(!(puzzle & 0x19018000l))
           doSolve32(16, 0x10000l, puzzle | 0x19018000l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x8018000l)) {
         if(!(puzzle & 0x1a018000l))
           doSolve32(16, 0x10000l, puzzle | 0x1a018000l, partsUsed | 0x800l);
         if(!(puzzle & 0x80d8000l))
           doSolve32(16, 0x10000l, puzzle | 0x80d8000l, partsUsed | 0x800l);
         if(!(puzzle & 0x58018000l))
           doSolve32(16, 0x10000l, puzzle | 0x58018000l, partsUsed | 0x800l);
         if(!(puzzle & 0x18058000l))
           doSolve32(16, 0x10000l, puzzle | 0x18058000l, partsUsed | 0x800l);
         if(!(puzzle & 0x18098000l))
           doSolve32(16, 0x10000l, puzzle | 0x18098000l, partsUsed | 0x800l);
         if(!(puzzle & 0x98018000l))
           doSolve32(16, 0x10000l, puzzle | 0x98018000l, partsUsed | 0x800l);
         if(!(puzzle & 0x48058000l))
           doSolve32(16, 0x10000l, puzzle | 0x48058000l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x60008000l)) {
         if(!(puzzle & 0x6c008000l))
           doSolve32(16, 0x10000l, puzzle | 0x6c008000l, partsUsed | 0x800l);
         if(!(puzzle & 0x68048000l))
           doSolve32(16, 0x10000l, puzzle | 0x68048000l, partsUsed | 0x800l);
         if(!(puzzle & 0x60068000l))
           doSolve32(16, 0x10000l, puzzle | 0x60068000l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x40048000l)) {
         if(!(puzzle & 0x4c048000l))
           doSolve32(16, 0x10000l, puzzle | 0x4c048000l, partsUsed | 0x800l);
         if(!(puzzle & 0x48068000l))
           doSolve32(16, 0x10000l, puzzle | 0x48068000l, partsUsed | 0x800l);
         if(!(puzzle & 0xc00c8000l))
           doSolve32(16, 0x10000l, puzzle | 0xc00c8000l, partsUsed | 0x800l);
         if(!(puzzle & 0x58048000l))
           doSolve32(16, 0x10000l, puzzle | 0x58048000l, partsUsed | 0x800l);
         if(!(puzzle & 0x480c8000l))
           doSolve32(16, 0x10000l, puzzle | 0x480c8000l, partsUsed | 0x800l);
         if(!(puzzle & 0x400d8000l))
           doSolve32(16, 0x10000l, puzzle | 0x400d8000l, partsUsed | 0x800l);
         if(!(puzzle & 0xc8048000l))
           doSolve32(16, 0x10000l, puzzle | 0xc8048000l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x98000l)) {
         if(!(puzzle & 0x90098000l))
           doSolve32(16, 0x10000l, puzzle | 0x90098000l, partsUsed | 0x800l);
         if(!(puzzle & 0x800d8000l))
           doSolve32(16, 0x10000l, puzzle | 0x800d8000l, partsUsed | 0x800l);
         if(!(puzzle & 0x100d8000l))
           doSolve32(16, 0x10000l, puzzle | 0x100d8000l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0xd8008000l))
        doSolve32(16, 0x10000l, puzzle | 0xd8008000l, partsUsed | 0x800l);
    piece12at15at32:
    break;
   case 16:
      if (partsUsed & 0x40l) goto piece7at16at32;
      if(!(puzzle & 0x92410000l))
        doSolve32(17, 0x20000l, puzzle | 0x92410000l, partsUsed | 0x40l);
    piece7at16at32:
      if (partsUsed & 0x100l) goto piece9at16at32;
      if(!(puzzle & 0x19010000l))
        doSolve32(17, 0x20000l, puzzle | 0x19010000l, partsUsed | 0x100l);
      if(!(puzzle & 0x18090000l))
        doSolve32(17, 0x20000l, puzzle | 0x18090000l, partsUsed | 0x100l);
      if(!(puzzle & 0x400d0000l))
        doSolve32(17, 0x20000l, puzzle | 0x400d0000l, partsUsed | 0x100l);
      if(!(puzzle & 0xd0010000l))
        doSolve32(17, 0x20000l, puzzle | 0xd0010000l, partsUsed | 0x100l);
    piece9at16at32:
      if (partsUsed & 0x400l) goto piece11at16at32;
      if(!(puzzle & 0x92090000l))
        doSolve32(17, 0x20000l, puzzle | 0x92090000l, partsUsed | 0x400l);
    piece11at16at32:
      if (partsUsed & 0x800l) goto piece12at16at32;
      if (!(puzzle & 0x18010000l)) {
         if(!(puzzle & 0x1b010000l))
           doSolve32(17, 0x20000l, puzzle | 0x1b010000l, partsUsed | 0x800l);
         if(!(puzzle & 0xd8010000l))
           doSolve32(17, 0x20000l, puzzle | 0xd8010000l, partsUsed | 0x800l);
         if(!(puzzle & 0x98090000l))
           doSolve32(17, 0x20000l, puzzle | 0x98090000l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x80090000l)) {
         if(!(puzzle & 0xd0090000l))
           doSolve32(17, 0x20000l, puzzle | 0xd0090000l, partsUsed | 0x800l);
         if(!(puzzle & 0xc00d0000l))
           doSolve32(17, 0x20000l, puzzle | 0xc00d0000l, partsUsed | 0x800l);
         if(!(puzzle & 0x900d0000l))
           doSolve32(17, 0x20000l, puzzle | 0x900d0000l, partsUsed | 0x800l);
      }
    piece12at16at32:
    break;
   case 17:
      if (partsUsed & 0x20l) goto piece6at17at32;
      if(!(puzzle & 0x24920000l))
        doSolve32(18, 0x40000l, puzzle | 0x24920000l, partsUsed | 0x20l);
    piece6at17at32:
      if (partsUsed & 0x80l) goto piece8at17at32;
      if(!(puzzle & 0xe00a0000l))
        doSolve32(18, 0x40000l, puzzle | 0xe00a0000l, partsUsed | 0x80l);
      if(!(puzzle & 0xa00e0000l))
        doSolve32(18, 0x40000l, puzzle | 0xa00e0000l, partsUsed | 0x80l);
    piece8at17at32:
      if (partsUsed & 0x100l) goto piece9at17at32;
      if(!(puzzle & 0x48060000l))
        doSolve32(18, 0x40000l, puzzle | 0x48060000l, partsUsed | 0x100l);
      if(!(puzzle & 0x2c020000l))
        doSolve32(18, 0x40000l, puzzle | 0x2c020000l, partsUsed | 0x100l);
    piece9at17at32:
      if (partsUsed & 0x400l) goto piece11at17at32;
      if (!(puzzle & 0x40060000l)) {
         if(!(puzzle & 0xe0060000l))
           doSolve32(18, 0x40000l, puzzle | 0xe0060000l, partsUsed | 0x400l);
         if(!(puzzle & 0x600e0000l))
           doSolve32(18, 0x40000l, puzzle | 0x600e0000l, partsUsed | 0x400l);
         if(!(puzzle & 0xc00e0000l))
           doSolve32(18, 0x40000l, puzzle | 0xc00e0000l, partsUsed | 0x400l);
      }
    piece11at17at32:
      if (partsUsed & 0x800l) goto piece12at17at32;
      if (!(puzzle & 0x60020000l)) {
         if(!(puzzle & 0x6c020000l))
           doSolve32(18, 0x40000l, puzzle | 0x6c020000l, partsUsed | 0x800l);
         if(!(puzzle & 0x68060000l))
           doSolve32(18, 0x40000l, puzzle | 0x68060000l, partsUsed | 0x800l);
         if(!(puzzle & 0x64060000l))
           doSolve32(18, 0x40000l, puzzle | 0x64060000l, partsUsed | 0x800l);
      }
    piece12at17at32:
    break;
   case 18:
      if (partsUsed & 0x20l) goto piece6at18at32;
      if(!(puzzle & 0x49240000l))
        doSolve32(19, 0x80000l, puzzle | 0x49240000l, partsUsed | 0x20l);
    piece6at18at32:
      if (partsUsed & 0x100l) goto piece9at18at32;
      if(!(puzzle & 0x58040000l))
        doSolve32(19, 0x80000l, puzzle | 0x58040000l, partsUsed | 0x100l);
      if(!(puzzle & 0x64040000l))
        doSolve32(19, 0x80000l, puzzle | 0x64040000l, partsUsed | 0x100l);
      if(!(puzzle & 0x900c0000l))
        doSolve32(19, 0x80000l, puzzle | 0x900c0000l, partsUsed | 0x100l);
    piece9at18at32:
      if (partsUsed & 0x400l) goto piece11at18at32;
      if(!(puzzle & 0xe00c0000l))
        doSolve32(19, 0x80000l, puzzle | 0xe00c0000l, partsUsed | 0x400l);
    piece11at18at32:
      if (partsUsed & 0x800l) goto piece12at18at32;
      if (!(puzzle & 0x48040000l)) {
         if(!(puzzle & 0x6c040000l))
           doSolve32(19, 0x80000l, puzzle | 0x6c040000l, partsUsed | 0x800l);
         if(!(puzzle & 0xd8040000l))
           doSolve32(19, 0x80000l, puzzle | 0xd8040000l, partsUsed | 0x800l);
         if(!(puzzle & 0xc80c0000l))
           doSolve32(19, 0x80000l, puzzle | 0xc80c0000l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0xd00c0000l))
        doSolve32(19, 0x80000l, puzzle | 0xd00c0000l, partsUsed | 0x800l);
    piece12at18at32:
    break;
   case 19:
      if (partsUsed & 0x20l) goto piece6at19at32;
      if(!(puzzle & 0x92480000l))
        doSolve32(20, 0x100000l, puzzle | 0x92480000l, partsUsed | 0x20l);
    piece6at19at32:
      if (partsUsed & 0x100l) goto piece9at19at32;
      if(!(puzzle & 0xc8080000l))
        doSolve32(20, 0x100000l, puzzle | 0xc8080000l, partsUsed | 0x100l);
    piece9at19at32:
      if (partsUsed & 0x800l) goto piece12at19at32;
      if(!(puzzle & 0xd8080000l))
        doSolve32(20, 0x100000l, puzzle | 0xd8080000l, partsUsed | 0x800l);
    piece12at19at32:
    break;
   case 20:
      if (partsUsed & 0x2l) goto piece2at20at32;
      if(!(puzzle & 0x13300000l))
        doSolve32(21, 0x200000l, puzzle | 0x13300000l, partsUsed | 0x2l);
      if(!(puzzle & 0x19900000l))
        doSolve32(21, 0x200000l, puzzle | 0x19900000l, partsUsed | 0x2l);
    piece2at20at32:
      if (partsUsed & 0x4l) goto piece3at20at32;
      if(!(puzzle & 0x9700000l))
        doSolve32(21, 0x200000l, puzzle | 0x9700000l, partsUsed | 0x4l);
      if(!(puzzle & 0x7900000l))
        doSolve32(21, 0x200000l, puzzle | 0x7900000l, partsUsed | 0x4l);
    piece3at20at32:
      if (partsUsed & 0x8l) goto piece4at20at32;
      if(!(puzzle & 0x19300000l))
        doSolve32(21, 0x200000l, puzzle | 0x19300000l, partsUsed | 0x8l);
      if(!(puzzle & 0x13900000l))
        doSolve32(21, 0x200000l, puzzle | 0x13900000l, partsUsed | 0x8l);
    piece4at20at32:
      if (partsUsed & 0x10l) goto piece5at20at32;
      if(!(puzzle & 0x2cb00000l))
        doSolve32(21, 0x200000l, puzzle | 0x2cb00000l, partsUsed | 0x10l);
      if(!(puzzle & 0x65900000l))
        doSolve32(21, 0x200000l, puzzle | 0x65900000l, partsUsed | 0x10l);
      if(!(puzzle & 0x4d300000l))
        doSolve32(21, 0x200000l, puzzle | 0x4d300000l, partsUsed | 0x10l);
    piece5at20at32:
      if (partsUsed & 0x20l) goto piece6at20at32;
      if(!(puzzle & 0x24b00000l))
        doSolve32(21, 0x200000l, puzzle | 0x24b00000l, partsUsed | 0x20l);
      if(!(puzzle & 0x64900000l))
        doSolve32(21, 0x200000l, puzzle | 0x64900000l, partsUsed | 0x20l);
      if(!(puzzle & 0x49300000l))
        doSolve32(21, 0x200000l, puzzle | 0x49300000l, partsUsed | 0x20l);
    piece6at20at32:
      if (partsUsed & 0x40l) goto piece7at20at32;
      if(!(puzzle & 0x25900000l))
        doSolve32(21, 0x200000l, puzzle | 0x25900000l, partsUsed | 0x40l);
      if(!(puzzle & 0x2c900000l))
        doSolve32(21, 0x200000l, puzzle | 0x2c900000l, partsUsed | 0x40l);
    piece7at20at32:
      if (partsUsed & 0x80l) goto piece8at20at32;
      if(!(puzzle & 0x3d00000l))
        doSolve32(21, 0x200000l, puzzle | 0x3d00000l, partsUsed | 0x80l);
      if(!(puzzle & 0x2f00000l))
        doSolve32(21, 0x200000l, puzzle | 0x2f00000l, partsUsed | 0x80l);
      if(!(puzzle & 0xcb00000l))
        doSolve32(21, 0x200000l, puzzle | 0xcb00000l, partsUsed | 0x80l);
      if(!(puzzle & 0xd300000l))
        doSolve32(21, 0x200000l, puzzle | 0xd300000l, partsUsed | 0x80l);
    piece8at20at32:
      if (partsUsed & 0x200l) goto piece10at20at32;
      if(!(puzzle & 0xb300000l))
        doSolve32(21, 0x200000l, puzzle | 0xb300000l, partsUsed | 0x200l);
      if(!(puzzle & 0xb900000l))
        doSolve32(21, 0x200000l, puzzle | 0xb900000l, partsUsed | 0x200l);
    piece10at20at32:
      if (partsUsed & 0x400l) goto piece11at20at32;
      if (!(puzzle & 0x1300000l)) {
         if(!(puzzle & 0x3700000l))
           doSolve32(21, 0x200000l, puzzle | 0x3700000l, partsUsed | 0x400l);
         if(!(puzzle & 0x1f00000l))
           doSolve32(21, 0x200000l, puzzle | 0x1f00000l, partsUsed | 0x400l);
         if(!(puzzle & 0x5b00000l))
           doSolve32(21, 0x200000l, puzzle | 0x5b00000l, partsUsed | 0x400l);
         if(!(puzzle & 0x9b00000l))
           doSolve32(21, 0x200000l, puzzle | 0x9b00000l, partsUsed | 0x400l);
         if(!(puzzle & 0x3b00000l))
           doSolve32(21, 0x200000l, puzzle | 0x3b00000l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0xd900000l))
        doSolve32(21, 0x200000l, puzzle | 0xd900000l, partsUsed | 0x400l);
    piece11at20at32:
    break;
   case 21:
      if (partsUsed & 0x2l) goto piece2at21at32;
      if(!(puzzle & 0x5e00000l))
        doSolve32(22, 0x400000l, puzzle | 0x5e00000l, partsUsed | 0x2l);
    piece2at21at32:
      if (partsUsed & 0x4l) goto piece3at21at32;
      if(!(puzzle & 0x1d200000l))
        doSolve32(22, 0x400000l, puzzle | 0x1d200000l, partsUsed | 0x4l);
    piece3at21at32:
      if (partsUsed & 0x8l) goto piece4at21at32;
      if(!(puzzle & 0xd600000l))
        doSolve32(22, 0x400000l, puzzle | 0xd600000l, partsUsed | 0x8l);
    piece4at21at32:
      if (partsUsed & 0x10l) goto piece5at21at32;
      if (!(puzzle & 0x49200000l)) {
         if(!(puzzle & 0x69a00000l))
           doSolve32(22, 0x400000l, puzzle | 0x69a00000l, partsUsed | 0x10l);
         if(!(puzzle & 0xcb200000l))
           doSolve32(22, 0x400000l, puzzle | 0xcb200000l, partsUsed | 0x10l);
         if(!(puzzle & 0x59600000l))
           doSolve32(22, 0x400000l, puzzle | 0x59600000l, partsUsed | 0x10l);
      }
      if(!(puzzle & 0x9a600000l))
        doSolve32(22, 0x400000l, puzzle | 0x9a600000l, partsUsed | 0x10l);
    piece5at21at32:
      if (partsUsed & 0x20l) goto piece6at21at32;
      if (!(puzzle & 0x49200000l)) {
         if(!(puzzle & 0x69200000l))
           doSolve32(22, 0x400000l, puzzle | 0x69200000l, partsUsed | 0x20l);
         if(!(puzzle & 0x49600000l))
           doSolve32(22, 0x400000l, puzzle | 0x49600000l, partsUsed | 0x20l);
         if(!(puzzle & 0xc9200000l))
           doSolve32(22, 0x400000l, puzzle | 0xc9200000l, partsUsed | 0x20l);
      }
      if(!(puzzle & 0x92600000l))
        doSolve32(22, 0x400000l, puzzle | 0x92600000l, partsUsed | 0x20l);
    piece6at21at32:
      if (partsUsed & 0x40l) goto piece7at21at32;
      if (!(puzzle & 0x49200000l)) {
         if(!(puzzle & 0x49a00000l))
           doSolve32(22, 0x400000l, puzzle | 0x49a00000l, partsUsed | 0x40l);
         if(!(puzzle & 0x4d200000l))
           doSolve32(22, 0x400000l, puzzle | 0x4d200000l, partsUsed | 0x40l);
         if(!(puzzle & 0x4b200000l))
           doSolve32(22, 0x400000l, puzzle | 0x4b200000l, partsUsed | 0x40l);
         if(!(puzzle & 0x59200000l))
           doSolve32(22, 0x400000l, puzzle | 0x59200000l, partsUsed | 0x40l);
      }
    piece7at21at32:
      if (partsUsed & 0x80l) goto piece8at21at32;
      if(!(puzzle & 0x19600000l))
        doSolve32(22, 0x400000l, puzzle | 0x19600000l, partsUsed | 0x80l);
      if(!(puzzle & 0x1a600000l))
        doSolve32(22, 0x400000l, puzzle | 0x1a600000l, partsUsed | 0x80l);
    piece8at21at32:
      if (partsUsed & 0x200l) goto piece10at21at32;
      if (!(puzzle & 0x1a00000l)) {
         if(!(puzzle & 0x13a00000l))
           doSolve32(22, 0x400000l, puzzle | 0x13a00000l, partsUsed | 0x200l);
         if(!(puzzle & 0x7a00000l))
           doSolve32(22, 0x400000l, puzzle | 0x7a00000l, partsUsed | 0x200l);
         if(!(puzzle & 0x9e00000l))
           doSolve32(22, 0x400000l, puzzle | 0x9e00000l, partsUsed | 0x200l);
         if(!(puzzle & 0x19a00000l))
           doSolve32(22, 0x400000l, puzzle | 0x19a00000l, partsUsed | 0x200l);
      }
      if(!(puzzle & 0xf200000l))
        doSolve32(22, 0x400000l, puzzle | 0xf200000l, partsUsed | 0x200l);
    piece10at21at32:
      if (partsUsed & 0x400l) goto piece11at21at32;
      if (!(puzzle & 0x3200000l)) {
         if(!(puzzle & 0xb600000l))
           doSolve32(22, 0x400000l, puzzle | 0xb600000l, partsUsed | 0x400l);
         if(!(puzzle & 0x13600000l))
           doSolve32(22, 0x400000l, puzzle | 0x13600000l, partsUsed | 0x400l);
         if(!(puzzle & 0x1b200000l))
           doSolve32(22, 0x400000l, puzzle | 0x1b200000l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x3e00000l))
        doSolve32(22, 0x400000l, puzzle | 0x3e00000l, partsUsed | 0x400l);
      if(!(puzzle & 0xda00000l))
        doSolve32(22, 0x400000l, puzzle | 0xda00000l, partsUsed | 0x400l);
    piece11at21at32:
    break;
   case 22:
      if (partsUsed & 0x2l) goto piece2at22at32;
      if(!(puzzle & 0xf400000l))
        doSolve32(23, 0x800000l, puzzle | 0xf400000l, partsUsed | 0x2l);
    piece2at22at32:
      if (partsUsed & 0x4l) goto piece3at22at32;
      if(!(puzzle & 0x13c00000l))
        doSolve32(23, 0x800000l, puzzle | 0x13c00000l, partsUsed | 0x4l);
    piece3at22at32:
      if (partsUsed & 0x8l) goto piece4at22at32;
      if(!(puzzle & 0x7c00000l))
        doSolve32(23, 0x800000l, puzzle | 0x7c00000l, partsUsed | 0x8l);
    piece4at22at32:
      if (partsUsed & 0x10l) goto piece5at22at32;
      if(!(puzzle & 0xd3400000l))
        doSolve32(23, 0x800000l, puzzle | 0xd3400000l, partsUsed | 0x10l);
    piece5at22at32:
      if (partsUsed & 0x20l) goto piece6at22at32;
      if(!(puzzle & 0xd2400000l))
        doSolve32(23, 0x800000l, puzzle | 0xd2400000l, partsUsed | 0x20l);
    piece6at22at32:
      if (partsUsed & 0x40l) goto piece7at22at32;
      if(!(puzzle & 0x93400000l))
        doSolve32(23, 0x800000l, puzzle | 0x93400000l, partsUsed | 0x40l);
      if(!(puzzle & 0x9a400000l))
        doSolve32(23, 0x800000l, puzzle | 0x9a400000l, partsUsed | 0x40l);
    piece7at22at32:
      if (partsUsed & 0x200l) goto piece10at22at32;
      if(!(puzzle & 0xbc00000l))
        doSolve32(23, 0x800000l, puzzle | 0xbc00000l, partsUsed | 0x200l);
    piece10at22at32:
      if (partsUsed & 0x400l) goto piece11at22at32;
      if(!(puzzle & 0x1b400000l))
        doSolve32(23, 0x800000l, puzzle | 0x1b400000l, partsUsed | 0x400l);
    piece11at22at32:
    break;
   case 23:
      if (partsUsed & 0x2l) goto piece2at23at32;
      if(!(puzzle & 0x99800000l))
        doSolve32(24, 0x1000000l, puzzle | 0x99800000l, partsUsed | 0x2l);
      if(!(puzzle & 0xcc800000l))
        doSolve32(24, 0x1000000l, puzzle | 0xcc800000l, partsUsed | 0x2l);
    piece2at23at32:
      if (partsUsed & 0x4l) goto piece3at23at32;
      if(!(puzzle & 0x4b800000l))
        doSolve32(24, 0x1000000l, puzzle | 0x4b800000l, partsUsed | 0x4l);
      if(!(puzzle & 0x3c800000l))
        doSolve32(24, 0x1000000l, puzzle | 0x3c800000l, partsUsed | 0x4l);
    piece3at23at32:
      if (partsUsed & 0x8l) goto piece4at23at32;
      if(!(puzzle & 0xc9800000l))
        doSolve32(24, 0x1000000l, puzzle | 0xc9800000l, partsUsed | 0x8l);
      if(!(puzzle & 0x9c800000l))
        doSolve32(24, 0x1000000l, puzzle | 0x9c800000l, partsUsed | 0x8l);
    piece4at23at32:
      if (partsUsed & 0x80l) goto piece8at23at32;
      if(!(puzzle & 0x1e800000l))
        doSolve32(24, 0x1000000l, puzzle | 0x1e800000l, partsUsed | 0x80l);
      if(!(puzzle & 0x17800000l))
        doSolve32(24, 0x1000000l, puzzle | 0x17800000l, partsUsed | 0x80l);
      if(!(puzzle & 0x65800000l))
        doSolve32(24, 0x1000000l, puzzle | 0x65800000l, partsUsed | 0x80l);
      if(!(puzzle & 0x69800000l))
        doSolve32(24, 0x1000000l, puzzle | 0x69800000l, partsUsed | 0x80l);
    piece8at23at32:
      if (partsUsed & 0x200l) goto piece10at23at32;
      if(!(puzzle & 0x59800000l))
        doSolve32(24, 0x1000000l, puzzle | 0x59800000l, partsUsed | 0x200l);
      if(!(puzzle & 0x5c800000l))
        doSolve32(24, 0x1000000l, puzzle | 0x5c800000l, partsUsed | 0x200l);
    piece10at23at32:
      if (partsUsed & 0x400l) goto piece11at23at32;
      if (!(puzzle & 0x9800000l)) {
         if(!(puzzle & 0x1b800000l))
           doSolve32(24, 0x1000000l, puzzle | 0x1b800000l, partsUsed | 0x400l);
         if(!(puzzle & 0xf800000l))
           doSolve32(24, 0x1000000l, puzzle | 0xf800000l, partsUsed | 0x400l);
         if(!(puzzle & 0x2d800000l))
           doSolve32(24, 0x1000000l, puzzle | 0x2d800000l, partsUsed | 0x400l);
         if(!(puzzle & 0x4d800000l))
           doSolve32(24, 0x1000000l, puzzle | 0x4d800000l, partsUsed | 0x400l);
         if(!(puzzle & 0x1d800000l))
           doSolve32(24, 0x1000000l, puzzle | 0x1d800000l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x6c800000l))
        doSolve32(24, 0x1000000l, puzzle | 0x6c800000l, partsUsed | 0x400l);
    piece11at23at32:
    break;
   case 24:
      if (partsUsed & 0x2l) goto piece2at24at32;
      if(!(puzzle & 0x2f000000l))
        doSolve32(25, 0x2000000l, puzzle | 0x2f000000l, partsUsed | 0x2l);
    piece2at24at32:
      if (partsUsed & 0x4l) goto piece3at24at32;
      if(!(puzzle & 0xe9000000l))
        doSolve32(25, 0x2000000l, puzzle | 0xe9000000l, partsUsed | 0x4l);
    piece3at24at32:
      if (partsUsed & 0x8l) goto piece4at24at32;
      if(!(puzzle & 0x6b000000l))
        doSolve32(25, 0x2000000l, puzzle | 0x6b000000l, partsUsed | 0x8l);
    piece4at24at32:
      if (partsUsed & 0x80l) goto piece8at24at32;
      if(!(puzzle & 0xcb000000l))
        doSolve32(25, 0x2000000l, puzzle | 0xcb000000l, partsUsed | 0x80l);
      if(!(puzzle & 0xd3000000l))
        doSolve32(25, 0x2000000l, puzzle | 0xd3000000l, partsUsed | 0x80l);
    piece8at24at32:
      if (partsUsed & 0x200l) goto piece10at24at32;
      if (!(puzzle & 0xd000000l)) {
         if(!(puzzle & 0x9d000000l))
           doSolve32(25, 0x2000000l, puzzle | 0x9d000000l, partsUsed | 0x200l);
         if(!(puzzle & 0x3d000000l))
           doSolve32(25, 0x2000000l, puzzle | 0x3d000000l, partsUsed | 0x200l);
         if(!(puzzle & 0x4f000000l))
           doSolve32(25, 0x2000000l, puzzle | 0x4f000000l, partsUsed | 0x200l);
         if(!(puzzle & 0xcd000000l))
           doSolve32(25, 0x2000000l, puzzle | 0xcd000000l, partsUsed | 0x200l);
      }
      if(!(puzzle & 0x79000000l))
        doSolve32(25, 0x2000000l, puzzle | 0x79000000l, partsUsed | 0x200l);
    piece10at24at32:
      if (partsUsed & 0x400l) goto piece11at24at32;
      if (!(puzzle & 0x19000000l)) {
         if(!(puzzle & 0x5b000000l))
           doSolve32(25, 0x2000000l, puzzle | 0x5b000000l, partsUsed | 0x400l);
         if(!(puzzle & 0x9b000000l))
           doSolve32(25, 0x2000000l, puzzle | 0x9b000000l, partsUsed | 0x400l);
         if(!(puzzle & 0xd9000000l))
           doSolve32(25, 0x2000000l, puzzle | 0xd9000000l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x1f000000l))
        doSolve32(25, 0x2000000l, puzzle | 0x1f000000l, partsUsed | 0x400l);
      if(!(puzzle & 0x6d000000l))
        doSolve32(25, 0x2000000l, puzzle | 0x6d000000l, partsUsed | 0x400l);
    piece11at24at32:
    break;
   case 25:
      if (partsUsed & 0x2l) goto piece2at25at32;
      if(!(puzzle & 0x7a000000l))
        doSolve32(26, 0x4000000l, puzzle | 0x7a000000l, partsUsed | 0x2l);
    piece2at25at32:
      if (partsUsed & 0x4l) goto piece3at25at32;
      if(!(puzzle & 0x9e000000l))
        doSolve32(26, 0x4000000l, puzzle | 0x9e000000l, partsUsed | 0x4l);
    piece3at25at32:
      if (partsUsed & 0x8l) goto piece4at25at32;
      if(!(puzzle & 0x3e000000l))
        doSolve32(26, 0x4000000l, puzzle | 0x3e000000l, partsUsed | 0x8l);
    piece4at25at32:
      if (partsUsed & 0x200l) goto piece10at25at32;
      if(!(puzzle & 0x5e000000l))
        doSolve32(26, 0x4000000l, puzzle | 0x5e000000l, partsUsed | 0x200l);
    piece10at25at32:
      if (partsUsed & 0x400l) goto piece11at25at32;
      if(!(puzzle & 0xda000000l))
        doSolve32(26, 0x4000000l, puzzle | 0xda000000l, partsUsed | 0x400l);
    piece11at25at32:
    break;
   case 26:
      if (partsUsed & 0x80l) goto piece8at26at32;
      if(!(puzzle & 0xbc000000l))
        doSolve32(27, 0x8000000l, puzzle | 0xbc000000l, partsUsed | 0x80l);
      if(!(puzzle & 0xf4000000l))
        doSolve32(27, 0x8000000l, puzzle | 0xf4000000l, partsUsed | 0x80l);
    piece8at26at32:
      if (partsUsed & 0x400l) goto piece11at26at32;
      if (!(puzzle & 0x4c000000l)) {
         if(!(puzzle & 0xdc000000l))
           doSolve32(27, 0x8000000l, puzzle | 0xdc000000l, partsUsed | 0x400l);
         if(!(puzzle & 0x7c000000l))
           doSolve32(27, 0x8000000l, puzzle | 0x7c000000l, partsUsed | 0x400l);
         if(!(puzzle & 0xec000000l))
           doSolve32(27, 0x8000000l, puzzle | 0xec000000l, partsUsed | 0x400l);
      }
    piece11at26at32:
    break;
   case 27:
      if (partsUsed & 0x400l) goto piece11at27at32;
      if(!(puzzle & 0xf8000000l))
        doSolve32(28, 0x10000000l, puzzle | 0xf8000000l, partsUsed | 0x400l);
    piece11at27at32:
    break;
  }
}


void altDoSolve(int start, long long startMask, long long puzzle, int partsUsed)
{
  for (;(puzzle & startMask) && start < NUM_LO_BITS; start++, startMask <<= 1);

  if(start == NUM_LO_BITS) {
    altDoSolve32(0, 1, puzzle>>NUM_LO_BITS, partsUsed);
    return;
  }

  switch(start) {
   case 4:
      if (partsUsed & 0x2l) goto altpiece2at4;
      if(!(puzzle & 0x480090010ll))
        altDoSolve(5, 0x20ll, puzzle | 0x480090010ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x400480090ll))
        altDoSolve(5, 0x20ll, puzzle | 0x400480090ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1330ll))
        altDoSolve(5, 0x20ll, puzzle | 0x1330ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1990ll))
        altDoSolve(5, 0x20ll, puzzle | 0x1990ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x40060030ll))
        altDoSolve(5, 0x20ll, puzzle | 0x40060030ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x60030010ll))
        altDoSolve(5, 0x20ll, puzzle | 0x60030010ll,
                   partsUsed | 0x2l);
    altpiece2at4:
      if (partsUsed & 0x4l) goto altpiece3at4;
      if(!(puzzle & 0x10490010ll))
        altDoSolve(5, 0x20ll, puzzle | 0x10490010ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x10070010ll))
        altDoSolve(5, 0x20ll, puzzle | 0x10070010ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x80080490ll))
        altDoSolve(5, 0x20ll, puzzle | 0x80080490ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x790ll))
        altDoSolve(5, 0x20ll, puzzle | 0x790ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x970ll))
        altDoSolve(5, 0x20ll, puzzle | 0x970ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x20020070ll))
        altDoSolve(5, 0x20ll, puzzle | 0x20020070ll,
                   partsUsed | 0x4l);
    altpiece3at4:
      if (partsUsed & 0x8l) goto altpiece4at4;
      if(!(puzzle & 0x400490010ll))
        altDoSolve(5, 0x20ll, puzzle | 0x400490010ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x480080090ll))
        altDoSolve(5, 0x20ll, puzzle | 0x480080090ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1930ll))
        altDoSolve(5, 0x20ll, puzzle | 0x1930ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1390ll))
        altDoSolve(5, 0x20ll, puzzle | 0x1390ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x40070010ll))
        altDoSolve(5, 0x20ll, puzzle | 0x40070010ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x60020030ll))
        altDoSolve(5, 0x20ll, puzzle | 0x60020030ll,
                   partsUsed | 0x8l);
    altpiece4at4:
      if (partsUsed & 0x10l) goto altpiece5at4;
      if (!(puzzle & 0x2490ll)) {
         if(!(puzzle & 0x6590ll))
           altDoSolve(5, 0x20ll, puzzle | 0x6590ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x2082490ll))
           altDoSolve(5, 0x20ll, puzzle | 0x2082490ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x2cb0ll))
           altDoSolve(5, 0x20ll, puzzle | 0x2cb0ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x90010090010ll))
        altDoSolve(5, 0x20ll, puzzle | 0x90010090010ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x80090080090ll))
        altDoSolve(5, 0x20ll, puzzle | 0x80090080090ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x412490ll))
        altDoSolve(5, 0x20ll, puzzle | 0x412490ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x10090010090ll))
        altDoSolve(5, 0x20ll, puzzle | 0x10090010090ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x4d30ll))
        altDoSolve(5, 0x20ll, puzzle | 0x4d30ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x2490410ll))
        altDoSolve(5, 0x20ll, puzzle | 0x2490410ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x30010030010ll))
        altDoSolve(5, 0x20ll, puzzle | 0x30010030010ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x10030010030ll))
        altDoSolve(5, 0x20ll, puzzle | 0x10030010030ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x20030020030ll))
        altDoSolve(5, 0x20ll, puzzle | 0x20030020030ll,
                   partsUsed | 0x10l);
    altpiece5at4:
      if (partsUsed & 0x20l) goto altpiece6at4;
      if (!(puzzle & 0x10010010010ll)) {
         if(!(puzzle & 0x10010010090ll))
           altDoSolve(5, 0x20ll, puzzle | 0x10010010090ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x90010010010ll))
           altDoSolve(5, 0x20ll, puzzle | 0x90010010010ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x10010010030ll))
           altDoSolve(5, 0x20ll, puzzle | 0x10010010030ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x30010010010ll))
           altDoSolve(5, 0x20ll, puzzle | 0x30010010010ll,
                   partsUsed | 0x20l);
      }
      if (!(puzzle & 0x2490ll)) {
         if(!(puzzle & 0x2002490ll))
           altDoSolve(5, 0x20ll, puzzle | 0x2002490ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x24b0ll))
           altDoSolve(5, 0x20ll, puzzle | 0x24b0ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x12490ll))
           altDoSolve(5, 0x20ll, puzzle | 0x12490ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x6490ll))
           altDoSolve(5, 0x20ll, puzzle | 0x6490ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x2490010ll))
        altDoSolve(5, 0x20ll, puzzle | 0x2490010ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x4930ll))
        altDoSolve(5, 0x20ll, puzzle | 0x4930ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x80080080090ll))
        altDoSolve(5, 0x20ll, puzzle | 0x80080080090ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x20020020030ll))
        altDoSolve(5, 0x20ll, puzzle | 0x20020020030ll,
                   partsUsed | 0x20l);
    altpiece6at4:
      if (partsUsed & 0x40l) goto altpiece7at4;
      if (!(puzzle & 0x10010010010ll)) {
         if(!(puzzle & 0x10010090010ll))
           altDoSolve(5, 0x20ll, puzzle | 0x10010090010ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x10090010010ll))
           altDoSolve(5, 0x20ll, puzzle | 0x10090010010ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x10010030010ll))
           altDoSolve(5, 0x20ll, puzzle | 0x10010030010ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x10030010010ll))
           altDoSolve(5, 0x20ll, puzzle | 0x10030010010ll,
                   partsUsed | 0x40l);
      }
      if (!(puzzle & 0x2490ll)) {
         if(!(puzzle & 0x402490ll))
           altDoSolve(5, 0x20ll, puzzle | 0x402490ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2590ll))
           altDoSolve(5, 0x20ll, puzzle | 0x2590ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x82490ll))
           altDoSolve(5, 0x20ll, puzzle | 0x82490ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2c90ll))
           altDoSolve(5, 0x20ll, puzzle | 0x2c90ll,
                   partsUsed | 0x40l);
      }
    altpiece7at4:
      if (partsUsed & 0x80l) goto altpiece8at4;
      if(!(puzzle & 0x90010090ll))
        altDoSolve(5, 0x20ll, puzzle | 0x90010090ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x90080090ll))
        altDoSolve(5, 0x20ll, puzzle | 0x90080090ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x490410ll))
        altDoSolve(5, 0x20ll, puzzle | 0x490410ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x410490ll))
        altDoSolve(5, 0x20ll, puzzle | 0x410490ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x2f0ll))
        altDoSolve(5, 0x20ll, puzzle | 0x2f0ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xcb0ll))
        altDoSolve(5, 0x20ll, puzzle | 0xcb0ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xd30ll))
        altDoSolve(5, 0x20ll, puzzle | 0xd30ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x3d0ll))
        altDoSolve(5, 0x20ll, puzzle | 0x3d0ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x30020030ll))
        altDoSolve(5, 0x20ll, puzzle | 0x30020030ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x30010030ll))
        altDoSolve(5, 0x20ll, puzzle | 0x30010030ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x70050ll))
        altDoSolve(5, 0x20ll, puzzle | 0x70050ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x50070ll))
        altDoSolve(5, 0x20ll, puzzle | 0x50070ll,
                   partsUsed | 0x80l);
    altpiece8at4:
      if (partsUsed & 0x100l) goto altpiece9at4;
      if(!(puzzle & 0x100130ll))
        altDoSolve(5, 0x20ll, puzzle | 0x100130ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x130010ll))
        altDoSolve(5, 0x20ll, puzzle | 0x130010ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x200b0ll))
        altDoSolve(5, 0x20ll, puzzle | 0x200b0ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x90030ll))
        altDoSolve(5, 0x20ll, puzzle | 0x90030ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x180090ll))
        altDoSolve(5, 0x20ll, puzzle | 0x180090ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x10190ll))
        altDoSolve(5, 0x20ll, puzzle | 0x10190ll,
                   partsUsed | 0x100l);
    altpiece9at4:
      if (partsUsed & 0x200l) goto altpiece10at4;
      if(!(puzzle & 0x80490010ll))
        altDoSolve(5, 0x20ll, puzzle | 0x80490010ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x80480090ll))
        altDoSolve(5, 0x20ll, puzzle | 0x80480090ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0xb30ll))
        altDoSolve(5, 0x20ll, puzzle | 0xb30ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0xb90ll))
        altDoSolve(5, 0x20ll, puzzle | 0xb90ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x20060030ll))
        altDoSolve(5, 0x20ll, puzzle | 0x20060030ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x20070010ll))
        altDoSolve(5, 0x20ll, puzzle | 0x20070010ll,
                   partsUsed | 0x200l);
    altpiece10at4:
      if (partsUsed & 0x400l) goto altpiece11at4;
      if (!(puzzle & 0x80090ll)) {
         if(!(puzzle & 0x10090090ll))
           altDoSolve(5, 0x20ll, puzzle | 0x10090090ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x490090ll))
           altDoSolve(5, 0x20ll, puzzle | 0x490090ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x480490ll))
           altDoSolve(5, 0x20ll, puzzle | 0x480490ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x90490ll))
           altDoSolve(5, 0x20ll, puzzle | 0x90490ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x80090090ll))
           altDoSolve(5, 0x20ll, puzzle | 0x80090090ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x130ll)) {
         if(!(puzzle & 0x370ll))
           altDoSolve(5, 0x20ll, puzzle | 0x370ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1f0ll))
           altDoSolve(5, 0x20ll, puzzle | 0x1f0ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x5b0ll))
           altDoSolve(5, 0x20ll, puzzle | 0x5b0ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x9b0ll))
           altDoSolve(5, 0x20ll, puzzle | 0x9b0ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x3b0ll))
           altDoSolve(5, 0x20ll, puzzle | 0x3b0ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x10010010ll)) {
         if(!(puzzle & 0x90090010ll))
           altDoSolve(5, 0x20ll, puzzle | 0x90090010ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x10030030ll))
           altDoSolve(5, 0x20ll, puzzle | 0x10030030ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x30030010ll))
           altDoSolve(5, 0x20ll, puzzle | 0x30030010ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x20030ll)) {
         if(!(puzzle & 0x20030030ll))
           altDoSolve(5, 0x20ll, puzzle | 0x20030030ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x70030ll))
           altDoSolve(5, 0x20ll, puzzle | 0x70030ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x30070ll))
           altDoSolve(5, 0x20ll, puzzle | 0x30070ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x60070ll))
           altDoSolve(5, 0x20ll, puzzle | 0x60070ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0xd90ll))
        altDoSolve(5, 0x20ll, puzzle | 0xd90ll,
                   partsUsed | 0x400l);
    altpiece11at4:
    break;
   case 5:
      if (partsUsed & 0x2l) goto altpiece2at5;
      if(!(puzzle & 0x5e0ll))
        altDoSolve(6, 0x40ll, puzzle | 0x5e0ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x10030060ll))
        altDoSolve(6, 0x40ll, puzzle | 0x10030060ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x900120020ll))
        altDoSolve(6, 0x40ll, puzzle | 0x900120020ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x800900120ll))
        altDoSolve(6, 0x40ll, puzzle | 0x800900120ll,
                   partsUsed | 0x2l);
    altpiece2at5:
      if (partsUsed & 0x4l) goto altpiece3at5;
      if(!(puzzle & 0x1d20ll))
        altDoSolve(6, 0x40ll, puzzle | 0x1d20ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x100100920ll))
        altDoSolve(6, 0x40ll, puzzle | 0x100100920ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x70020020ll))
        altDoSolve(6, 0x40ll, puzzle | 0x70020020ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x20920020ll))
        altDoSolve(6, 0x40ll, puzzle | 0x20920020ll,
                   partsUsed | 0x4l);
    altpiece3at5:
      if (partsUsed & 0x8l) goto altpiece4at5;
      if(!(puzzle & 0xd60ll))
        altDoSolve(6, 0x40ll, puzzle | 0xd60ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x30020060ll))
        altDoSolve(6, 0x40ll, puzzle | 0x30020060ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x800920020ll))
        altDoSolve(6, 0x40ll, puzzle | 0x800920020ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x900100120ll))
        altDoSolve(6, 0x40ll, puzzle | 0x900100120ll,
                   partsUsed | 0x8l);
    altpiece4at5:
      if (partsUsed & 0x10l) goto altpiece5at5;
      if (!(puzzle & 0x4920ll)) {
         if(!(puzzle & 0x69a0ll))
           altDoSolve(6, 0x40ll, puzzle | 0x69a0ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x824920ll))
           altDoSolve(6, 0x40ll, puzzle | 0x824920ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0xcb20ll))
           altDoSolve(6, 0x40ll, puzzle | 0xcb20ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x4104920ll))
           altDoSolve(6, 0x40ll, puzzle | 0x4104920ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x5960ll))
           altDoSolve(6, 0x40ll, puzzle | 0x5960ll,
                   partsUsed | 0x10l);
      }
      if (!(puzzle & 0x20020020020ll)) {
         if(!(puzzle & 0x30020030020ll))
           altDoSolve(6, 0x40ll, puzzle | 0x30020030020ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x120020120020ll))
           altDoSolve(6, 0x40ll, puzzle | 0x120020120020ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x20120020120ll))
           altDoSolve(6, 0x40ll, puzzle | 0x20120020120ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x60020060020ll))
           altDoSolve(6, 0x40ll, puzzle | 0x60020060020ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x20060020060ll))
           altDoSolve(6, 0x40ll, puzzle | 0x20060020060ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x100120100120ll))
        altDoSolve(6, 0x40ll, puzzle | 0x100120100120ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x9a60ll))
        altDoSolve(6, 0x40ll, puzzle | 0x9a60ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x4920820ll))
        altDoSolve(6, 0x40ll, puzzle | 0x4920820ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x40060040060ll))
        altDoSolve(6, 0x40ll, puzzle | 0x40060040060ll,
                   partsUsed | 0x10l);
    altpiece5at5:
      if (partsUsed & 0x20l) goto altpiece6at5;
      if (!(puzzle & 0x4920ll)) {
         if(!(puzzle & 0x6920ll))
           altDoSolve(6, 0x40ll, puzzle | 0x6920ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x4004920ll))
           altDoSolve(6, 0x40ll, puzzle | 0x4004920ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x4960ll))
           altDoSolve(6, 0x40ll, puzzle | 0x4960ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x24920ll))
           altDoSolve(6, 0x40ll, puzzle | 0x24920ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0xc920ll))
           altDoSolve(6, 0x40ll, puzzle | 0xc920ll,
                   partsUsed | 0x20l);
      }
      if (!(puzzle & 0x20020020020ll)) {
         if(!(puzzle & 0x30020020020ll))
           altDoSolve(6, 0x40ll, puzzle | 0x30020020020ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x20020020120ll))
           altDoSolve(6, 0x40ll, puzzle | 0x20020020120ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x120020020020ll))
           altDoSolve(6, 0x40ll, puzzle | 0x120020020020ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x20020020060ll))
           altDoSolve(6, 0x40ll, puzzle | 0x20020020060ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x60020020020ll))
           altDoSolve(6, 0x40ll, puzzle | 0x60020020020ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x4920020ll))
        altDoSolve(6, 0x40ll, puzzle | 0x4920020ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x9260ll))
        altDoSolve(6, 0x40ll, puzzle | 0x9260ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x100100100120ll))
        altDoSolve(6, 0x40ll, puzzle | 0x100100100120ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x40040040060ll))
        altDoSolve(6, 0x40ll, puzzle | 0x40040040060ll,
                   partsUsed | 0x20l);
    altpiece6at5:
      if (partsUsed & 0x40l) goto altpiece7at5;
      if (!(puzzle & 0x4920ll)) {
         if(!(puzzle & 0x49a0ll))
           altDoSolve(6, 0x40ll, puzzle | 0x49a0ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4d20ll))
           altDoSolve(6, 0x40ll, puzzle | 0x4d20ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x804920ll))
           altDoSolve(6, 0x40ll, puzzle | 0x804920ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4b20ll))
           altDoSolve(6, 0x40ll, puzzle | 0x4b20ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x104920ll))
           altDoSolve(6, 0x40ll, puzzle | 0x104920ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x5920ll))
           altDoSolve(6, 0x40ll, puzzle | 0x5920ll,
                   partsUsed | 0x40l);
      }
      if (!(puzzle & 0x20020020020ll)) {
         if(!(puzzle & 0x20020030020ll))
           altDoSolve(6, 0x40ll, puzzle | 0x20020030020ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x20030020020ll))
           altDoSolve(6, 0x40ll, puzzle | 0x20030020020ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x20020120020ll))
           altDoSolve(6, 0x40ll, puzzle | 0x20020120020ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x20120020020ll))
           altDoSolve(6, 0x40ll, puzzle | 0x20120020020ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x20020060020ll))
           altDoSolve(6, 0x40ll, puzzle | 0x20020060020ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x20060020020ll))
           altDoSolve(6, 0x40ll, puzzle | 0x20060020020ll,
                   partsUsed | 0x40l);
      }
    altpiece7at5:
      if (partsUsed & 0x80l) goto altpiece8at5;
      if(!(puzzle & 0x120020120ll))
        altDoSolve(6, 0x40ll, puzzle | 0x120020120ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x120100120ll))
        altDoSolve(6, 0x40ll, puzzle | 0x120100120ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x920820ll))
        altDoSolve(6, 0x40ll, puzzle | 0x920820ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x820920ll))
        altDoSolve(6, 0x40ll, puzzle | 0x820920ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1960ll))
        altDoSolve(6, 0x40ll, puzzle | 0x1960ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1a60ll))
        altDoSolve(6, 0x40ll, puzzle | 0x1a60ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x60040060ll))
        altDoSolve(6, 0x40ll, puzzle | 0x60040060ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x60020060ll))
        altDoSolve(6, 0x40ll, puzzle | 0x60020060ll,
                   partsUsed | 0x80l);
    altpiece8at5:
      if (partsUsed & 0x100l) goto altpiece9at5;
      if(!(puzzle & 0x801a0ll))
        altDoSolve(6, 0x40ll, puzzle | 0x801a0ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x30120ll))
        altDoSolve(6, 0x40ll, puzzle | 0x30120ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x20320ll))
        altDoSolve(6, 0x40ll, puzzle | 0x20320ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1a0020ll))
        altDoSolve(6, 0x40ll, puzzle | 0x1a0020ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x120060ll))
        altDoSolve(6, 0x40ll, puzzle | 0x120060ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x260020ll))
        altDoSolve(6, 0x40ll, puzzle | 0x260020ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x40160ll))
        altDoSolve(6, 0x40ll, puzzle | 0x40160ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x300120ll))
        altDoSolve(6, 0x40ll, puzzle | 0x300120ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x200260ll))
        altDoSolve(6, 0x40ll, puzzle | 0x200260ll,
                   partsUsed | 0x100l);
    altpiece9at5:
      if (partsUsed & 0x200l) goto altpiece10at5;
      if (!(puzzle & 0x1a0ll)) {
         if(!(puzzle & 0x13a0ll))
           altDoSolve(6, 0x40ll, puzzle | 0x13a0ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x7a0ll))
           altDoSolve(6, 0x40ll, puzzle | 0x7a0ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x9e0ll))
           altDoSolve(6, 0x40ll, puzzle | 0x9e0ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x19a0ll))
           altDoSolve(6, 0x40ll, puzzle | 0x19a0ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x20020020ll)) {
         if(!(puzzle & 0x20030060ll))
           altDoSolve(6, 0x40ll, puzzle | 0x20030060ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x60030020ll))
           altDoSolve(6, 0x40ll, puzzle | 0x60030020ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x30060020ll))
           altDoSolve(6, 0x40ll, puzzle | 0x30060020ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0xf20ll))
        altDoSolve(6, 0x40ll, puzzle | 0xf20ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x10070020ll))
        altDoSolve(6, 0x40ll, puzzle | 0x10070020ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x40070020ll))
        altDoSolve(6, 0x40ll, puzzle | 0x40070020ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100920020ll))
        altDoSolve(6, 0x40ll, puzzle | 0x100920020ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100900120ll))
        altDoSolve(6, 0x40ll, puzzle | 0x100900120ll,
                   partsUsed | 0x200l);
    altpiece10at5:
      if (partsUsed & 0x400l) goto altpiece11at5;
      if (!(puzzle & 0x100120ll)) {
         if(!(puzzle & 0x20120120ll))
           altDoSolve(6, 0x40ll, puzzle | 0x20120120ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x920120ll))
           altDoSolve(6, 0x40ll, puzzle | 0x920120ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x900920ll))
           altDoSolve(6, 0x40ll, puzzle | 0x900920ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x120920ll))
           altDoSolve(6, 0x40ll, puzzle | 0x120920ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x100120120ll))
           altDoSolve(6, 0x40ll, puzzle | 0x100120120ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x20020020ll)) {
         if(!(puzzle & 0x120120020ll))
           altDoSolve(6, 0x40ll, puzzle | 0x120120020ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x20060060ll))
           altDoSolve(6, 0x40ll, puzzle | 0x20060060ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x60060020ll))
           altDoSolve(6, 0x40ll, puzzle | 0x60060020ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x320ll)) {
         if(!(puzzle & 0xb60ll))
           altDoSolve(6, 0x40ll, puzzle | 0xb60ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1360ll))
           altDoSolve(6, 0x40ll, puzzle | 0x1360ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1b20ll))
           altDoSolve(6, 0x40ll, puzzle | 0x1b20ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x3e0ll))
        altDoSolve(6, 0x40ll, puzzle | 0x3e0ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0xda0ll))
        altDoSolve(6, 0x40ll, puzzle | 0xda0ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x70060ll))
        altDoSolve(6, 0x40ll, puzzle | 0x70060ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x30030020ll))
        altDoSolve(6, 0x40ll, puzzle | 0x30030020ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x40060060ll))
        altDoSolve(6, 0x40ll, puzzle | 0x40060060ll,
                   partsUsed | 0x400l);
    altpiece11at5:
    break;
   case 6:
      if (partsUsed & 0x2l) goto altpiece2at6;
      if(!(puzzle & 0xf40ll))
        altDoSolve(7, 0x80ll, puzzle | 0xf40ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x30060040ll))
        altDoSolve(7, 0x80ll, puzzle | 0x30060040ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1200240040ll))
        altDoSolve(7, 0x80ll, puzzle | 0x1200240040ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1001200240ll))
        altDoSolve(7, 0x80ll, puzzle | 0x1001200240ll,
                   partsUsed | 0x2l);
    altpiece2at6:
      if (partsUsed & 0x4l) goto altpiece3at6;
      if(!(puzzle & 0x13c0ll))
        altDoSolve(7, 0x80ll, puzzle | 0x13c0ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x200201240ll))
        altDoSolve(7, 0x80ll, puzzle | 0x200201240ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x40070040ll))
        altDoSolve(7, 0x80ll, puzzle | 0x40070040ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x41240040ll))
        altDoSolve(7, 0x80ll, puzzle | 0x41240040ll,
                   partsUsed | 0x4l);
    altpiece3at6:
      if (partsUsed & 0x8l) goto altpiece4at6;
      if(!(puzzle & 0x7c0ll))
        altDoSolve(7, 0x80ll, puzzle | 0x7c0ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x10070040ll))
        altDoSolve(7, 0x80ll, puzzle | 0x10070040ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1001240040ll))
        altDoSolve(7, 0x80ll, puzzle | 0x1001240040ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1200200240ll))
        altDoSolve(7, 0x80ll, puzzle | 0x1200200240ll,
                   partsUsed | 0x8l);
    altpiece4at6:
      if (partsUsed & 0x10l) goto altpiece5at6;
      if (!(puzzle & 0x9240ll)) {
         if(!(puzzle & 0xd340ll))
           altDoSolve(7, 0x80ll, puzzle | 0xd340ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x1049240ll))
           altDoSolve(7, 0x80ll, puzzle | 0x1049240ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x8209240ll))
           altDoSolve(7, 0x80ll, puzzle | 0x8209240ll,
                   partsUsed | 0x10l);
      }
      if (!(puzzle & 0x40040040040ll)) {
         if(!(puzzle & 0x60040060040ll))
           altDoSolve(7, 0x80ll, puzzle | 0x60040060040ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x240040240040ll))
           altDoSolve(7, 0x80ll, puzzle | 0x240040240040ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x40240040240ll))
           altDoSolve(7, 0x80ll, puzzle | 0x40240040240ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x200240200240ll))
        altDoSolve(7, 0x80ll, puzzle | 0x200240200240ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x9241040ll))
        altDoSolve(7, 0x80ll, puzzle | 0x9241040ll,
                   partsUsed | 0x10l);
    altpiece5at6:
      if (partsUsed & 0x20l) goto altpiece6at6;
      if (!(puzzle & 0x9240ll)) {
         if(!(puzzle & 0xd240ll))
           altDoSolve(7, 0x80ll, puzzle | 0xd240ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x8009240ll))
           altDoSolve(7, 0x80ll, puzzle | 0x8009240ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x49240ll))
           altDoSolve(7, 0x80ll, puzzle | 0x49240ll,
                   partsUsed | 0x20l);
      }
      if (!(puzzle & 0x40040040040ll)) {
         if(!(puzzle & 0x60040040040ll))
           altDoSolve(7, 0x80ll, puzzle | 0x60040040040ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x40040040240ll))
           altDoSolve(7, 0x80ll, puzzle | 0x40040040240ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x240040040040ll))
           altDoSolve(7, 0x80ll, puzzle | 0x240040040040ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x9240040ll))
        altDoSolve(7, 0x80ll, puzzle | 0x9240040ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x200200200240ll))
        altDoSolve(7, 0x80ll, puzzle | 0x200200200240ll,
                   partsUsed | 0x20l);
    altpiece6at6:
      if (partsUsed & 0x40l) goto altpiece7at6;
      if (!(puzzle & 0x9240ll)) {
         if(!(puzzle & 0x9340ll))
           altDoSolve(7, 0x80ll, puzzle | 0x9340ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x9a40ll))
           altDoSolve(7, 0x80ll, puzzle | 0x9a40ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x1009240ll))
           altDoSolve(7, 0x80ll, puzzle | 0x1009240ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x209240ll))
           altDoSolve(7, 0x80ll, puzzle | 0x209240ll,
                   partsUsed | 0x40l);
      }
      if (!(puzzle & 0x40040040040ll)) {
         if(!(puzzle & 0x40040060040ll))
           altDoSolve(7, 0x80ll, puzzle | 0x40040060040ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x40060040040ll))
           altDoSolve(7, 0x80ll, puzzle | 0x40060040040ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x40040240040ll))
           altDoSolve(7, 0x80ll, puzzle | 0x40040240040ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x40240040040ll))
           altDoSolve(7, 0x80ll, puzzle | 0x40240040040ll,
                   partsUsed | 0x40l);
      }
    altpiece7at6:
      if (partsUsed & 0x80l) goto altpiece8at6;
      if(!(puzzle & 0x240040240ll))
        altDoSolve(7, 0x80ll, puzzle | 0x240040240ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x240200240ll))
        altDoSolve(7, 0x80ll, puzzle | 0x240200240ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1241040ll))
        altDoSolve(7, 0x80ll, puzzle | 0x1241040ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1041240ll))
        altDoSolve(7, 0x80ll, puzzle | 0x1041240ll,
                   partsUsed | 0x80l);
    altpiece8at6:
      if (partsUsed & 0x100l) goto altpiece9at6;
      if(!(puzzle & 0x340040ll))
        altDoSolve(7, 0x80ll, puzzle | 0x340040ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x100340ll))
        altDoSolve(7, 0x80ll, puzzle | 0x100340ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x60240ll))
        altDoSolve(7, 0x80ll, puzzle | 0x60240ll,
                   partsUsed | 0x100l);
    altpiece9at6:
      if (partsUsed & 0x200l) goto altpiece10at6;
      if(!(puzzle & 0xbc0ll))
        altDoSolve(7, 0x80ll, puzzle | 0xbc0ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x20070040ll))
        altDoSolve(7, 0x80ll, puzzle | 0x20070040ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x201240040ll))
        altDoSolve(7, 0x80ll, puzzle | 0x201240040ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x201200240ll))
        altDoSolve(7, 0x80ll, puzzle | 0x201200240ll,
                   partsUsed | 0x200l);
    altpiece10at6:
      if (partsUsed & 0x400l) goto altpiece11at6;
      if (!(puzzle & 0x1240ll)) {
         if(!(puzzle & 0x1b40ll))
           altDoSolve(7, 0x80ll, puzzle | 0x1b40ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1201240ll))
           altDoSolve(7, 0x80ll, puzzle | 0x1201240ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x241240ll))
           altDoSolve(7, 0x80ll, puzzle | 0x241240ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x40040040ll)) {
         if(!(puzzle & 0x60060040ll))
           altDoSolve(7, 0x80ll, puzzle | 0x60060040ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x40240240ll))
           altDoSolve(7, 0x80ll, puzzle | 0x40240240ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x240240040ll))
           altDoSolve(7, 0x80ll, puzzle | 0x240240040ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x1240240ll))
        altDoSolve(7, 0x80ll, puzzle | 0x1240240ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x200240240ll))
        altDoSolve(7, 0x80ll, puzzle | 0x200240240ll,
                   partsUsed | 0x400l);
    altpiece11at6:
    break;
   case 7:
      if (partsUsed & 0x2l) goto altpiece2at7;
      if(!(puzzle & 0x10090480ll))
        altDoSolve(8, 0x100ll, puzzle | 0x10090480ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x2400480080ll))
        altDoSolve(8, 0x100ll, puzzle | 0x2400480080ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x2002400480ll))
        altDoSolve(8, 0x100ll, puzzle | 0x2002400480ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x9980ll))
        altDoSolve(8, 0x100ll, puzzle | 0x9980ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0xcc80ll))
        altDoSolve(8, 0x100ll, puzzle | 0xcc80ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x200300180ll))
        altDoSolve(8, 0x100ll, puzzle | 0x200300180ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x300180080ll))
        altDoSolve(8, 0x100ll, puzzle | 0x300180080ll,
                   partsUsed | 0x2l);
    altpiece2at7:
      if (partsUsed & 0x4l) goto altpiece3at7;
      if (!(puzzle & 0x80080080ll)) {
         if(!(puzzle & 0x490080080ll))
           altDoSolve(8, 0x100ll, puzzle | 0x490080080ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x82480080ll))
           altDoSolve(8, 0x100ll, puzzle | 0x82480080ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x80380080ll))
           altDoSolve(8, 0x100ll, puzzle | 0x80380080ll,
                   partsUsed | 0x4l);
      }
      if(!(puzzle & 0x400402480ll))
        altDoSolve(8, 0x100ll, puzzle | 0x400402480ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x3c80ll))
        altDoSolve(8, 0x100ll, puzzle | 0x3c80ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x4b80ll))
        altDoSolve(8, 0x100ll, puzzle | 0x4b80ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x100100380ll))
        altDoSolve(8, 0x100ll, puzzle | 0x100100380ll,
                   partsUsed | 0x4l);
    altpiece3at7:
      if (partsUsed & 0x8l) goto altpiece4at7;
      if(!(puzzle & 0x90080480ll))
        altDoSolve(8, 0x100ll, puzzle | 0x90080480ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x2002480080ll))
        altDoSolve(8, 0x100ll, puzzle | 0x2002480080ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x2400400480ll))
        altDoSolve(8, 0x100ll, puzzle | 0x2400400480ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0xc980ll))
        altDoSolve(8, 0x100ll, puzzle | 0xc980ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x9c80ll))
        altDoSolve(8, 0x100ll, puzzle | 0x9c80ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x200380080ll))
        altDoSolve(8, 0x100ll, puzzle | 0x200380080ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x300100180ll))
        altDoSolve(8, 0x100ll, puzzle | 0x300100180ll,
                   partsUsed | 0x8l);
    altpiece4at7:
      if (partsUsed & 0x10l) goto altpiece5at7;
      if (!(puzzle & 0x80080080080ll)) {
         if(!(puzzle & 0x80480080480ll))
           altDoSolve(8, 0x100ll, puzzle | 0x80480080480ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x180080180080ll))
           altDoSolve(8, 0x100ll, puzzle | 0x180080180080ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x80180080180ll))
           altDoSolve(8, 0x100ll, puzzle | 0x80180080180ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x2492080ll))
        altDoSolve(8, 0x100ll, puzzle | 0x2492080ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x90080090080ll))
        altDoSolve(8, 0x100ll, puzzle | 0x90080090080ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x480080480080ll))
        altDoSolve(8, 0x100ll, puzzle | 0x480080480080ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x400480400480ll))
        altDoSolve(8, 0x100ll, puzzle | 0x400480400480ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x100180100180ll))
        altDoSolve(8, 0x100ll, puzzle | 0x100180100180ll,
                   partsUsed | 0x10l);
    altpiece5at7:
      if (partsUsed & 0x20l) goto altpiece6at7;
      if (!(puzzle & 0x80080080080ll)) {
         if(!(puzzle & 0x90080080080ll))
           altDoSolve(8, 0x100ll, puzzle | 0x90080080080ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x80080080480ll))
           altDoSolve(8, 0x100ll, puzzle | 0x80080080480ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x480080080080ll))
           altDoSolve(8, 0x100ll, puzzle | 0x480080080080ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x80080080180ll))
           altDoSolve(8, 0x100ll, puzzle | 0x80080080180ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x180080080080ll))
           altDoSolve(8, 0x100ll, puzzle | 0x180080080080ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x400400400480ll))
        altDoSolve(8, 0x100ll, puzzle | 0x400400400480ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x100100100180ll))
        altDoSolve(8, 0x100ll, puzzle | 0x100100100180ll,
                   partsUsed | 0x20l);
    altpiece6at7:
      if (partsUsed & 0x40l) goto altpiece7at7;
      if (!(puzzle & 0x80080080080ll)) {
         if(!(puzzle & 0x80090080080ll))
           altDoSolve(8, 0x100ll, puzzle | 0x80090080080ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x80080480080ll))
           altDoSolve(8, 0x100ll, puzzle | 0x80080480080ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x80480080080ll))
           altDoSolve(8, 0x100ll, puzzle | 0x80480080080ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x80080180080ll))
           altDoSolve(8, 0x100ll, puzzle | 0x80080180080ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x80180080080ll))
           altDoSolve(8, 0x100ll, puzzle | 0x80180080080ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x2490080ll))
        altDoSolve(8, 0x100ll, puzzle | 0x2490080ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x80080090080ll))
        altDoSolve(8, 0x100ll, puzzle | 0x80080090080ll,
                   partsUsed | 0x40l);
    altpiece7at7:
      if (partsUsed & 0x80l) goto altpiece8at7;
      if(!(puzzle & 0x480080480ll))
        altDoSolve(8, 0x100ll, puzzle | 0x480080480ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x480400480ll))
        altDoSolve(8, 0x100ll, puzzle | 0x480400480ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x2482080ll))
        altDoSolve(8, 0x100ll, puzzle | 0x2482080ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x2082480ll))
        altDoSolve(8, 0x100ll, puzzle | 0x2082480ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1780ll))
        altDoSolve(8, 0x100ll, puzzle | 0x1780ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x6580ll))
        altDoSolve(8, 0x100ll, puzzle | 0x6580ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x6980ll))
        altDoSolve(8, 0x100ll, puzzle | 0x6980ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1e80ll))
        altDoSolve(8, 0x100ll, puzzle | 0x1e80ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x180100180ll))
        altDoSolve(8, 0x100ll, puzzle | 0x180100180ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x180080180ll))
        altDoSolve(8, 0x100ll, puzzle | 0x180080180ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x380280ll))
        altDoSolve(8, 0x100ll, puzzle | 0x380280ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x280380ll))
        altDoSolve(8, 0x100ll, puzzle | 0x280380ll,
                   partsUsed | 0x80l);
    altpiece8at7:
      if (partsUsed & 0x100l) goto altpiece9at7;
      if(!(puzzle & 0x120180ll))
        altDoSolve(8, 0x100ll, puzzle | 0x120180ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x100580ll))
        altDoSolve(8, 0x100ll, puzzle | 0x100580ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xb0080ll))
        altDoSolve(8, 0x100ll, puzzle | 0xb0080ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x980080ll))
        altDoSolve(8, 0x100ll, puzzle | 0x980080ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x480180ll))
        altDoSolve(8, 0x100ll, puzzle | 0x480180ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xc00480ll))
        altDoSolve(8, 0x100ll, puzzle | 0xc00480ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x80c80ll))
        altDoSolve(8, 0x100ll, puzzle | 0x80c80ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x800980ll))
        altDoSolve(8, 0x100ll, puzzle | 0x800980ll,
                   partsUsed | 0x100l);
    altpiece9at7:
      if (partsUsed & 0x200l) goto altpiece10at7;
      if (!(puzzle & 0x80080080ll)) {
         if(!(puzzle & 0x80090480ll))
           altDoSolve(8, 0x100ll, puzzle | 0x80090480ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x90480080ll))
           altDoSolve(8, 0x100ll, puzzle | 0x90480080ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x480090080ll))
           altDoSolve(8, 0x100ll, puzzle | 0x480090080ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x480080ll)) {
         if(!(puzzle & 0x400490080ll))
           altDoSolve(8, 0x100ll, puzzle | 0x400490080ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x10490080ll))
           altDoSolve(8, 0x100ll, puzzle | 0x10490080ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x402480080ll))
           altDoSolve(8, 0x100ll, puzzle | 0x402480080ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x5980ll))
        altDoSolve(8, 0x100ll, puzzle | 0x5980ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x5c80ll))
        altDoSolve(8, 0x100ll, puzzle | 0x5c80ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x402400480ll))
        altDoSolve(8, 0x100ll, puzzle | 0x402400480ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100300180ll))
        altDoSolve(8, 0x100ll, puzzle | 0x100300180ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100380080ll))
        altDoSolve(8, 0x100ll, puzzle | 0x100380080ll,
                   partsUsed | 0x200l);
    altpiece10at7:
      if (partsUsed & 0x400l) goto altpiece11at7;
      if (!(puzzle & 0x400480ll)) {
         if(!(puzzle & 0x80480480ll))
           altDoSolve(8, 0x100ll, puzzle | 0x80480480ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2480480ll))
           altDoSolve(8, 0x100ll, puzzle | 0x2480480ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2402480ll))
           altDoSolve(8, 0x100ll, puzzle | 0x2402480ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x482480ll))
           altDoSolve(8, 0x100ll, puzzle | 0x482480ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x400480480ll))
           altDoSolve(8, 0x100ll, puzzle | 0x400480480ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x980ll)) {
         if(!(puzzle & 0x1b80ll))
           altDoSolve(8, 0x100ll, puzzle | 0x1b80ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xf80ll))
           altDoSolve(8, 0x100ll, puzzle | 0xf80ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2d80ll))
           altDoSolve(8, 0x100ll, puzzle | 0x2d80ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x4d80ll))
           altDoSolve(8, 0x100ll, puzzle | 0x4d80ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1d80ll))
           altDoSolve(8, 0x100ll, puzzle | 0x1d80ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x80080080ll)) {
         if(!(puzzle & 0x480480080ll))
           altDoSolve(8, 0x100ll, puzzle | 0x480480080ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x80180180ll))
           altDoSolve(8, 0x100ll, puzzle | 0x80180180ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x180180080ll))
           altDoSolve(8, 0x100ll, puzzle | 0x180180080ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x100180ll)) {
         if(!(puzzle & 0x100180180ll))
           altDoSolve(8, 0x100ll, puzzle | 0x100180180ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x380180ll))
           altDoSolve(8, 0x100ll, puzzle | 0x380180ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x180380ll))
           altDoSolve(8, 0x100ll, puzzle | 0x180380ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x300380ll))
           altDoSolve(8, 0x100ll, puzzle | 0x300380ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x90090080ll))
        altDoSolve(8, 0x100ll, puzzle | 0x90090080ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x490480ll))
        altDoSolve(8, 0x100ll, puzzle | 0x490480ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x6c80ll))
        altDoSolve(8, 0x100ll, puzzle | 0x6c80ll,
                   partsUsed | 0x400l);
    altpiece11at7:
    break;
   case 8:
      if (partsUsed & 0x2l) goto altpiece2at8;
      if(!(puzzle & 0x20120900ll))
        altDoSolve(9, 0x200ll, puzzle | 0x20120900ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x2f00ll))
        altDoSolve(9, 0x200ll, puzzle | 0x2f00ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x80180300ll))
        altDoSolve(9, 0x200ll, puzzle | 0x80180300ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x4800900100ll))
        altDoSolve(9, 0x200ll, puzzle | 0x4800900100ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x4004800900ll))
        altDoSolve(9, 0x200ll, puzzle | 0x4004800900ll,
                   partsUsed | 0x2l);
    altpiece2at8:
      if (partsUsed & 0x4l) goto altpiece3at8;
      if (!(puzzle & 0x100100100ll)) {
         if(!(puzzle & 0x920100100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x920100100ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x380100100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x380100100ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x104900100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x104900100ll,
                   partsUsed | 0x4l);
      }
      if(!(puzzle & 0xe900ll))
        altDoSolve(9, 0x200ll, puzzle | 0xe900ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x800804900ll))
        altDoSolve(9, 0x200ll, puzzle | 0x800804900ll,
                   partsUsed | 0x4l);
    altpiece3at8:
      if (partsUsed & 0x8l) goto altpiece4at8;
      if(!(puzzle & 0x120100900ll))
        altDoSolve(9, 0x200ll, puzzle | 0x120100900ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x180100300ll))
        altDoSolve(9, 0x200ll, puzzle | 0x180100300ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x6b00ll))
        altDoSolve(9, 0x200ll, puzzle | 0x6b00ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x4004900100ll))
        altDoSolve(9, 0x200ll, puzzle | 0x4004900100ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x4800800900ll))
        altDoSolve(9, 0x200ll, puzzle | 0x4800800900ll,
                   partsUsed | 0x8l);
    altpiece4at8:
      if (partsUsed & 0x10l) goto altpiece5at8;
      if (!(puzzle & 0x100100100100ll)) {
         if(!(puzzle & 0x180100180100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x180100180100ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x900100900100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x900100900100ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x100900100900ll))
           altDoSolve(9, 0x200ll, puzzle | 0x100900100900ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x300100300100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x300100300100ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x100300100300ll))
           altDoSolve(9, 0x200ll, puzzle | 0x100300100300ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x4924100ll))
        altDoSolve(9, 0x200ll, puzzle | 0x4924100ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x120100120100ll))
        altDoSolve(9, 0x200ll, puzzle | 0x120100120100ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x800900800900ll))
        altDoSolve(9, 0x200ll, puzzle | 0x800900800900ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x200300200300ll))
        altDoSolve(9, 0x200ll, puzzle | 0x200300200300ll,
                   partsUsed | 0x10l);
    altpiece5at8:
      if (partsUsed & 0x20l) goto altpiece6at8;
      if (!(puzzle & 0x100100100100ll)) {
         if(!(puzzle & 0x120100100100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x120100100100ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x180100100100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x180100100100ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x100100100900ll))
           altDoSolve(9, 0x200ll, puzzle | 0x100100100900ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x900100100100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x900100100100ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x100100100300ll))
           altDoSolve(9, 0x200ll, puzzle | 0x100100100300ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x300100100100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x300100100100ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x800800800900ll))
        altDoSolve(9, 0x200ll, puzzle | 0x800800800900ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x200200200300ll))
        altDoSolve(9, 0x200ll, puzzle | 0x200200200300ll,
                   partsUsed | 0x20l);
    altpiece6at8:
      if (partsUsed & 0x40l) goto altpiece7at8;
      if (!(puzzle & 0x100100100100ll)) {
         if(!(puzzle & 0x100120100100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x100120100100ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100100180100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x100100180100ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100180100100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x100180100100ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100100900100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x100100900100ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100900100100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x100900100100ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100100300100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x100100300100ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100300100100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x100300100100ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x4920100ll))
        altDoSolve(9, 0x200ll, puzzle | 0x4920100ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x100100120100ll))
        altDoSolve(9, 0x200ll, puzzle | 0x100100120100ll,
                   partsUsed | 0x40l);
    altpiece7at8:
      if (partsUsed & 0x80l) goto altpiece8at8;
      if(!(puzzle & 0x900100900ll))
        altDoSolve(9, 0x200ll, puzzle | 0x900100900ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x900800900ll))
        altDoSolve(9, 0x200ll, puzzle | 0x900800900ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x4904100ll))
        altDoSolve(9, 0x200ll, puzzle | 0x4904100ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x4104900ll))
        altDoSolve(9, 0x200ll, puzzle | 0x4104900ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xcb00ll))
        altDoSolve(9, 0x200ll, puzzle | 0xcb00ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xd300ll))
        altDoSolve(9, 0x200ll, puzzle | 0xd300ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x300200300ll))
        altDoSolve(9, 0x200ll, puzzle | 0x300200300ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x300100300ll))
        altDoSolve(9, 0x200ll, puzzle | 0x300100300ll,
                   partsUsed | 0x80l);
    altpiece8at8:
      if (partsUsed & 0x100l) goto altpiece9at8;
      if(!(puzzle & 0x190100ll))
        altDoSolve(9, 0x200ll, puzzle | 0x190100ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x180900ll))
        altDoSolve(9, 0x200ll, puzzle | 0x180900ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x240300ll))
        altDoSolve(9, 0x200ll, puzzle | 0x240300ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x200b00ll))
        altDoSolve(9, 0x200ll, puzzle | 0x200b00ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x160100ll))
        altDoSolve(9, 0x200ll, puzzle | 0x160100ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x400d00ll))
        altDoSolve(9, 0x200ll, puzzle | 0x400d00ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xd00100ll))
        altDoSolve(9, 0x200ll, puzzle | 0xd00100ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x900300ll))
        altDoSolve(9, 0x200ll, puzzle | 0x900300ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1300100ll))
        altDoSolve(9, 0x200ll, puzzle | 0x1300100ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1800900ll))
        altDoSolve(9, 0x200ll, puzzle | 0x1800900ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x101900ll))
        altDoSolve(9, 0x200ll, puzzle | 0x101900ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1001300ll))
        altDoSolve(9, 0x200ll, puzzle | 0x1001300ll,
                   partsUsed | 0x100l);
    altpiece9at8:
      if (partsUsed & 0x200l) goto altpiece10at8;
      if (!(puzzle & 0x100100100ll)) {
         if(!(puzzle & 0x100120900ll))
           altDoSolve(9, 0x200ll, puzzle | 0x100120900ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x120900100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x120900100ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x900120100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x900120100ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x100180300ll))
           altDoSolve(9, 0x200ll, puzzle | 0x100180300ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x300180100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x300180100ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x180300100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x180300100ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x900100ll)) {
         if(!(puzzle & 0x800920100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x800920100ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x20920100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x20920100ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x804900100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x804900100ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0xd00ll)) {
         if(!(puzzle & 0x9d00ll))
           altDoSolve(9, 0x200ll, puzzle | 0x9d00ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x3d00ll))
           altDoSolve(9, 0x200ll, puzzle | 0x3d00ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x4f00ll))
           altDoSolve(9, 0x200ll, puzzle | 0x4f00ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0xcd00ll))
           altDoSolve(9, 0x200ll, puzzle | 0xcd00ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x7900ll))
        altDoSolve(9, 0x200ll, puzzle | 0x7900ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x80380100ll))
        altDoSolve(9, 0x200ll, puzzle | 0x80380100ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x200380100ll))
        altDoSolve(9, 0x200ll, puzzle | 0x200380100ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x804800900ll))
        altDoSolve(9, 0x200ll, puzzle | 0x804800900ll,
                   partsUsed | 0x200l);
    altpiece10at8:
      if (partsUsed & 0x400l) goto altpiece11at8;
      if (!(puzzle & 0x800900ll)) {
         if(!(puzzle & 0x100900900ll))
           altDoSolve(9, 0x200ll, puzzle | 0x100900900ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x4900900ll))
           altDoSolve(9, 0x200ll, puzzle | 0x4900900ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x4804900ll))
           altDoSolve(9, 0x200ll, puzzle | 0x4804900ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x904900ll))
           altDoSolve(9, 0x200ll, puzzle | 0x904900ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x800900900ll))
           altDoSolve(9, 0x200ll, puzzle | 0x800900900ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x100100100ll)) {
         if(!(puzzle & 0x900900100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x900900100ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x100300300ll))
           altDoSolve(9, 0x200ll, puzzle | 0x100300300ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x300300100ll))
           altDoSolve(9, 0x200ll, puzzle | 0x300300100ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x1900ll)) {
         if(!(puzzle & 0x5b00ll))
           altDoSolve(9, 0x200ll, puzzle | 0x5b00ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x9b00ll))
           altDoSolve(9, 0x200ll, puzzle | 0x9b00ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xd900ll))
           altDoSolve(9, 0x200ll, puzzle | 0xd900ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x120120100ll))
        altDoSolve(9, 0x200ll, puzzle | 0x120120100ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x920900ll))
        altDoSolve(9, 0x200ll, puzzle | 0x920900ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x1f00ll))
        altDoSolve(9, 0x200ll, puzzle | 0x1f00ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x6d00ll))
        altDoSolve(9, 0x200ll, puzzle | 0x6d00ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x380300ll))
        altDoSolve(9, 0x200ll, puzzle | 0x380300ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x180180100ll))
        altDoSolve(9, 0x200ll, puzzle | 0x180180100ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x200300300ll))
        altDoSolve(9, 0x200ll, puzzle | 0x200300300ll,
                   partsUsed | 0x400l);
    altpiece11at8:
    break;
   case 9:
      if (partsUsed & 0x2l) goto altpiece2at9;
      if(!(puzzle & 0x40241200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x40241200ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x7a00ll))
        altDoSolve(10, 0x400ll, puzzle | 0x7a00ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x180300200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x180300200ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x9001200200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x9001200200ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x8009001200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x8009001200ll,
                   partsUsed | 0x2l);
    altpiece2at9:
      if (partsUsed & 0x4l) goto altpiece3at9;
      if (!(puzzle & 0x200200200ll)) {
         if(!(puzzle & 0x1240200200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x1240200200ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x200380200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x200380200ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x209200200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x209200200ll,
                   partsUsed | 0x4l);
      }
      if(!(puzzle & 0x9e00ll))
        altDoSolve(10, 0x400ll, puzzle | 0x9e00ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x1001009200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x1001009200ll,
                   partsUsed | 0x4l);
    altpiece3at9:
      if (partsUsed & 0x8l) goto altpiece4at9;
      if(!(puzzle & 0x240201200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x240201200ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x3e00ll))
        altDoSolve(10, 0x400ll, puzzle | 0x3e00ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x80380200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x80380200ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x8009200200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x8009200200ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x9001001200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x9001001200ll,
                   partsUsed | 0x8l);
    altpiece4at9:
      if (partsUsed & 0x10l) goto altpiece5at9;
      if (!(puzzle & 0x200200200200ll)) {
         if(!(puzzle & 0x300200300200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x300200300200ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x1200201200200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x1200201200200ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x201200201200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x201200201200ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x9248200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x9248200ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x240200240200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x240200240200ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x1001201001200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x1001201001200ll,
                   partsUsed | 0x10l);
    altpiece5at9:
      if (partsUsed & 0x20l) goto altpiece6at9;
      if (!(puzzle & 0x200200200200ll)) {
         if(!(puzzle & 0x240200200200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x240200200200ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x300200200200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x300200200200ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x200200201200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x200200201200ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x1200200200200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x1200200200200ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x1001001001200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x1001001001200ll,
                   partsUsed | 0x20l);
    altpiece6at9:
      if (partsUsed & 0x40l) goto altpiece7at9;
      if (!(puzzle & 0x200200200200ll)) {
         if(!(puzzle & 0x200240200200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x200240200200ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x200200300200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x200200300200ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x200300200200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x200300200200ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x200201200200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x200201200200ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x201200200200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x201200200200ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x9240200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x9240200ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x200200240200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x200200240200ll,
                   partsUsed | 0x40l);
    altpiece7at9:
      if (partsUsed & 0x80l) goto altpiece8at9;
      if(!(puzzle & 0x1200201200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x1200201200ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1201001200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x1201001200ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x9208200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x9208200ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x8209200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x8209200ll,
                   partsUsed | 0x80l);
    altpiece8at9:
      if (partsUsed & 0x100l) goto altpiece9at9;
      if(!(puzzle & 0x320200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x320200ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x301200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x301200ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x801a00ll))
        altDoSolve(10, 0x400ll, puzzle | 0x801a00ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1a00200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x1a00200ll,
                   partsUsed | 0x100l);
    altpiece9at9:
      if (partsUsed & 0x200l) goto altpiece10at9;
      if (!(puzzle & 0x200200200ll)) {
         if(!(puzzle & 0x200241200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x200241200ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x241200200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x241200200ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x1200240200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x1200240200ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x1200200ll)) {
         if(!(puzzle & 0x1001240200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x1001240200ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x41240200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x41240200ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x1009200200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x1009200200ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x5e00ll))
        altDoSolve(10, 0x400ll, puzzle | 0x5e00ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100380200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x100380200ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x1009001200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x1009001200ll,
                   partsUsed | 0x200l);
    altpiece10at9:
      if (partsUsed & 0x400l) goto altpiece11at9;
      if (!(puzzle & 0x9200ll)) {
         if(!(puzzle & 0xda00ll))
           altDoSolve(10, 0x400ll, puzzle | 0xda00ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x9009200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x9009200ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1209200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x1209200ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x200200200ll)) {
         if(!(puzzle & 0x300300200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x300300200ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x201201200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x201201200ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1201200200ll))
           altDoSolve(10, 0x400ll, puzzle | 0x1201200200ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x240240200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x240240200ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x1241200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x1241200ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x9201200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x9201200ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x1001201200ll))
        altDoSolve(10, 0x400ll, puzzle | 0x1001201200ll,
                   partsUsed | 0x400l);
    altpiece11at9:
    break;
   case 10:
      if (partsUsed & 0x2l) goto altpiece2at10;
      if(!(puzzle & 0x90480400ll))
        altDoSolve(11, 0x800ll, puzzle | 0x90480400ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x80482400ll))
        altDoSolve(11, 0x800ll, puzzle | 0x80482400ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1001800c00ll))
        altDoSolve(11, 0x800ll, puzzle | 0x1001800c00ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1800c00400ll))
        altDoSolve(11, 0x800ll, puzzle | 0x1800c00400ll,
                   partsUsed | 0x2l);
    altpiece2at10:
      if (partsUsed & 0x4l) goto altpiece3at10;
      if (!(puzzle & 0x400400400ll)) {
         if(!(puzzle & 0x400490400ll))
           altDoSolve(11, 0x800ll, puzzle | 0x400490400ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x2480400400ll))
           altDoSolve(11, 0x800ll, puzzle | 0x2480400400ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x401c00400ll))
           altDoSolve(11, 0x800ll, puzzle | 0x401c00400ll,
                   partsUsed | 0x4l);
      }
      if(!(puzzle & 0x800801c00ll))
        altDoSolve(11, 0x800ll, puzzle | 0x800801c00ll,
                   partsUsed | 0x4l);
    altpiece3at10:
      if (partsUsed & 0x8l) goto altpiece4at10;
      if(!(puzzle & 0x10490400ll))
        altDoSolve(11, 0x800ll, puzzle | 0x10490400ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x480402400ll))
        altDoSolve(11, 0x800ll, puzzle | 0x480402400ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1001c00400ll))
        altDoSolve(11, 0x800ll, puzzle | 0x1001c00400ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1800800c00ll))
        altDoSolve(11, 0x800ll, puzzle | 0x1800800c00ll,
                   partsUsed | 0x8l);
    altpiece4at10:
      if (partsUsed & 0x10l) goto altpiece5at10;
      if (!(puzzle & 0x400400400400ll)) {
         if(!(puzzle & 0x480400480400ll))
           altDoSolve(11, 0x800ll, puzzle | 0x480400480400ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x2400402400400ll))
           altDoSolve(11, 0x800ll, puzzle | 0x2400402400400ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x402400402400ll))
           altDoSolve(11, 0x800ll, puzzle | 0x402400402400ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0xc00400c00400ll))
           altDoSolve(11, 0x800ll, puzzle | 0xc00400c00400ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x400c00400c00ll))
           altDoSolve(11, 0x800ll, puzzle | 0x400c00400c00ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x2002402002400ll))
        altDoSolve(11, 0x800ll, puzzle | 0x2002402002400ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x800c00800c00ll))
        altDoSolve(11, 0x800ll, puzzle | 0x800c00800c00ll,
                   partsUsed | 0x10l);
    altpiece5at10:
      if (partsUsed & 0x20l) goto altpiece6at10;
      if (!(puzzle & 0x400400400400ll)) {
         if(!(puzzle & 0x480400400400ll))
           altDoSolve(11, 0x800ll, puzzle | 0x480400400400ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x400400402400ll))
           altDoSolve(11, 0x800ll, puzzle | 0x400400402400ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x2400400400400ll))
           altDoSolve(11, 0x800ll, puzzle | 0x2400400400400ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x400400400c00ll))
           altDoSolve(11, 0x800ll, puzzle | 0x400400400c00ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0xc00400400400ll))
           altDoSolve(11, 0x800ll, puzzle | 0xc00400400400ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x2002002002400ll))
        altDoSolve(11, 0x800ll, puzzle | 0x2002002002400ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x800800800c00ll))
        altDoSolve(11, 0x800ll, puzzle | 0x800800800c00ll,
                   partsUsed | 0x20l);
    altpiece6at10:
      if (partsUsed & 0x40l) goto altpiece7at10;
      if (!(puzzle & 0x400400400400ll)) {
         if(!(puzzle & 0x400480400400ll))
           altDoSolve(11, 0x800ll, puzzle | 0x400480400400ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x400402400400ll))
           altDoSolve(11, 0x800ll, puzzle | 0x400402400400ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x402400400400ll))
           altDoSolve(11, 0x800ll, puzzle | 0x402400400400ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x400400c00400ll))
           altDoSolve(11, 0x800ll, puzzle | 0x400400c00400ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x400c00400400ll))
           altDoSolve(11, 0x800ll, puzzle | 0x400c00400400ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x2490400ll))
        altDoSolve(11, 0x800ll, puzzle | 0x2490400ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x400400480400ll))
        altDoSolve(11, 0x800ll, puzzle | 0x400400480400ll,
                   partsUsed | 0x40l);
    altpiece7at10:
      if (partsUsed & 0x80l) goto altpiece8at10;
      if(!(puzzle & 0x2400402400ll))
        altDoSolve(11, 0x800ll, puzzle | 0x2400402400ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x2402002400ll))
        altDoSolve(11, 0x800ll, puzzle | 0x2402002400ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xbc00ll))
        altDoSolve(11, 0x800ll, puzzle | 0xbc00ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xf400ll))
        altDoSolve(11, 0x800ll, puzzle | 0xf400ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xc00800c00ll))
        altDoSolve(11, 0x800ll, puzzle | 0xc00800c00ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xc00400c00ll))
        altDoSolve(11, 0x800ll, puzzle | 0xc00400c00ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1c01400ll))
        altDoSolve(11, 0x800ll, puzzle | 0x1c01400ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1401c00ll))
        altDoSolve(11, 0x800ll, puzzle | 0x1401c00ll,
                   partsUsed | 0x80l);
    altpiece8at10:
      if (partsUsed & 0x100l) goto altpiece9at10;
      if(!(puzzle & 0x900c00ll))
        altDoSolve(11, 0x800ll, puzzle | 0x900c00ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x802c00ll))
        altDoSolve(11, 0x800ll, puzzle | 0x802c00ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x580400ll))
        altDoSolve(11, 0x800ll, puzzle | 0x580400ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x4c00400ll))
        altDoSolve(11, 0x800ll, puzzle | 0x4c00400ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x2400c00ll))
        altDoSolve(11, 0x800ll, puzzle | 0x2400c00ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x6002400ll))
        altDoSolve(11, 0x800ll, puzzle | 0x6002400ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x406400ll))
        altDoSolve(11, 0x800ll, puzzle | 0x406400ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x4004c00ll))
        altDoSolve(11, 0x800ll, puzzle | 0x4004c00ll,
                   partsUsed | 0x100l);
    altpiece9at10:
      if (partsUsed & 0x200l) goto altpiece10at10;
      if (!(puzzle & 0x480400ll)) {
         if(!(puzzle & 0x80490400ll))
           altDoSolve(11, 0x800ll, puzzle | 0x80490400ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x400482400ll))
           altDoSolve(11, 0x800ll, puzzle | 0x400482400ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x2002480400ll))
           altDoSolve(11, 0x800ll, puzzle | 0x2002480400ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x2400480400ll))
           altDoSolve(11, 0x800ll, puzzle | 0x2400480400ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x82480400ll))
           altDoSolve(11, 0x800ll, puzzle | 0x82480400ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x482400400ll))
        altDoSolve(11, 0x800ll, puzzle | 0x482400400ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x801800c00ll))
        altDoSolve(11, 0x800ll, puzzle | 0x801800c00ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x801c00400ll))
        altDoSolve(11, 0x800ll, puzzle | 0x801c00400ll,
                   partsUsed | 0x200l);
    altpiece10at10:
      if (partsUsed & 0x400l) goto altpiece11at10;
      if (!(puzzle & 0x2400400ll)) {
         if(!(puzzle & 0x402402400ll))
           altDoSolve(11, 0x800ll, puzzle | 0x402402400ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2402400400ll))
           altDoSolve(11, 0x800ll, puzzle | 0x2402400400ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2002402400ll))
           altDoSolve(11, 0x800ll, puzzle | 0x2002402400ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x4c00ll)) {
         if(!(puzzle & 0xdc00ll))
           altDoSolve(11, 0x800ll, puzzle | 0xdc00ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x7c00ll))
           altDoSolve(11, 0x800ll, puzzle | 0x7c00ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xec00ll))
           altDoSolve(11, 0x800ll, puzzle | 0xec00ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0xc00400ll)) {
         if(!(puzzle & 0x800c00c00ll))
           altDoSolve(11, 0x800ll, puzzle | 0x800c00c00ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x400c00c00ll))
           altDoSolve(11, 0x800ll, puzzle | 0x400c00c00ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1c00c00ll))
           altDoSolve(11, 0x800ll, puzzle | 0x1c00c00ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xc00c00400ll))
           altDoSolve(11, 0x800ll, puzzle | 0xc00c00400ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xc01c00ll))
           altDoSolve(11, 0x800ll, puzzle | 0xc01c00ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x480480400ll))
        altDoSolve(11, 0x800ll, puzzle | 0x480480400ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x2482400ll))
        altDoSolve(11, 0x800ll, puzzle | 0x2482400ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x1801c00ll))
        altDoSolve(11, 0x800ll, puzzle | 0x1801c00ll,
                   partsUsed | 0x400l);
    altpiece11at10:
    break;
   case 11:
      if (partsUsed & 0x2l) goto altpiece2at11;
      if(!(puzzle & 0x400c01800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x400c01800ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x120900800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x120900800ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x100904800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x100904800ll,
                   partsUsed | 0x2l);
    altpiece2at11:
      if (partsUsed & 0x4l) goto altpiece3at11;
      if (!(puzzle & 0x800800800ll)) {
         if(!(puzzle & 0x800920800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x800920800ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x4900800800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x4900800800ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x1c00800800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x1c00800800ll,
                   partsUsed | 0x4l);
      }
    altpiece3at11:
      if (partsUsed & 0x8l) goto altpiece4at11;
      if(!(puzzle & 0x20920800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x20920800ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x900804800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x900804800ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0xc00801800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0xc00801800ll,
                   partsUsed | 0x8l);
    altpiece4at11:
      if (partsUsed & 0x10l) goto altpiece5at11;
      if (!(puzzle & 0x800800800800ll)) {
         if(!(puzzle & 0x900800900800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x900800900800ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0xc00800c00800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0xc00800c00800ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x4800804800800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x4800804800800ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x804800804800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x804800804800ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x1800801800800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x1800801800800ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x801800801800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x801800801800ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x4004804004800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x4004804004800ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x1001801001800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x1001801001800ll,
                   partsUsed | 0x10l);
    altpiece5at11:
      if (partsUsed & 0x20l) goto altpiece6at11;
      if (!(puzzle & 0x800800800800ll)) {
         if(!(puzzle & 0x900800800800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x900800800800ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0xc00800800800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0xc00800800800ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x800800804800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x800800804800ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x4800800800800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x4800800800800ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x800800801800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x800800801800ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x1800800800800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x1800800800800ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x4004004004800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x4004004004800ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x1001001001800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x1001001001800ll,
                   partsUsed | 0x20l);
    altpiece6at11:
      if (partsUsed & 0x40l) goto altpiece7at11;
      if (!(puzzle & 0x800800800800ll)) {
         if(!(puzzle & 0x800900800800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x800900800800ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800800c00800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x800800c00800ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800c00800800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x800c00800800ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800804800800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x800804800800ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x804800800800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x804800800800ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800801800800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x800801800800ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x801800800800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x801800800800ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x4920800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x4920800ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x800800900800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x800800900800ll,
                   partsUsed | 0x40l);
    altpiece7at11:
      if (partsUsed & 0x80l) goto altpiece8at11;
      if(!(puzzle & 0x4800804800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x4800804800ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x4804004800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x4804004800ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1801001800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x1801001800ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1800801800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x1800801800ll,
                   partsUsed | 0x80l);
    altpiece8at11:
      if (partsUsed & 0x100l) goto altpiece9at11;
      if(!(puzzle & 0xc80800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0xc80800ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xc04800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0xc04800ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1201800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x1201800ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1005800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x1005800ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xb00800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0xb00800ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x2006800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x2006800ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x6800800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x6800800ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x4801800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x4801800ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x9800800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x9800800ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xc004800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0xc004800ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x80c800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x80c800ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x8009800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x8009800ll,
                   partsUsed | 0x100l);
    altpiece9at11:
      if (partsUsed & 0x200l) goto altpiece10at11;
      if (!(puzzle & 0x900800ll)) {
         if(!(puzzle & 0x100920800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x100920800ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x800904800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x800904800ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x4004900800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x4004900800ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x4800900800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x4800900800ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x104900800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x104900800ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x800800800ll)) {
         if(!(puzzle & 0x904800800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x904800800ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x800c01800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x800c01800ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x1800c00800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x1800c00800ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0xc01800800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0xc01800800ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x401c00800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x401c00800ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x1001c00800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x1001c00800ll,
                   partsUsed | 0x200l);
    altpiece10at11:
      if (partsUsed & 0x400l) goto altpiece11at11;
      if (!(puzzle & 0x4800800ll)) {
         if(!(puzzle & 0x804804800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x804804800ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x4804800800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x4804800800ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x4004804800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x4004804800ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x1800800ll)) {
         if(!(puzzle & 0x1001801800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x1001801800ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x801801800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x801801800ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1801800800ll))
           altDoSolve(12, 0x1000ll, puzzle | 0x1801800800ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x900900800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x900900800ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x4904800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x4904800ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0xf800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0xf800ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x1c01800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0x1c01800ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0xc00c00800ll))
        altDoSolve(12, 0x1000ll, puzzle | 0xc00c00800ll,
                   partsUsed | 0x400l);
    altpiece11at11:
    break;
   case 12:
      if (partsUsed & 0x2l) goto altpiece2at12;
      if(!(puzzle & 0xc01801000ll))
        altDoSolve(13, 0x2000ll, puzzle | 0xc01801000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x241201000ll))
        altDoSolve(13, 0x2000ll, puzzle | 0x241201000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x201209000ll))
        altDoSolve(13, 0x2000ll, puzzle | 0x201209000ll,
                   partsUsed | 0x2l);
    altpiece2at12:
      if (partsUsed & 0x4l) goto altpiece3at12;
      if (!(puzzle & 0x1001001000ll)) {
         if(!(puzzle & 0x1001241000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x1001241000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x9201001000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x9201001000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x1001c01000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x1001c01000ll,
                   partsUsed | 0x4l);
      }
    altpiece3at12:
      if (partsUsed & 0x8l) goto altpiece4at12;
      if(!(puzzle & 0x401c01000ll))
        altDoSolve(13, 0x2000ll, puzzle | 0x401c01000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x41241000ll))
        altDoSolve(13, 0x2000ll, puzzle | 0x41241000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1201009000ll))
        altDoSolve(13, 0x2000ll, puzzle | 0x1201009000ll,
                   partsUsed | 0x8l);
    altpiece4at12:
      if (partsUsed & 0x10l) goto altpiece5at12;
      if (!(puzzle & 0x1001001001000ll)) {
         if(!(puzzle & 0x1201001201000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x1201001201000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x1801001801000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x1801001801000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x9001009001000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x9001009001000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x1009001009000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x1009001009000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x8009008009000ll))
        altDoSolve(13, 0x2000ll, puzzle | 0x8009008009000ll,
                   partsUsed | 0x10l);
    altpiece5at12:
      if (partsUsed & 0x20l) goto altpiece6at12;
      if (!(puzzle & 0x1001001001000ll)) {
         if(!(puzzle & 0x1201001001000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x1201001001000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x1801001001000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x1801001001000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x1001001009000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x1001001009000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x9001001001000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x9001001001000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x8008008009000ll))
        altDoSolve(13, 0x2000ll, puzzle | 0x8008008009000ll,
                   partsUsed | 0x20l);
    altpiece6at12:
      if (partsUsed & 0x40l) goto altpiece7at12;
      if (!(puzzle & 0x1001001001000ll)) {
         if(!(puzzle & 0x1001201001000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x1001201001000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x1001001801000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x1001001801000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x1001801001000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x1001801001000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x1001009001000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x1001009001000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x1009001001000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x1009001001000ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x9241000ll))
        altDoSolve(13, 0x2000ll, puzzle | 0x9241000ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x1001001201000ll))
        altDoSolve(13, 0x2000ll, puzzle | 0x1001001201000ll,
                   partsUsed | 0x40l);
    altpiece7at12:
      if (partsUsed & 0x80l) goto altpiece8at12;
      if(!(puzzle & 0x9001009000ll))
        altDoSolve(13, 0x2000ll, puzzle | 0x9001009000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x9008009000ll))
        altDoSolve(13, 0x2000ll, puzzle | 0x9008009000ll,
                   partsUsed | 0x80l);
    altpiece8at12:
      if (partsUsed & 0x100l) goto altpiece9at12;
      if(!(puzzle & 0x1901000ll))
        altDoSolve(13, 0x2000ll, puzzle | 0x1901000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1809000ll))
        altDoSolve(13, 0x2000ll, puzzle | 0x1809000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x400d000ll))
        altDoSolve(13, 0x2000ll, puzzle | 0x400d000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xd001000ll))
        altDoSolve(13, 0x2000ll, puzzle | 0xd001000ll,
                   partsUsed | 0x100l);
    altpiece9at12:
      if (partsUsed & 0x200l) goto altpiece10at12;
      if (!(puzzle & 0x1201000ll)) {
         if(!(puzzle & 0x201241000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x201241000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x1001209000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x1001209000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x8009201000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x8009201000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x9001201000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x9001201000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x209201000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x209201000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x1209001000ll))
        altDoSolve(13, 0x2000ll, puzzle | 0x1209001000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x801c01000ll))
        altDoSolve(13, 0x2000ll, puzzle | 0x801c01000ll,
                   partsUsed | 0x200l);
    altpiece10at12:
      if (partsUsed & 0x400l) goto altpiece11at12;
      if (!(puzzle & 0x1001001000ll)) {
         if(!(puzzle & 0x1801801000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x1801801000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1009009000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x1009009000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x9009001000ll))
           altDoSolve(13, 0x2000ll, puzzle | 0x9009001000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x1201201000ll))
        altDoSolve(13, 0x2000ll, puzzle | 0x1201201000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x9209000ll))
        altDoSolve(13, 0x2000ll, puzzle | 0x9209000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x8009009000ll))
        altDoSolve(13, 0x2000ll, puzzle | 0x8009009000ll,
                   partsUsed | 0x400l);
    altpiece11at12:
    break;
   case 13:
      if (partsUsed & 0x2l) goto altpiece2at13;
      if(!(puzzle & 0x482402000ll))
        altDoSolve(14, 0x4000ll, puzzle | 0x482402000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x800c006000ll))
        altDoSolve(14, 0x4000ll, puzzle | 0x800c006000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0xc006002000ll))
        altDoSolve(14, 0x4000ll, puzzle | 0xc006002000ll,
                   partsUsed | 0x2l);
    altpiece2at13:
      if (partsUsed & 0x4l) goto altpiece3at13;
      if(!(puzzle & 0x400400e000ll))
        altDoSolve(14, 0x4000ll, puzzle | 0x400400e000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x2002482000ll))
        altDoSolve(14, 0x4000ll, puzzle | 0x2002482000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x200e002000ll))
        altDoSolve(14, 0x4000ll, puzzle | 0x200e002000ll,
                   partsUsed | 0x4l);
    altpiece3at13:
      if (partsUsed & 0x8l) goto altpiece4at13;
      if(!(puzzle & 0x82482000ll))
        altDoSolve(14, 0x4000ll, puzzle | 0x82482000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x800e002000ll))
        altDoSolve(14, 0x4000ll, puzzle | 0x800e002000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0xc004006000ll))
        altDoSolve(14, 0x4000ll, puzzle | 0xc004006000ll,
                   partsUsed | 0x8l);
    altpiece4at13:
      if (partsUsed & 0x10l) goto altpiece5at13;
      if (!(puzzle & 0x2002002002000ll)) {
         if(!(puzzle & 0x2402002402000ll))
           altDoSolve(14, 0x4000ll, puzzle | 0x2402002402000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x6002006002000ll))
           altDoSolve(14, 0x4000ll, puzzle | 0x6002006002000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x2006002006000ll))
           altDoSolve(14, 0x4000ll, puzzle | 0x2006002006000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x4006004006000ll))
        altDoSolve(14, 0x4000ll, puzzle | 0x4006004006000ll,
                   partsUsed | 0x10l);
    altpiece5at13:
      if (partsUsed & 0x20l) goto altpiece6at13;
      if (!(puzzle & 0x2002002002000ll)) {
         if(!(puzzle & 0x2402002002000ll))
           altDoSolve(14, 0x4000ll, puzzle | 0x2402002002000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x2002002006000ll))
           altDoSolve(14, 0x4000ll, puzzle | 0x2002002006000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x6002002002000ll))
           altDoSolve(14, 0x4000ll, puzzle | 0x6002002002000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x2492000ll))
        altDoSolve(14, 0x4000ll, puzzle | 0x2492000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x4004004006000ll))
        altDoSolve(14, 0x4000ll, puzzle | 0x4004004006000ll,
                   partsUsed | 0x20l);
    altpiece6at13:
      if (partsUsed & 0x40l) goto altpiece7at13;
      if (!(puzzle & 0x2002002002000ll)) {
         if(!(puzzle & 0x2002402002000ll))
           altDoSolve(14, 0x4000ll, puzzle | 0x2002402002000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2002002402000ll))
           altDoSolve(14, 0x4000ll, puzzle | 0x2002002402000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2002006002000ll))
           altDoSolve(14, 0x4000ll, puzzle | 0x2002006002000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2006002002000ll))
           altDoSolve(14, 0x4000ll, puzzle | 0x2006002002000ll,
                   partsUsed | 0x40l);
      }
    altpiece7at13:
      if (partsUsed & 0x80l) goto altpiece8at13;
      if(!(puzzle & 0x6004006000ll))
        altDoSolve(14, 0x4000ll, puzzle | 0x6004006000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x6002006000ll))
        altDoSolve(14, 0x4000ll, puzzle | 0x6002006000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xe00a000ll))
        altDoSolve(14, 0x4000ll, puzzle | 0xe00a000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xa00e000ll))
        altDoSolve(14, 0x4000ll, puzzle | 0xa00e000ll,
                   partsUsed | 0x80l);
    altpiece8at13:
      if (partsUsed & 0x100l) goto altpiece9at13;
      if(!(puzzle & 0x4806000ll))
        altDoSolve(14, 0x4000ll, puzzle | 0x4806000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x2c02000ll))
        altDoSolve(14, 0x4000ll, puzzle | 0x2c02000ll,
                   partsUsed | 0x100l);
    altpiece9at13:
      if (partsUsed & 0x200l) goto altpiece10at13;
      if(!(puzzle & 0x402482000ll))
        altDoSolve(14, 0x4000ll, puzzle | 0x402482000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x400c006000ll))
        altDoSolve(14, 0x4000ll, puzzle | 0x400c006000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x400e002000ll))
        altDoSolve(14, 0x4000ll, puzzle | 0x400e002000ll,
                   partsUsed | 0x200l);
    altpiece10at13:
      if (partsUsed & 0x400l) goto altpiece11at13;
      if (!(puzzle & 0x2002002000ll)) {
         if(!(puzzle & 0x2402402000ll))
           altDoSolve(14, 0x4000ll, puzzle | 0x2402402000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2006006000ll))
           altDoSolve(14, 0x4000ll, puzzle | 0x2006006000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x6006002000ll))
           altDoSolve(14, 0x4000ll, puzzle | 0x6006002000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x4006000ll)) {
         if(!(puzzle & 0x4006006000ll))
           altDoSolve(14, 0x4000ll, puzzle | 0x4006006000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xe006000ll))
           altDoSolve(14, 0x4000ll, puzzle | 0xe006000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x600e000ll))
           altDoSolve(14, 0x4000ll, puzzle | 0x600e000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xc00e000ll))
           altDoSolve(14, 0x4000ll, puzzle | 0xc00e000ll,
                   partsUsed | 0x400l);
      }
    altpiece11at13:
    break;
   case 14:
      if (partsUsed & 0x2l) goto altpiece2at14;
      if(!(puzzle & 0x904804000ll))
        altDoSolve(15, 0x8000ll, puzzle | 0x904804000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x200600c000ll))
        altDoSolve(15, 0x8000ll, puzzle | 0x200600c000ll,
                   partsUsed | 0x2l);
    altpiece2at14:
      if (partsUsed & 0x4l) goto altpiece3at14;
      if(!(puzzle & 0x4004904000ll))
        altDoSolve(15, 0x8000ll, puzzle | 0x4004904000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0xe004004000ll))
        altDoSolve(15, 0x8000ll, puzzle | 0xe004004000ll,
                   partsUsed | 0x4l);
    altpiece3at14:
      if (partsUsed & 0x8l) goto altpiece4at14;
      if(!(puzzle & 0x104904000ll))
        altDoSolve(15, 0x8000ll, puzzle | 0x104904000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x600400c000ll))
        altDoSolve(15, 0x8000ll, puzzle | 0x600400c000ll,
                   partsUsed | 0x8l);
    altpiece4at14:
      if (partsUsed & 0x10l) goto altpiece5at14;
      if (!(puzzle & 0x4004004004000ll)) {
         if(!(puzzle & 0x4804004804000ll))
           altDoSolve(15, 0x8000ll, puzzle | 0x4804004804000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x6004006004000ll))
           altDoSolve(15, 0x8000ll, puzzle | 0x6004006004000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0xc00400c004000ll))
           altDoSolve(15, 0x8000ll, puzzle | 0xc00400c004000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x400c00400c000ll))
           altDoSolve(15, 0x8000ll, puzzle | 0x400c00400c000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x800c00800c000ll))
        altDoSolve(15, 0x8000ll, puzzle | 0x800c00800c000ll,
                   partsUsed | 0x10l);
    altpiece5at14:
      if (partsUsed & 0x20l) goto altpiece6at14;
      if (!(puzzle & 0x4004004004000ll)) {
         if(!(puzzle & 0x4804004004000ll))
           altDoSolve(15, 0x8000ll, puzzle | 0x4804004004000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x6004004004000ll))
           altDoSolve(15, 0x8000ll, puzzle | 0x6004004004000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x400400400c000ll))
           altDoSolve(15, 0x8000ll, puzzle | 0x400400400c000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0xc004004004000ll))
           altDoSolve(15, 0x8000ll, puzzle | 0xc004004004000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x4924000ll))
        altDoSolve(15, 0x8000ll, puzzle | 0x4924000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x800800800c000ll))
        altDoSolve(15, 0x8000ll, puzzle | 0x800800800c000ll,
                   partsUsed | 0x20l);
    altpiece6at14:
      if (partsUsed & 0x40l) goto altpiece7at14;
      if (!(puzzle & 0x4004004004000ll)) {
         if(!(puzzle & 0x4004804004000ll))
           altDoSolve(15, 0x8000ll, puzzle | 0x4004804004000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4004004804000ll))
           altDoSolve(15, 0x8000ll, puzzle | 0x4004004804000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4004006004000ll))
           altDoSolve(15, 0x8000ll, puzzle | 0x4004006004000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4006004004000ll))
           altDoSolve(15, 0x8000ll, puzzle | 0x4006004004000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x400400c004000ll))
           altDoSolve(15, 0x8000ll, puzzle | 0x400400c004000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x400c004004000ll))
           altDoSolve(15, 0x8000ll, puzzle | 0x400c004004000ll,
                   partsUsed | 0x40l);
      }
    altpiece7at14:
      if (partsUsed & 0x80l) goto altpiece8at14;
      if(!(puzzle & 0xc00800c000ll))
        altDoSolve(15, 0x8000ll, puzzle | 0xc00800c000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xc00400c000ll))
        altDoSolve(15, 0x8000ll, puzzle | 0xc00400c000ll,
                   partsUsed | 0x80l);
    altpiece8at14:
      if (partsUsed & 0x100l) goto altpiece9at14;
      if(!(puzzle & 0x5804000ll))
        altDoSolve(15, 0x8000ll, puzzle | 0x5804000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x6404000ll))
        altDoSolve(15, 0x8000ll, puzzle | 0x6404000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x900c000ll))
        altDoSolve(15, 0x8000ll, puzzle | 0x900c000ll,
                   partsUsed | 0x100l);
    altpiece9at14:
      if (partsUsed & 0x200l) goto altpiece10at14;
      if (!(puzzle & 0x4004004000ll)) {
         if(!(puzzle & 0x400600c000ll))
           altDoSolve(15, 0x8000ll, puzzle | 0x400600c000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0xc006004000ll))
           altDoSolve(15, 0x8000ll, puzzle | 0xc006004000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x600c004000ll))
           altDoSolve(15, 0x8000ll, puzzle | 0x600c004000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x804904000ll))
        altDoSolve(15, 0x8000ll, puzzle | 0x804904000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x200e004000ll))
        altDoSolve(15, 0x8000ll, puzzle | 0x200e004000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x800e004000ll))
        altDoSolve(15, 0x8000ll, puzzle | 0x800e004000ll,
                   partsUsed | 0x200l);
    altpiece10at14:
      if (partsUsed & 0x400l) goto altpiece11at14;
      if (!(puzzle & 0x4004004000ll)) {
         if(!(puzzle & 0x4804804000ll))
           altDoSolve(15, 0x8000ll, puzzle | 0x4804804000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x6006004000ll))
           altDoSolve(15, 0x8000ll, puzzle | 0x6006004000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x400c00c000ll))
           altDoSolve(15, 0x8000ll, puzzle | 0x400c00c000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xc00c004000ll))
           altDoSolve(15, 0x8000ll, puzzle | 0xc00c004000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0xe00c000ll))
        altDoSolve(15, 0x8000ll, puzzle | 0xe00c000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x800c00c000ll))
        altDoSolve(15, 0x8000ll, puzzle | 0x800c00c000ll,
                   partsUsed | 0x400l);
    altpiece11at14:
    break;
   case 15:
      if (partsUsed & 0x2l) goto altpiece2at15;
      if(!(puzzle & 0x1209008000ll))
        altDoSolve(16, 0x10000ll, puzzle | 0x1209008000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x600c008000ll))
        altDoSolve(16, 0x10000ll, puzzle | 0x600c008000ll,
                   partsUsed | 0x2l);
    altpiece2at15:
      if (partsUsed & 0x4l) goto altpiece3at15;
      if(!(puzzle & 0x8009208000ll))
        altDoSolve(16, 0x10000ll, puzzle | 0x8009208000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x800e008000ll))
        altDoSolve(16, 0x10000ll, puzzle | 0x800e008000ll,
                   partsUsed | 0x4l);
    altpiece3at15:
      if (partsUsed & 0x8l) goto altpiece4at15;
      if(!(puzzle & 0x209208000ll))
        altDoSolve(16, 0x10000ll, puzzle | 0x209208000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x200e008000ll))
        altDoSolve(16, 0x10000ll, puzzle | 0x200e008000ll,
                   partsUsed | 0x8l);
    altpiece4at15:
      if (partsUsed & 0x10l) goto altpiece5at15;
      if(!(puzzle & 0x9008009008000ll))
        altDoSolve(16, 0x10000ll, puzzle | 0x9008009008000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0xc00800c008000ll))
        altDoSolve(16, 0x10000ll, puzzle | 0xc00800c008000ll,
                   partsUsed | 0x10l);
    altpiece5at15:
      if (partsUsed & 0x20l) goto altpiece6at15;
      if(!(puzzle & 0x9248000ll))
        altDoSolve(16, 0x10000ll, puzzle | 0x9248000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x9008008008000ll))
        altDoSolve(16, 0x10000ll, puzzle | 0x9008008008000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0xc008008008000ll))
        altDoSolve(16, 0x10000ll, puzzle | 0xc008008008000ll,
                   partsUsed | 0x20l);
    altpiece6at15:
      if (partsUsed & 0x40l) goto altpiece7at15;
      if (!(puzzle & 0x8008008008000ll)) {
         if(!(puzzle & 0x8009008008000ll))
           altDoSolve(16, 0x10000ll, puzzle | 0x8009008008000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x8008009008000ll))
           altDoSolve(16, 0x10000ll, puzzle | 0x8008009008000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800800c008000ll))
           altDoSolve(16, 0x10000ll, puzzle | 0x800800c008000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800c008008000ll))
           altDoSolve(16, 0x10000ll, puzzle | 0x800c008008000ll,
                   partsUsed | 0x40l);
      }
    altpiece7at15:
      if (partsUsed & 0x100l) goto altpiece9at15;
      if(!(puzzle & 0xc808000ll))
        altDoSolve(16, 0x10000ll, puzzle | 0xc808000ll,
                   partsUsed | 0x100l);
    altpiece9at15:
      if (partsUsed & 0x200l) goto altpiece10at15;
      if(!(puzzle & 0x1009208000ll))
        altDoSolve(16, 0x10000ll, puzzle | 0x1009208000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x400e008000ll))
        altDoSolve(16, 0x10000ll, puzzle | 0x400e008000ll,
                   partsUsed | 0x200l);
    altpiece10at15:
      if (partsUsed & 0x400l) goto altpiece11at15;
      if(!(puzzle & 0x9009008000ll))
        altDoSolve(16, 0x10000ll, puzzle | 0x9009008000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0xc00c008000ll))
        altDoSolve(16, 0x10000ll, puzzle | 0xc00c008000ll,
                   partsUsed | 0x400l);
    altpiece11at15:
    break;
   case 16:
      if (partsUsed & 0x2l) goto altpiece2at16;
      if(!(puzzle & 0x480090010000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x480090010000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x400480090000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x400480090000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1330000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x1330000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1990000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x1990000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x40060030000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x40060030000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x60030010000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x60030010000ll,
                   partsUsed | 0x2l);
    altpiece2at16:
      if (partsUsed & 0x4l) goto altpiece3at16;
      if(!(puzzle & 0x10490010000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x10490010000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x10070010000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x10070010000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x80080490000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x80080490000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x790000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x790000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x970000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x970000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x20020070000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x20020070000ll,
                   partsUsed | 0x4l);
    altpiece3at16:
      if (partsUsed & 0x8l) goto altpiece4at16;
      if(!(puzzle & 0x400490010000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x400490010000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x480080090000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x480080090000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1930000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x1930000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1390000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x1390000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x40070010000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x40070010000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x60020030000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x60020030000ll,
                   partsUsed | 0x8l);
    altpiece4at16:
      if (partsUsed & 0x10l) goto altpiece5at16;
      if (!(puzzle & 0x2490000ll)) {
         if(!(puzzle & 0x6590000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x6590000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x2082490000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x2082490000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x2cb0000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x2cb0000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x90010090010000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x90010090010000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x80090080090000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x80090080090000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x412490000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x412490000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x10090010090000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x10090010090000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x4d30000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x4d30000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x2490410000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x2490410000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x30010030010000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x30010030010000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x10030010030000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x10030010030000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x20030020030000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x20030020030000ll,
                   partsUsed | 0x10l);
    altpiece5at16:
      if (partsUsed & 0x20l) goto altpiece6at16;
      if (!(puzzle & 0x10010010010000ll)) {
         if(!(puzzle & 0x10010010090000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x10010010090000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x90010010010000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x90010010010000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x10010010030000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x10010010030000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x30010010010000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x30010010010000ll,
                   partsUsed | 0x20l);
      }
      if (!(puzzle & 0x2490000ll)) {
         if(!(puzzle & 0x2002490000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x2002490000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x24b0000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x24b0000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x12490000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x12490000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x6490000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x6490000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x2490010000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x2490010000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x4930000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x4930000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x80080080090000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x80080080090000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x20020020030000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x20020020030000ll,
                   partsUsed | 0x20l);
    altpiece6at16:
      if (partsUsed & 0x40l) goto altpiece7at16;
      if (!(puzzle & 0x10010010010000ll)) {
         if(!(puzzle & 0x10010090010000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x10010090010000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x10090010010000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x10090010010000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x10010030010000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x10010030010000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x10030010010000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x10030010010000ll,
                   partsUsed | 0x40l);
      }
      if (!(puzzle & 0x2490000ll)) {
         if(!(puzzle & 0x402490000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x402490000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2590000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x2590000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x82490000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x82490000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2c90000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x2c90000ll,
                   partsUsed | 0x40l);
      }
    altpiece7at16:
      if (partsUsed & 0x80l) goto altpiece8at16;
      if(!(puzzle & 0x90010090000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x90010090000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x90080090000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x90080090000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x490410000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x490410000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x410490000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x410490000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x2f0000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x2f0000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xcb0000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0xcb0000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xd30000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0xd30000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x3d0000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x3d0000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x30020030000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x30020030000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x30010030000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x30010030000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x70050000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x70050000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x50070000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x50070000ll,
                   partsUsed | 0x80l);
    altpiece8at16:
      if (partsUsed & 0x100l) goto altpiece9at16;
      if(!(puzzle & 0x100130000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x100130000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x130010000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x130010000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x200b0000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x200b0000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x90030000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x90030000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x180090000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x180090000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x10190000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x10190000ll,
                   partsUsed | 0x100l);
    altpiece9at16:
      if (partsUsed & 0x200l) goto altpiece10at16;
      if(!(puzzle & 0x80490010000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x80490010000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x80480090000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x80480090000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0xb30000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0xb30000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0xb90000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0xb90000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x20060030000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x20060030000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x20070010000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0x20070010000ll,
                   partsUsed | 0x200l);
    altpiece10at16:
      if (partsUsed & 0x400l) goto altpiece11at16;
      if (!(puzzle & 0x80090000ll)) {
         if(!(puzzle & 0x10090090000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x10090090000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x490090000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x490090000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x480490000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x480490000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x90490000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x90490000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x80090090000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x80090090000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x130000ll)) {
         if(!(puzzle & 0x370000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x370000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1f0000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x1f0000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x5b0000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x5b0000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x9b0000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x9b0000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x3b0000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x3b0000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x10010010000ll)) {
         if(!(puzzle & 0x90090010000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x90090010000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x10030030000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x10030030000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x30030010000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x30030010000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x20030000ll)) {
         if(!(puzzle & 0x20030030000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x20030030000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x70030000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x70030000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x30070000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x30070000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x60070000ll))
           altDoSolve(17, 0x20000ll, puzzle | 0x60070000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0xd90000ll))
        altDoSolve(17, 0x20000ll, puzzle | 0xd90000ll,
                   partsUsed | 0x400l);
    altpiece11at16:
    break;
   case 17:
      if (partsUsed & 0x2l) goto altpiece2at17;
      if(!(puzzle & 0x5e0000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x5e0000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x10030060000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x10030060000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x900120020000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x900120020000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x800900120000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x800900120000ll,
                   partsUsed | 0x2l);
    altpiece2at17:
      if (partsUsed & 0x4l) goto altpiece3at17;
      if(!(puzzle & 0x1d20000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x1d20000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x100100920000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x100100920000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x70020020000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x70020020000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x20920020000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x20920020000ll,
                   partsUsed | 0x4l);
    altpiece3at17:
      if (partsUsed & 0x8l) goto altpiece4at17;
      if(!(puzzle & 0xd60000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0xd60000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x30020060000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x30020060000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x800920020000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x800920020000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x900100120000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x900100120000ll,
                   partsUsed | 0x8l);
    altpiece4at17:
      if (partsUsed & 0x10l) goto altpiece5at17;
      if (!(puzzle & 0x4920000ll)) {
         if(!(puzzle & 0x69a0000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x69a0000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x824920000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x824920000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0xcb20000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0xcb20000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x4104920000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x4104920000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x5960000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x5960000ll,
                   partsUsed | 0x10l);
      }
      if (!(puzzle & 0x20020020020000ll)) {
         if(!(puzzle & 0x30020030020000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x30020030020000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x120020120020000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x120020120020000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x20120020120000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x20120020120000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x60020060020000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x60020060020000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x20060020060000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x20060020060000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x100120100120000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x100120100120000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x9a60000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x9a60000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x4920820000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x4920820000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x40060040060000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x40060040060000ll,
                   partsUsed | 0x10l);
    altpiece5at17:
      if (partsUsed & 0x20l) goto altpiece6at17;
      if (!(puzzle & 0x4920000ll)) {
         if(!(puzzle & 0x6920000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x6920000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x4004920000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x4004920000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x4960000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x4960000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x24920000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x24920000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0xc920000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0xc920000ll,
                   partsUsed | 0x20l);
      }
      if (!(puzzle & 0x20020020020000ll)) {
         if(!(puzzle & 0x30020020020000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x30020020020000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x20020020120000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x20020020120000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x120020020020000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x120020020020000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x20020020060000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x20020020060000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x60020020020000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x60020020020000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x4920020000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x4920020000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x9260000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x9260000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x100100100120000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x100100100120000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x40040040060000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x40040040060000ll,
                   partsUsed | 0x20l);
    altpiece6at17:
      if (partsUsed & 0x40l) goto altpiece7at17;
      if (!(puzzle & 0x4920000ll)) {
         if(!(puzzle & 0x49a0000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x49a0000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4d20000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x4d20000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x804920000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x804920000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4b20000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x4b20000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x104920000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x104920000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x5920000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x5920000ll,
                   partsUsed | 0x40l);
      }
      if (!(puzzle & 0x20020020020000ll)) {
         if(!(puzzle & 0x20020030020000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x20020030020000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x20030020020000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x20030020020000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x20020120020000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x20020120020000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x20120020020000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x20120020020000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x20020060020000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x20020060020000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x20060020020000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x20060020020000ll,
                   partsUsed | 0x40l);
      }
    altpiece7at17:
      if (partsUsed & 0x80l) goto altpiece8at17;
      if(!(puzzle & 0x120020120000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x120020120000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x120100120000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x120100120000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x920820000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x920820000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x820920000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x820920000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1960000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x1960000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1a60000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x1a60000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x60040060000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x60040060000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x60020060000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x60020060000ll,
                   partsUsed | 0x80l);
    altpiece8at17:
      if (partsUsed & 0x100l) goto altpiece9at17;
      if(!(puzzle & 0x801a0000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x801a0000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x30120000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x30120000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x20320000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x20320000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1a0020000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x1a0020000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x120060000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x120060000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x260020000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x260020000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x40160000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x40160000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x300120000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x300120000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x200260000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x200260000ll,
                   partsUsed | 0x100l);
    altpiece9at17:
      if (partsUsed & 0x200l) goto altpiece10at17;
      if (!(puzzle & 0x1a0000ll)) {
         if(!(puzzle & 0x13a0000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x13a0000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x7a0000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x7a0000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x9e0000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x9e0000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x19a0000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x19a0000ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x20020020000ll)) {
         if(!(puzzle & 0x20030060000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x20030060000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x60030020000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x60030020000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x30060020000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x30060020000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0xf20000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0xf20000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x10070020000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x10070020000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x40070020000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x40070020000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100920020000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x100920020000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100900120000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x100900120000ll,
                   partsUsed | 0x200l);
    altpiece10at17:
      if (partsUsed & 0x400l) goto altpiece11at17;
      if (!(puzzle & 0x100120000ll)) {
         if(!(puzzle & 0x20120120000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x20120120000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x920120000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x920120000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x900920000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x900920000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x120920000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x120920000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x100120120000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x100120120000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x20020020000ll)) {
         if(!(puzzle & 0x120120020000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x120120020000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x20060060000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x20060060000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x60060020000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x60060020000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x320000ll)) {
         if(!(puzzle & 0xb60000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0xb60000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1360000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x1360000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1b20000ll))
           altDoSolve(18, 0x40000ll, puzzle | 0x1b20000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x3e0000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x3e0000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0xda0000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0xda0000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x70060000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x70060000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x30030020000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x30030020000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x40060060000ll))
        altDoSolve(18, 0x40000ll, puzzle | 0x40060060000ll,
                   partsUsed | 0x400l);
    altpiece11at17:
    break;
   case 18:
      if (partsUsed & 0x2l) goto altpiece2at18;
      if(!(puzzle & 0xf40000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0xf40000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x30060040000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x30060040000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1200240040000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x1200240040000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1001200240000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x1001200240000ll,
                   partsUsed | 0x2l);
    altpiece2at18:
      if (partsUsed & 0x4l) goto altpiece3at18;
      if(!(puzzle & 0x13c0000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x13c0000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x200201240000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x200201240000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x40070040000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x40070040000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x41240040000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x41240040000ll,
                   partsUsed | 0x4l);
    altpiece3at18:
      if (partsUsed & 0x8l) goto altpiece4at18;
      if(!(puzzle & 0x7c0000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x7c0000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x10070040000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x10070040000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1001240040000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x1001240040000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1200200240000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x1200200240000ll,
                   partsUsed | 0x8l);
    altpiece4at18:
      if (partsUsed & 0x10l) goto altpiece5at18;
      if (!(puzzle & 0x9240000ll)) {
         if(!(puzzle & 0xd340000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0xd340000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x1049240000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x1049240000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x8209240000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x8209240000ll,
                   partsUsed | 0x10l);
      }
      if (!(puzzle & 0x40040040040000ll)) {
         if(!(puzzle & 0x60040060040000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x60040060040000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x240040240040000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x240040240040000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x40240040240000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x40240040240000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x200240200240000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x200240200240000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x9241040000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x9241040000ll,
                   partsUsed | 0x10l);
    altpiece5at18:
      if (partsUsed & 0x20l) goto altpiece6at18;
      if (!(puzzle & 0x9240000ll)) {
         if(!(puzzle & 0xd240000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0xd240000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x8009240000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x8009240000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x49240000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x49240000ll,
                   partsUsed | 0x20l);
      }
      if (!(puzzle & 0x40040040040000ll)) {
         if(!(puzzle & 0x60040040040000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x60040040040000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x40040040240000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x40040040240000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x240040040040000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x240040040040000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x9240040000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x9240040000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x200200200240000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x200200200240000ll,
                   partsUsed | 0x20l);
    altpiece6at18:
      if (partsUsed & 0x40l) goto altpiece7at18;
      if (!(puzzle & 0x9240000ll)) {
         if(!(puzzle & 0x9340000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x9340000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x9a40000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x9a40000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x1009240000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x1009240000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x209240000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x209240000ll,
                   partsUsed | 0x40l);
      }
      if (!(puzzle & 0x40040040040000ll)) {
         if(!(puzzle & 0x40040060040000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x40040060040000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x40060040040000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x40060040040000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x40040240040000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x40040240040000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x40240040040000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x40240040040000ll,
                   partsUsed | 0x40l);
      }
    altpiece7at18:
      if (partsUsed & 0x80l) goto altpiece8at18;
      if(!(puzzle & 0x240040240000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x240040240000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x240200240000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x240200240000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1241040000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x1241040000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1041240000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x1041240000ll,
                   partsUsed | 0x80l);
    altpiece8at18:
      if (partsUsed & 0x100l) goto altpiece9at18;
      if(!(puzzle & 0x340040000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x340040000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x100340000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x100340000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x60240000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x60240000ll,
                   partsUsed | 0x100l);
    altpiece9at18:
      if (partsUsed & 0x200l) goto altpiece10at18;
      if(!(puzzle & 0xbc0000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0xbc0000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x20070040000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x20070040000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x201240040000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x201240040000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x201200240000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x201200240000ll,
                   partsUsed | 0x200l);
    altpiece10at18:
      if (partsUsed & 0x400l) goto altpiece11at18;
      if (!(puzzle & 0x1240000ll)) {
         if(!(puzzle & 0x1b40000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x1b40000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1201240000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x1201240000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x241240000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x241240000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x40040040000ll)) {
         if(!(puzzle & 0x60060040000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x60060040000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x40240240000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x40240240000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x240240040000ll))
           altDoSolve(19, 0x80000ll, puzzle | 0x240240040000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x1240240000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x1240240000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x200240240000ll))
        altDoSolve(19, 0x80000ll, puzzle | 0x200240240000ll,
                   partsUsed | 0x400l);
    altpiece11at18:
    break;
   case 19:
      if (partsUsed & 0x2l) goto altpiece2at19;
      if(!(puzzle & 0x10090480000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x10090480000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x2400480080000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x2400480080000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x2002400480000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x2002400480000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x9980000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x9980000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0xcc80000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0xcc80000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x200300180000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x200300180000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x300180080000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x300180080000ll,
                   partsUsed | 0x2l);
    altpiece2at19:
      if (partsUsed & 0x4l) goto altpiece3at19;
      if (!(puzzle & 0x80080080000ll)) {
         if(!(puzzle & 0x490080080000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x490080080000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x82480080000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x82480080000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x80380080000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x80380080000ll,
                   partsUsed | 0x4l);
      }
      if(!(puzzle & 0x400402480000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x400402480000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x3c80000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x3c80000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x4b80000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x4b80000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x100100380000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x100100380000ll,
                   partsUsed | 0x4l);
    altpiece3at19:
      if (partsUsed & 0x8l) goto altpiece4at19;
      if(!(puzzle & 0x90080480000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x90080480000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x2002480080000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x2002480080000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x2400400480000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x2400400480000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0xc980000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0xc980000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x9c80000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x9c80000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x200380080000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x200380080000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x300100180000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x300100180000ll,
                   partsUsed | 0x8l);
    altpiece4at19:
      if (partsUsed & 0x10l) goto altpiece5at19;
      if (!(puzzle & 0x80080080080000ll)) {
         if(!(puzzle & 0x80480080480000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x80480080480000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x180080180080000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x180080180080000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x80180080180000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x80180080180000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x2492080000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x2492080000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x90080090080000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x90080090080000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x480080480080000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x480080480080000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x400480400480000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x400480400480000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x100180100180000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x100180100180000ll,
                   partsUsed | 0x10l);
    altpiece5at19:
      if (partsUsed & 0x20l) goto altpiece6at19;
      if (!(puzzle & 0x80080080080000ll)) {
         if(!(puzzle & 0x90080080080000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x90080080080000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x80080080480000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x80080080480000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x480080080080000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x480080080080000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x80080080180000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x80080080180000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x180080080080000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x180080080080000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x400400400480000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x400400400480000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x100100100180000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x100100100180000ll,
                   partsUsed | 0x20l);
    altpiece6at19:
      if (partsUsed & 0x40l) goto altpiece7at19;
      if (!(puzzle & 0x80080080080000ll)) {
         if(!(puzzle & 0x80090080080000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x80090080080000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x80080480080000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x80080480080000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x80480080080000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x80480080080000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x80080180080000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x80080180080000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x80180080080000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x80180080080000ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x2490080000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x2490080000ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x80080090080000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x80080090080000ll,
                   partsUsed | 0x40l);
    altpiece7at19:
      if (partsUsed & 0x80l) goto altpiece8at19;
      if(!(puzzle & 0x480080480000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x480080480000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x480400480000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x480400480000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x2482080000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x2482080000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x2082480000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x2082480000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1780000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x1780000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x6580000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x6580000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x6980000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x6980000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1e80000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x1e80000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x180100180000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x180100180000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x180080180000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x180080180000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x380280000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x380280000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x280380000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x280380000ll,
                   partsUsed | 0x80l);
    altpiece8at19:
      if (partsUsed & 0x100l) goto altpiece9at19;
      if(!(puzzle & 0x120180000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x120180000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x100580000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x100580000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xb0080000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0xb0080000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x980080000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x980080000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x480180000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x480180000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xc00480000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0xc00480000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x80c80000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x80c80000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x800980000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x800980000ll,
                   partsUsed | 0x100l);
    altpiece9at19:
      if (partsUsed & 0x200l) goto altpiece10at19;
      if (!(puzzle & 0x80080080000ll)) {
         if(!(puzzle & 0x80090480000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x80090480000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x90480080000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x90480080000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x480090080000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x480090080000ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x480080000ll)) {
         if(!(puzzle & 0x400490080000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x400490080000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x10490080000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x10490080000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x402480080000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x402480080000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x5980000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x5980000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x5c80000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x5c80000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x402400480000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x402400480000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100300180000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x100300180000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100380080000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x100380080000ll,
                   partsUsed | 0x200l);
    altpiece10at19:
      if (partsUsed & 0x400l) goto altpiece11at19;
      if (!(puzzle & 0x400480000ll)) {
         if(!(puzzle & 0x80480480000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x80480480000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2480480000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x2480480000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2402480000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x2402480000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x482480000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x482480000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x400480480000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x400480480000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x980000ll)) {
         if(!(puzzle & 0x1b80000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x1b80000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xf80000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0xf80000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2d80000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x2d80000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x4d80000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x4d80000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1d80000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x1d80000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x80080080000ll)) {
         if(!(puzzle & 0x480480080000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x480480080000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x80180180000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x80180180000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x180180080000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x180180080000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x100180000ll)) {
         if(!(puzzle & 0x100180180000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x100180180000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x380180000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x380180000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x180380000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x180380000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x300380000ll))
           altDoSolve(20, 0x100000ll, puzzle | 0x300380000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x90090080000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x90090080000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x490480000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x490480000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x6c80000ll))
        altDoSolve(20, 0x100000ll, puzzle | 0x6c80000ll,
                   partsUsed | 0x400l);
    altpiece11at19:
    break;
   case 20:
      if (partsUsed & 0x2l) goto altpiece2at20;
      if(!(puzzle & 0x20120900000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x20120900000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x2f00000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x2f00000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x80180300000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x80180300000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x4800900100000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x4800900100000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x4004800900000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x4004800900000ll,
                   partsUsed | 0x2l);
    altpiece2at20:
      if (partsUsed & 0x4l) goto altpiece3at20;
      if (!(puzzle & 0x100100100000ll)) {
         if(!(puzzle & 0x920100100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x920100100000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x380100100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x380100100000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x104900100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x104900100000ll,
                   partsUsed | 0x4l);
      }
      if(!(puzzle & 0xe900000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0xe900000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x800804900000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x800804900000ll,
                   partsUsed | 0x4l);
    altpiece3at20:
      if (partsUsed & 0x8l) goto altpiece4at20;
      if(!(puzzle & 0x120100900000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x120100900000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x180100300000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x180100300000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x6b00000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x6b00000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x4004900100000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x4004900100000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x4800800900000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x4800800900000ll,
                   partsUsed | 0x8l);
    altpiece4at20:
      if (partsUsed & 0x10l) goto altpiece5at20;
      if (!(puzzle & 0x100100100100000ll)) {
         if(!(puzzle & 0x180100180100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x180100180100000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x900100900100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x900100900100000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x100900100900000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x100900100900000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x300100300100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x300100300100000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x100300100300000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x100300100300000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x4924100000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x4924100000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x120100120100000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x120100120100000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x800900800900000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x800900800900000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x200300200300000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x200300200300000ll,
                   partsUsed | 0x10l);
    altpiece5at20:
      if (partsUsed & 0x20l) goto altpiece6at20;
      if (!(puzzle & 0x100100100100000ll)) {
         if(!(puzzle & 0x120100100100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x120100100100000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x180100100100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x180100100100000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x100100100900000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x100100100900000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x900100100100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x900100100100000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x100100100300000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x100100100300000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x300100100100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x300100100100000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x800800800900000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x800800800900000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x200200200300000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x200200200300000ll,
                   partsUsed | 0x20l);
    altpiece6at20:
      if (partsUsed & 0x40l) goto altpiece7at20;
      if (!(puzzle & 0x100100100100000ll)) {
         if(!(puzzle & 0x100120100100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x100120100100000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100100180100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x100100180100000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100180100100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x100180100100000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100100900100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x100100900100000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100900100100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x100900100100000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100100300100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x100100300100000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x100300100100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x100300100100000ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x4920100000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x4920100000ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x100100120100000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x100100120100000ll,
                   partsUsed | 0x40l);
    altpiece7at20:
      if (partsUsed & 0x80l) goto altpiece8at20;
      if(!(puzzle & 0x900100900000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x900100900000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x900800900000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x900800900000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x4904100000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x4904100000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x4104900000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x4104900000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xcb00000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0xcb00000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xd300000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0xd300000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x300200300000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x300200300000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x300100300000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x300100300000ll,
                   partsUsed | 0x80l);
    altpiece8at20:
      if (partsUsed & 0x100l) goto altpiece9at20;
      if(!(puzzle & 0x190100000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x190100000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x180900000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x180900000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x240300000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x240300000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x200b00000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x200b00000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x160100000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x160100000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x400d00000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x400d00000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xd00100000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0xd00100000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x900300000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x900300000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1300100000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x1300100000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1800900000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x1800900000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x101900000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x101900000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1001300000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x1001300000ll,
                   partsUsed | 0x100l);
    altpiece9at20:
      if (partsUsed & 0x200l) goto altpiece10at20;
      if (!(puzzle & 0x100100100000ll)) {
         if(!(puzzle & 0x100120900000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x100120900000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x120900100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x120900100000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x900120100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x900120100000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x100180300000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x100180300000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x300180100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x300180100000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x180300100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x180300100000ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x900100000ll)) {
         if(!(puzzle & 0x800920100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x800920100000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x20920100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x20920100000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x804900100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x804900100000ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0xd00000ll)) {
         if(!(puzzle & 0x9d00000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x9d00000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x3d00000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x3d00000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x4f00000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x4f00000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0xcd00000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0xcd00000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x7900000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x7900000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x80380100000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x80380100000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x200380100000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x200380100000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x804800900000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x804800900000ll,
                   partsUsed | 0x200l);
    altpiece10at20:
      if (partsUsed & 0x400l) goto altpiece11at20;
      if (!(puzzle & 0x800900000ll)) {
         if(!(puzzle & 0x100900900000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x100900900000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x4900900000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x4900900000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x4804900000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x4804900000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x904900000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x904900000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x800900900000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x800900900000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x100100100000ll)) {
         if(!(puzzle & 0x900900100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x900900100000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x100300300000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x100300300000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x300300100000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x300300100000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x1900000ll)) {
         if(!(puzzle & 0x5b00000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x5b00000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x9b00000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0x9b00000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xd900000ll))
           altDoSolve(21, 0x200000ll, puzzle | 0xd900000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x120120100000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x120120100000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x920900000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x920900000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x1f00000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x1f00000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x6d00000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x6d00000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x380300000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x380300000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x180180100000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x180180100000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x200300300000ll))
        altDoSolve(21, 0x200000ll, puzzle | 0x200300300000ll,
                   partsUsed | 0x400l);
    altpiece11at20:
    break;
   case 21:
      if (partsUsed & 0x2l) goto altpiece2at21;
      if(!(puzzle & 0x40241200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x40241200000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x7a00000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x7a00000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x180300200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x180300200000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x9001200200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x9001200200000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x8009001200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x8009001200000ll,
                   partsUsed | 0x2l);
    altpiece2at21:
      if (partsUsed & 0x4l) goto altpiece3at21;
      if (!(puzzle & 0x200200200000ll)) {
         if(!(puzzle & 0x1240200200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x1240200200000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x200380200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x200380200000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x209200200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x209200200000ll,
                   partsUsed | 0x4l);
      }
      if(!(puzzle & 0x9e00000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x9e00000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x1001009200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x1001009200000ll,
                   partsUsed | 0x4l);
    altpiece3at21:
      if (partsUsed & 0x8l) goto altpiece4at21;
      if(!(puzzle & 0x240201200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x240201200000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x3e00000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x3e00000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x80380200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x80380200000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x8009200200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x8009200200000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x9001001200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x9001001200000ll,
                   partsUsed | 0x8l);
    altpiece4at21:
      if (partsUsed & 0x10l) goto altpiece5at21;
      if (!(puzzle & 0x200200200200000ll)) {
         if(!(puzzle & 0x300200300200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x300200300200000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x1200201200200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x1200201200200000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x201200201200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x201200201200000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x9248200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x9248200000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x240200240200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x240200240200000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x1001201001200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x1001201001200000ll,
                   partsUsed | 0x10l);
    altpiece5at21:
      if (partsUsed & 0x20l) goto altpiece6at21;
      if (!(puzzle & 0x200200200200000ll)) {
         if(!(puzzle & 0x240200200200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x240200200200000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x300200200200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x300200200200000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x200200201200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x200200201200000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x1200200200200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x1200200200200000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x1001001001200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x1001001001200000ll,
                   partsUsed | 0x20l);
    altpiece6at21:
      if (partsUsed & 0x40l) goto altpiece7at21;
      if (!(puzzle & 0x200200200200000ll)) {
         if(!(puzzle & 0x200240200200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x200240200200000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x200200300200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x200200300200000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x200300200200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x200300200200000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x200201200200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x200201200200000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x201200200200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x201200200200000ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x9240200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x9240200000ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x200200240200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x200200240200000ll,
                   partsUsed | 0x40l);
    altpiece7at21:
      if (partsUsed & 0x80l) goto altpiece8at21;
      if(!(puzzle & 0x1200201200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x1200201200000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1201001200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x1201001200000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x9208200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x9208200000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x8209200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x8209200000ll,
                   partsUsed | 0x80l);
    altpiece8at21:
      if (partsUsed & 0x100l) goto altpiece9at21;
      if(!(puzzle & 0x320200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x320200000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x301200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x301200000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x801a00000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x801a00000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1a00200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x1a00200000ll,
                   partsUsed | 0x100l);
    altpiece9at21:
      if (partsUsed & 0x200l) goto altpiece10at21;
      if (!(puzzle & 0x200200200000ll)) {
         if(!(puzzle & 0x200241200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x200241200000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x241200200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x241200200000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x1200240200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x1200240200000ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x1200200000ll)) {
         if(!(puzzle & 0x1001240200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x1001240200000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x41240200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x41240200000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x1009200200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x1009200200000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x5e00000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x5e00000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100380200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x100380200000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x1009001200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x1009001200000ll,
                   partsUsed | 0x200l);
    altpiece10at21:
      if (partsUsed & 0x400l) goto altpiece11at21;
      if (!(puzzle & 0x9200000ll)) {
         if(!(puzzle & 0xda00000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0xda00000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x9009200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x9009200000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1209200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x1209200000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x200200200000ll)) {
         if(!(puzzle & 0x300300200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x300300200000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x201201200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x201201200000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1201200200000ll))
           altDoSolve(22, 0x400000ll, puzzle | 0x1201200200000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x240240200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x240240200000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x1241200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x1241200000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x9201200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x9201200000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x1001201200000ll))
        altDoSolve(22, 0x400000ll, puzzle | 0x1001201200000ll,
                   partsUsed | 0x400l);
    altpiece11at21:
    break;
   case 22:
      if (partsUsed & 0x2l) goto altpiece2at22;
      if(!(puzzle & 0x90480400000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x90480400000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x80482400000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x80482400000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1001800c00000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x1001800c00000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1800c00400000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x1800c00400000ll,
                   partsUsed | 0x2l);
    altpiece2at22:
      if (partsUsed & 0x4l) goto altpiece3at22;
      if (!(puzzle & 0x400400400000ll)) {
         if(!(puzzle & 0x400490400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x400490400000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x2480400400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x2480400400000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x401c00400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x401c00400000ll,
                   partsUsed | 0x4l);
      }
      if(!(puzzle & 0x800801c00000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x800801c00000ll,
                   partsUsed | 0x4l);
    altpiece3at22:
      if (partsUsed & 0x8l) goto altpiece4at22;
      if(!(puzzle & 0x10490400000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x10490400000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x480402400000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x480402400000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1001c00400000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x1001c00400000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1800800c00000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x1800800c00000ll,
                   partsUsed | 0x8l);
    altpiece4at22:
      if (partsUsed & 0x10l) goto altpiece5at22;
      if (!(puzzle & 0x400400400400000ll)) {
         if(!(puzzle & 0x480400480400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x480400480400000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x2400402400400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x2400402400400000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x402400402400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x402400402400000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0xc00400c00400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0xc00400c00400000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x400c00400c00000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x400c00400c00000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x2002402002400000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x2002402002400000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x800c00800c00000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x800c00800c00000ll,
                   partsUsed | 0x10l);
    altpiece5at22:
      if (partsUsed & 0x20l) goto altpiece6at22;
      if (!(puzzle & 0x400400400400000ll)) {
         if(!(puzzle & 0x480400400400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x480400400400000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x400400402400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x400400402400000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x2400400400400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x2400400400400000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x400400400c00000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x400400400c00000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0xc00400400400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0xc00400400400000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x2002002002400000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x2002002002400000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x800800800c00000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x800800800c00000ll,
                   partsUsed | 0x20l);
    altpiece6at22:
      if (partsUsed & 0x40l) goto altpiece7at22;
      if (!(puzzle & 0x400400400400000ll)) {
         if(!(puzzle & 0x400480400400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x400480400400000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x400402400400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x400402400400000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x402400400400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x402400400400000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x400400c00400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x400400c00400000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x400c00400400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x400c00400400000ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x2490400000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x2490400000ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x400400480400000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x400400480400000ll,
                   partsUsed | 0x40l);
    altpiece7at22:
      if (partsUsed & 0x80l) goto altpiece8at22;
      if(!(puzzle & 0x2400402400000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x2400402400000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x2402002400000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x2402002400000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xbc00000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0xbc00000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xf400000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0xf400000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xc00800c00000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0xc00800c00000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xc00400c00000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0xc00400c00000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1c01400000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x1c01400000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1401c00000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x1401c00000ll,
                   partsUsed | 0x80l);
    altpiece8at22:
      if (partsUsed & 0x100l) goto altpiece9at22;
      if(!(puzzle & 0x900c00000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x900c00000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x802c00000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x802c00000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x580400000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x580400000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x4c00400000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x4c00400000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x2400c00000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x2400c00000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x6002400000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x6002400000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x406400000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x406400000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x4004c00000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x4004c00000ll,
                   partsUsed | 0x100l);
    altpiece9at22:
      if (partsUsed & 0x200l) goto altpiece10at22;
      if (!(puzzle & 0x480400000ll)) {
         if(!(puzzle & 0x80490400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x80490400000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x400482400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x400482400000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x2002480400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x2002480400000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x2400480400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x2400480400000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x82480400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x82480400000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x482400400000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x482400400000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x801800c00000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x801800c00000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x801c00400000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x801c00400000ll,
                   partsUsed | 0x200l);
    altpiece10at22:
      if (partsUsed & 0x400l) goto altpiece11at22;
      if (!(puzzle & 0x2400400000ll)) {
         if(!(puzzle & 0x402402400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x402402400000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2402400400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x2402400400000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2002402400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x2002402400000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x4c00000ll)) {
         if(!(puzzle & 0xdc00000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0xdc00000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x7c00000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x7c00000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xec00000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0xec00000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0xc00400000ll)) {
         if(!(puzzle & 0x800c00c00000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x800c00c00000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x400c00c00000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x400c00c00000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1c00c00000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0x1c00c00000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xc00c00400000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0xc00c00400000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xc01c00000ll))
           altDoSolve(23, 0x800000ll, puzzle | 0xc01c00000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x480480400000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x480480400000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x2482400000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x2482400000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x1801c00000ll))
        altDoSolve(23, 0x800000ll, puzzle | 0x1801c00000ll,
                   partsUsed | 0x400l);
    altpiece11at22:
    break;
   case 23:
      if (partsUsed & 0x2l) goto altpiece2at23;
      if(!(puzzle & 0x400c01800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x400c01800000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x120900800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x120900800000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x100904800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x100904800000ll,
                   partsUsed | 0x2l);
    altpiece2at23:
      if (partsUsed & 0x4l) goto altpiece3at23;
      if (!(puzzle & 0x800800800000ll)) {
         if(!(puzzle & 0x800920800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x800920800000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x4900800800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x4900800800000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x1c00800800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x1c00800800000ll,
                   partsUsed | 0x4l);
      }
    altpiece3at23:
      if (partsUsed & 0x8l) goto altpiece4at23;
      if(!(puzzle & 0x20920800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x20920800000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x900804800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x900804800000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0xc00801800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0xc00801800000ll,
                   partsUsed | 0x8l);
    altpiece4at23:
      if (partsUsed & 0x10l) goto altpiece5at23;
      if (!(puzzle & 0x800800800800000ll)) {
         if(!(puzzle & 0x900800900800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x900800900800000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0xc00800c00800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0xc00800c00800000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x4800804800800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x4800804800800000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x804800804800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x804800804800000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x1800801800800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x1800801800800000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x801800801800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x801800801800000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x4004804004800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x4004804004800000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x1001801001800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x1001801001800000ll,
                   partsUsed | 0x10l);
    altpiece5at23:
      if (partsUsed & 0x20l) goto altpiece6at23;
      if (!(puzzle & 0x800800800800000ll)) {
         if(!(puzzle & 0x900800800800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x900800800800000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0xc00800800800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0xc00800800800000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x800800804800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x800800804800000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x4800800800800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x4800800800800000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x800800801800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x800800801800000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x1800800800800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x1800800800800000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x4004004004800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x4004004004800000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x1001001001800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x1001001001800000ll,
                   partsUsed | 0x20l);
    altpiece6at23:
      if (partsUsed & 0x40l) goto altpiece7at23;
      if (!(puzzle & 0x800800800800000ll)) {
         if(!(puzzle & 0x800900800800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x800900800800000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800800c00800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x800800c00800000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800c00800800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x800c00800800000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800804800800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x800804800800000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x804800800800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x804800800800000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800801800800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x800801800800000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x801800800800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x801800800800000ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x4920800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x4920800000ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x800800900800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x800800900800000ll,
                   partsUsed | 0x40l);
    altpiece7at23:
      if (partsUsed & 0x80l) goto altpiece8at23;
      if(!(puzzle & 0x4800804800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x4800804800000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x4804004800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x4804004800000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1801001800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x1801001800000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1800801800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x1800801800000ll,
                   partsUsed | 0x80l);
    altpiece8at23:
      if (partsUsed & 0x100l) goto altpiece9at23;
      if(!(puzzle & 0xc80800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0xc80800000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xc04800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0xc04800000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1201800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x1201800000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1005800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x1005800000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xb00800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0xb00800000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x2006800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x2006800000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x6800800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x6800800000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x4801800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x4801800000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x9800800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x9800800000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xc004800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0xc004800000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x80c800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x80c800000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x8009800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x8009800000ll,
                   partsUsed | 0x100l);
    altpiece9at23:
      if (partsUsed & 0x200l) goto altpiece10at23;
      if (!(puzzle & 0x900800000ll)) {
         if(!(puzzle & 0x100920800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x100920800000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x800904800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x800904800000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x4004900800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x4004900800000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x4800900800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x4800900800000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x104900800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x104900800000ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x800800800000ll)) {
         if(!(puzzle & 0x904800800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x904800800000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x800c01800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x800c01800000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x1800c00800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x1800c00800000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0xc01800800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0xc01800800000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x401c00800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x401c00800000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x1001c00800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x1001c00800000ll,
                   partsUsed | 0x200l);
    altpiece10at23:
      if (partsUsed & 0x400l) goto altpiece11at23;
      if (!(puzzle & 0x4800800000ll)) {
         if(!(puzzle & 0x804804800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x804804800000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x4804800800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x4804800800000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x4004804800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x4004804800000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x1800800000ll)) {
         if(!(puzzle & 0x1001801800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x1001801800000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x801801800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x801801800000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1801800800000ll))
           altDoSolve(24, 0x1000000ll, puzzle | 0x1801800800000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x900900800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x900900800000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x4904800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x4904800000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0xf800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0xf800000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x1c01800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0x1c01800000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0xc00c00800000ll))
        altDoSolve(24, 0x1000000ll, puzzle | 0xc00c00800000ll,
                   partsUsed | 0x400l);
    altpiece11at23:
    break;
   case 24:
      if (partsUsed & 0x2l) goto altpiece2at24;
      if(!(puzzle & 0xc01801000000ll))
        altDoSolve(25, 0x2000000ll, puzzle | 0xc01801000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x241201000000ll))
        altDoSolve(25, 0x2000000ll, puzzle | 0x241201000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x201209000000ll))
        altDoSolve(25, 0x2000000ll, puzzle | 0x201209000000ll,
                   partsUsed | 0x2l);
    altpiece2at24:
      if (partsUsed & 0x4l) goto altpiece3at24;
      if (!(puzzle & 0x1001001000000ll)) {
         if(!(puzzle & 0x1001241000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x1001241000000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x9201001000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x9201001000000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x1001c01000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x1001c01000000ll,
                   partsUsed | 0x4l);
      }
    altpiece3at24:
      if (partsUsed & 0x8l) goto altpiece4at24;
      if(!(puzzle & 0x401c01000000ll))
        altDoSolve(25, 0x2000000ll, puzzle | 0x401c01000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x41241000000ll))
        altDoSolve(25, 0x2000000ll, puzzle | 0x41241000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1201009000000ll))
        altDoSolve(25, 0x2000000ll, puzzle | 0x1201009000000ll,
                   partsUsed | 0x8l);
    altpiece4at24:
      if (partsUsed & 0x10l) goto altpiece5at24;
      if (!(puzzle & 0x1001001001000000ll)) {
         if(!(puzzle & 0x1201001201000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x1201001201000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x1801001801000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x1801001801000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x9001009001000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x9001009001000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x1009001009000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x1009001009000000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x8009008009000000ll))
        altDoSolve(25, 0x2000000ll, puzzle | 0x8009008009000000ll,
                   partsUsed | 0x10l);
    altpiece5at24:
      if (partsUsed & 0x20l) goto altpiece6at24;
      if (!(puzzle & 0x1001001001000000ll)) {
         if(!(puzzle & 0x1201001001000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x1201001001000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x1801001001000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x1801001001000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x1001001009000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x1001001009000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x9001001001000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x9001001001000000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x8008008009000000ll))
        altDoSolve(25, 0x2000000ll, puzzle | 0x8008008009000000ll,
                   partsUsed | 0x20l);
    altpiece6at24:
      if (partsUsed & 0x40l) goto altpiece7at24;
      if (!(puzzle & 0x1001001001000000ll)) {
         if(!(puzzle & 0x1001201001000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x1001201001000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x1001001801000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x1001001801000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x1001801001000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x1001801001000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x1001009001000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x1001009001000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x1009001001000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x1009001001000000ll,
                   partsUsed | 0x40l);
      }
      if(!(puzzle & 0x9241000000ll))
        altDoSolve(25, 0x2000000ll, puzzle | 0x9241000000ll,
                   partsUsed | 0x40l);
      if(!(puzzle & 0x1001001201000000ll))
        altDoSolve(25, 0x2000000ll, puzzle | 0x1001001201000000ll,
                   partsUsed | 0x40l);
    altpiece7at24:
      if (partsUsed & 0x80l) goto altpiece8at24;
      if(!(puzzle & 0x9001009000000ll))
        altDoSolve(25, 0x2000000ll, puzzle | 0x9001009000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x9008009000000ll))
        altDoSolve(25, 0x2000000ll, puzzle | 0x9008009000000ll,
                   partsUsed | 0x80l);
    altpiece8at24:
      if (partsUsed & 0x100l) goto altpiece9at24;
      if(!(puzzle & 0x1901000000ll))
        altDoSolve(25, 0x2000000ll, puzzle | 0x1901000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1809000000ll))
        altDoSolve(25, 0x2000000ll, puzzle | 0x1809000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x400d000000ll))
        altDoSolve(25, 0x2000000ll, puzzle | 0x400d000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xd001000000ll))
        altDoSolve(25, 0x2000000ll, puzzle | 0xd001000000ll,
                   partsUsed | 0x100l);
    altpiece9at24:
      if (partsUsed & 0x200l) goto altpiece10at24;
      if (!(puzzle & 0x1201000000ll)) {
         if(!(puzzle & 0x201241000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x201241000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x1001209000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x1001209000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x8009201000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x8009201000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x9001201000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x9001201000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x209201000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x209201000000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x1209001000000ll))
        altDoSolve(25, 0x2000000ll, puzzle | 0x1209001000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x801c01000000ll))
        altDoSolve(25, 0x2000000ll, puzzle | 0x801c01000000ll,
                   partsUsed | 0x200l);
    altpiece10at24:
      if (partsUsed & 0x400l) goto altpiece11at24;
      if (!(puzzle & 0x1001001000000ll)) {
         if(!(puzzle & 0x1801801000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x1801801000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1009009000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x1009009000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x9009001000000ll))
           altDoSolve(25, 0x2000000ll, puzzle | 0x9009001000000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x1201201000000ll))
        altDoSolve(25, 0x2000000ll, puzzle | 0x1201201000000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x9209000000ll))
        altDoSolve(25, 0x2000000ll, puzzle | 0x9209000000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x8009009000000ll))
        altDoSolve(25, 0x2000000ll, puzzle | 0x8009009000000ll,
                   partsUsed | 0x400l);
    altpiece11at24:
    break;
   case 25:
      if (partsUsed & 0x2l) goto altpiece2at25;
      if(!(puzzle & 0x482402000000ll))
        altDoSolve(26, 0x4000000ll, puzzle | 0x482402000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x800c006000000ll))
        altDoSolve(26, 0x4000000ll, puzzle | 0x800c006000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0xc006002000000ll))
        altDoSolve(26, 0x4000000ll, puzzle | 0xc006002000000ll,
                   partsUsed | 0x2l);
    altpiece2at25:
      if (partsUsed & 0x4l) goto altpiece3at25;
      if(!(puzzle & 0x400400e000000ll))
        altDoSolve(26, 0x4000000ll, puzzle | 0x400400e000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x2002482000000ll))
        altDoSolve(26, 0x4000000ll, puzzle | 0x2002482000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x200e002000000ll))
        altDoSolve(26, 0x4000000ll, puzzle | 0x200e002000000ll,
                   partsUsed | 0x4l);
    altpiece3at25:
      if (partsUsed & 0x8l) goto altpiece4at25;
      if(!(puzzle & 0x82482000000ll))
        altDoSolve(26, 0x4000000ll, puzzle | 0x82482000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x800e002000000ll))
        altDoSolve(26, 0x4000000ll, puzzle | 0x800e002000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0xc004006000000ll))
        altDoSolve(26, 0x4000000ll, puzzle | 0xc004006000000ll,
                   partsUsed | 0x8l);
    altpiece4at25:
      if (partsUsed & 0x10l) goto altpiece5at25;
      if (!(puzzle & 0x2002002002000000ll)) {
         if(!(puzzle & 0x2402002402000000ll))
           altDoSolve(26, 0x4000000ll, puzzle | 0x2402002402000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x6002006002000000ll))
           altDoSolve(26, 0x4000000ll, puzzle | 0x6002006002000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x2006002006000000ll))
           altDoSolve(26, 0x4000000ll, puzzle | 0x2006002006000000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x4006004006000000ll))
        altDoSolve(26, 0x4000000ll, puzzle | 0x4006004006000000ll,
                   partsUsed | 0x10l);
    altpiece5at25:
      if (partsUsed & 0x20l) goto altpiece6at25;
      if (!(puzzle & 0x2002002002000000ll)) {
         if(!(puzzle & 0x2402002002000000ll))
           altDoSolve(26, 0x4000000ll, puzzle | 0x2402002002000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x2002002006000000ll))
           altDoSolve(26, 0x4000000ll, puzzle | 0x2002002006000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x6002002002000000ll))
           altDoSolve(26, 0x4000000ll, puzzle | 0x6002002002000000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x2492000000ll))
        altDoSolve(26, 0x4000000ll, puzzle | 0x2492000000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x4004004006000000ll))
        altDoSolve(26, 0x4000000ll, puzzle | 0x4004004006000000ll,
                   partsUsed | 0x20l);
    altpiece6at25:
      if (partsUsed & 0x40l) goto altpiece7at25;
      if (!(puzzle & 0x2002002002000000ll)) {
         if(!(puzzle & 0x2002402002000000ll))
           altDoSolve(26, 0x4000000ll, puzzle | 0x2002402002000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2002002402000000ll))
           altDoSolve(26, 0x4000000ll, puzzle | 0x2002002402000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2002006002000000ll))
           altDoSolve(26, 0x4000000ll, puzzle | 0x2002006002000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2006002002000000ll))
           altDoSolve(26, 0x4000000ll, puzzle | 0x2006002002000000ll,
                   partsUsed | 0x40l);
      }
    altpiece7at25:
      if (partsUsed & 0x80l) goto altpiece8at25;
      if(!(puzzle & 0x6004006000000ll))
        altDoSolve(26, 0x4000000ll, puzzle | 0x6004006000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x6002006000000ll))
        altDoSolve(26, 0x4000000ll, puzzle | 0x6002006000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xe00a000000ll))
        altDoSolve(26, 0x4000000ll, puzzle | 0xe00a000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xa00e000000ll))
        altDoSolve(26, 0x4000000ll, puzzle | 0xa00e000000ll,
                   partsUsed | 0x80l);
    altpiece8at25:
      if (partsUsed & 0x100l) goto altpiece9at25;
      if(!(puzzle & 0x4806000000ll))
        altDoSolve(26, 0x4000000ll, puzzle | 0x4806000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x2c02000000ll))
        altDoSolve(26, 0x4000000ll, puzzle | 0x2c02000000ll,
                   partsUsed | 0x100l);
    altpiece9at25:
      if (partsUsed & 0x200l) goto altpiece10at25;
      if(!(puzzle & 0x402482000000ll))
        altDoSolve(26, 0x4000000ll, puzzle | 0x402482000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x400c006000000ll))
        altDoSolve(26, 0x4000000ll, puzzle | 0x400c006000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x400e002000000ll))
        altDoSolve(26, 0x4000000ll, puzzle | 0x400e002000000ll,
                   partsUsed | 0x200l);
    altpiece10at25:
      if (partsUsed & 0x400l) goto altpiece11at25;
      if (!(puzzle & 0x2002002000000ll)) {
         if(!(puzzle & 0x2402402000000ll))
           altDoSolve(26, 0x4000000ll, puzzle | 0x2402402000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2006006000000ll))
           altDoSolve(26, 0x4000000ll, puzzle | 0x2006006000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x6006002000000ll))
           altDoSolve(26, 0x4000000ll, puzzle | 0x6006002000000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x4006000000ll)) {
         if(!(puzzle & 0x4006006000000ll))
           altDoSolve(26, 0x4000000ll, puzzle | 0x4006006000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xe006000000ll))
           altDoSolve(26, 0x4000000ll, puzzle | 0xe006000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x600e000000ll))
           altDoSolve(26, 0x4000000ll, puzzle | 0x600e000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xc00e000000ll))
           altDoSolve(26, 0x4000000ll, puzzle | 0xc00e000000ll,
                   partsUsed | 0x400l);
      }
    altpiece11at25:
    break;
   case 26:
      if (partsUsed & 0x2l) goto altpiece2at26;
      if(!(puzzle & 0x904804000000ll))
        altDoSolve(27, 0x8000000ll, puzzle | 0x904804000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x200600c000000ll))
        altDoSolve(27, 0x8000000ll, puzzle | 0x200600c000000ll,
                   partsUsed | 0x2l);
    altpiece2at26:
      if (partsUsed & 0x4l) goto altpiece3at26;
      if(!(puzzle & 0x4004904000000ll))
        altDoSolve(27, 0x8000000ll, puzzle | 0x4004904000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0xe004004000000ll))
        altDoSolve(27, 0x8000000ll, puzzle | 0xe004004000000ll,
                   partsUsed | 0x4l);
    altpiece3at26:
      if (partsUsed & 0x8l) goto altpiece4at26;
      if(!(puzzle & 0x104904000000ll))
        altDoSolve(27, 0x8000000ll, puzzle | 0x104904000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x600400c000000ll))
        altDoSolve(27, 0x8000000ll, puzzle | 0x600400c000000ll,
                   partsUsed | 0x8l);
    altpiece4at26:
      if (partsUsed & 0x10l) goto altpiece5at26;
      if (!(puzzle & 0x4004004004000000ll)) {
         if(!(puzzle & 0x4804004804000000ll))
           altDoSolve(27, 0x8000000ll, puzzle | 0x4804004804000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x6004006004000000ll))
           altDoSolve(27, 0x8000000ll, puzzle | 0x6004006004000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0xc00400c004000000ll))
           altDoSolve(27, 0x8000000ll, puzzle | 0xc00400c004000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x400c00400c000000ll))
           altDoSolve(27, 0x8000000ll, puzzle | 0x400c00400c000000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x800c00800c000000ll))
        altDoSolve(27, 0x8000000ll, puzzle | 0x800c00800c000000ll,
                   partsUsed | 0x10l);
    altpiece5at26:
      if (partsUsed & 0x20l) goto altpiece6at26;
      if (!(puzzle & 0x4004004004000000ll)) {
         if(!(puzzle & 0x4804004004000000ll))
           altDoSolve(27, 0x8000000ll, puzzle | 0x4804004004000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x6004004004000000ll))
           altDoSolve(27, 0x8000000ll, puzzle | 0x6004004004000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x400400400c000000ll))
           altDoSolve(27, 0x8000000ll, puzzle | 0x400400400c000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0xc004004004000000ll))
           altDoSolve(27, 0x8000000ll, puzzle | 0xc004004004000000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x4924000000ll))
        altDoSolve(27, 0x8000000ll, puzzle | 0x4924000000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x800800800c000000ll))
        altDoSolve(27, 0x8000000ll, puzzle | 0x800800800c000000ll,
                   partsUsed | 0x20l);
    altpiece6at26:
      if (partsUsed & 0x40l) goto altpiece7at26;
      if (!(puzzle & 0x4004004004000000ll)) {
         if(!(puzzle & 0x4004804004000000ll))
           altDoSolve(27, 0x8000000ll, puzzle | 0x4004804004000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4004004804000000ll))
           altDoSolve(27, 0x8000000ll, puzzle | 0x4004004804000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4004006004000000ll))
           altDoSolve(27, 0x8000000ll, puzzle | 0x4004006004000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4006004004000000ll))
           altDoSolve(27, 0x8000000ll, puzzle | 0x4006004004000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x400400c004000000ll))
           altDoSolve(27, 0x8000000ll, puzzle | 0x400400c004000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x400c004004000000ll))
           altDoSolve(27, 0x8000000ll, puzzle | 0x400c004004000000ll,
                   partsUsed | 0x40l);
      }
    altpiece7at26:
      if (partsUsed & 0x80l) goto altpiece8at26;
      if(!(puzzle & 0xc00800c000000ll))
        altDoSolve(27, 0x8000000ll, puzzle | 0xc00800c000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xc00400c000000ll))
        altDoSolve(27, 0x8000000ll, puzzle | 0xc00400c000000ll,
                   partsUsed | 0x80l);
    altpiece8at26:
      if (partsUsed & 0x100l) goto altpiece9at26;
      if(!(puzzle & 0x5804000000ll))
        altDoSolve(27, 0x8000000ll, puzzle | 0x5804000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x6404000000ll))
        altDoSolve(27, 0x8000000ll, puzzle | 0x6404000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x900c000000ll))
        altDoSolve(27, 0x8000000ll, puzzle | 0x900c000000ll,
                   partsUsed | 0x100l);
    altpiece9at26:
      if (partsUsed & 0x200l) goto altpiece10at26;
      if (!(puzzle & 0x4004004000000ll)) {
         if(!(puzzle & 0x400600c000000ll))
           altDoSolve(27, 0x8000000ll, puzzle | 0x400600c000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0xc006004000000ll))
           altDoSolve(27, 0x8000000ll, puzzle | 0xc006004000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x600c004000000ll))
           altDoSolve(27, 0x8000000ll, puzzle | 0x600c004000000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x804904000000ll))
        altDoSolve(27, 0x8000000ll, puzzle | 0x804904000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x200e004000000ll))
        altDoSolve(27, 0x8000000ll, puzzle | 0x200e004000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x800e004000000ll))
        altDoSolve(27, 0x8000000ll, puzzle | 0x800e004000000ll,
                   partsUsed | 0x200l);
    altpiece10at26:
      if (partsUsed & 0x400l) goto altpiece11at26;
      if (!(puzzle & 0x4004004000000ll)) {
         if(!(puzzle & 0x4804804000000ll))
           altDoSolve(27, 0x8000000ll, puzzle | 0x4804804000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x6006004000000ll))
           altDoSolve(27, 0x8000000ll, puzzle | 0x6006004000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x400c00c000000ll))
           altDoSolve(27, 0x8000000ll, puzzle | 0x400c00c000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xc00c004000000ll))
           altDoSolve(27, 0x8000000ll, puzzle | 0xc00c004000000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0xe00c000000ll))
        altDoSolve(27, 0x8000000ll, puzzle | 0xe00c000000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x800c00c000000ll))
        altDoSolve(27, 0x8000000ll, puzzle | 0x800c00c000000ll,
                   partsUsed | 0x400l);
    altpiece11at26:
    break;
   case 27:
      if (partsUsed & 0x2l) goto altpiece2at27;
      if(!(puzzle & 0x1209008000000ll))
        altDoSolve(28, 0x10000000ll, puzzle | 0x1209008000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x600c008000000ll))
        altDoSolve(28, 0x10000000ll, puzzle | 0x600c008000000ll,
                   partsUsed | 0x2l);
    altpiece2at27:
      if (partsUsed & 0x4l) goto altpiece3at27;
      if(!(puzzle & 0x8009208000000ll))
        altDoSolve(28, 0x10000000ll, puzzle | 0x8009208000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x800e008000000ll))
        altDoSolve(28, 0x10000000ll, puzzle | 0x800e008000000ll,
                   partsUsed | 0x4l);
    altpiece3at27:
      if (partsUsed & 0x8l) goto altpiece4at27;
      if(!(puzzle & 0x209208000000ll))
        altDoSolve(28, 0x10000000ll, puzzle | 0x209208000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x200e008000000ll))
        altDoSolve(28, 0x10000000ll, puzzle | 0x200e008000000ll,
                   partsUsed | 0x8l);
    altpiece4at27:
      if (partsUsed & 0x10l) goto altpiece5at27;
      if(!(puzzle & 0x9008009008000000ll))
        altDoSolve(28, 0x10000000ll, puzzle | 0x9008009008000000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0xc00800c008000000ll))
        altDoSolve(28, 0x10000000ll, puzzle | 0xc00800c008000000ll,
                   partsUsed | 0x10l);
    altpiece5at27:
      if (partsUsed & 0x20l) goto altpiece6at27;
      if(!(puzzle & 0x9248000000ll))
        altDoSolve(28, 0x10000000ll, puzzle | 0x9248000000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x9008008008000000ll))
        altDoSolve(28, 0x10000000ll, puzzle | 0x9008008008000000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0xc008008008000000ll))
        altDoSolve(28, 0x10000000ll, puzzle | 0xc008008008000000ll,
                   partsUsed | 0x20l);
    altpiece6at27:
      if (partsUsed & 0x40l) goto altpiece7at27;
      if (!(puzzle & 0x8008008008000000ll)) {
         if(!(puzzle & 0x8009008008000000ll))
           altDoSolve(28, 0x10000000ll, puzzle | 0x8009008008000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x8008009008000000ll))
           altDoSolve(28, 0x10000000ll, puzzle | 0x8008009008000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800800c008000000ll))
           altDoSolve(28, 0x10000000ll, puzzle | 0x800800c008000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x800c008008000000ll))
           altDoSolve(28, 0x10000000ll, puzzle | 0x800c008008000000ll,
                   partsUsed | 0x40l);
      }
    altpiece7at27:
      if (partsUsed & 0x100l) goto altpiece9at27;
      if(!(puzzle & 0xc808000000ll))
        altDoSolve(28, 0x10000000ll, puzzle | 0xc808000000ll,
                   partsUsed | 0x100l);
    altpiece9at27:
      if (partsUsed & 0x200l) goto altpiece10at27;
      if(!(puzzle & 0x1009208000000ll))
        altDoSolve(28, 0x10000000ll, puzzle | 0x1009208000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x400e008000000ll))
        altDoSolve(28, 0x10000000ll, puzzle | 0x400e008000000ll,
                   partsUsed | 0x200l);
    altpiece10at27:
      if (partsUsed & 0x400l) goto altpiece11at27;
      if(!(puzzle & 0x9009008000000ll))
        altDoSolve(28, 0x10000000ll, puzzle | 0x9009008000000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0xc00c008000000ll))
        altDoSolve(28, 0x10000000ll, puzzle | 0xc00c008000000ll,
                   partsUsed | 0x400l);
    altpiece11at27:
    break;
   case 28:
      if (partsUsed & 0x2l) goto altpiece2at28;
      if(!(puzzle & 0x480090010000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x480090010000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x400480090000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x400480090000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1330000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x1330000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1990000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x1990000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x40060030000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x40060030000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x60030010000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x60030010000000ll,
                   partsUsed | 0x2l);
    altpiece2at28:
      if (partsUsed & 0x4l) goto altpiece3at28;
      if(!(puzzle & 0x10490010000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x10490010000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x10070010000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x10070010000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x80080490000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x80080490000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x790000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x790000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x970000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x970000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x20020070000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x20020070000000ll,
                   partsUsed | 0x4l);
    altpiece3at28:
      if (partsUsed & 0x8l) goto altpiece4at28;
      if(!(puzzle & 0x400490010000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x400490010000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x480080090000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x480080090000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1930000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x1930000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1390000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x1390000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x40070010000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x40070010000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x60020030000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x60020030000000ll,
                   partsUsed | 0x8l);
    altpiece4at28:
      if (partsUsed & 0x10l) goto altpiece5at28;
      if (!(puzzle & 0x2490000000ll)) {
         if(!(puzzle & 0x412490000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x412490000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x6590000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x6590000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x2082490000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x2082490000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x2cb0000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x2cb0000000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x4d30000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x4d30000000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x2490410000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x2490410000000ll,
                   partsUsed | 0x10l);
    altpiece5at28:
      if (partsUsed & 0x20l) goto altpiece6at28;
      if (!(puzzle & 0x2490000000ll)) {
         if(!(puzzle & 0x2002490000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x2002490000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x24b0000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x24b0000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x12490000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x12490000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x6490000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x6490000000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x2490010000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x2490010000000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x4930000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x4930000000ll,
                   partsUsed | 0x20l);
    altpiece6at28:
      if (partsUsed & 0x40l) goto altpiece7at28;
      if (!(puzzle & 0x2490000000ll)) {
         if(!(puzzle & 0x402490000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x402490000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2590000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x2590000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x82490000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x82490000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x2c90000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x2c90000000ll,
                   partsUsed | 0x40l);
      }
    altpiece7at28:
      if (partsUsed & 0x80l) goto altpiece8at28;
      if(!(puzzle & 0x90010090000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x90010090000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x90080090000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x90080090000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x490410000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x490410000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x410490000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x410490000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x2f0000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x2f0000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xcb0000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0xcb0000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0xd30000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0xd30000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x3d0000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x3d0000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x30020030000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x30020030000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x30010030000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x30010030000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x70050000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x70050000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x50070000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x50070000000ll,
                   partsUsed | 0x80l);
    altpiece8at28:
      if (partsUsed & 0x100l) goto altpiece9at28;
      if(!(puzzle & 0x100130000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x100130000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x130010000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x130010000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x200b0000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x200b0000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x90030000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x90030000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x180090000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x180090000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x10190000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x10190000000ll,
                   partsUsed | 0x100l);
    altpiece9at28:
      if (partsUsed & 0x200l) goto altpiece10at28;
      if(!(puzzle & 0x80490010000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x80490010000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x80480090000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x80480090000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0xb30000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0xb30000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0xb90000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0xb90000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x20060030000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x20060030000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x20070010000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x20070010000000ll,
                   partsUsed | 0x200l);
    altpiece10at28:
      if (partsUsed & 0x400l) goto altpiece11at28;
      if (!(puzzle & 0x80090000000ll)) {
         if(!(puzzle & 0x10090090000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x10090090000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x490090000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x490090000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x480490000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x480490000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x90490000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x90490000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x80090090000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x80090090000000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x130000000ll)) {
         if(!(puzzle & 0x370000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x370000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1f0000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x1f0000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x5b0000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x5b0000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x9b0000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x9b0000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x3b0000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x3b0000000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x10010010000000ll)) {
         if(!(puzzle & 0x90090010000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x90090010000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x10030030000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x10030030000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x30030010000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x30030010000000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x20030000000ll)) {
         if(!(puzzle & 0x20030030000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x20030030000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x70030000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x70030000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x30070000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x30070000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x60070000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x60070000000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0xd90000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0xd90000000ll,
                   partsUsed | 0x400l);
    altpiece11at28:
      if (partsUsed & 0x800l) goto altpiece13at28;
      if (!(puzzle & 0x10030000000ll)) {
         if(!(puzzle & 0x101b0000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x101b0000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xb0030000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0xb0030000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x300b0000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x300b0000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x30130000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x30130000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x130030000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x130030000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x900b0000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x900b0000000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x100110000000ll)) {
         if(!(puzzle & 0x180190000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x180190000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x120130000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x120130000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1001b0000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x1001b0000000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x80090000000ll)) {
         if(!(puzzle & 0xb0090000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0xb0090000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x90190000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x90190000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x801b0000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x801b0000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x190090000000ll))
           altDoSolve(29, 0x20000000ll, puzzle | 0x190090000000ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0x201b0000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x201b0000000ll,
                   partsUsed | 0x800l);
      if(!(puzzle & 0x1b0010000000ll))
        altDoSolve(29, 0x20000000ll, puzzle | 0x1b0010000000ll,
                   partsUsed | 0x800l);
    altpiece13at28:
    break;
   case 29:
      if (partsUsed & 0x2l) goto altpiece2at29;
      if(!(puzzle & 0x5e0000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x5e0000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x10030060000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x10030060000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x900120020000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x900120020000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x800900120000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x800900120000000ll,
                   partsUsed | 0x2l);
    altpiece2at29:
      if (partsUsed & 0x4l) goto altpiece3at29;
      if(!(puzzle & 0x1d20000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x1d20000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x100100920000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x100100920000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x70020020000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x70020020000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x20920020000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x20920020000000ll,
                   partsUsed | 0x4l);
    altpiece3at29:
      if (partsUsed & 0x8l) goto altpiece4at29;
      if(!(puzzle & 0xd60000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0xd60000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x30020060000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x30020060000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x800920020000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x800920020000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x900100120000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x900100120000000ll,
                   partsUsed | 0x8l);
    altpiece4at29:
      if (partsUsed & 0x10l) goto altpiece5at29;
      if (!(puzzle & 0x4920000000ll)) {
         if(!(puzzle & 0x69a0000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x69a0000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x824920000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x824920000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0xcb20000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0xcb20000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x4104920000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x4104920000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x5960000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x5960000000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x9a60000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x9a60000000ll,
                   partsUsed | 0x10l);
      if(!(puzzle & 0x4920820000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x4920820000000ll,
                   partsUsed | 0x10l);
    altpiece5at29:
      if (partsUsed & 0x20l) goto altpiece6at29;
      if (!(puzzle & 0x4920000000ll)) {
         if(!(puzzle & 0x6920000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x6920000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x4004920000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x4004920000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x4960000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x4960000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x24920000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x24920000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0xc920000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0xc920000000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x4920020000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x4920020000000ll,
                   partsUsed | 0x20l);
      if(!(puzzle & 0x9260000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x9260000000ll,
                   partsUsed | 0x20l);
    altpiece6at29:
      if (partsUsed & 0x40l) goto altpiece7at29;
      if (!(puzzle & 0x4920000000ll)) {
         if(!(puzzle & 0x49a0000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x49a0000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4d20000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x4d20000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x804920000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x804920000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x4b20000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x4b20000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x104920000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x104920000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x5920000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x5920000000ll,
                   partsUsed | 0x40l);
      }
    altpiece7at29:
      if (partsUsed & 0x80l) goto altpiece8at29;
      if(!(puzzle & 0x120020120000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x120020120000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x120100120000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x120100120000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x920820000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x920820000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x820920000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x820920000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1960000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x1960000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1a60000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x1a60000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x60040060000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x60040060000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x60020060000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x60020060000000ll,
                   partsUsed | 0x80l);
    altpiece8at29:
      if (partsUsed & 0x100l) goto altpiece9at29;
      if(!(puzzle & 0x801a0000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x801a0000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x30120000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x30120000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x20320000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x20320000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x1a0020000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x1a0020000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x120060000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x120060000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x260020000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x260020000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x40160000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x40160000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x300120000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x300120000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x200260000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x200260000000ll,
                   partsUsed | 0x100l);
    altpiece9at29:
      if (partsUsed & 0x200l) goto altpiece10at29;
      if (!(puzzle & 0x1a0000000ll)) {
         if(!(puzzle & 0x13a0000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x13a0000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x7a0000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x7a0000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x9e0000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x9e0000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x19a0000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x19a0000000ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x20020020000000ll)) {
         if(!(puzzle & 0x20030060000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x20030060000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x60030020000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x60030020000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x30060020000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x30060020000000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0xf20000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0xf20000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x10070020000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x10070020000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x40070020000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x40070020000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100920020000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x100920020000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100900120000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x100900120000000ll,
                   partsUsed | 0x200l);
    altpiece10at29:
      if (partsUsed & 0x400l) goto altpiece11at29;
      if (!(puzzle & 0x100120000000ll)) {
         if(!(puzzle & 0x20120120000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x20120120000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x920120000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x920120000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x900920000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x900920000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x120920000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x120920000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x100120120000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x100120120000000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x20020020000000ll)) {
         if(!(puzzle & 0x120120020000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x120120020000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x20060060000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x20060060000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x60060020000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x60060020000000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x320000000ll)) {
         if(!(puzzle & 0xb60000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0xb60000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1360000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x1360000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1b20000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x1b20000000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x3e0000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x3e0000000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0xda0000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0xda0000000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x70060000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x70060000000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x30030020000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x30030020000000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x40060060000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x40060060000000ll,
                   partsUsed | 0x400l);
    altpiece11at29:
      if (partsUsed & 0x800l) goto altpiece13at29;
      if (!(puzzle & 0x20060000000ll)) {
         if(!(puzzle & 0x20360000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x20360000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x160060000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x160060000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x60160000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x60160000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x60260000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x60260000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x260060000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x260060000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x120160000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x120160000000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x200220000000ll)) {
         if(!(puzzle & 0x300320000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x300320000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x240260000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x240260000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x200360000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x200360000000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x100120000000ll)) {
         if(!(puzzle & 0x160120000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x160120000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x120320000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x120320000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x100360000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x100360000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x320120000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x320120000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1a0120000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x1a0120000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1801a0000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x1801a0000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x130120000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x130120000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1201a0000000ll))
           altDoSolve(30, 0x40000000ll, puzzle | 0x1201a0000000ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0x40360000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x40360000000ll,
                   partsUsed | 0x800l);
      if(!(puzzle & 0x360020000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x360020000000ll,
                   partsUsed | 0x800l);
      if(!(puzzle & 0x1b0020000000ll))
        altDoSolve(30, 0x40000000ll, puzzle | 0x1b0020000000ll,
                   partsUsed | 0x800l);
    altpiece13at29:
    break;
   case 30:
      if (partsUsed & 0x2l) goto altpiece2at30;
      if(!(puzzle & 0xf40000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0xf40000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x30060040000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x30060040000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1200240040000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x1200240040000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x1001200240000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x1001200240000000ll,
                   partsUsed | 0x2l);
    altpiece2at30:
      if (partsUsed & 0x4l) goto altpiece3at30;
      if(!(puzzle & 0x13c0000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x13c0000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x200201240000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x200201240000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x40070040000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x40070040000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x41240040000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x41240040000000ll,
                   partsUsed | 0x4l);
    altpiece3at30:
      if (partsUsed & 0x8l) goto altpiece4at30;
      if(!(puzzle & 0x7c0000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x7c0000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x10070040000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x10070040000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1001240040000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x1001240040000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x1200200240000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x1200200240000000ll,
                   partsUsed | 0x8l);
    altpiece4at30:
      if (partsUsed & 0x10l) goto altpiece5at30;
      if (!(puzzle & 0x9240000000ll)) {
         if(!(puzzle & 0xd340000000ll))
           altDoSolve(31, 0x80000000ll, puzzle | 0xd340000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x1049240000000ll))
           altDoSolve(31, 0x80000000ll, puzzle | 0x1049240000000ll,
                   partsUsed | 0x10l);
         if(!(puzzle & 0x8209240000000ll))
           altDoSolve(31, 0x80000000ll, puzzle | 0x8209240000000ll,
                   partsUsed | 0x10l);
      }
      if(!(puzzle & 0x9241040000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x9241040000000ll,
                   partsUsed | 0x10l);
    altpiece5at30:
      if (partsUsed & 0x20l) goto altpiece6at30;
      if (!(puzzle & 0x9240000000ll)) {
         if(!(puzzle & 0xd240000000ll))
           altDoSolve(31, 0x80000000ll, puzzle | 0xd240000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x8009240000000ll))
           altDoSolve(31, 0x80000000ll, puzzle | 0x8009240000000ll,
                   partsUsed | 0x20l);
         if(!(puzzle & 0x49240000000ll))
           altDoSolve(31, 0x80000000ll, puzzle | 0x49240000000ll,
                   partsUsed | 0x20l);
      }
      if(!(puzzle & 0x9240040000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x9240040000000ll,
                   partsUsed | 0x20l);
    altpiece6at30:
      if (partsUsed & 0x40l) goto altpiece7at30;
      if (!(puzzle & 0x9240000000ll)) {
         if(!(puzzle & 0x9340000000ll))
           altDoSolve(31, 0x80000000ll, puzzle | 0x9340000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x9a40000000ll))
           altDoSolve(31, 0x80000000ll, puzzle | 0x9a40000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x1009240000000ll))
           altDoSolve(31, 0x80000000ll, puzzle | 0x1009240000000ll,
                   partsUsed | 0x40l);
         if(!(puzzle & 0x209240000000ll))
           altDoSolve(31, 0x80000000ll, puzzle | 0x209240000000ll,
                   partsUsed | 0x40l);
      }
    altpiece7at30:
      if (partsUsed & 0x80l) goto altpiece8at30;
      if(!(puzzle & 0x240040240000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x240040240000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x240200240000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x240200240000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1241040000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x1241040000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1041240000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x1041240000000ll,
                   partsUsed | 0x80l);
    altpiece8at30:
      if (partsUsed & 0x100l) goto altpiece9at30;
      if(!(puzzle & 0x340040000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x340040000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x100340000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x100340000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x60240000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x60240000000ll,
                   partsUsed | 0x100l);
    altpiece9at30:
      if (partsUsed & 0x200l) goto altpiece10at30;
      if(!(puzzle & 0xbc0000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0xbc0000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x20070040000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x20070040000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x201240040000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x201240040000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x201200240000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x201200240000000ll,
                   partsUsed | 0x200l);
    altpiece10at30:
      if (partsUsed & 0x400l) goto altpiece11at30;
      if (!(puzzle & 0x1240000000ll)) {
         if(!(puzzle & 0x1b40000000ll))
           altDoSolve(31, 0x80000000ll, puzzle | 0x1b40000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1201240000000ll))
           altDoSolve(31, 0x80000000ll, puzzle | 0x1201240000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x241240000000ll))
           altDoSolve(31, 0x80000000ll, puzzle | 0x241240000000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x40040040000000ll)) {
         if(!(puzzle & 0x60060040000000ll))
           altDoSolve(31, 0x80000000ll, puzzle | 0x60060040000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x40240240000000ll))
           altDoSolve(31, 0x80000000ll, puzzle | 0x40240240000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x240240040000000ll))
           altDoSolve(31, 0x80000000ll, puzzle | 0x240240040000000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x1240240000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x1240240000000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x200240240000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x200240240000000ll,
                   partsUsed | 0x400l);
    altpiece11at30:
      if (partsUsed & 0x800l) goto altpiece13at30;
      if (!(puzzle & 0x300040000000ll)) {
         if(!(puzzle & 0x360040000000ll))
           altDoSolve(31, 0x80000000ll, puzzle | 0x360040000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x340240000000ll))
           altDoSolve(31, 0x80000000ll, puzzle | 0x340240000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x300340000000ll))
           altDoSolve(31, 0x80000000ll, puzzle | 0x300340000000ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0x260240000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x260240000000ll,
                   partsUsed | 0x800l);
      if(!(puzzle & 0x240340000000ll))
        altDoSolve(31, 0x80000000ll, puzzle | 0x240340000000ll,
                   partsUsed | 0x800l);
    altpiece13at30:
    break;
   case 31:
      if (partsUsed & 0x2l) goto altpiece2at31;
      if(!(puzzle & 0x10090480000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x10090480000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x2400480080000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x2400480080000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x2002400480000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x2002400480000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x9980000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x9980000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0xcc80000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0xcc80000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x200300180000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x200300180000000ll,
                   partsUsed | 0x2l);
      if(!(puzzle & 0x300180080000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x300180080000000ll,
                   partsUsed | 0x2l);
    altpiece2at31:
      if (partsUsed & 0x4l) goto altpiece3at31;
      if (!(puzzle & 0x80080080000000ll)) {
         if(!(puzzle & 0x490080080000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x490080080000000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x82480080000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x82480080000000ll,
                   partsUsed | 0x4l);
         if(!(puzzle & 0x80380080000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x80380080000000ll,
                   partsUsed | 0x4l);
      }
      if(!(puzzle & 0x400402480000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x400402480000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x3c80000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x3c80000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x4b80000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x4b80000000ll,
                   partsUsed | 0x4l);
      if(!(puzzle & 0x100100380000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x100100380000000ll,
                   partsUsed | 0x4l);
    altpiece3at31:
      if (partsUsed & 0x8l) goto altpiece4at31;
      if(!(puzzle & 0x90080480000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x90080480000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x2002480080000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x2002480080000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x2400400480000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x2400400480000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0xc980000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0xc980000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x9c80000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x9c80000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x200380080000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x200380080000000ll,
                   partsUsed | 0x8l);
      if(!(puzzle & 0x300100180000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x300100180000000ll,
                   partsUsed | 0x8l);
    altpiece4at31:
      if (partsUsed & 0x10l) goto altpiece5at31;
      if(!(puzzle & 0x2492080000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x2492080000000ll,
                   partsUsed | 0x10l);
    altpiece5at31:
      if (partsUsed & 0x40l) goto altpiece7at31;
      if(!(puzzle & 0x2490080000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x2490080000000ll,
                   partsUsed | 0x40l);
    altpiece7at31:
      if (partsUsed & 0x80l) goto altpiece8at31;
      if(!(puzzle & 0x480080480000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x480080480000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x480400480000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x480400480000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x2482080000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x2482080000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x2082480000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x2082480000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1780000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x1780000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x6580000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x6580000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x6980000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x6980000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x1e80000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x1e80000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x180100180000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x180100180000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x180080180000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x180080180000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x380280000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x380280000000ll,
                   partsUsed | 0x80l);
      if(!(puzzle & 0x280380000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x280380000000ll,
                   partsUsed | 0x80l);
    altpiece8at31:
      if (partsUsed & 0x100l) goto altpiece9at31;
      if(!(puzzle & 0x120180000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x120180000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x100580000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x100580000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xb0080000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0xb0080000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x980080000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x980080000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x480180000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x480180000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0xc00480000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0xc00480000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x80c80000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x80c80000000ll,
                   partsUsed | 0x100l);
      if(!(puzzle & 0x800980000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x800980000000ll,
                   partsUsed | 0x100l);
    altpiece9at31:
      if (partsUsed & 0x200l) goto altpiece10at31;
      if (!(puzzle & 0x80080080000000ll)) {
         if(!(puzzle & 0x80090480000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x80090480000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x90480080000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x90480080000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x480090080000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x480090080000000ll,
                   partsUsed | 0x200l);
      }
      if (!(puzzle & 0x480080000000ll)) {
         if(!(puzzle & 0x400490080000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x400490080000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x10490080000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x10490080000000ll,
                   partsUsed | 0x200l);
         if(!(puzzle & 0x402480080000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x402480080000000ll,
                   partsUsed | 0x200l);
      }
      if(!(puzzle & 0x5980000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x5980000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x5c80000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x5c80000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x402400480000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x402400480000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100300180000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x100300180000000ll,
                   partsUsed | 0x200l);
      if(!(puzzle & 0x100380080000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x100380080000000ll,
                   partsUsed | 0x200l);
    altpiece10at31:
      if (partsUsed & 0x400l) goto altpiece11at31;
      if (!(puzzle & 0x400480000000ll)) {
         if(!(puzzle & 0x80480480000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x80480480000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2480480000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x2480480000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2402480000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x2402480000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x482480000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x482480000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x400480480000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x400480480000000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x980000000ll)) {
         if(!(puzzle & 0x1b80000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x1b80000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0xf80000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0xf80000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x2d80000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x2d80000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x4d80000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x4d80000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x1d80000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x1d80000000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x80080080000000ll)) {
         if(!(puzzle & 0x480480080000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x480480080000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x80180180000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x80180180000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x180180080000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x180180080000000ll,
                   partsUsed | 0x400l);
      }
      if (!(puzzle & 0x100180000000ll)) {
         if(!(puzzle & 0x100180180000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x100180180000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x380180000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x380180000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x180380000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x180380000000ll,
                   partsUsed | 0x400l);
         if(!(puzzle & 0x300380000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x300380000000ll,
                   partsUsed | 0x400l);
      }
      if(!(puzzle & 0x90090080000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x90090080000000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x490480000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x490480000000ll,
                   partsUsed | 0x400l);
      if(!(puzzle & 0x6c80000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x6c80000000ll,
                   partsUsed | 0x400l);
    altpiece11at31:
      if (partsUsed & 0x800l) goto altpiece13at31;
      if (!(puzzle & 0x80180000000ll)) {
         if(!(puzzle & 0x80d80000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x80d80000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x580180000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x580180000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x180580000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x180580000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x180980000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x180980000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x980180000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x980180000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x480580000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x480580000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x1a0180000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x1a0180000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x190180000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x190180000000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x800880000000ll)) {
         if(!(puzzle & 0xc00c80000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0xc00c80000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x900980000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x900980000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x800d80000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x800d80000000ll,
                   partsUsed | 0x800l);
      }
      if (!(puzzle & 0x400480000000ll)) {
         if(!(puzzle & 0x580480000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x580480000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x480c80000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x480c80000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0x400d80000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0x400d80000000ll,
                   partsUsed | 0x800l);
         if(!(puzzle & 0xc80480000000ll))
           altDoSolve(32, 0x100000000ll, puzzle | 0xc80480000000ll,
                   partsUsed | 0x800l);
      }
      if(!(puzzle & 0x100d80000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x100d80000000ll,
                   partsUsed | 0x800l);
      if(!(puzzle & 0xd80080000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0xd80080000000ll,
                   partsUsed | 0x800l);
      if(!(puzzle & 0x1b0080000000ll))
        altDoSolve(32, 0x100000000ll, puzzle | 0x1b0080000000ll,
                   partsUsed | 0x800l);
    altpiece13at31:
    break;
  }
}


void altDoSolve32(int start, int startMask, int puzzle, int partsUsed)
{
  for (;(puzzle & startMask) && start < NUM_HI_BITS; start++, startMask <<= 1);
  if(partsUsed == (1<<12)-1) {
    if(!(++numSolutions%4099)) { write(0, ".", 1); }
    if(!((numSolutions) % (409963/10))) {
      percent++;
      printf("\r%d%% done, %ld seconds so far; expecting %.0lf seconds.\n", 
	     percent*10, (clock()-startTime)/1000000, ((double)(clock()-startTime)/100000/percent)*correction[percent-1]);
      fflush(stdout);
    }
    return;
  }
  switch(start) {
   case 0:
      if (partsUsed & 0x2l) goto altpiece2at0at32;
      if(!(puzzle & 0x201209l))
        altDoSolve32(1, 0x2l, puzzle | 0x201209l, partsUsed | 0x2l);
      if(!(puzzle & 0x2fl))
        altDoSolve32(1, 0x2l, puzzle | 0x2fl, partsUsed | 0x2l);
      if(!(puzzle & 0x801803l))
        altDoSolve32(1, 0x2l, puzzle | 0x801803l, partsUsed | 0x2l);
      if(!(puzzle & 0x48009001l))
        altDoSolve32(1, 0x2l, puzzle | 0x48009001l, partsUsed | 0x2l);
      if(!(puzzle & 0x40048009l))
        altDoSolve32(1, 0x2l, puzzle | 0x40048009l, partsUsed | 0x2l);
    altpiece2at0at32:
      if (partsUsed & 0x4l) goto altpiece3at0at32;
      if (!(puzzle & 0x1001001l)) {
         if(!(puzzle & 0x9201001l))
           altDoSolve32(1, 0x2l, puzzle | 0x9201001l, partsUsed | 0x4l);
         if(!(puzzle & 0x3801001l))
           altDoSolve32(1, 0x2l, puzzle | 0x3801001l, partsUsed | 0x4l);
         if(!(puzzle & 0x1049001l))
           altDoSolve32(1, 0x2l, puzzle | 0x1049001l, partsUsed | 0x4l);
      }
      if(!(puzzle & 0xe9l))
        altDoSolve32(1, 0x2l, puzzle | 0xe9l, partsUsed | 0x4l);
      if(!(puzzle & 0x8008049l))
        altDoSolve32(1, 0x2l, puzzle | 0x8008049l, partsUsed | 0x4l);
    altpiece3at0at32:
      if (partsUsed & 0x8l) goto altpiece4at0at32;
      if(!(puzzle & 0x1201009l))
        altDoSolve32(1, 0x2l, puzzle | 0x1201009l, partsUsed | 0x8l);
      if(!(puzzle & 0x1801003l))
        altDoSolve32(1, 0x2l, puzzle | 0x1801003l, partsUsed | 0x8l);
      if(!(puzzle & 0x6bl))
        altDoSolve32(1, 0x2l, puzzle | 0x6bl, partsUsed | 0x8l);
      if(!(puzzle & 0x40049001l))
        altDoSolve32(1, 0x2l, puzzle | 0x40049001l, partsUsed | 0x8l);
      if(!(puzzle & 0x48008009l))
        altDoSolve32(1, 0x2l, puzzle | 0x48008009l, partsUsed | 0x8l);
    altpiece4at0at32:
      if (partsUsed & 0x10l) goto altpiece5at0at32;
      if(!(puzzle & 0x49241l))
        altDoSolve32(1, 0x2l, puzzle | 0x49241l, partsUsed | 0x10l);
    altpiece5at0at32:
      if (partsUsed & 0x40l) goto altpiece7at0at32;
      if(!(puzzle & 0x49201l))
        altDoSolve32(1, 0x2l, puzzle | 0x49201l, partsUsed | 0x40l);
    altpiece7at0at32:
      if (partsUsed & 0x80l) goto altpiece8at0at32;
      if(!(puzzle & 0x9001009l))
        altDoSolve32(1, 0x2l, puzzle | 0x9001009l, partsUsed | 0x80l);
      if(!(puzzle & 0x9008009l))
        altDoSolve32(1, 0x2l, puzzle | 0x9008009l, partsUsed | 0x80l);
      if(!(puzzle & 0x49041l))
        altDoSolve32(1, 0x2l, puzzle | 0x49041l, partsUsed | 0x80l);
      if(!(puzzle & 0x41049l))
        altDoSolve32(1, 0x2l, puzzle | 0x41049l, partsUsed | 0x80l);
      if(!(puzzle & 0xcbl))
        altDoSolve32(1, 0x2l, puzzle | 0xcbl, partsUsed | 0x80l);
      if(!(puzzle & 0xd3l))
        altDoSolve32(1, 0x2l, puzzle | 0xd3l, partsUsed | 0x80l);
      if(!(puzzle & 0x3002003l))
        altDoSolve32(1, 0x2l, puzzle | 0x3002003l, partsUsed | 0x80l);
      if(!(puzzle & 0x3001003l))
        altDoSolve32(1, 0x2l, puzzle | 0x3001003l, partsUsed | 0x80l);
    altpiece8at0at32:
      if (partsUsed & 0x100l) goto altpiece9at0at32;
      if(!(puzzle & 0x1901l))
        altDoSolve32(1, 0x2l, puzzle | 0x1901l, partsUsed | 0x100l);
      if(!(puzzle & 0x1809l))
        altDoSolve32(1, 0x2l, puzzle | 0x1809l, partsUsed | 0x100l);
      if(!(puzzle & 0x2403l))
        altDoSolve32(1, 0x2l, puzzle | 0x2403l, partsUsed | 0x100l);
      if(!(puzzle & 0x200bl))
        altDoSolve32(1, 0x2l, puzzle | 0x200bl, partsUsed | 0x100l);
      if(!(puzzle & 0x1601l))
        altDoSolve32(1, 0x2l, puzzle | 0x1601l, partsUsed | 0x100l);
      if(!(puzzle & 0x400dl))
        altDoSolve32(1, 0x2l, puzzle | 0x400dl, partsUsed | 0x100l);
      if(!(puzzle & 0xd001l))
        altDoSolve32(1, 0x2l, puzzle | 0xd001l, partsUsed | 0x100l);
      if(!(puzzle & 0x9003l))
        altDoSolve32(1, 0x2l, puzzle | 0x9003l, partsUsed | 0x100l);
      if(!(puzzle & 0x13001l))
        altDoSolve32(1, 0x2l, puzzle | 0x13001l, partsUsed | 0x100l);
      if(!(puzzle & 0x18009l))
        altDoSolve32(1, 0x2l, puzzle | 0x18009l, partsUsed | 0x100l);
      if(!(puzzle & 0x1019l))
        altDoSolve32(1, 0x2l, puzzle | 0x1019l, partsUsed | 0x100l);
      if(!(puzzle & 0x10013l))
        altDoSolve32(1, 0x2l, puzzle | 0x10013l, partsUsed | 0x100l);
    altpiece9at0at32:
      if (partsUsed & 0x200l) goto altpiece10at0at32;
      if (!(puzzle & 0x1001001l)) {
         if(!(puzzle & 0x1001209l))
           altDoSolve32(1, 0x2l, puzzle | 0x1001209l, partsUsed | 0x200l);
         if(!(puzzle & 0x1209001l))
           altDoSolve32(1, 0x2l, puzzle | 0x1209001l, partsUsed | 0x200l);
         if(!(puzzle & 0x9001201l))
           altDoSolve32(1, 0x2l, puzzle | 0x9001201l, partsUsed | 0x200l);
         if(!(puzzle & 0x1001803l))
           altDoSolve32(1, 0x2l, puzzle | 0x1001803l, partsUsed | 0x200l);
         if(!(puzzle & 0x3001801l))
           altDoSolve32(1, 0x2l, puzzle | 0x3001801l, partsUsed | 0x200l);
         if(!(puzzle & 0x1803001l))
           altDoSolve32(1, 0x2l, puzzle | 0x1803001l, partsUsed | 0x200l);
      }
      if (!(puzzle & 0x9001l)) {
         if(!(puzzle & 0x8009201l))
           altDoSolve32(1, 0x2l, puzzle | 0x8009201l, partsUsed | 0x200l);
         if(!(puzzle & 0x209201l))
           altDoSolve32(1, 0x2l, puzzle | 0x209201l, partsUsed | 0x200l);
         if(!(puzzle & 0x8049001l))
           altDoSolve32(1, 0x2l, puzzle | 0x8049001l, partsUsed | 0x200l);
      }
      if (!(puzzle & 0xdl)) {
         if(!(puzzle & 0x9dl))
           altDoSolve32(1, 0x2l, puzzle | 0x9dl, partsUsed | 0x200l);
         if(!(puzzle & 0x3dl))
           altDoSolve32(1, 0x2l, puzzle | 0x3dl, partsUsed | 0x200l);
         if(!(puzzle & 0x4fl))
           altDoSolve32(1, 0x2l, puzzle | 0x4fl, partsUsed | 0x200l);
         if(!(puzzle & 0xcdl))
           altDoSolve32(1, 0x2l, puzzle | 0xcdl, partsUsed | 0x200l);
      }
      if(!(puzzle & 0x79l))
        altDoSolve32(1, 0x2l, puzzle | 0x79l, partsUsed | 0x200l);
      if(!(puzzle & 0x803801l))
        altDoSolve32(1, 0x2l, puzzle | 0x803801l, partsUsed | 0x200l);
      if(!(puzzle & 0x2003801l))
        altDoSolve32(1, 0x2l, puzzle | 0x2003801l, partsUsed | 0x200l);
      if(!(puzzle & 0x8048009l))
        altDoSolve32(1, 0x2l, puzzle | 0x8048009l, partsUsed | 0x200l);
    altpiece10at0at32:
      if (partsUsed & 0x400l) goto altpiece11at0at32;
      if (!(puzzle & 0x8009l)) {
         if(!(puzzle & 0x1009009l))
           altDoSolve32(1, 0x2l, puzzle | 0x1009009l, partsUsed | 0x400l);
         if(!(puzzle & 0x49009l))
           altDoSolve32(1, 0x2l, puzzle | 0x49009l, partsUsed | 0x400l);
         if(!(puzzle & 0x48049l))
           altDoSolve32(1, 0x2l, puzzle | 0x48049l, partsUsed | 0x400l);
         if(!(puzzle & 0x9049l))
           altDoSolve32(1, 0x2l, puzzle | 0x9049l, partsUsed | 0x400l);
         if(!(puzzle & 0x8009009l))
           altDoSolve32(1, 0x2l, puzzle | 0x8009009l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x1001001l)) {
         if(!(puzzle & 0x9009001l))
           altDoSolve32(1, 0x2l, puzzle | 0x9009001l, partsUsed | 0x400l);
         if(!(puzzle & 0x1003003l))
           altDoSolve32(1, 0x2l, puzzle | 0x1003003l, partsUsed | 0x400l);
         if(!(puzzle & 0x3003001l))
           altDoSolve32(1, 0x2l, puzzle | 0x3003001l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x19l)) {
         if(!(puzzle & 0x5bl))
           altDoSolve32(1, 0x2l, puzzle | 0x5bl, partsUsed | 0x400l);
         if(!(puzzle & 0x9bl))
           altDoSolve32(1, 0x2l, puzzle | 0x9bl, partsUsed | 0x400l);
         if(!(puzzle & 0xd9l))
           altDoSolve32(1, 0x2l, puzzle | 0xd9l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x1201201l))
        altDoSolve32(1, 0x2l, puzzle | 0x1201201l, partsUsed | 0x400l);
      if(!(puzzle & 0x9209l))
        altDoSolve32(1, 0x2l, puzzle | 0x9209l, partsUsed | 0x400l);
      if(!(puzzle & 0x1fl))
        altDoSolve32(1, 0x2l, puzzle | 0x1fl, partsUsed | 0x400l);
      if(!(puzzle & 0x6dl))
        altDoSolve32(1, 0x2l, puzzle | 0x6dl, partsUsed | 0x400l);
      if(!(puzzle & 0x3803l))
        altDoSolve32(1, 0x2l, puzzle | 0x3803l, partsUsed | 0x400l);
      if(!(puzzle & 0x1801801l))
        altDoSolve32(1, 0x2l, puzzle | 0x1801801l, partsUsed | 0x400l);
      if(!(puzzle & 0x2003003l))
        altDoSolve32(1, 0x2l, puzzle | 0x2003003l, partsUsed | 0x400l);
    altpiece11at0at32:
      if (partsUsed & 0x800l) goto altpiece13at0at32;
      if (!(puzzle & 0x1003l)) {
         if(!(puzzle & 0x101bl))
           altDoSolve32(1, 0x2l, puzzle | 0x101bl, partsUsed | 0x800l);
         if(!(puzzle & 0xb003l))
           altDoSolve32(1, 0x2l, puzzle | 0xb003l, partsUsed | 0x800l);
         if(!(puzzle & 0x300bl))
           altDoSolve32(1, 0x2l, puzzle | 0x300bl, partsUsed | 0x800l);
         if(!(puzzle & 0x3013l))
           altDoSolve32(1, 0x2l, puzzle | 0x3013l, partsUsed | 0x800l);
         if(!(puzzle & 0x13003l))
           altDoSolve32(1, 0x2l, puzzle | 0x13003l, partsUsed | 0x800l);
         if(!(puzzle & 0x900bl))
           altDoSolve32(1, 0x2l, puzzle | 0x900bl, partsUsed | 0x800l);
         if(!(puzzle & 0x3403l))
           altDoSolve32(1, 0x2l, puzzle | 0x3403l, partsUsed | 0x800l);
         if(!(puzzle & 0x3203l))
           altDoSolve32(1, 0x2l, puzzle | 0x3203l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x10011l)) {
         if(!(puzzle & 0x18019l))
           altDoSolve32(1, 0x2l, puzzle | 0x18019l, partsUsed | 0x800l);
         if(!(puzzle & 0x12013l))
           altDoSolve32(1, 0x2l, puzzle | 0x12013l, partsUsed | 0x800l);
         if(!(puzzle & 0x1001bl))
           altDoSolve32(1, 0x2l, puzzle | 0x1001bl, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x8009l)) {
         if(!(puzzle & 0xb009l))
           altDoSolve32(1, 0x2l, puzzle | 0xb009l, partsUsed | 0x800l);
         if(!(puzzle & 0x9019l))
           altDoSolve32(1, 0x2l, puzzle | 0x9019l, partsUsed | 0x800l);
         if(!(puzzle & 0x801bl))
           altDoSolve32(1, 0x2l, puzzle | 0x801bl, partsUsed | 0x800l);
         if(!(puzzle & 0x19009l))
           altDoSolve32(1, 0x2l, puzzle | 0x19009l, partsUsed | 0x800l);
         if(!(puzzle & 0xd009l))
           altDoSolve32(1, 0x2l, puzzle | 0xd009l, partsUsed | 0x800l);
         if(!(puzzle & 0xc00dl))
           altDoSolve32(1, 0x2l, puzzle | 0xc00dl, partsUsed | 0x800l);
         if(!(puzzle & 0x9809l))
           altDoSolve32(1, 0x2l, puzzle | 0x9809l, partsUsed | 0x800l);
         if(!(puzzle & 0x900dl))
           altDoSolve32(1, 0x2l, puzzle | 0x900dl, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x201bl))
        altDoSolve32(1, 0x2l, puzzle | 0x201bl, partsUsed | 0x800l);
      if(!(puzzle & 0x1b001l))
        altDoSolve32(1, 0x2l, puzzle | 0x1b001l, partsUsed | 0x800l);
      if(!(puzzle & 0xd801l))
        altDoSolve32(1, 0x2l, puzzle | 0xd801l, partsUsed | 0x800l);
      if(!(puzzle & 0x3601l))
        altDoSolve32(1, 0x2l, puzzle | 0x3601l, partsUsed | 0x800l);
      if(!(puzzle & 0x1b01l))
        altDoSolve32(1, 0x2l, puzzle | 0x1b01l, partsUsed | 0x800l);
    altpiece13at0at32:
    break;
   case 1:
      if (partsUsed & 0x2l) goto altpiece2at1at32;
      if(!(puzzle & 0x402412l))
        altDoSolve32(2, 0x4l, puzzle | 0x402412l, partsUsed | 0x2l);
      if(!(puzzle & 0x7al))
        altDoSolve32(2, 0x4l, puzzle | 0x7al, partsUsed | 0x2l);
      if(!(puzzle & 0x1803002l))
        altDoSolve32(2, 0x4l, puzzle | 0x1803002l, partsUsed | 0x2l);
      if(!(puzzle & 0x90012002l))
        altDoSolve32(2, 0x4l, puzzle | 0x90012002l, partsUsed | 0x2l);
      if(!(puzzle & 0x80090012l))
        altDoSolve32(2, 0x4l, puzzle | 0x80090012l, partsUsed | 0x2l);
    altpiece2at1at32:
      if (partsUsed & 0x4l) goto altpiece3at1at32;
      if (!(puzzle & 0x2002002l)) {
         if(!(puzzle & 0x12402002l))
           altDoSolve32(2, 0x4l, puzzle | 0x12402002l, partsUsed | 0x4l);
         if(!(puzzle & 0x2003802l))
           altDoSolve32(2, 0x4l, puzzle | 0x2003802l, partsUsed | 0x4l);
         if(!(puzzle & 0x2092002l))
           altDoSolve32(2, 0x4l, puzzle | 0x2092002l, partsUsed | 0x4l);
      }
      if(!(puzzle & 0x9el))
        altDoSolve32(2, 0x4l, puzzle | 0x9el, partsUsed | 0x4l);
      if(!(puzzle & 0x10010092l))
        altDoSolve32(2, 0x4l, puzzle | 0x10010092l, partsUsed | 0x4l);
    altpiece3at1at32:
      if (partsUsed & 0x8l) goto altpiece4at1at32;
      if(!(puzzle & 0x2402012l))
        altDoSolve32(2, 0x4l, puzzle | 0x2402012l, partsUsed | 0x8l);
      if(!(puzzle & 0x3el))
        altDoSolve32(2, 0x4l, puzzle | 0x3el, partsUsed | 0x8l);
      if(!(puzzle & 0x803802l))
        altDoSolve32(2, 0x4l, puzzle | 0x803802l, partsUsed | 0x8l);
      if(!(puzzle & 0x80092002l))
        altDoSolve32(2, 0x4l, puzzle | 0x80092002l, partsUsed | 0x8l);
      if(!(puzzle & 0x90010012l))
        altDoSolve32(2, 0x4l, puzzle | 0x90010012l, partsUsed | 0x8l);
    altpiece4at1at32:
      if (partsUsed & 0x10l) goto altpiece5at1at32;
      if(!(puzzle & 0x92482l))
        altDoSolve32(2, 0x4l, puzzle | 0x92482l, partsUsed | 0x10l);
    altpiece5at1at32:
      if (partsUsed & 0x40l) goto altpiece7at1at32;
      if(!(puzzle & 0x92402l))
        altDoSolve32(2, 0x4l, puzzle | 0x92402l, partsUsed | 0x40l);
    altpiece7at1at32:
      if (partsUsed & 0x80l) goto altpiece8at1at32;
      if(!(puzzle & 0x12002012l))
        altDoSolve32(2, 0x4l, puzzle | 0x12002012l, partsUsed | 0x80l);
      if(!(puzzle & 0x12010012l))
        altDoSolve32(2, 0x4l, puzzle | 0x12010012l, partsUsed | 0x80l);
      if(!(puzzle & 0x92082l))
        altDoSolve32(2, 0x4l, puzzle | 0x92082l, partsUsed | 0x80l);
      if(!(puzzle & 0x82092l))
        altDoSolve32(2, 0x4l, puzzle | 0x82092l, partsUsed | 0x80l);
    altpiece8at1at32:
      if (partsUsed & 0x100l) goto altpiece9at1at32;
      if(!(puzzle & 0x3202l))
        altDoSolve32(2, 0x4l, puzzle | 0x3202l, partsUsed | 0x100l);
      if(!(puzzle & 0x3012l))
        altDoSolve32(2, 0x4l, puzzle | 0x3012l, partsUsed | 0x100l);
      if(!(puzzle & 0x801al))
        altDoSolve32(2, 0x4l, puzzle | 0x801al, partsUsed | 0x100l);
      if(!(puzzle & 0x1a002l))
        altDoSolve32(2, 0x4l, puzzle | 0x1a002l, partsUsed | 0x100l);
    altpiece9at1at32:
      if (partsUsed & 0x200l) goto altpiece10at1at32;
      if (!(puzzle & 0x2002002l)) {
         if(!(puzzle & 0x2002412l))
           altDoSolve32(2, 0x4l, puzzle | 0x2002412l, partsUsed | 0x200l);
         if(!(puzzle & 0x2412002l))
           altDoSolve32(2, 0x4l, puzzle | 0x2412002l, partsUsed | 0x200l);
         if(!(puzzle & 0x12002402l))
           altDoSolve32(2, 0x4l, puzzle | 0x12002402l, partsUsed | 0x200l);
      }
      if (!(puzzle & 0x12002l)) {
         if(!(puzzle & 0x10012402l))
           altDoSolve32(2, 0x4l, puzzle | 0x10012402l, partsUsed | 0x200l);
         if(!(puzzle & 0x412402l))
           altDoSolve32(2, 0x4l, puzzle | 0x412402l, partsUsed | 0x200l);
         if(!(puzzle & 0x10092002l))
           altDoSolve32(2, 0x4l, puzzle | 0x10092002l, partsUsed | 0x200l);
      }
      if(!(puzzle & 0x5el))
        altDoSolve32(2, 0x4l, puzzle | 0x5el, partsUsed | 0x200l);
      if(!(puzzle & 0x1003802l))
        altDoSolve32(2, 0x4l, puzzle | 0x1003802l, partsUsed | 0x200l);
      if(!(puzzle & 0x10090012l))
        altDoSolve32(2, 0x4l, puzzle | 0x10090012l, partsUsed | 0x200l);
    altpiece10at1at32:
      if (partsUsed & 0x400l) goto altpiece11at1at32;
      if (!(puzzle & 0x92l)) {
         if(!(puzzle & 0xdal))
           altDoSolve32(2, 0x4l, puzzle | 0xdal, partsUsed | 0x400l);
         if(!(puzzle & 0x90092l))
           altDoSolve32(2, 0x4l, puzzle | 0x90092l, partsUsed | 0x400l);
         if(!(puzzle & 0x12092l))
           altDoSolve32(2, 0x4l, puzzle | 0x12092l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x2002002l)) {
         if(!(puzzle & 0x3003002l))
           altDoSolve32(2, 0x4l, puzzle | 0x3003002l, partsUsed | 0x400l);
         if(!(puzzle & 0x2012012l))
           altDoSolve32(2, 0x4l, puzzle | 0x2012012l, partsUsed | 0x400l);
         if(!(puzzle & 0x12012002l))
           altDoSolve32(2, 0x4l, puzzle | 0x12012002l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x2402402l))
        altDoSolve32(2, 0x4l, puzzle | 0x2402402l, partsUsed | 0x400l);
      if(!(puzzle & 0x12412l))
        altDoSolve32(2, 0x4l, puzzle | 0x12412l, partsUsed | 0x400l);
      if(!(puzzle & 0x92012l))
        altDoSolve32(2, 0x4l, puzzle | 0x92012l, partsUsed | 0x400l);
      if(!(puzzle & 0x10012012l))
        altDoSolve32(2, 0x4l, puzzle | 0x10012012l, partsUsed | 0x400l);
    altpiece11at1at32:
      if (partsUsed & 0x800l) goto altpiece13at1at32;
      if (!(puzzle & 0x18002l)) {
         if(!(puzzle & 0x1b002l))
           altDoSolve32(2, 0x4l, puzzle | 0x1b002l, partsUsed | 0x800l);
         if(!(puzzle & 0x1a012l))
           altDoSolve32(2, 0x4l, puzzle | 0x1a012l, partsUsed | 0x800l);
         if(!(puzzle & 0x1801al))
           altDoSolve32(2, 0x4l, puzzle | 0x1801al, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x13012l))
        altDoSolve32(2, 0x4l, puzzle | 0x13012l, partsUsed | 0x800l);
      if(!(puzzle & 0x1201al))
        altDoSolve32(2, 0x4l, puzzle | 0x1201al, partsUsed | 0x800l);
      if(!(puzzle & 0x3602l))
        altDoSolve32(2, 0x4l, puzzle | 0x3602l, partsUsed | 0x800l);
    altpiece13at1at32:
    break;
   case 2:
      if (partsUsed & 0x2l) goto altpiece2at2at32;
      if(!(puzzle & 0x904804l))
        altDoSolve32(3, 0x8l, puzzle | 0x904804l, partsUsed | 0x2l);
      if(!(puzzle & 0x804824l))
        altDoSolve32(3, 0x8l, puzzle | 0x804824l, partsUsed | 0x2l);
      if(!(puzzle & 0x1001800cl))
        altDoSolve32(3, 0x8l, puzzle | 0x1001800cl, partsUsed | 0x2l);
      if(!(puzzle & 0x1800c004l))
        altDoSolve32(3, 0x8l, puzzle | 0x1800c004l, partsUsed | 0x2l);
    altpiece2at2at32:
      if (partsUsed & 0x4l) goto altpiece3at2at32;
      if (!(puzzle & 0x4004004l)) {
         if(!(puzzle & 0x4004904l))
           altDoSolve32(3, 0x8l, puzzle | 0x4004904l, partsUsed | 0x4l);
         if(!(puzzle & 0x24804004l))
           altDoSolve32(3, 0x8l, puzzle | 0x24804004l, partsUsed | 0x4l);
         if(!(puzzle & 0x401c004l))
           altDoSolve32(3, 0x8l, puzzle | 0x401c004l, partsUsed | 0x4l);
      }
      if(!(puzzle & 0x800801cl))
        altDoSolve32(3, 0x8l, puzzle | 0x800801cl, partsUsed | 0x4l);
    altpiece3at2at32:
      if (partsUsed & 0x8l) goto altpiece4at2at32;
      if(!(puzzle & 0x104904l))
        altDoSolve32(3, 0x8l, puzzle | 0x104904l, partsUsed | 0x8l);
      if(!(puzzle & 0x4804024l))
        altDoSolve32(3, 0x8l, puzzle | 0x4804024l, partsUsed | 0x8l);
      if(!(puzzle & 0x1001c004l))
        altDoSolve32(3, 0x8l, puzzle | 0x1001c004l, partsUsed | 0x8l);
      if(!(puzzle & 0x1800800cl))
        altDoSolve32(3, 0x8l, puzzle | 0x1800800cl, partsUsed | 0x8l);
    altpiece4at2at32:
      if (partsUsed & 0x40l) goto altpiece7at2at32;
      if(!(puzzle & 0x24904l))
        altDoSolve32(3, 0x8l, puzzle | 0x24904l, partsUsed | 0x40l);
    altpiece7at2at32:
      if (partsUsed & 0x80l) goto altpiece8at2at32;
      if(!(puzzle & 0x24004024l))
        altDoSolve32(3, 0x8l, puzzle | 0x24004024l, partsUsed | 0x80l);
      if(!(puzzle & 0x24020024l))
        altDoSolve32(3, 0x8l, puzzle | 0x24020024l, partsUsed | 0x80l);
      if(!(puzzle & 0xbcl))
        altDoSolve32(3, 0x8l, puzzle | 0xbcl, partsUsed | 0x80l);
      if(!(puzzle & 0xf4l))
        altDoSolve32(3, 0x8l, puzzle | 0xf4l, partsUsed | 0x80l);
      if(!(puzzle & 0xc00800cl))
        altDoSolve32(3, 0x8l, puzzle | 0xc00800cl, partsUsed | 0x80l);
      if(!(puzzle & 0xc00400cl))
        altDoSolve32(3, 0x8l, puzzle | 0xc00400cl, partsUsed | 0x80l);
      if(!(puzzle & 0x1c014l))
        altDoSolve32(3, 0x8l, puzzle | 0x1c014l, partsUsed | 0x80l);
      if(!(puzzle & 0x1401cl))
        altDoSolve32(3, 0x8l, puzzle | 0x1401cl, partsUsed | 0x80l);
    altpiece8at2at32:
      if (partsUsed & 0x100l) goto altpiece9at2at32;
      if(!(puzzle & 0x900cl))
        altDoSolve32(3, 0x8l, puzzle | 0x900cl, partsUsed | 0x100l);
      if(!(puzzle & 0x802cl))
        altDoSolve32(3, 0x8l, puzzle | 0x802cl, partsUsed | 0x100l);
      if(!(puzzle & 0x5804l))
        altDoSolve32(3, 0x8l, puzzle | 0x5804l, partsUsed | 0x100l);
      if(!(puzzle & 0x4c004l))
        altDoSolve32(3, 0x8l, puzzle | 0x4c004l, partsUsed | 0x100l);
      if(!(puzzle & 0x2400cl))
        altDoSolve32(3, 0x8l, puzzle | 0x2400cl, partsUsed | 0x100l);
      if(!(puzzle & 0x60024l))
        altDoSolve32(3, 0x8l, puzzle | 0x60024l, partsUsed | 0x100l);
      if(!(puzzle & 0x4064l))
        altDoSolve32(3, 0x8l, puzzle | 0x4064l, partsUsed | 0x100l);
      if(!(puzzle & 0x4004cl))
        altDoSolve32(3, 0x8l, puzzle | 0x4004cl, partsUsed | 0x100l);
    altpiece9at2at32:
      if (partsUsed & 0x200l) goto altpiece10at2at32;
      if (!(puzzle & 0x4804l)) {
         if(!(puzzle & 0x804904l))
           altDoSolve32(3, 0x8l, puzzle | 0x804904l, partsUsed | 0x200l);
         if(!(puzzle & 0x4004824l))
           altDoSolve32(3, 0x8l, puzzle | 0x4004824l, partsUsed | 0x200l);
         if(!(puzzle & 0x20024804l))
           altDoSolve32(3, 0x8l, puzzle | 0x20024804l, partsUsed | 0x200l);
         if(!(puzzle & 0x24004804l))
           altDoSolve32(3, 0x8l, puzzle | 0x24004804l, partsUsed | 0x200l);
         if(!(puzzle & 0x824804l))
           altDoSolve32(3, 0x8l, puzzle | 0x824804l, partsUsed | 0x200l);
      }
      if(!(puzzle & 0x4824004l))
        altDoSolve32(3, 0x8l, puzzle | 0x4824004l, partsUsed | 0x200l);
      if(!(puzzle & 0x801800cl))
        altDoSolve32(3, 0x8l, puzzle | 0x801800cl, partsUsed | 0x200l);
      if(!(puzzle & 0x801c004l))
        altDoSolve32(3, 0x8l, puzzle | 0x801c004l, partsUsed | 0x200l);
    altpiece10at2at32:
      if (partsUsed & 0x400l) goto altpiece11at2at32;
      if (!(puzzle & 0x24004l)) {
         if(!(puzzle & 0x4024024l))
           altDoSolve32(3, 0x8l, puzzle | 0x4024024l, partsUsed | 0x400l);
         if(!(puzzle & 0x24024004l))
           altDoSolve32(3, 0x8l, puzzle | 0x24024004l, partsUsed | 0x400l);
         if(!(puzzle & 0x20024024l))
           altDoSolve32(3, 0x8l, puzzle | 0x20024024l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x4cl)) {
         if(!(puzzle & 0xdcl))
           altDoSolve32(3, 0x8l, puzzle | 0xdcl, partsUsed | 0x400l);
         if(!(puzzle & 0x7cl))
           altDoSolve32(3, 0x8l, puzzle | 0x7cl, partsUsed | 0x400l);
         if(!(puzzle & 0xecl))
           altDoSolve32(3, 0x8l, puzzle | 0xecl, partsUsed | 0x400l);
      }
      if (!(puzzle & 0xc004l)) {
         if(!(puzzle & 0x800c00cl))
           altDoSolve32(3, 0x8l, puzzle | 0x800c00cl, partsUsed | 0x400l);
         if(!(puzzle & 0x400c00cl))
           altDoSolve32(3, 0x8l, puzzle | 0x400c00cl, partsUsed | 0x400l);
         if(!(puzzle & 0x1c00cl))
           altDoSolve32(3, 0x8l, puzzle | 0x1c00cl, partsUsed | 0x400l);
         if(!(puzzle & 0xc00c004l))
           altDoSolve32(3, 0x8l, puzzle | 0xc00c004l, partsUsed | 0x400l);
         if(!(puzzle & 0xc01cl))
           altDoSolve32(3, 0x8l, puzzle | 0xc01cl, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x4804804l))
        altDoSolve32(3, 0x8l, puzzle | 0x4804804l, partsUsed | 0x400l);
      if(!(puzzle & 0x24824l))
        altDoSolve32(3, 0x8l, puzzle | 0x24824l, partsUsed | 0x400l);
      if(!(puzzle & 0x1801cl))
        altDoSolve32(3, 0x8l, puzzle | 0x1801cl, partsUsed | 0x400l);
    altpiece11at2at32:
      if (partsUsed & 0x800l) goto altpiece13at2at32;
      if (!(puzzle & 0x400cl)) {
         if(!(puzzle & 0x406cl))
           altDoSolve32(3, 0x8l, puzzle | 0x406cl, partsUsed | 0x800l);
         if(!(puzzle & 0x2c00cl))
           altDoSolve32(3, 0x8l, puzzle | 0x2c00cl, partsUsed | 0x800l);
         if(!(puzzle & 0xc02cl))
           altDoSolve32(3, 0x8l, puzzle | 0xc02cl, partsUsed | 0x800l);
         if(!(puzzle & 0xc04cl))
           altDoSolve32(3, 0x8l, puzzle | 0xc04cl, partsUsed | 0x800l);
         if(!(puzzle & 0x4c00cl))
           altDoSolve32(3, 0x8l, puzzle | 0x4c00cl, partsUsed | 0x800l);
         if(!(puzzle & 0x2402cl))
           altDoSolve32(3, 0x8l, puzzle | 0x2402cl, partsUsed | 0x800l);
         if(!(puzzle & 0xd00cl))
           altDoSolve32(3, 0x8l, puzzle | 0xd00cl, partsUsed | 0x800l);
         if(!(puzzle & 0xc80cl))
           altDoSolve32(3, 0x8l, puzzle | 0xc80cl, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x40044l)) {
         if(!(puzzle & 0x60064l))
           altDoSolve32(3, 0x8l, puzzle | 0x60064l, partsUsed | 0x800l);
         if(!(puzzle & 0x4804cl))
           altDoSolve32(3, 0x8l, puzzle | 0x4804cl, partsUsed | 0x800l);
         if(!(puzzle & 0x4006cl))
           altDoSolve32(3, 0x8l, puzzle | 0x4006cl, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x20024l)) {
         if(!(puzzle & 0x2c024l))
           altDoSolve32(3, 0x8l, puzzle | 0x2c024l, partsUsed | 0x800l);
         if(!(puzzle & 0x24064l))
           altDoSolve32(3, 0x8l, puzzle | 0x24064l, partsUsed | 0x800l);
         if(!(puzzle & 0x2006cl))
           altDoSolve32(3, 0x8l, puzzle | 0x2006cl, partsUsed | 0x800l);
         if(!(puzzle & 0x64024l))
           altDoSolve32(3, 0x8l, puzzle | 0x64024l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x806cl))
        altDoSolve32(3, 0x8l, puzzle | 0x806cl, partsUsed | 0x800l);
      if(!(puzzle & 0x6c004l))
        altDoSolve32(3, 0x8l, puzzle | 0x6c004l, partsUsed | 0x800l);
      if(!(puzzle & 0xd804l))
        altDoSolve32(3, 0x8l, puzzle | 0xd804l, partsUsed | 0x800l);
    altpiece13at2at32:
    break;
   case 3:
      if (partsUsed & 0x2l) goto altpiece2at3at32;
      if(!(puzzle & 0x400c018l))
        altDoSolve32(4, 0x10l, puzzle | 0x400c018l, partsUsed | 0x2l);
      if(!(puzzle & 0x1209008l))
        altDoSolve32(4, 0x10l, puzzle | 0x1209008l, partsUsed | 0x2l);
      if(!(puzzle & 0x1009048l))
        altDoSolve32(4, 0x10l, puzzle | 0x1009048l, partsUsed | 0x2l);
    altpiece2at3at32:
      if (partsUsed & 0x4l) goto altpiece3at3at32;
      if (!(puzzle & 0x8008008l)) {
         if(!(puzzle & 0x8009208l))
           altDoSolve32(4, 0x10l, puzzle | 0x8009208l, partsUsed | 0x4l);
         if(!(puzzle & 0x49008008l))
           altDoSolve32(4, 0x10l, puzzle | 0x49008008l, partsUsed | 0x4l);
         if(!(puzzle & 0x1c008008l))
           altDoSolve32(4, 0x10l, puzzle | 0x1c008008l, partsUsed | 0x4l);
      }
    altpiece3at3at32:
      if (partsUsed & 0x8l) goto altpiece4at3at32;
      if(!(puzzle & 0x209208l))
        altDoSolve32(4, 0x10l, puzzle | 0x209208l, partsUsed | 0x8l);
      if(!(puzzle & 0x9008048l))
        altDoSolve32(4, 0x10l, puzzle | 0x9008048l, partsUsed | 0x8l);
      if(!(puzzle & 0xc008018l))
        altDoSolve32(4, 0x10l, puzzle | 0xc008018l, partsUsed | 0x8l);
    altpiece4at3at32:
      if (partsUsed & 0x40l) goto altpiece7at3at32;
      if(!(puzzle & 0x49208l))
        altDoSolve32(4, 0x10l, puzzle | 0x49208l, partsUsed | 0x40l);
    altpiece7at3at32:
      if (partsUsed & 0x80l) goto altpiece8at3at32;
      if(!(puzzle & 0x48008048l))
        altDoSolve32(4, 0x10l, puzzle | 0x48008048l, partsUsed | 0x80l);
      if(!(puzzle & 0x48040048l))
        altDoSolve32(4, 0x10l, puzzle | 0x48040048l, partsUsed | 0x80l);
      if(!(puzzle & 0x18010018l))
        altDoSolve32(4, 0x10l, puzzle | 0x18010018l, partsUsed | 0x80l);
      if(!(puzzle & 0x18008018l))
        altDoSolve32(4, 0x10l, puzzle | 0x18008018l, partsUsed | 0x80l);
    altpiece8at3at32:
      if (partsUsed & 0x100l) goto altpiece9at3at32;
      if(!(puzzle & 0xc808l))
        altDoSolve32(4, 0x10l, puzzle | 0xc808l, partsUsed | 0x100l);
      if(!(puzzle & 0xc048l))
        altDoSolve32(4, 0x10l, puzzle | 0xc048l, partsUsed | 0x100l);
      if(!(puzzle & 0x12018l))
        altDoSolve32(4, 0x10l, puzzle | 0x12018l, partsUsed | 0x100l);
      if(!(puzzle & 0x10058l))
        altDoSolve32(4, 0x10l, puzzle | 0x10058l, partsUsed | 0x100l);
      if(!(puzzle & 0xb008l))
        altDoSolve32(4, 0x10l, puzzle | 0xb008l, partsUsed | 0x100l);
      if(!(puzzle & 0x20068l))
        altDoSolve32(4, 0x10l, puzzle | 0x20068l, partsUsed | 0x100l);
      if(!(puzzle & 0x68008l))
        altDoSolve32(4, 0x10l, puzzle | 0x68008l, partsUsed | 0x100l);
      if(!(puzzle & 0x48018l))
        altDoSolve32(4, 0x10l, puzzle | 0x48018l, partsUsed | 0x100l);
      if(!(puzzle & 0x98008l))
        altDoSolve32(4, 0x10l, puzzle | 0x98008l, partsUsed | 0x100l);
      if(!(puzzle & 0xc0048l))
        altDoSolve32(4, 0x10l, puzzle | 0xc0048l, partsUsed | 0x100l);
      if(!(puzzle & 0x80c8l))
        altDoSolve32(4, 0x10l, puzzle | 0x80c8l, partsUsed | 0x100l);
      if(!(puzzle & 0x80098l))
        altDoSolve32(4, 0x10l, puzzle | 0x80098l, partsUsed | 0x100l);
    altpiece9at3at32:
      if (partsUsed & 0x200l) goto altpiece10at3at32;
      if (!(puzzle & 0x9008l)) {
         if(!(puzzle & 0x1009208l))
           altDoSolve32(4, 0x10l, puzzle | 0x1009208l, partsUsed | 0x200l);
         if(!(puzzle & 0x8009048l))
           altDoSolve32(4, 0x10l, puzzle | 0x8009048l, partsUsed | 0x200l);
         if(!(puzzle & 0x40049008l))
           altDoSolve32(4, 0x10l, puzzle | 0x40049008l, partsUsed | 0x200l);
         if(!(puzzle & 0x48009008l))
           altDoSolve32(4, 0x10l, puzzle | 0x48009008l, partsUsed | 0x200l);
         if(!(puzzle & 0x1049008l))
           altDoSolve32(4, 0x10l, puzzle | 0x1049008l, partsUsed | 0x200l);
      }
      if (!(puzzle & 0x8008008l)) {
         if(!(puzzle & 0x9048008l))
           altDoSolve32(4, 0x10l, puzzle | 0x9048008l, partsUsed | 0x200l);
         if(!(puzzle & 0x800c018l))
           altDoSolve32(4, 0x10l, puzzle | 0x800c018l, partsUsed | 0x200l);
         if(!(puzzle & 0x1800c008l))
           altDoSolve32(4, 0x10l, puzzle | 0x1800c008l, partsUsed | 0x200l);
         if(!(puzzle & 0xc018008l))
           altDoSolve32(4, 0x10l, puzzle | 0xc018008l, partsUsed | 0x200l);
      }
      if(!(puzzle & 0x401c008l))
        altDoSolve32(4, 0x10l, puzzle | 0x401c008l, partsUsed | 0x200l);
      if(!(puzzle & 0x1001c008l))
        altDoSolve32(4, 0x10l, puzzle | 0x1001c008l, partsUsed | 0x200l);
    altpiece10at3at32:
      if (partsUsed & 0x400l) goto altpiece11at3at32;
      if (!(puzzle & 0x48008l)) {
         if(!(puzzle & 0x8048048l))
           altDoSolve32(4, 0x10l, puzzle | 0x8048048l, partsUsed | 0x400l);
         if(!(puzzle & 0x48048008l))
           altDoSolve32(4, 0x10l, puzzle | 0x48048008l, partsUsed | 0x400l);
         if(!(puzzle & 0x40048048l))
           altDoSolve32(4, 0x10l, puzzle | 0x40048048l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x18008l)) {
         if(!(puzzle & 0x10018018l))
           altDoSolve32(4, 0x10l, puzzle | 0x10018018l, partsUsed | 0x400l);
         if(!(puzzle & 0x8018018l))
           altDoSolve32(4, 0x10l, puzzle | 0x8018018l, partsUsed | 0x400l);
         if(!(puzzle & 0x18018008l))
           altDoSolve32(4, 0x10l, puzzle | 0x18018008l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x9009008l))
        altDoSolve32(4, 0x10l, puzzle | 0x9009008l, partsUsed | 0x400l);
      if(!(puzzle & 0x49048l))
        altDoSolve32(4, 0x10l, puzzle | 0x49048l, partsUsed | 0x400l);
      if(!(puzzle & 0xf8l))
        altDoSolve32(4, 0x10l, puzzle | 0xf8l, partsUsed | 0x400l);
      if(!(puzzle & 0x1c018l))
        altDoSolve32(4, 0x10l, puzzle | 0x1c018l, partsUsed | 0x400l);
      if(!(puzzle & 0xc00c008l))
        altDoSolve32(4, 0x10l, puzzle | 0xc00c008l, partsUsed | 0x400l);
    altpiece11at3at32:
      if (partsUsed & 0x800l) goto altpiece13at3at32;
      if (!(puzzle & 0x8018l)) {
         if(!(puzzle & 0x80d8l))
           altDoSolve32(4, 0x10l, puzzle | 0x80d8l, partsUsed | 0x800l);
         if(!(puzzle & 0x58018l))
           altDoSolve32(4, 0x10l, puzzle | 0x58018l, partsUsed | 0x800l);
         if(!(puzzle & 0x18058l))
           altDoSolve32(4, 0x10l, puzzle | 0x18058l, partsUsed | 0x800l);
         if(!(puzzle & 0x18098l))
           altDoSolve32(4, 0x10l, puzzle | 0x18098l, partsUsed | 0x800l);
         if(!(puzzle & 0x98018l))
           altDoSolve32(4, 0x10l, puzzle | 0x98018l, partsUsed | 0x800l);
         if(!(puzzle & 0x48058l))
           altDoSolve32(4, 0x10l, puzzle | 0x48058l, partsUsed | 0x800l);
         if(!(puzzle & 0x1a018l))
           altDoSolve32(4, 0x10l, puzzle | 0x1a018l, partsUsed | 0x800l);
         if(!(puzzle & 0x19018l))
           altDoSolve32(4, 0x10l, puzzle | 0x19018l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x80088l)) {
         if(!(puzzle & 0xc00c8l))
           altDoSolve32(4, 0x10l, puzzle | 0xc00c8l, partsUsed | 0x800l);
         if(!(puzzle & 0x90098l))
           altDoSolve32(4, 0x10l, puzzle | 0x90098l, partsUsed | 0x800l);
         if(!(puzzle & 0x800d8l))
           altDoSolve32(4, 0x10l, puzzle | 0x800d8l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x40048l)) {
         if(!(puzzle & 0x58048l))
           altDoSolve32(4, 0x10l, puzzle | 0x58048l, partsUsed | 0x800l);
         if(!(puzzle & 0x480c8l))
           altDoSolve32(4, 0x10l, puzzle | 0x480c8l, partsUsed | 0x800l);
         if(!(puzzle & 0x400d8l))
           altDoSolve32(4, 0x10l, puzzle | 0x400d8l, partsUsed | 0x800l);
         if(!(puzzle & 0xc8048l))
           altDoSolve32(4, 0x10l, puzzle | 0xc8048l, partsUsed | 0x800l);
         if(!(puzzle & 0x68048l))
           altDoSolve32(4, 0x10l, puzzle | 0x68048l, partsUsed | 0x800l);
         if(!(puzzle & 0x60068l))
           altDoSolve32(4, 0x10l, puzzle | 0x60068l, partsUsed | 0x800l);
         if(!(puzzle & 0x4c048l))
           altDoSolve32(4, 0x10l, puzzle | 0x4c048l, partsUsed | 0x800l);
         if(!(puzzle & 0x48068l))
           altDoSolve32(4, 0x10l, puzzle | 0x48068l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x100d8l))
        altDoSolve32(4, 0x10l, puzzle | 0x100d8l, partsUsed | 0x800l);
      if(!(puzzle & 0xd8008l))
        altDoSolve32(4, 0x10l, puzzle | 0xd8008l, partsUsed | 0x800l);
      if(!(puzzle & 0x6c008l))
        altDoSolve32(4, 0x10l, puzzle | 0x6c008l, partsUsed | 0x800l);
      if(!(puzzle & 0x1b008l))
        altDoSolve32(4, 0x10l, puzzle | 0x1b008l, partsUsed | 0x800l);
      if(!(puzzle & 0xd808l))
        altDoSolve32(4, 0x10l, puzzle | 0xd808l, partsUsed | 0x800l);
    altpiece13at3at32:
    break;
   case 4:
      if (partsUsed & 0x2l) goto altpiece2at4at32;
      if(!(puzzle & 0xc018010l))
        altDoSolve32(5, 0x20l, puzzle | 0xc018010l, partsUsed | 0x2l);
      if(!(puzzle & 0x2412010l))
        altDoSolve32(5, 0x20l, puzzle | 0x2412010l, partsUsed | 0x2l);
      if(!(puzzle & 0x2012090l))
        altDoSolve32(5, 0x20l, puzzle | 0x2012090l, partsUsed | 0x2l);
    altpiece2at4at32:
      if (partsUsed & 0x4l) goto altpiece3at4at32;
      if (!(puzzle & 0x10010010l)) {
         if(!(puzzle & 0x10012410l))
           altDoSolve32(5, 0x20l, puzzle | 0x10012410l, partsUsed | 0x4l);
         if(!(puzzle & 0x92010010l))
           altDoSolve32(5, 0x20l, puzzle | 0x92010010l, partsUsed | 0x4l);
         if(!(puzzle & 0x1001c010l))
           altDoSolve32(5, 0x20l, puzzle | 0x1001c010l, partsUsed | 0x4l);
      }
    altpiece3at4at32:
      if (partsUsed & 0x8l) goto altpiece4at4at32;
      if(!(puzzle & 0x401c010l))
        altDoSolve32(5, 0x20l, puzzle | 0x401c010l, partsUsed | 0x8l);
      if(!(puzzle & 0x412410l))
        altDoSolve32(5, 0x20l, puzzle | 0x412410l, partsUsed | 0x8l);
      if(!(puzzle & 0x12010090l))
        altDoSolve32(5, 0x20l, puzzle | 0x12010090l, partsUsed | 0x8l);
    altpiece4at4at32:
      if (partsUsed & 0x40l) goto altpiece7at4at32;
      if(!(puzzle & 0x92410l))
        altDoSolve32(5, 0x20l, puzzle | 0x92410l, partsUsed | 0x40l);
    altpiece7at4at32:
      if (partsUsed & 0x80l) goto altpiece8at4at32;
      if(!(puzzle & 0x90010090l))
        altDoSolve32(5, 0x20l, puzzle | 0x90010090l, partsUsed | 0x80l);
      if(!(puzzle & 0x90080090l))
        altDoSolve32(5, 0x20l, puzzle | 0x90080090l, partsUsed | 0x80l);
    altpiece8at4at32:
      if (partsUsed & 0x100l) goto altpiece9at4at32;
      if(!(puzzle & 0x19010l))
        altDoSolve32(5, 0x20l, puzzle | 0x19010l, partsUsed | 0x100l);
      if(!(puzzle & 0x18090l))
        altDoSolve32(5, 0x20l, puzzle | 0x18090l, partsUsed | 0x100l);
      if(!(puzzle & 0x400d0l))
        altDoSolve32(5, 0x20l, puzzle | 0x400d0l, partsUsed | 0x100l);
      if(!(puzzle & 0xd0010l))
        altDoSolve32(5, 0x20l, puzzle | 0xd0010l, partsUsed | 0x100l);
    altpiece9at4at32:
      if (partsUsed & 0x200l) goto altpiece10at4at32;
      if (!(puzzle & 0x12010l)) {
         if(!(puzzle & 0x2012410l))
           altDoSolve32(5, 0x20l, puzzle | 0x2012410l, partsUsed | 0x200l);
         if(!(puzzle & 0x10012090l))
           altDoSolve32(5, 0x20l, puzzle | 0x10012090l, partsUsed | 0x200l);
         if(!(puzzle & 0x80092010l))
           altDoSolve32(5, 0x20l, puzzle | 0x80092010l, partsUsed | 0x200l);
         if(!(puzzle & 0x90012010l))
           altDoSolve32(5, 0x20l, puzzle | 0x90012010l, partsUsed | 0x200l);
         if(!(puzzle & 0x2092010l))
           altDoSolve32(5, 0x20l, puzzle | 0x2092010l, partsUsed | 0x200l);
      }
      if(!(puzzle & 0x12090010l))
        altDoSolve32(5, 0x20l, puzzle | 0x12090010l, partsUsed | 0x200l);
      if(!(puzzle & 0x801c010l))
        altDoSolve32(5, 0x20l, puzzle | 0x801c010l, partsUsed | 0x200l);
    altpiece10at4at32:
      if (partsUsed & 0x400l) goto altpiece11at4at32;
      if (!(puzzle & 0x10010010l)) {
         if(!(puzzle & 0x18018010l))
           altDoSolve32(5, 0x20l, puzzle | 0x18018010l, partsUsed | 0x400l);
         if(!(puzzle & 0x10090090l))
           altDoSolve32(5, 0x20l, puzzle | 0x10090090l, partsUsed | 0x400l);
         if(!(puzzle & 0x90090010l))
           altDoSolve32(5, 0x20l, puzzle | 0x90090010l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x12012010l))
        altDoSolve32(5, 0x20l, puzzle | 0x12012010l, partsUsed | 0x400l);
      if(!(puzzle & 0x92090l))
        altDoSolve32(5, 0x20l, puzzle | 0x92090l, partsUsed | 0x400l);
      if(!(puzzle & 0x80090090l))
        altDoSolve32(5, 0x20l, puzzle | 0x80090090l, partsUsed | 0x400l);
    altpiece11at4at32:
      if (partsUsed & 0x800l) goto altpiece13at4at32;
      if (!(puzzle & 0xc0010l)) {
         if(!(puzzle & 0xd8010l))
           altDoSolve32(5, 0x20l, puzzle | 0xd8010l, partsUsed | 0x800l);
         if(!(puzzle & 0xd0090l))
           altDoSolve32(5, 0x20l, puzzle | 0xd0090l, partsUsed | 0x800l);
         if(!(puzzle & 0xc00d0l))
           altDoSolve32(5, 0x20l, puzzle | 0xc00d0l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x98090l))
        altDoSolve32(5, 0x20l, puzzle | 0x98090l, partsUsed | 0x800l);
      if(!(puzzle & 0x900d0l))
        altDoSolve32(5, 0x20l, puzzle | 0x900d0l, partsUsed | 0x800l);
      if(!(puzzle & 0x1b010l))
        altDoSolve32(5, 0x20l, puzzle | 0x1b010l, partsUsed | 0x800l);
    altpiece13at4at32:
    break;
   case 5:
      if (partsUsed & 0x2l) goto altpiece2at5at32;
      if(!(puzzle & 0x4824020l))
        altDoSolve32(6, 0x40l, puzzle | 0x4824020l, partsUsed | 0x2l);
      if(!(puzzle & 0x800c0060l))
        altDoSolve32(6, 0x40l, puzzle | 0x800c0060l, partsUsed | 0x2l);
      if(!(puzzle & 0xc0060020l))
        altDoSolve32(6, 0x40l, puzzle | 0xc0060020l, partsUsed | 0x2l);
    altpiece2at5at32:
      if (partsUsed & 0x4l) goto altpiece3at5at32;
      if(!(puzzle & 0x400400e0l))
        altDoSolve32(6, 0x40l, puzzle | 0x400400e0l, partsUsed | 0x4l);
      if(!(puzzle & 0x20024820l))
        altDoSolve32(6, 0x40l, puzzle | 0x20024820l, partsUsed | 0x4l);
      if(!(puzzle & 0x200e0020l))
        altDoSolve32(6, 0x40l, puzzle | 0x200e0020l, partsUsed | 0x4l);
    altpiece3at5at32:
      if (partsUsed & 0x8l) goto altpiece4at5at32;
      if(!(puzzle & 0x824820l))
        altDoSolve32(6, 0x40l, puzzle | 0x824820l, partsUsed | 0x8l);
      if(!(puzzle & 0x800e0020l))
        altDoSolve32(6, 0x40l, puzzle | 0x800e0020l, partsUsed | 0x8l);
      if(!(puzzle & 0xc0040060l))
        altDoSolve32(6, 0x40l, puzzle | 0xc0040060l, partsUsed | 0x8l);
    altpiece4at5at32:
      if (partsUsed & 0x20l) goto altpiece6at5at32;
      if(!(puzzle & 0x24920l))
        altDoSolve32(6, 0x40l, puzzle | 0x24920l, partsUsed | 0x20l);
    altpiece6at5at32:
      if (partsUsed & 0x80l) goto altpiece8at5at32;
      if(!(puzzle & 0x60040060l))
        altDoSolve32(6, 0x40l, puzzle | 0x60040060l, partsUsed | 0x80l);
      if(!(puzzle & 0x60020060l))
        altDoSolve32(6, 0x40l, puzzle | 0x60020060l, partsUsed | 0x80l);
      if(!(puzzle & 0xe00a0l))
        altDoSolve32(6, 0x40l, puzzle | 0xe00a0l, partsUsed | 0x80l);
      if(!(puzzle & 0xa00e0l))
        altDoSolve32(6, 0x40l, puzzle | 0xa00e0l, partsUsed | 0x80l);
    altpiece8at5at32:
      if (partsUsed & 0x100l) goto altpiece9at5at32;
      if(!(puzzle & 0x48060l))
        altDoSolve32(6, 0x40l, puzzle | 0x48060l, partsUsed | 0x100l);
      if(!(puzzle & 0x2c020l))
        altDoSolve32(6, 0x40l, puzzle | 0x2c020l, partsUsed | 0x100l);
    altpiece9at5at32:
      if (partsUsed & 0x200l) goto altpiece10at5at32;
      if(!(puzzle & 0x4024820l))
        altDoSolve32(6, 0x40l, puzzle | 0x4024820l, partsUsed | 0x200l);
      if(!(puzzle & 0x400c0060l))
        altDoSolve32(6, 0x40l, puzzle | 0x400c0060l, partsUsed | 0x200l);
      if(!(puzzle & 0x400e0020l))
        altDoSolve32(6, 0x40l, puzzle | 0x400e0020l, partsUsed | 0x200l);
    altpiece10at5at32:
      if (partsUsed & 0x400l) goto altpiece11at5at32;
      if (!(puzzle & 0x20020020l)) {
         if(!(puzzle & 0x24024020l))
           altDoSolve32(6, 0x40l, puzzle | 0x24024020l, partsUsed | 0x400l);
         if(!(puzzle & 0x20060060l))
           altDoSolve32(6, 0x40l, puzzle | 0x20060060l, partsUsed | 0x400l);
         if(!(puzzle & 0x60060020l))
           altDoSolve32(6, 0x40l, puzzle | 0x60060020l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x40060l)) {
         if(!(puzzle & 0x40060060l))
           altDoSolve32(6, 0x40l, puzzle | 0x40060060l, partsUsed | 0x400l);
         if(!(puzzle & 0xe0060l))
           altDoSolve32(6, 0x40l, puzzle | 0xe0060l, partsUsed | 0x400l);
         if(!(puzzle & 0x600e0l))
           altDoSolve32(6, 0x40l, puzzle | 0x600e0l, partsUsed | 0x400l);
         if(!(puzzle & 0xc00e0l))
           altDoSolve32(6, 0x40l, puzzle | 0xc00e0l, partsUsed | 0x400l);
      }
    altpiece11at5at32:
      if (partsUsed & 0x800l) goto altpiece13at5at32;
      if (!(puzzle & 0x60020l)) {
         if(!(puzzle & 0x6c020l))
           altDoSolve32(6, 0x40l, puzzle | 0x6c020l, partsUsed | 0x800l);
         if(!(puzzle & 0x68060l))
           altDoSolve32(6, 0x40l, puzzle | 0x68060l, partsUsed | 0x800l);
         if(!(puzzle & 0x64060l))
           altDoSolve32(6, 0x40l, puzzle | 0x64060l, partsUsed | 0x800l);
      }
    altpiece13at5at32:
    break;
   case 6:
      if (partsUsed & 0x2l) goto altpiece2at6at32;
      if(!(puzzle & 0x9048040l))
        altDoSolve32(7, 0x80l, puzzle | 0x9048040l, partsUsed | 0x2l);
      if(!(puzzle & 0x200600c0l))
        altDoSolve32(7, 0x80l, puzzle | 0x200600c0l, partsUsed | 0x2l);
    altpiece2at6at32:
      if (partsUsed & 0x4l) goto altpiece3at6at32;
      if(!(puzzle & 0x40049040l))
        altDoSolve32(7, 0x80l, puzzle | 0x40049040l, partsUsed | 0x4l);
      if(!(puzzle & 0xe0040040l))
        altDoSolve32(7, 0x80l, puzzle | 0xe0040040l, partsUsed | 0x4l);
    altpiece3at6at32:
      if (partsUsed & 0x8l) goto altpiece4at6at32;
      if(!(puzzle & 0x1049040l))
        altDoSolve32(7, 0x80l, puzzle | 0x1049040l, partsUsed | 0x8l);
      if(!(puzzle & 0x600400c0l))
        altDoSolve32(7, 0x80l, puzzle | 0x600400c0l, partsUsed | 0x8l);
    altpiece4at6at32:
      if (partsUsed & 0x20l) goto altpiece6at6at32;
      if(!(puzzle & 0x49240l))
        altDoSolve32(7, 0x80l, puzzle | 0x49240l, partsUsed | 0x20l);
    altpiece6at6at32:
      if (partsUsed & 0x80l) goto altpiece8at6at32;
      if(!(puzzle & 0xc00800c0l))
        altDoSolve32(7, 0x80l, puzzle | 0xc00800c0l, partsUsed | 0x80l);
      if(!(puzzle & 0xc00400c0l))
        altDoSolve32(7, 0x80l, puzzle | 0xc00400c0l, partsUsed | 0x80l);
    altpiece8at6at32:
      if (partsUsed & 0x100l) goto altpiece9at6at32;
      if(!(puzzle & 0x58040l))
        altDoSolve32(7, 0x80l, puzzle | 0x58040l, partsUsed | 0x100l);
      if(!(puzzle & 0x64040l))
        altDoSolve32(7, 0x80l, puzzle | 0x64040l, partsUsed | 0x100l);
      if(!(puzzle & 0x900c0l))
        altDoSolve32(7, 0x80l, puzzle | 0x900c0l, partsUsed | 0x100l);
    altpiece9at6at32:
      if (partsUsed & 0x200l) goto altpiece10at6at32;
      if (!(puzzle & 0x40040040l)) {
         if(!(puzzle & 0x400600c0l))
           altDoSolve32(7, 0x80l, puzzle | 0x400600c0l, partsUsed | 0x200l);
         if(!(puzzle & 0xc0060040l))
           altDoSolve32(7, 0x80l, puzzle | 0xc0060040l, partsUsed | 0x200l);
         if(!(puzzle & 0x600c0040l))
           altDoSolve32(7, 0x80l, puzzle | 0x600c0040l, partsUsed | 0x200l);
      }
      if(!(puzzle & 0x8049040l))
        altDoSolve32(7, 0x80l, puzzle | 0x8049040l, partsUsed | 0x200l);
      if(!(puzzle & 0x200e0040l))
        altDoSolve32(7, 0x80l, puzzle | 0x200e0040l, partsUsed | 0x200l);
      if(!(puzzle & 0x800e0040l))
        altDoSolve32(7, 0x80l, puzzle | 0x800e0040l, partsUsed | 0x200l);
    altpiece10at6at32:
      if (partsUsed & 0x400l) goto altpiece11at6at32;
      if (!(puzzle & 0x40040040l)) {
         if(!(puzzle & 0x48048040l))
           altDoSolve32(7, 0x80l, puzzle | 0x48048040l, partsUsed | 0x400l);
         if(!(puzzle & 0x60060040l))
           altDoSolve32(7, 0x80l, puzzle | 0x60060040l, partsUsed | 0x400l);
         if(!(puzzle & 0x400c00c0l))
           altDoSolve32(7, 0x80l, puzzle | 0x400c00c0l, partsUsed | 0x400l);
         if(!(puzzle & 0xc00c0040l))
           altDoSolve32(7, 0x80l, puzzle | 0xc00c0040l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0xe00c0l))
        altDoSolve32(7, 0x80l, puzzle | 0xe00c0l, partsUsed | 0x400l);
      if(!(puzzle & 0x800c00c0l))
        altDoSolve32(7, 0x80l, puzzle | 0x800c00c0l, partsUsed | 0x400l);
    altpiece11at6at32:
      if (partsUsed & 0x800l) goto altpiece13at6at32;
      if (!(puzzle & 0xc0040l)) {
         if(!(puzzle & 0xd8040l))
           altDoSolve32(7, 0x80l, puzzle | 0xd8040l, partsUsed | 0x800l);
         if(!(puzzle & 0xd00c0l))
           altDoSolve32(7, 0x80l, puzzle | 0xd00c0l, partsUsed | 0x800l);
         if(!(puzzle & 0xc80c0l))
           altDoSolve32(7, 0x80l, puzzle | 0xc80c0l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x6c040l))
        altDoSolve32(7, 0x80l, puzzle | 0x6c040l, partsUsed | 0x800l);
    altpiece13at6at32:
    break;
   case 7:
      if (partsUsed & 0x2l) goto altpiece2at7at32;
      if(!(puzzle & 0x12090080l))
        altDoSolve32(8, 0x100l, puzzle | 0x12090080l, partsUsed | 0x2l);
      if(!(puzzle & 0x600c0080l))
        altDoSolve32(8, 0x100l, puzzle | 0x600c0080l, partsUsed | 0x2l);
    altpiece2at7at32:
      if (partsUsed & 0x4l) goto altpiece3at7at32;
      if(!(puzzle & 0x80092080l))
        altDoSolve32(8, 0x100l, puzzle | 0x80092080l, partsUsed | 0x4l);
      if(!(puzzle & 0x800e0080l))
        altDoSolve32(8, 0x100l, puzzle | 0x800e0080l, partsUsed | 0x4l);
    altpiece3at7at32:
      if (partsUsed & 0x8l) goto altpiece4at7at32;
      if(!(puzzle & 0x2092080l))
        altDoSolve32(8, 0x100l, puzzle | 0x2092080l, partsUsed | 0x8l);
      if(!(puzzle & 0x200e0080l))
        altDoSolve32(8, 0x100l, puzzle | 0x200e0080l, partsUsed | 0x8l);
    altpiece4at7at32:
      if (partsUsed & 0x20l) goto altpiece6at7at32;
      if(!(puzzle & 0x92480l))
        altDoSolve32(8, 0x100l, puzzle | 0x92480l, partsUsed | 0x20l);
    altpiece6at7at32:
      if (partsUsed & 0x100l) goto altpiece9at7at32;
      if(!(puzzle & 0xc8080l))
        altDoSolve32(8, 0x100l, puzzle | 0xc8080l, partsUsed | 0x100l);
    altpiece9at7at32:
      if (partsUsed & 0x200l) goto altpiece10at7at32;
      if(!(puzzle & 0x10092080l))
        altDoSolve32(8, 0x100l, puzzle | 0x10092080l, partsUsed | 0x200l);
      if(!(puzzle & 0x400e0080l))
        altDoSolve32(8, 0x100l, puzzle | 0x400e0080l, partsUsed | 0x200l);
    altpiece10at7at32:
      if (partsUsed & 0x400l) goto altpiece11at7at32;
      if(!(puzzle & 0x90090080l))
        altDoSolve32(8, 0x100l, puzzle | 0x90090080l, partsUsed | 0x400l);
      if(!(puzzle & 0xc00c0080l))
        altDoSolve32(8, 0x100l, puzzle | 0xc00c0080l, partsUsed | 0x400l);
    altpiece11at7at32:
      if (partsUsed & 0x800l) goto altpiece13at7at32;
      if(!(puzzle & 0xd8080l))
        altDoSolve32(8, 0x100l, puzzle | 0xd8080l, partsUsed | 0x800l);
    altpiece13at7at32:
    break;
   case 8:
      if (partsUsed & 0x2l) goto altpiece2at8at32;
      if(!(puzzle & 0x13300l))
        altDoSolve32(9, 0x200l, puzzle | 0x13300l, partsUsed | 0x2l);
      if(!(puzzle & 0x19900l))
        altDoSolve32(9, 0x200l, puzzle | 0x19900l, partsUsed | 0x2l);
    altpiece2at8at32:
      if (partsUsed & 0x4l) goto altpiece3at8at32;
      if(!(puzzle & 0x9700l))
        altDoSolve32(9, 0x200l, puzzle | 0x9700l, partsUsed | 0x4l);
      if(!(puzzle & 0x7900l))
        altDoSolve32(9, 0x200l, puzzle | 0x7900l, partsUsed | 0x4l);
    altpiece3at8at32:
      if (partsUsed & 0x8l) goto altpiece4at8at32;
      if(!(puzzle & 0x19300l))
        altDoSolve32(9, 0x200l, puzzle | 0x19300l, partsUsed | 0x8l);
      if(!(puzzle & 0x13900l))
        altDoSolve32(9, 0x200l, puzzle | 0x13900l, partsUsed | 0x8l);
    altpiece4at8at32:
      if (partsUsed & 0x10l) goto altpiece5at8at32;
      if (!(puzzle & 0x24900l)) {
         if(!(puzzle & 0x4124900l))
           altDoSolve32(9, 0x200l, puzzle | 0x4124900l, partsUsed | 0x10l);
         if(!(puzzle & 0x65900l))
           altDoSolve32(9, 0x200l, puzzle | 0x65900l, partsUsed | 0x10l);
         if(!(puzzle & 0x20824900l))
           altDoSolve32(9, 0x200l, puzzle | 0x20824900l, partsUsed | 0x10l);
         if(!(puzzle & 0x2cb00l))
           altDoSolve32(9, 0x200l, puzzle | 0x2cb00l, partsUsed | 0x10l);
      }
      if(!(puzzle & 0x4d300l))
        altDoSolve32(9, 0x200l, puzzle | 0x4d300l, partsUsed | 0x10l);
      if(!(puzzle & 0x24904100l))
        altDoSolve32(9, 0x200l, puzzle | 0x24904100l, partsUsed | 0x10l);
    altpiece5at8at32:
      if (partsUsed & 0x20l) goto altpiece6at8at32;
      if (!(puzzle & 0x24900l)) {
         if(!(puzzle & 0x20024900l))
           altDoSolve32(9, 0x200l, puzzle | 0x20024900l, partsUsed | 0x20l);
         if(!(puzzle & 0x24b00l))
           altDoSolve32(9, 0x200l, puzzle | 0x24b00l, partsUsed | 0x20l);
         if(!(puzzle & 0x124900l))
           altDoSolve32(9, 0x200l, puzzle | 0x124900l, partsUsed | 0x20l);
         if(!(puzzle & 0x64900l))
           altDoSolve32(9, 0x200l, puzzle | 0x64900l, partsUsed | 0x20l);
      }
      if(!(puzzle & 0x24900100l))
        altDoSolve32(9, 0x200l, puzzle | 0x24900100l, partsUsed | 0x20l);
      if(!(puzzle & 0x49300l))
        altDoSolve32(9, 0x200l, puzzle | 0x49300l, partsUsed | 0x20l);
    altpiece6at8at32:
      if (partsUsed & 0x40l) goto altpiece7at8at32;
      if (!(puzzle & 0x24900l)) {
         if(!(puzzle & 0x4024900l))
           altDoSolve32(9, 0x200l, puzzle | 0x4024900l, partsUsed | 0x40l);
         if(!(puzzle & 0x25900l))
           altDoSolve32(9, 0x200l, puzzle | 0x25900l, partsUsed | 0x40l);
         if(!(puzzle & 0x824900l))
           altDoSolve32(9, 0x200l, puzzle | 0x824900l, partsUsed | 0x40l);
         if(!(puzzle & 0x2c900l))
           altDoSolve32(9, 0x200l, puzzle | 0x2c900l, partsUsed | 0x40l);
      }
    altpiece7at8at32:
      if (partsUsed & 0x80l) goto altpiece8at8at32;
      if(!(puzzle & 0x4904100l))
        altDoSolve32(9, 0x200l, puzzle | 0x4904100l, partsUsed | 0x80l);
      if(!(puzzle & 0x4104900l))
        altDoSolve32(9, 0x200l, puzzle | 0x4104900l, partsUsed | 0x80l);
      if(!(puzzle & 0x2f00l))
        altDoSolve32(9, 0x200l, puzzle | 0x2f00l, partsUsed | 0x80l);
      if(!(puzzle & 0xcb00l))
        altDoSolve32(9, 0x200l, puzzle | 0xcb00l, partsUsed | 0x80l);
      if(!(puzzle & 0xd300l))
        altDoSolve32(9, 0x200l, puzzle | 0xd300l, partsUsed | 0x80l);
      if(!(puzzle & 0x3d00l))
        altDoSolve32(9, 0x200l, puzzle | 0x3d00l, partsUsed | 0x80l);
      if(!(puzzle & 0x700500l))
        altDoSolve32(9, 0x200l, puzzle | 0x700500l, partsUsed | 0x80l);
      if(!(puzzle & 0x500700l))
        altDoSolve32(9, 0x200l, puzzle | 0x500700l, partsUsed | 0x80l);
    altpiece8at8at32:
      if (partsUsed & 0x100l) goto altpiece9at8at32;
      if(!(puzzle & 0x1001300l))
        altDoSolve32(9, 0x200l, puzzle | 0x1001300l, partsUsed | 0x100l);
      if(!(puzzle & 0x1300100l))
        altDoSolve32(9, 0x200l, puzzle | 0x1300100l, partsUsed | 0x100l);
      if(!(puzzle & 0x200b00l))
        altDoSolve32(9, 0x200l, puzzle | 0x200b00l, partsUsed | 0x100l);
      if(!(puzzle & 0x900300l))
        altDoSolve32(9, 0x200l, puzzle | 0x900300l, partsUsed | 0x100l);
      if(!(puzzle & 0x1800900l))
        altDoSolve32(9, 0x200l, puzzle | 0x1800900l, partsUsed | 0x100l);
      if(!(puzzle & 0x101900l))
        altDoSolve32(9, 0x200l, puzzle | 0x101900l, partsUsed | 0x100l);
    altpiece9at8at32:
      if (partsUsed & 0x200l) goto altpiece10at8at32;
      if(!(puzzle & 0xb300l))
        altDoSolve32(9, 0x200l, puzzle | 0xb300l, partsUsed | 0x200l);
      if(!(puzzle & 0xb900l))
        altDoSolve32(9, 0x200l, puzzle | 0xb900l, partsUsed | 0x200l);
    altpiece10at8at32:
      if (partsUsed & 0x400l) goto altpiece11at8at32;
      if (!(puzzle & 0x800900l)) {
         if(!(puzzle & 0x4900900l))
           altDoSolve32(9, 0x200l, puzzle | 0x4900900l, partsUsed | 0x400l);
         if(!(puzzle & 0x4804900l))
           altDoSolve32(9, 0x200l, puzzle | 0x4804900l, partsUsed | 0x400l);
         if(!(puzzle & 0x904900l))
           altDoSolve32(9, 0x200l, puzzle | 0x904900l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x1300l)) {
         if(!(puzzle & 0x3700l))
           altDoSolve32(9, 0x200l, puzzle | 0x3700l, partsUsed | 0x400l);
         if(!(puzzle & 0x1f00l))
           altDoSolve32(9, 0x200l, puzzle | 0x1f00l, partsUsed | 0x400l);
         if(!(puzzle & 0x5b00l))
           altDoSolve32(9, 0x200l, puzzle | 0x5b00l, partsUsed | 0x400l);
         if(!(puzzle & 0x9b00l))
           altDoSolve32(9, 0x200l, puzzle | 0x9b00l, partsUsed | 0x400l);
         if(!(puzzle & 0x3b00l))
           altDoSolve32(9, 0x200l, puzzle | 0x3b00l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x200300l)) {
         if(!(puzzle & 0x700300l))
           altDoSolve32(9, 0x200l, puzzle | 0x700300l, partsUsed | 0x400l);
         if(!(puzzle & 0x300700l))
           altDoSolve32(9, 0x200l, puzzle | 0x300700l, partsUsed | 0x400l);
         if(!(puzzle & 0x600700l))
           altDoSolve32(9, 0x200l, puzzle | 0x600700l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0xd900l))
        altDoSolve32(9, 0x200l, puzzle | 0xd900l, partsUsed | 0x400l);
    altpiece11at8at32:
      if (partsUsed & 0x800l) goto altpiece13at8at32;
      if (!(puzzle & 0x100300l)) {
         if(!(puzzle & 0x101b00l))
           altDoSolve32(9, 0x200l, puzzle | 0x101b00l, partsUsed | 0x800l);
         if(!(puzzle & 0xb00300l))
           altDoSolve32(9, 0x200l, puzzle | 0xb00300l, partsUsed | 0x800l);
         if(!(puzzle & 0x300b00l))
           altDoSolve32(9, 0x200l, puzzle | 0x300b00l, partsUsed | 0x800l);
         if(!(puzzle & 0x301300l))
           altDoSolve32(9, 0x200l, puzzle | 0x301300l, partsUsed | 0x800l);
         if(!(puzzle & 0x1300300l))
           altDoSolve32(9, 0x200l, puzzle | 0x1300300l, partsUsed | 0x800l);
         if(!(puzzle & 0x900b00l))
           altDoSolve32(9, 0x200l, puzzle | 0x900b00l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x1001100l)) {
         if(!(puzzle & 0x1801900l))
           altDoSolve32(9, 0x200l, puzzle | 0x1801900l, partsUsed | 0x800l);
         if(!(puzzle & 0x1201300l))
           altDoSolve32(9, 0x200l, puzzle | 0x1201300l, partsUsed | 0x800l);
         if(!(puzzle & 0x1001b00l))
           altDoSolve32(9, 0x200l, puzzle | 0x1001b00l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x800900l)) {
         if(!(puzzle & 0xb00900l))
           altDoSolve32(9, 0x200l, puzzle | 0xb00900l, partsUsed | 0x800l);
         if(!(puzzle & 0x901900l))
           altDoSolve32(9, 0x200l, puzzle | 0x901900l, partsUsed | 0x800l);
         if(!(puzzle & 0x801b00l))
           altDoSolve32(9, 0x200l, puzzle | 0x801b00l, partsUsed | 0x800l);
         if(!(puzzle & 0x1900900l))
           altDoSolve32(9, 0x200l, puzzle | 0x1900900l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x201b00l))
        altDoSolve32(9, 0x200l, puzzle | 0x201b00l, partsUsed | 0x800l);
      if(!(puzzle & 0x1b00100l))
        altDoSolve32(9, 0x200l, puzzle | 0x1b00100l, partsUsed | 0x800l);
    altpiece13at8at32:
    break;
   case 9:
      if (partsUsed & 0x2l) goto altpiece2at9at32;
      if(!(puzzle & 0x5e00l))
        altDoSolve32(10, 0x400l, puzzle | 0x5e00l, partsUsed | 0x2l);
    altpiece2at9at32:
      if (partsUsed & 0x4l) goto altpiece3at9at32;
      if(!(puzzle & 0x1d200l))
        altDoSolve32(10, 0x400l, puzzle | 0x1d200l, partsUsed | 0x4l);
    altpiece3at9at32:
      if (partsUsed & 0x8l) goto altpiece4at9at32;
      if(!(puzzle & 0xd600l))
        altDoSolve32(10, 0x400l, puzzle | 0xd600l, partsUsed | 0x8l);
    altpiece4at9at32:
      if (partsUsed & 0x10l) goto altpiece5at9at32;
      if (!(puzzle & 0x49200l)) {
         if(!(puzzle & 0x69a00l))
           altDoSolve32(10, 0x400l, puzzle | 0x69a00l, partsUsed | 0x10l);
         if(!(puzzle & 0x8249200l))
           altDoSolve32(10, 0x400l, puzzle | 0x8249200l, partsUsed | 0x10l);
         if(!(puzzle & 0xcb200l))
           altDoSolve32(10, 0x400l, puzzle | 0xcb200l, partsUsed | 0x10l);
         if(!(puzzle & 0x41049200l))
           altDoSolve32(10, 0x400l, puzzle | 0x41049200l, partsUsed | 0x10l);
         if(!(puzzle & 0x59600l))
           altDoSolve32(10, 0x400l, puzzle | 0x59600l, partsUsed | 0x10l);
      }
      if(!(puzzle & 0x9a600l))
        altDoSolve32(10, 0x400l, puzzle | 0x9a600l, partsUsed | 0x10l);
      if(!(puzzle & 0x49208200l))
        altDoSolve32(10, 0x400l, puzzle | 0x49208200l, partsUsed | 0x10l);
    altpiece5at9at32:
      if (partsUsed & 0x20l) goto altpiece6at9at32;
      if (!(puzzle & 0x49200l)) {
         if(!(puzzle & 0x69200l))
           altDoSolve32(10, 0x400l, puzzle | 0x69200l, partsUsed | 0x20l);
         if(!(puzzle & 0x40049200l))
           altDoSolve32(10, 0x400l, puzzle | 0x40049200l, partsUsed | 0x20l);
         if(!(puzzle & 0x49600l))
           altDoSolve32(10, 0x400l, puzzle | 0x49600l, partsUsed | 0x20l);
         if(!(puzzle & 0x249200l))
           altDoSolve32(10, 0x400l, puzzle | 0x249200l, partsUsed | 0x20l);
         if(!(puzzle & 0xc9200l))
           altDoSolve32(10, 0x400l, puzzle | 0xc9200l, partsUsed | 0x20l);
      }
      if(!(puzzle & 0x49200200l))
        altDoSolve32(10, 0x400l, puzzle | 0x49200200l, partsUsed | 0x20l);
      if(!(puzzle & 0x92600l))
        altDoSolve32(10, 0x400l, puzzle | 0x92600l, partsUsed | 0x20l);
    altpiece6at9at32:
      if (partsUsed & 0x40l) goto altpiece7at9at32;
      if (!(puzzle & 0x49200l)) {
         if(!(puzzle & 0x49a00l))
           altDoSolve32(10, 0x400l, puzzle | 0x49a00l, partsUsed | 0x40l);
         if(!(puzzle & 0x4d200l))
           altDoSolve32(10, 0x400l, puzzle | 0x4d200l, partsUsed | 0x40l);
         if(!(puzzle & 0x8049200l))
           altDoSolve32(10, 0x400l, puzzle | 0x8049200l, partsUsed | 0x40l);
         if(!(puzzle & 0x4b200l))
           altDoSolve32(10, 0x400l, puzzle | 0x4b200l, partsUsed | 0x40l);
         if(!(puzzle & 0x1049200l))
           altDoSolve32(10, 0x400l, puzzle | 0x1049200l, partsUsed | 0x40l);
         if(!(puzzle & 0x59200l))
           altDoSolve32(10, 0x400l, puzzle | 0x59200l, partsUsed | 0x40l);
      }
    altpiece7at9at32:
      if (partsUsed & 0x80l) goto altpiece8at9at32;
      if(!(puzzle & 0x9208200l))
        altDoSolve32(10, 0x400l, puzzle | 0x9208200l, partsUsed | 0x80l);
      if(!(puzzle & 0x8209200l))
        altDoSolve32(10, 0x400l, puzzle | 0x8209200l, partsUsed | 0x80l);
      if(!(puzzle & 0x19600l))
        altDoSolve32(10, 0x400l, puzzle | 0x19600l, partsUsed | 0x80l);
      if(!(puzzle & 0x1a600l))
        altDoSolve32(10, 0x400l, puzzle | 0x1a600l, partsUsed | 0x80l);
    altpiece8at9at32:
      if (partsUsed & 0x100l) goto altpiece9at9at32;
      if(!(puzzle & 0x801a00l))
        altDoSolve32(10, 0x400l, puzzle | 0x801a00l, partsUsed | 0x100l);
      if(!(puzzle & 0x301200l))
        altDoSolve32(10, 0x400l, puzzle | 0x301200l, partsUsed | 0x100l);
      if(!(puzzle & 0x203200l))
        altDoSolve32(10, 0x400l, puzzle | 0x203200l, partsUsed | 0x100l);
      if(!(puzzle & 0x1a00200l))
        altDoSolve32(10, 0x400l, puzzle | 0x1a00200l, partsUsed | 0x100l);
      if(!(puzzle & 0x1200600l))
        altDoSolve32(10, 0x400l, puzzle | 0x1200600l, partsUsed | 0x100l);
      if(!(puzzle & 0x2600200l))
        altDoSolve32(10, 0x400l, puzzle | 0x2600200l, partsUsed | 0x100l);
      if(!(puzzle & 0x401600l))
        altDoSolve32(10, 0x400l, puzzle | 0x401600l, partsUsed | 0x100l);
      if(!(puzzle & 0x3001200l))
        altDoSolve32(10, 0x400l, puzzle | 0x3001200l, partsUsed | 0x100l);
      if(!(puzzle & 0x2002600l))
        altDoSolve32(10, 0x400l, puzzle | 0x2002600l, partsUsed | 0x100l);
    altpiece9at9at32:
      if (partsUsed & 0x200l) goto altpiece10at9at32;
      if (!(puzzle & 0x1a00l)) {
         if(!(puzzle & 0x13a00l))
           altDoSolve32(10, 0x400l, puzzle | 0x13a00l, partsUsed | 0x200l);
         if(!(puzzle & 0x7a00l))
           altDoSolve32(10, 0x400l, puzzle | 0x7a00l, partsUsed | 0x200l);
         if(!(puzzle & 0x9e00l))
           altDoSolve32(10, 0x400l, puzzle | 0x9e00l, partsUsed | 0x200l);
         if(!(puzzle & 0x19a00l))
           altDoSolve32(10, 0x400l, puzzle | 0x19a00l, partsUsed | 0x200l);
      }
      if(!(puzzle & 0xf200l))
        altDoSolve32(10, 0x400l, puzzle | 0xf200l, partsUsed | 0x200l);
    altpiece10at9at32:
      if (partsUsed & 0x400l) goto altpiece11at9at32;
      if (!(puzzle & 0x1001200l)) {
         if(!(puzzle & 0x9201200l))
           altDoSolve32(10, 0x400l, puzzle | 0x9201200l, partsUsed | 0x400l);
         if(!(puzzle & 0x9009200l))
           altDoSolve32(10, 0x400l, puzzle | 0x9009200l, partsUsed | 0x400l);
         if(!(puzzle & 0x1209200l))
           altDoSolve32(10, 0x400l, puzzle | 0x1209200l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x3200l)) {
         if(!(puzzle & 0xb600l))
           altDoSolve32(10, 0x400l, puzzle | 0xb600l, partsUsed | 0x400l);
         if(!(puzzle & 0x13600l))
           altDoSolve32(10, 0x400l, puzzle | 0x13600l, partsUsed | 0x400l);
         if(!(puzzle & 0x1b200l))
           altDoSolve32(10, 0x400l, puzzle | 0x1b200l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x3e00l))
        altDoSolve32(10, 0x400l, puzzle | 0x3e00l, partsUsed | 0x400l);
      if(!(puzzle & 0xda00l))
        altDoSolve32(10, 0x400l, puzzle | 0xda00l, partsUsed | 0x400l);
      if(!(puzzle & 0x700600l))
        altDoSolve32(10, 0x400l, puzzle | 0x700600l, partsUsed | 0x400l);
    altpiece11at9at32:
      if (partsUsed & 0x800l) goto altpiece13at9at32;
      if (!(puzzle & 0x200600l)) {
         if(!(puzzle & 0x203600l))
           altDoSolve32(10, 0x400l, puzzle | 0x203600l, partsUsed | 0x800l);
         if(!(puzzle & 0x1600600l))
           altDoSolve32(10, 0x400l, puzzle | 0x1600600l, partsUsed | 0x800l);
         if(!(puzzle & 0x601600l))
           altDoSolve32(10, 0x400l, puzzle | 0x601600l, partsUsed | 0x800l);
         if(!(puzzle & 0x602600l))
           altDoSolve32(10, 0x400l, puzzle | 0x602600l, partsUsed | 0x800l);
         if(!(puzzle & 0x2600600l))
           altDoSolve32(10, 0x400l, puzzle | 0x2600600l, partsUsed | 0x800l);
         if(!(puzzle & 0x1201600l))
           altDoSolve32(10, 0x400l, puzzle | 0x1201600l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x2002200l)) {
         if(!(puzzle & 0x3003200l))
           altDoSolve32(10, 0x400l, puzzle | 0x3003200l, partsUsed | 0x800l);
         if(!(puzzle & 0x2402600l))
           altDoSolve32(10, 0x400l, puzzle | 0x2402600l, partsUsed | 0x800l);
         if(!(puzzle & 0x2003600l))
           altDoSolve32(10, 0x400l, puzzle | 0x2003600l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x1001200l)) {
         if(!(puzzle & 0x1601200l))
           altDoSolve32(10, 0x400l, puzzle | 0x1601200l, partsUsed | 0x800l);
         if(!(puzzle & 0x1203200l))
           altDoSolve32(10, 0x400l, puzzle | 0x1203200l, partsUsed | 0x800l);
         if(!(puzzle & 0x1003600l))
           altDoSolve32(10, 0x400l, puzzle | 0x1003600l, partsUsed | 0x800l);
         if(!(puzzle & 0x3201200l))
           altDoSolve32(10, 0x400l, puzzle | 0x3201200l, partsUsed | 0x800l);
         if(!(puzzle & 0x1a01200l))
           altDoSolve32(10, 0x400l, puzzle | 0x1a01200l, partsUsed | 0x800l);
         if(!(puzzle & 0x1801a00l))
           altDoSolve32(10, 0x400l, puzzle | 0x1801a00l, partsUsed | 0x800l);
         if(!(puzzle & 0x1301200l))
           altDoSolve32(10, 0x400l, puzzle | 0x1301200l, partsUsed | 0x800l);
         if(!(puzzle & 0x1201a00l))
           altDoSolve32(10, 0x400l, puzzle | 0x1201a00l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x403600l))
        altDoSolve32(10, 0x400l, puzzle | 0x403600l, partsUsed | 0x800l);
      if(!(puzzle & 0x3600200l))
        altDoSolve32(10, 0x400l, puzzle | 0x3600200l, partsUsed | 0x800l);
      if(!(puzzle & 0x1b00200l))
        altDoSolve32(10, 0x400l, puzzle | 0x1b00200l, partsUsed | 0x800l);
    altpiece13at9at32:
    break;
   case 10:
      if (partsUsed & 0x2l) goto altpiece2at10at32;
      if(!(puzzle & 0xf400l))
        altDoSolve32(11, 0x800l, puzzle | 0xf400l, partsUsed | 0x2l);
    altpiece2at10at32:
      if (partsUsed & 0x4l) goto altpiece3at10at32;
      if(!(puzzle & 0x13c00l))
        altDoSolve32(11, 0x800l, puzzle | 0x13c00l, partsUsed | 0x4l);
    altpiece3at10at32:
      if (partsUsed & 0x8l) goto altpiece4at10at32;
      if(!(puzzle & 0x7c00l))
        altDoSolve32(11, 0x800l, puzzle | 0x7c00l, partsUsed | 0x8l);
    altpiece4at10at32:
      if (partsUsed & 0x10l) goto altpiece5at10at32;
      if (!(puzzle & 0x92400l)) {
         if(!(puzzle & 0xd3400l))
           altDoSolve32(11, 0x800l, puzzle | 0xd3400l, partsUsed | 0x10l);
         if(!(puzzle & 0x10492400l))
           altDoSolve32(11, 0x800l, puzzle | 0x10492400l, partsUsed | 0x10l);
         if(!(puzzle & 0x82092400l))
           altDoSolve32(11, 0x800l, puzzle | 0x82092400l, partsUsed | 0x10l);
      }
      if(!(puzzle & 0x92410400l))
        altDoSolve32(11, 0x800l, puzzle | 0x92410400l, partsUsed | 0x10l);
    altpiece5at10at32:
      if (partsUsed & 0x20l) goto altpiece6at10at32;
      if (!(puzzle & 0x92400l)) {
         if(!(puzzle & 0xd2400l))
           altDoSolve32(11, 0x800l, puzzle | 0xd2400l, partsUsed | 0x20l);
         if(!(puzzle & 0x80092400l))
           altDoSolve32(11, 0x800l, puzzle | 0x80092400l, partsUsed | 0x20l);
         if(!(puzzle & 0x492400l))
           altDoSolve32(11, 0x800l, puzzle | 0x492400l, partsUsed | 0x20l);
      }
      if(!(puzzle & 0x92400400l))
        altDoSolve32(11, 0x800l, puzzle | 0x92400400l, partsUsed | 0x20l);
    altpiece6at10at32:
      if (partsUsed & 0x40l) goto altpiece7at10at32;
      if (!(puzzle & 0x92400l)) {
         if(!(puzzle & 0x93400l))
           altDoSolve32(11, 0x800l, puzzle | 0x93400l, partsUsed | 0x40l);
         if(!(puzzle & 0x9a400l))
           altDoSolve32(11, 0x800l, puzzle | 0x9a400l, partsUsed | 0x40l);
         if(!(puzzle & 0x10092400l))
           altDoSolve32(11, 0x800l, puzzle | 0x10092400l, partsUsed | 0x40l);
         if(!(puzzle & 0x2092400l))
           altDoSolve32(11, 0x800l, puzzle | 0x2092400l, partsUsed | 0x40l);
      }
    altpiece7at10at32:
      if (partsUsed & 0x80l) goto altpiece8at10at32;
      if(!(puzzle & 0x12410400l))
        altDoSolve32(11, 0x800l, puzzle | 0x12410400l, partsUsed | 0x80l);
      if(!(puzzle & 0x10412400l))
        altDoSolve32(11, 0x800l, puzzle | 0x10412400l, partsUsed | 0x80l);
    altpiece8at10at32:
      if (partsUsed & 0x100l) goto altpiece9at10at32;
      if(!(puzzle & 0x3400400l))
        altDoSolve32(11, 0x800l, puzzle | 0x3400400l, partsUsed | 0x100l);
      if(!(puzzle & 0x1003400l))
        altDoSolve32(11, 0x800l, puzzle | 0x1003400l, partsUsed | 0x100l);
      if(!(puzzle & 0x602400l))
        altDoSolve32(11, 0x800l, puzzle | 0x602400l, partsUsed | 0x100l);
    altpiece9at10at32:
      if (partsUsed & 0x200l) goto altpiece10at10at32;
      if(!(puzzle & 0xbc00l))
        altDoSolve32(11, 0x800l, puzzle | 0xbc00l, partsUsed | 0x200l);
    altpiece10at10at32:
      if (partsUsed & 0x400l) goto altpiece11at10at32;
      if (!(puzzle & 0x12400l)) {
         if(!(puzzle & 0x1b400l))
           altDoSolve32(11, 0x800l, puzzle | 0x1b400l, partsUsed | 0x400l);
         if(!(puzzle & 0x12012400l))
           altDoSolve32(11, 0x800l, puzzle | 0x12012400l, partsUsed | 0x400l);
         if(!(puzzle & 0x2412400l))
           altDoSolve32(11, 0x800l, puzzle | 0x2412400l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x12402400l))
        altDoSolve32(11, 0x800l, puzzle | 0x12402400l, partsUsed | 0x400l);
    altpiece11at10at32:
      if (partsUsed & 0x800l) goto altpiece13at10at32;
      if (!(puzzle & 0x3000400l)) {
         if(!(puzzle & 0x3600400l))
           altDoSolve32(11, 0x800l, puzzle | 0x3600400l, partsUsed | 0x800l);
         if(!(puzzle & 0x3402400l))
           altDoSolve32(11, 0x800l, puzzle | 0x3402400l, partsUsed | 0x800l);
         if(!(puzzle & 0x3003400l))
           altDoSolve32(11, 0x800l, puzzle | 0x3003400l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x2602400l))
        altDoSolve32(11, 0x800l, puzzle | 0x2602400l, partsUsed | 0x800l);
      if(!(puzzle & 0x2403400l))
        altDoSolve32(11, 0x800l, puzzle | 0x2403400l, partsUsed | 0x800l);
    altpiece13at10at32:
    break;
   case 11:
      if (partsUsed & 0x2l) goto altpiece2at11at32;
      if(!(puzzle & 0x99800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x99800l, partsUsed | 0x2l);
      if(!(puzzle & 0xcc800l))
        altDoSolve32(12, 0x1000l, puzzle | 0xcc800l, partsUsed | 0x2l);
    altpiece2at11at32:
      if (partsUsed & 0x4l) goto altpiece3at11at32;
      if(!(puzzle & 0x4b800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x4b800l, partsUsed | 0x4l);
      if(!(puzzle & 0x3c800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x3c800l, partsUsed | 0x4l);
    altpiece3at11at32:
      if (partsUsed & 0x8l) goto altpiece4at11at32;
      if(!(puzzle & 0xc9800l))
        altDoSolve32(12, 0x1000l, puzzle | 0xc9800l, partsUsed | 0x8l);
      if(!(puzzle & 0x9c800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x9c800l, partsUsed | 0x8l);
    altpiece4at11at32:
      if (partsUsed & 0x10l) goto altpiece5at11at32;
      if(!(puzzle & 0x24920800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x24920800l, partsUsed | 0x10l);
    altpiece5at11at32:
      if (partsUsed & 0x40l) goto altpiece7at11at32;
      if(!(puzzle & 0x24900800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x24900800l, partsUsed | 0x40l);
    altpiece7at11at32:
      if (partsUsed & 0x80l) goto altpiece8at11at32;
      if(!(puzzle & 0x24820800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x24820800l, partsUsed | 0x80l);
      if(!(puzzle & 0x20824800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x20824800l, partsUsed | 0x80l);
      if(!(puzzle & 0x17800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x17800l, partsUsed | 0x80l);
      if(!(puzzle & 0x65800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x65800l, partsUsed | 0x80l);
      if(!(puzzle & 0x69800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x69800l, partsUsed | 0x80l);
      if(!(puzzle & 0x1e800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x1e800l, partsUsed | 0x80l);
      if(!(puzzle & 0x3802800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x3802800l, partsUsed | 0x80l);
      if(!(puzzle & 0x2803800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x2803800l, partsUsed | 0x80l);
    altpiece8at11at32:
      if (partsUsed & 0x100l) goto altpiece9at11at32;
      if(!(puzzle & 0x1201800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x1201800l, partsUsed | 0x100l);
      if(!(puzzle & 0x1005800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x1005800l, partsUsed | 0x100l);
      if(!(puzzle & 0xb00800l))
        altDoSolve32(12, 0x1000l, puzzle | 0xb00800l, partsUsed | 0x100l);
      if(!(puzzle & 0x9800800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x9800800l, partsUsed | 0x100l);
      if(!(puzzle & 0x4801800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x4801800l, partsUsed | 0x100l);
      if(!(puzzle & 0xc004800l))
        altDoSolve32(12, 0x1000l, puzzle | 0xc004800l, partsUsed | 0x100l);
      if(!(puzzle & 0x80c800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x80c800l, partsUsed | 0x100l);
      if(!(puzzle & 0x8009800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x8009800l, partsUsed | 0x100l);
    altpiece9at11at32:
      if (partsUsed & 0x200l) goto altpiece10at11at32;
      if(!(puzzle & 0x59800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x59800l, partsUsed | 0x200l);
      if(!(puzzle & 0x5c800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x5c800l, partsUsed | 0x200l);
    altpiece10at11at32:
      if (partsUsed & 0x400l) goto altpiece11at11at32;
      if (!(puzzle & 0x4004800l)) {
         if(!(puzzle & 0x4904800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x4904800l, partsUsed | 0x400l);
         if(!(puzzle & 0x24804800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x24804800l, partsUsed | 0x400l);
         if(!(puzzle & 0x24024800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x24024800l, partsUsed | 0x400l);
         if(!(puzzle & 0x4824800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x4824800l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x9800l)) {
         if(!(puzzle & 0x1b800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x1b800l, partsUsed | 0x400l);
         if(!(puzzle & 0xf800l))
           altDoSolve32(12, 0x1000l, puzzle | 0xf800l, partsUsed | 0x400l);
         if(!(puzzle & 0x2d800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x2d800l, partsUsed | 0x400l);
         if(!(puzzle & 0x4d800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x4d800l, partsUsed | 0x400l);
         if(!(puzzle & 0x1d800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x1d800l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x1001800l)) {
         if(!(puzzle & 0x3801800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x3801800l, partsUsed | 0x400l);
         if(!(puzzle & 0x1803800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x1803800l, partsUsed | 0x400l);
         if(!(puzzle & 0x3003800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x3003800l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x6c800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x6c800l, partsUsed | 0x400l);
    altpiece11at11at32:
      if (partsUsed & 0x800l) goto altpiece13at11at32;
      if (!(puzzle & 0x801800l)) {
         if(!(puzzle & 0x80d800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x80d800l, partsUsed | 0x800l);
         if(!(puzzle & 0x5801800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x5801800l, partsUsed | 0x800l);
         if(!(puzzle & 0x1805800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x1805800l, partsUsed | 0x800l);
         if(!(puzzle & 0x1809800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x1809800l, partsUsed | 0x800l);
         if(!(puzzle & 0x9801800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x9801800l, partsUsed | 0x800l);
         if(!(puzzle & 0x4805800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x4805800l, partsUsed | 0x800l);
         if(!(puzzle & 0x1a01800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x1a01800l, partsUsed | 0x800l);
         if(!(puzzle & 0x1901800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x1901800l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x8008800l)) {
         if(!(puzzle & 0xc00c800l))
           altDoSolve32(12, 0x1000l, puzzle | 0xc00c800l, partsUsed | 0x800l);
         if(!(puzzle & 0x9009800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x9009800l, partsUsed | 0x800l);
         if(!(puzzle & 0x800d800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x800d800l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x4004800l)) {
         if(!(puzzle & 0x5804800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x5804800l, partsUsed | 0x800l);
         if(!(puzzle & 0x480c800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x480c800l, partsUsed | 0x800l);
         if(!(puzzle & 0x400d800l))
           altDoSolve32(12, 0x1000l, puzzle | 0x400d800l, partsUsed | 0x800l);
         if(!(puzzle & 0xc804800l))
           altDoSolve32(12, 0x1000l, puzzle | 0xc804800l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x100d800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x100d800l, partsUsed | 0x800l);
      if(!(puzzle & 0xd800800l))
        altDoSolve32(12, 0x1000l, puzzle | 0xd800800l, partsUsed | 0x800l);
      if(!(puzzle & 0x1b00800l))
        altDoSolve32(12, 0x1000l, puzzle | 0x1b00800l, partsUsed | 0x800l);
    altpiece13at11at32:
    break;
   case 12:
      if (partsUsed & 0x2l) goto altpiece2at12at32;
      if(!(puzzle & 0x2f000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x2f000l, partsUsed | 0x2l);
    altpiece2at12at32:
      if (partsUsed & 0x4l) goto altpiece3at12at32;
      if(!(puzzle & 0xe9000l))
        altDoSolve32(13, 0x2000l, puzzle | 0xe9000l, partsUsed | 0x4l);
    altpiece3at12at32:
      if (partsUsed & 0x8l) goto altpiece4at12at32;
      if(!(puzzle & 0x6b000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x6b000l, partsUsed | 0x8l);
    altpiece4at12at32:
      if (partsUsed & 0x10l) goto altpiece5at12at32;
      if(!(puzzle & 0x49241000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x49241000l, partsUsed | 0x10l);
    altpiece5at12at32:
      if (partsUsed & 0x40l) goto altpiece7at12at32;
      if(!(puzzle & 0x49201000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x49201000l, partsUsed | 0x40l);
    altpiece7at12at32:
      if (partsUsed & 0x80l) goto altpiece8at12at32;
      if(!(puzzle & 0x49041000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x49041000l, partsUsed | 0x80l);
      if(!(puzzle & 0x41049000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x41049000l, partsUsed | 0x80l);
      if(!(puzzle & 0xcb000l))
        altDoSolve32(13, 0x2000l, puzzle | 0xcb000l, partsUsed | 0x80l);
      if(!(puzzle & 0xd3000l))
        altDoSolve32(13, 0x2000l, puzzle | 0xd3000l, partsUsed | 0x80l);
    altpiece8at12at32:
      if (partsUsed & 0x100l) goto altpiece9at12at32;
      if(!(puzzle & 0x1901000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x1901000l, partsUsed | 0x100l);
      if(!(puzzle & 0x1809000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x1809000l, partsUsed | 0x100l);
      if(!(puzzle & 0x2403000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x2403000l, partsUsed | 0x100l);
      if(!(puzzle & 0x200b000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x200b000l, partsUsed | 0x100l);
      if(!(puzzle & 0x1601000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x1601000l, partsUsed | 0x100l);
      if(!(puzzle & 0x400d000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x400d000l, partsUsed | 0x100l);
      if(!(puzzle & 0xd001000l))
        altDoSolve32(13, 0x2000l, puzzle | 0xd001000l, partsUsed | 0x100l);
      if(!(puzzle & 0x9003000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x9003000l, partsUsed | 0x100l);
      if(!(puzzle & 0x13001000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x13001000l, partsUsed | 0x100l);
      if(!(puzzle & 0x18009000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x18009000l, partsUsed | 0x100l);
      if(!(puzzle & 0x1019000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x1019000l, partsUsed | 0x100l);
      if(!(puzzle & 0x10013000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x10013000l, partsUsed | 0x100l);
    altpiece9at12at32:
      if (partsUsed & 0x200l) goto altpiece10at12at32;
      if (!(puzzle & 0xd000l)) {
         if(!(puzzle & 0x9d000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x9d000l, partsUsed | 0x200l);
         if(!(puzzle & 0x3d000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x3d000l, partsUsed | 0x200l);
         if(!(puzzle & 0x4f000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x4f000l, partsUsed | 0x200l);
         if(!(puzzle & 0xcd000l))
           altDoSolve32(13, 0x2000l, puzzle | 0xcd000l, partsUsed | 0x200l);
      }
      if(!(puzzle & 0x79000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x79000l, partsUsed | 0x200l);
    altpiece10at12at32:
      if (partsUsed & 0x400l) goto altpiece11at12at32;
      if (!(puzzle & 0x8009000l)) {
         if(!(puzzle & 0x9209000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x9209000l, partsUsed | 0x400l);
         if(!(puzzle & 0x49009000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x49009000l, partsUsed | 0x400l);
         if(!(puzzle & 0x48049000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x48049000l, partsUsed | 0x400l);
         if(!(puzzle & 0x9049000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x9049000l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x19000l)) {
         if(!(puzzle & 0x5b000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x5b000l, partsUsed | 0x400l);
         if(!(puzzle & 0x9b000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x9b000l, partsUsed | 0x400l);
         if(!(puzzle & 0xd9000l))
           altDoSolve32(13, 0x2000l, puzzle | 0xd9000l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x1f000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x1f000l, partsUsed | 0x400l);
      if(!(puzzle & 0x6d000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x6d000l, partsUsed | 0x400l);
      if(!(puzzle & 0x3803000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x3803000l, partsUsed | 0x400l);
    altpiece11at12at32:
      if (partsUsed & 0x800l) goto altpiece13at12at32;
      if (!(puzzle & 0x1003000l)) {
         if(!(puzzle & 0x101b000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x101b000l, partsUsed | 0x800l);
         if(!(puzzle & 0xb003000l))
           altDoSolve32(13, 0x2000l, puzzle | 0xb003000l, partsUsed | 0x800l);
         if(!(puzzle & 0x300b000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x300b000l, partsUsed | 0x800l);
         if(!(puzzle & 0x3013000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x3013000l, partsUsed | 0x800l);
         if(!(puzzle & 0x13003000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x13003000l, partsUsed | 0x800l);
         if(!(puzzle & 0x900b000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x900b000l, partsUsed | 0x800l);
         if(!(puzzle & 0x3403000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x3403000l, partsUsed | 0x800l);
         if(!(puzzle & 0x3203000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x3203000l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x10011000l)) {
         if(!(puzzle & 0x18019000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x18019000l, partsUsed | 0x800l);
         if(!(puzzle & 0x12013000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x12013000l, partsUsed | 0x800l);
         if(!(puzzle & 0x1001b000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x1001b000l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x8009000l)) {
         if(!(puzzle & 0xb009000l))
           altDoSolve32(13, 0x2000l, puzzle | 0xb009000l, partsUsed | 0x800l);
         if(!(puzzle & 0x9019000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x9019000l, partsUsed | 0x800l);
         if(!(puzzle & 0x801b000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x801b000l, partsUsed | 0x800l);
         if(!(puzzle & 0x19009000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x19009000l, partsUsed | 0x800l);
         if(!(puzzle & 0xd009000l))
           altDoSolve32(13, 0x2000l, puzzle | 0xd009000l, partsUsed | 0x800l);
         if(!(puzzle & 0xc00d000l))
           altDoSolve32(13, 0x2000l, puzzle | 0xc00d000l, partsUsed | 0x800l);
         if(!(puzzle & 0x9809000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x9809000l, partsUsed | 0x800l);
         if(!(puzzle & 0x900d000l))
           altDoSolve32(13, 0x2000l, puzzle | 0x900d000l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x201b000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x201b000l, partsUsed | 0x800l);
      if(!(puzzle & 0x1b001000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x1b001000l, partsUsed | 0x800l);
      if(!(puzzle & 0xd801000l))
        altDoSolve32(13, 0x2000l, puzzle | 0xd801000l, partsUsed | 0x800l);
      if(!(puzzle & 0x3601000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x3601000l, partsUsed | 0x800l);
      if(!(puzzle & 0x1b01000l))
        altDoSolve32(13, 0x2000l, puzzle | 0x1b01000l, partsUsed | 0x800l);
    altpiece13at12at32:
    break;
   case 13:
      if (partsUsed & 0x2l) goto altpiece2at13at32;
      if(!(puzzle & 0x7a000l))
        altDoSolve32(14, 0x4000l, puzzle | 0x7a000l, partsUsed | 0x2l);
    altpiece2at13at32:
      if (partsUsed & 0x4l) goto altpiece3at13at32;
      if(!(puzzle & 0x9e000l))
        altDoSolve32(14, 0x4000l, puzzle | 0x9e000l, partsUsed | 0x4l);
    altpiece3at13at32:
      if (partsUsed & 0x8l) goto altpiece4at13at32;
      if(!(puzzle & 0x3e000l))
        altDoSolve32(14, 0x4000l, puzzle | 0x3e000l, partsUsed | 0x8l);
    altpiece4at13at32:
      if (partsUsed & 0x10l) goto altpiece5at13at32;
      if(!(puzzle & 0x92482000l))
        altDoSolve32(14, 0x4000l, puzzle | 0x92482000l, partsUsed | 0x10l);
    altpiece5at13at32:
      if (partsUsed & 0x40l) goto altpiece7at13at32;
      if(!(puzzle & 0x92402000l))
        altDoSolve32(14, 0x4000l, puzzle | 0x92402000l, partsUsed | 0x40l);
    altpiece7at13at32:
      if (partsUsed & 0x80l) goto altpiece8at13at32;
      if(!(puzzle & 0x92082000l))
        altDoSolve32(14, 0x4000l, puzzle | 0x92082000l, partsUsed | 0x80l);
      if(!(puzzle & 0x82092000l))
        altDoSolve32(14, 0x4000l, puzzle | 0x82092000l, partsUsed | 0x80l);
    altpiece8at13at32:
      if (partsUsed & 0x100l) goto altpiece9at13at32;
      if(!(puzzle & 0x3202000l))
        altDoSolve32(14, 0x4000l, puzzle | 0x3202000l, partsUsed | 0x100l);
      if(!(puzzle & 0x3012000l))
        altDoSolve32(14, 0x4000l, puzzle | 0x3012000l, partsUsed | 0x100l);
      if(!(puzzle & 0x801a000l))
        altDoSolve32(14, 0x4000l, puzzle | 0x801a000l, partsUsed | 0x100l);
      if(!(puzzle & 0x1a002000l))
        altDoSolve32(14, 0x4000l, puzzle | 0x1a002000l, partsUsed | 0x100l);
    altpiece9at13at32:
      if (partsUsed & 0x200l) goto altpiece10at13at32;
      if(!(puzzle & 0x5e000l))
        altDoSolve32(14, 0x4000l, puzzle | 0x5e000l, partsUsed | 0x200l);
    altpiece10at13at32:
      if (partsUsed & 0x400l) goto altpiece11at13at32;
      if (!(puzzle & 0x10012000l)) {
         if(!(puzzle & 0x12412000l))
           altDoSolve32(14, 0x4000l, puzzle | 0x12412000l, partsUsed | 0x400l);
         if(!(puzzle & 0x92012000l))
           altDoSolve32(14, 0x4000l, puzzle | 0x92012000l, partsUsed | 0x400l);
         if(!(puzzle & 0x90092000l))
           altDoSolve32(14, 0x4000l, puzzle | 0x90092000l, partsUsed | 0x400l);
         if(!(puzzle & 0x12092000l))
           altDoSolve32(14, 0x4000l, puzzle | 0x12092000l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0xda000l))
        altDoSolve32(14, 0x4000l, puzzle | 0xda000l, partsUsed | 0x400l);
    altpiece11at13at32:
      if (partsUsed & 0x800l) goto altpiece13at13at32;
      if (!(puzzle & 0x18002000l)) {
         if(!(puzzle & 0x1b002000l))
           altDoSolve32(14, 0x4000l, puzzle | 0x1b002000l, partsUsed | 0x800l);
         if(!(puzzle & 0x1a012000l))
           altDoSolve32(14, 0x4000l, puzzle | 0x1a012000l, partsUsed | 0x800l);
         if(!(puzzle & 0x1801a000l))
           altDoSolve32(14, 0x4000l, puzzle | 0x1801a000l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x13012000l))
        altDoSolve32(14, 0x4000l, puzzle | 0x13012000l, partsUsed | 0x800l);
      if(!(puzzle & 0x1201a000l))
        altDoSolve32(14, 0x4000l, puzzle | 0x1201a000l, partsUsed | 0x800l);
      if(!(puzzle & 0x3602000l))
        altDoSolve32(14, 0x4000l, puzzle | 0x3602000l, partsUsed | 0x800l);
    altpiece13at13at32:
    break;
   case 14:
      if (partsUsed & 0x40l) goto altpiece7at14at32;
      if(!(puzzle & 0x24904000l))
        altDoSolve32(15, 0x8000l, puzzle | 0x24904000l, partsUsed | 0x40l);
    altpiece7at14at32:
      if (partsUsed & 0x80l) goto altpiece8at14at32;
      if(!(puzzle & 0xbc000l))
        altDoSolve32(15, 0x8000l, puzzle | 0xbc000l, partsUsed | 0x80l);
      if(!(puzzle & 0xf4000l))
        altDoSolve32(15, 0x8000l, puzzle | 0xf4000l, partsUsed | 0x80l);
      if(!(puzzle & 0x1c014000l))
        altDoSolve32(15, 0x8000l, puzzle | 0x1c014000l, partsUsed | 0x80l);
      if(!(puzzle & 0x1401c000l))
        altDoSolve32(15, 0x8000l, puzzle | 0x1401c000l, partsUsed | 0x80l);
    altpiece8at14at32:
      if (partsUsed & 0x100l) goto altpiece9at14at32;
      if(!(puzzle & 0x900c000l))
        altDoSolve32(15, 0x8000l, puzzle | 0x900c000l, partsUsed | 0x100l);
      if(!(puzzle & 0x802c000l))
        altDoSolve32(15, 0x8000l, puzzle | 0x802c000l, partsUsed | 0x100l);
      if(!(puzzle & 0x5804000l))
        altDoSolve32(15, 0x8000l, puzzle | 0x5804000l, partsUsed | 0x100l);
      if(!(puzzle & 0x4c004000l))
        altDoSolve32(15, 0x8000l, puzzle | 0x4c004000l, partsUsed | 0x100l);
      if(!(puzzle & 0x2400c000l))
        altDoSolve32(15, 0x8000l, puzzle | 0x2400c000l, partsUsed | 0x100l);
      if(!(puzzle & 0x60024000l))
        altDoSolve32(15, 0x8000l, puzzle | 0x60024000l, partsUsed | 0x100l);
      if(!(puzzle & 0x4064000l))
        altDoSolve32(15, 0x8000l, puzzle | 0x4064000l, partsUsed | 0x100l);
      if(!(puzzle & 0x4004c000l))
        altDoSolve32(15, 0x8000l, puzzle | 0x4004c000l, partsUsed | 0x100l);
    altpiece9at14at32:
      if (partsUsed & 0x400l) goto altpiece11at14at32;
      if (!(puzzle & 0x4c000l)) {
         if(!(puzzle & 0xdc000l))
           altDoSolve32(15, 0x8000l, puzzle | 0xdc000l, partsUsed | 0x400l);
         if(!(puzzle & 0x7c000l))
           altDoSolve32(15, 0x8000l, puzzle | 0x7c000l, partsUsed | 0x400l);
         if(!(puzzle & 0xec000l))
           altDoSolve32(15, 0x8000l, puzzle | 0xec000l, partsUsed | 0x400l);
      }
      if (!(puzzle & 0x800c000l)) {
         if(!(puzzle & 0x1c00c000l))
           altDoSolve32(15, 0x8000l, puzzle | 0x1c00c000l, partsUsed | 0x400l);
         if(!(puzzle & 0xc01c000l))
           altDoSolve32(15, 0x8000l, puzzle | 0xc01c000l, partsUsed | 0x400l);
         if(!(puzzle & 0x1801c000l))
           altDoSolve32(15, 0x8000l, puzzle | 0x1801c000l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x24824000l))
        altDoSolve32(15, 0x8000l, puzzle | 0x24824000l, partsUsed | 0x400l);
    altpiece11at14at32:
      if (partsUsed & 0x800l) goto altpiece13at14at32;
      if (!(puzzle & 0x400c000l)) {
         if(!(puzzle & 0x406c000l))
           altDoSolve32(15, 0x8000l, puzzle | 0x406c000l, partsUsed | 0x800l);
         if(!(puzzle & 0x2c00c000l))
           altDoSolve32(15, 0x8000l, puzzle | 0x2c00c000l, partsUsed | 0x800l);
         if(!(puzzle & 0xc02c000l))
           altDoSolve32(15, 0x8000l, puzzle | 0xc02c000l, partsUsed | 0x800l);
         if(!(puzzle & 0xc04c000l))
           altDoSolve32(15, 0x8000l, puzzle | 0xc04c000l, partsUsed | 0x800l);
         if(!(puzzle & 0x4c00c000l))
           altDoSolve32(15, 0x8000l, puzzle | 0x4c00c000l, partsUsed | 0x800l);
         if(!(puzzle & 0x2402c000l))
           altDoSolve32(15, 0x8000l, puzzle | 0x2402c000l, partsUsed | 0x800l);
         if(!(puzzle & 0xd00c000l))
           altDoSolve32(15, 0x8000l, puzzle | 0xd00c000l, partsUsed | 0x800l);
         if(!(puzzle & 0xc80c000l))
           altDoSolve32(15, 0x8000l, puzzle | 0xc80c000l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x40044000l)) {
         if(!(puzzle & 0x60064000l))
           altDoSolve32(15, 0x8000l, puzzle | 0x60064000l, partsUsed | 0x800l);
         if(!(puzzle & 0x4804c000l))
           altDoSolve32(15, 0x8000l, puzzle | 0x4804c000l, partsUsed | 0x800l);
         if(!(puzzle & 0x4006c000l))
           altDoSolve32(15, 0x8000l, puzzle | 0x4006c000l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x20024000l)) {
         if(!(puzzle & 0x2c024000l))
           altDoSolve32(15, 0x8000l, puzzle | 0x2c024000l, partsUsed | 0x800l);
         if(!(puzzle & 0x24064000l))
           altDoSolve32(15, 0x8000l, puzzle | 0x24064000l, partsUsed | 0x800l);
         if(!(puzzle & 0x2006c000l))
           altDoSolve32(15, 0x8000l, puzzle | 0x2006c000l, partsUsed | 0x800l);
         if(!(puzzle & 0x64024000l))
           altDoSolve32(15, 0x8000l, puzzle | 0x64024000l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x806c000l))
        altDoSolve32(15, 0x8000l, puzzle | 0x806c000l, partsUsed | 0x800l);
      if(!(puzzle & 0x6c004000l))
        altDoSolve32(15, 0x8000l, puzzle | 0x6c004000l, partsUsed | 0x800l);
      if(!(puzzle & 0xd804000l))
        altDoSolve32(15, 0x8000l, puzzle | 0xd804000l, partsUsed | 0x800l);
    altpiece13at14at32:
    break;
   case 15:
      if (partsUsed & 0x40l) goto altpiece7at15at32;
      if(!(puzzle & 0x49208000l))
        altDoSolve32(16, 0x10000l, puzzle | 0x49208000l, partsUsed | 0x40l);
    altpiece7at15at32:
      if (partsUsed & 0x100l) goto altpiece9at15at32;
      if(!(puzzle & 0xc808000l))
        altDoSolve32(16, 0x10000l, puzzle | 0xc808000l, partsUsed | 0x100l);
      if(!(puzzle & 0xc048000l))
        altDoSolve32(16, 0x10000l, puzzle | 0xc048000l, partsUsed | 0x100l);
      if(!(puzzle & 0x12018000l))
        altDoSolve32(16, 0x10000l, puzzle | 0x12018000l, partsUsed | 0x100l);
      if(!(puzzle & 0x10058000l))
        altDoSolve32(16, 0x10000l, puzzle | 0x10058000l, partsUsed | 0x100l);
      if(!(puzzle & 0xb008000l))
        altDoSolve32(16, 0x10000l, puzzle | 0xb008000l, partsUsed | 0x100l);
      if(!(puzzle & 0x20068000l))
        altDoSolve32(16, 0x10000l, puzzle | 0x20068000l, partsUsed | 0x100l);
      if(!(puzzle & 0x68008000l))
        altDoSolve32(16, 0x10000l, puzzle | 0x68008000l, partsUsed | 0x100l);
      if(!(puzzle & 0x48018000l))
        altDoSolve32(16, 0x10000l, puzzle | 0x48018000l, partsUsed | 0x100l);
      if(!(puzzle & 0x98008000l))
        altDoSolve32(16, 0x10000l, puzzle | 0x98008000l, partsUsed | 0x100l);
      if(!(puzzle & 0xc0048000l))
        altDoSolve32(16, 0x10000l, puzzle | 0xc0048000l, partsUsed | 0x100l);
      if(!(puzzle & 0x80c8000l))
        altDoSolve32(16, 0x10000l, puzzle | 0x80c8000l, partsUsed | 0x100l);
      if(!(puzzle & 0x80098000l))
        altDoSolve32(16, 0x10000l, puzzle | 0x80098000l, partsUsed | 0x100l);
    altpiece9at15at32:
      if (partsUsed & 0x400l) goto altpiece11at15at32;
      if(!(puzzle & 0x1c018000l))
        altDoSolve32(16, 0x10000l, puzzle | 0x1c018000l, partsUsed | 0x400l);
      if(!(puzzle & 0x49048000l))
        altDoSolve32(16, 0x10000l, puzzle | 0x49048000l, partsUsed | 0x400l);
      if(!(puzzle & 0xf8000l))
        altDoSolve32(16, 0x10000l, puzzle | 0xf8000l, partsUsed | 0x400l);
    altpiece11at15at32:
      if (partsUsed & 0x800l) goto altpiece13at15at32;
      if (!(puzzle & 0x8018000l)) {
         if(!(puzzle & 0x80d8000l))
           altDoSolve32(16, 0x10000l, puzzle | 0x80d8000l, partsUsed | 0x800l);
         if(!(puzzle & 0x58018000l))
           altDoSolve32(16, 0x10000l, puzzle | 0x58018000l, partsUsed | 0x800l);
         if(!(puzzle & 0x18058000l))
           altDoSolve32(16, 0x10000l, puzzle | 0x18058000l, partsUsed | 0x800l);
         if(!(puzzle & 0x18098000l))
           altDoSolve32(16, 0x10000l, puzzle | 0x18098000l, partsUsed | 0x800l);
         if(!(puzzle & 0x98018000l))
           altDoSolve32(16, 0x10000l, puzzle | 0x98018000l, partsUsed | 0x800l);
         if(!(puzzle & 0x48058000l))
           altDoSolve32(16, 0x10000l, puzzle | 0x48058000l, partsUsed | 0x800l);
         if(!(puzzle & 0x1a018000l))
           altDoSolve32(16, 0x10000l, puzzle | 0x1a018000l, partsUsed | 0x800l);
         if(!(puzzle & 0x19018000l))
           altDoSolve32(16, 0x10000l, puzzle | 0x19018000l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x80088000l)) {
         if(!(puzzle & 0xc00c8000l))
           altDoSolve32(16, 0x10000l, puzzle | 0xc00c8000l, partsUsed | 0x800l);
         if(!(puzzle & 0x90098000l))
           altDoSolve32(16, 0x10000l, puzzle | 0x90098000l, partsUsed | 0x800l);
         if(!(puzzle & 0x800d8000l))
           altDoSolve32(16, 0x10000l, puzzle | 0x800d8000l, partsUsed | 0x800l);
      }
      if (!(puzzle & 0x40048000l)) {
         if(!(puzzle & 0x58048000l))
           altDoSolve32(16, 0x10000l, puzzle | 0x58048000l, partsUsed | 0x800l);
         if(!(puzzle & 0x480c8000l))
           altDoSolve32(16, 0x10000l, puzzle | 0x480c8000l, partsUsed | 0x800l);
         if(!(puzzle & 0x400d8000l))
           altDoSolve32(16, 0x10000l, puzzle | 0x400d8000l, partsUsed | 0x800l);
         if(!(puzzle & 0xc8048000l))
           altDoSolve32(16, 0x10000l, puzzle | 0xc8048000l, partsUsed | 0x800l);
         if(!(puzzle & 0x68048000l))
           altDoSolve32(16, 0x10000l, puzzle | 0x68048000l, partsUsed | 0x800l);
         if(!(puzzle & 0x60068000l))
           altDoSolve32(16, 0x10000l, puzzle | 0x60068000l, partsUsed | 0x800l);
         if(!(puzzle & 0x4c048000l))
           altDoSolve32(16, 0x10000l, puzzle | 0x4c048000l, partsUsed | 0x800l);
         if(!(puzzle & 0x48068000l))
           altDoSolve32(16, 0x10000l, puzzle | 0x48068000l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x100d8000l))
        altDoSolve32(16, 0x10000l, puzzle | 0x100d8000l, partsUsed | 0x800l);
      if(!(puzzle & 0xd8008000l))
        altDoSolve32(16, 0x10000l, puzzle | 0xd8008000l, partsUsed | 0x800l);
      if(!(puzzle & 0x6c008000l))
        altDoSolve32(16, 0x10000l, puzzle | 0x6c008000l, partsUsed | 0x800l);
      if(!(puzzle & 0x1b008000l))
        altDoSolve32(16, 0x10000l, puzzle | 0x1b008000l, partsUsed | 0x800l);
      if(!(puzzle & 0xd808000l))
        altDoSolve32(16, 0x10000l, puzzle | 0xd808000l, partsUsed | 0x800l);
    altpiece13at15at32:
    break;
   case 16:
      if (partsUsed & 0x40l) goto altpiece7at16at32;
      if(!(puzzle & 0x92410000l))
        altDoSolve32(17, 0x20000l, puzzle | 0x92410000l, partsUsed | 0x40l);
    altpiece7at16at32:
      if (partsUsed & 0x100l) goto altpiece9at16at32;
      if(!(puzzle & 0x19010000l))
        altDoSolve32(17, 0x20000l, puzzle | 0x19010000l, partsUsed | 0x100l);
      if(!(puzzle & 0x18090000l))
        altDoSolve32(17, 0x20000l, puzzle | 0x18090000l, partsUsed | 0x100l);
      if(!(puzzle & 0x400d0000l))
        altDoSolve32(17, 0x20000l, puzzle | 0x400d0000l, partsUsed | 0x100l);
      if(!(puzzle & 0xd0010000l))
        altDoSolve32(17, 0x20000l, puzzle | 0xd0010000l, partsUsed | 0x100l);
    altpiece9at16at32:
      if (partsUsed & 0x400l) goto altpiece11at16at32;
      if(!(puzzle & 0x92090000l))
        altDoSolve32(17, 0x20000l, puzzle | 0x92090000l, partsUsed | 0x400l);
    altpiece11at16at32:
      if (partsUsed & 0x800l) goto altpiece13at16at32;
      if (!(puzzle & 0xc0010000l)) {
         if(!(puzzle & 0xd8010000l))
           altDoSolve32(17, 0x20000l, puzzle | 0xd8010000l, partsUsed | 0x800l);
         if(!(puzzle & 0xd0090000l))
           altDoSolve32(17, 0x20000l, puzzle | 0xd0090000l, partsUsed | 0x800l);
         if(!(puzzle & 0xc00d0000l))
           altDoSolve32(17, 0x20000l, puzzle | 0xc00d0000l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x98090000l))
        altDoSolve32(17, 0x20000l, puzzle | 0x98090000l, partsUsed | 0x800l);
      if(!(puzzle & 0x900d0000l))
        altDoSolve32(17, 0x20000l, puzzle | 0x900d0000l, partsUsed | 0x800l);
      if(!(puzzle & 0x1b010000l))
        altDoSolve32(17, 0x20000l, puzzle | 0x1b010000l, partsUsed | 0x800l);
    altpiece13at16at32:
    break;
   case 17:
      if (partsUsed & 0x20l) goto altpiece6at17at32;
      if(!(puzzle & 0x24920000l))
        altDoSolve32(18, 0x40000l, puzzle | 0x24920000l, partsUsed | 0x20l);
    altpiece6at17at32:
      if (partsUsed & 0x80l) goto altpiece8at17at32;
      if(!(puzzle & 0xe00a0000l))
        altDoSolve32(18, 0x40000l, puzzle | 0xe00a0000l, partsUsed | 0x80l);
      if(!(puzzle & 0xa00e0000l))
        altDoSolve32(18, 0x40000l, puzzle | 0xa00e0000l, partsUsed | 0x80l);
    altpiece8at17at32:
      if (partsUsed & 0x100l) goto altpiece9at17at32;
      if(!(puzzle & 0x48060000l))
        altDoSolve32(18, 0x40000l, puzzle | 0x48060000l, partsUsed | 0x100l);
      if(!(puzzle & 0x2c020000l))
        altDoSolve32(18, 0x40000l, puzzle | 0x2c020000l, partsUsed | 0x100l);
    altpiece9at17at32:
      if (partsUsed & 0x400l) goto altpiece11at17at32;
      if (!(puzzle & 0x40060000l)) {
         if(!(puzzle & 0xe0060000l))
           altDoSolve32(18, 0x40000l, puzzle | 0xe0060000l, partsUsed | 0x400l);
         if(!(puzzle & 0x600e0000l))
           altDoSolve32(18, 0x40000l, puzzle | 0x600e0000l, partsUsed | 0x400l);
         if(!(puzzle & 0xc00e0000l))
           altDoSolve32(18, 0x40000l, puzzle | 0xc00e0000l, partsUsed | 0x400l);
      }
    altpiece11at17at32:
      if (partsUsed & 0x800l) goto altpiece13at17at32;
      if (!(puzzle & 0x60020000l)) {
         if(!(puzzle & 0x6c020000l))
           altDoSolve32(18, 0x40000l, puzzle | 0x6c020000l, partsUsed | 0x800l);
         if(!(puzzle & 0x68060000l))
           altDoSolve32(18, 0x40000l, puzzle | 0x68060000l, partsUsed | 0x800l);
         if(!(puzzle & 0x64060000l))
           altDoSolve32(18, 0x40000l, puzzle | 0x64060000l, partsUsed | 0x800l);
      }
    altpiece13at17at32:
    break;
   case 18:
      if (partsUsed & 0x20l) goto altpiece6at18at32;
      if(!(puzzle & 0x49240000l))
        altDoSolve32(19, 0x80000l, puzzle | 0x49240000l, partsUsed | 0x20l);
    altpiece6at18at32:
      if (partsUsed & 0x100l) goto altpiece9at18at32;
      if(!(puzzle & 0x58040000l))
        altDoSolve32(19, 0x80000l, puzzle | 0x58040000l, partsUsed | 0x100l);
      if(!(puzzle & 0x64040000l))
        altDoSolve32(19, 0x80000l, puzzle | 0x64040000l, partsUsed | 0x100l);
      if(!(puzzle & 0x900c0000l))
        altDoSolve32(19, 0x80000l, puzzle | 0x900c0000l, partsUsed | 0x100l);
    altpiece9at18at32:
      if (partsUsed & 0x400l) goto altpiece11at18at32;
      if(!(puzzle & 0xe00c0000l))
        altDoSolve32(19, 0x80000l, puzzle | 0xe00c0000l, partsUsed | 0x400l);
    altpiece11at18at32:
      if (partsUsed & 0x800l) goto altpiece13at18at32;
      if (!(puzzle & 0xc0040000l)) {
         if(!(puzzle & 0xd8040000l))
           altDoSolve32(19, 0x80000l, puzzle | 0xd8040000l, partsUsed | 0x800l);
         if(!(puzzle & 0xd00c0000l))
           altDoSolve32(19, 0x80000l, puzzle | 0xd00c0000l, partsUsed | 0x800l);
         if(!(puzzle & 0xc80c0000l))
           altDoSolve32(19, 0x80000l, puzzle | 0xc80c0000l, partsUsed | 0x800l);
      }
      if(!(puzzle & 0x6c040000l))
        altDoSolve32(19, 0x80000l, puzzle | 0x6c040000l, partsUsed | 0x800l);
    altpiece13at18at32:
    break;
   case 19:
      if (partsUsed & 0x20l) goto altpiece6at19at32;
      if(!(puzzle & 0x92480000l))
        altDoSolve32(20, 0x100000l, puzzle | 0x92480000l, partsUsed | 0x20l);
    altpiece6at19at32:
      if (partsUsed & 0x100l) goto altpiece9at19at32;
      if(!(puzzle & 0xc8080000l))
        altDoSolve32(20, 0x100000l, puzzle | 0xc8080000l, partsUsed | 0x100l);
    altpiece9at19at32:
      if (partsUsed & 0x800l) goto altpiece13at19at32;
      if(!(puzzle & 0xd8080000l))
        altDoSolve32(20, 0x100000l, puzzle | 0xd8080000l, partsUsed | 0x800l);
    altpiece13at19at32:
    break;
   case 20:
      if (partsUsed & 0x2l) goto altpiece2at20at32;
      if(!(puzzle & 0x13300000l))
        altDoSolve32(21, 0x200000l, puzzle | 0x13300000l, partsUsed | 0x2l);
      if(!(puzzle & 0x19900000l))
        altDoSolve32(21, 0x200000l, puzzle | 0x19900000l, partsUsed | 0x2l);
    altpiece2at20at32:
      if (partsUsed & 0x4l) goto altpiece3at20at32;
      if(!(puzzle & 0x9700000l))
        altDoSolve32(21, 0x200000l, puzzle | 0x9700000l, partsUsed | 0x4l);
      if(!(puzzle & 0x7900000l))
        altDoSolve32(21, 0x200000l, puzzle | 0x7900000l, partsUsed | 0x4l);
    altpiece3at20at32:
      if (partsUsed & 0x8l) goto altpiece4at20at32;
      if(!(puzzle & 0x19300000l))
        altDoSolve32(21, 0x200000l, puzzle | 0x19300000l, partsUsed | 0x8l);
      if(!(puzzle & 0x13900000l))
        altDoSolve32(21, 0x200000l, puzzle | 0x13900000l, partsUsed | 0x8l);
    altpiece4at20at32:
      if (partsUsed & 0x10l) goto altpiece5at20at32;
      if(!(puzzle & 0x2cb00000l))
        altDoSolve32(21, 0x200000l, puzzle | 0x2cb00000l, partsUsed | 0x10l);
      if(!(puzzle & 0x65900000l))
        altDoSolve32(21, 0x200000l, puzzle | 0x65900000l, partsUsed | 0x10l);
      if(!(puzzle & 0x4d300000l))
        altDoSolve32(21, 0x200000l, puzzle | 0x4d300000l, partsUsed | 0x10l);
    altpiece5at20at32:
      if (partsUsed & 0x20l) goto altpiece6at20at32;
      if(!(puzzle & 0x24b00000l))
        altDoSolve32(21, 0x200000l, puzzle | 0x24b00000l, partsUsed | 0x20l);
      if(!(puzzle & 0x64900000l))
        altDoSolve32(21, 0x200000l, puzzle | 0x64900000l, partsUsed | 0x20l);
      if(!(puzzle & 0x49300000l))
        altDoSolve32(21, 0x200000l, puzzle | 0x49300000l, partsUsed | 0x20l);
    altpiece6at20at32:
      if (partsUsed & 0x40l) goto altpiece7at20at32;
      if(!(puzzle & 0x25900000l))
        altDoSolve32(21, 0x200000l, puzzle | 0x25900000l, partsUsed | 0x40l);
      if(!(puzzle & 0x2c900000l))
        altDoSolve32(21, 0x200000l, puzzle | 0x2c900000l, partsUsed | 0x40l);
    altpiece7at20at32:
      if (partsUsed & 0x80l) goto altpiece8at20at32;
      if(!(puzzle & 0x3d00000l))
        altDoSolve32(21, 0x200000l, puzzle | 0x3d00000l, partsUsed | 0x80l);
      if(!(puzzle & 0x2f00000l))
        altDoSolve32(21, 0x200000l, puzzle | 0x2f00000l, partsUsed | 0x80l);
      if(!(puzzle & 0xcb00000l))
        altDoSolve32(21, 0x200000l, puzzle | 0xcb00000l, partsUsed | 0x80l);
      if(!(puzzle & 0xd300000l))
        altDoSolve32(21, 0x200000l, puzzle | 0xd300000l, partsUsed | 0x80l);
    altpiece8at20at32:
      if (partsUsed & 0x200l) goto altpiece10at20at32;
      if(!(puzzle & 0xb300000l))
        altDoSolve32(21, 0x200000l, puzzle | 0xb300000l, partsUsed | 0x200l);
      if(!(puzzle & 0xb900000l))
        altDoSolve32(21, 0x200000l, puzzle | 0xb900000l, partsUsed | 0x200l);
    altpiece10at20at32:
      if (partsUsed & 0x400l) goto altpiece11at20at32;
      if (!(puzzle & 0x1300000l)) {
         if(!(puzzle & 0x3700000l))
           altDoSolve32(21, 0x200000l, puzzle | 0x3700000l, partsUsed | 0x400l);
         if(!(puzzle & 0x1f00000l))
           altDoSolve32(21, 0x200000l, puzzle | 0x1f00000l, partsUsed | 0x400l);
         if(!(puzzle & 0x5b00000l))
           altDoSolve32(21, 0x200000l, puzzle | 0x5b00000l, partsUsed | 0x400l);
         if(!(puzzle & 0x9b00000l))
           altDoSolve32(21, 0x200000l, puzzle | 0x9b00000l, partsUsed | 0x400l);
         if(!(puzzle & 0x3b00000l))
           altDoSolve32(21, 0x200000l, puzzle | 0x3b00000l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0xd900000l))
        altDoSolve32(21, 0x200000l, puzzle | 0xd900000l, partsUsed | 0x400l);
    altpiece11at20at32:
    break;
   case 21:
      if (partsUsed & 0x2l) goto altpiece2at21at32;
      if(!(puzzle & 0x5e00000l))
        altDoSolve32(22, 0x400000l, puzzle | 0x5e00000l, partsUsed | 0x2l);
    altpiece2at21at32:
      if (partsUsed & 0x4l) goto altpiece3at21at32;
      if(!(puzzle & 0x1d200000l))
        altDoSolve32(22, 0x400000l, puzzle | 0x1d200000l, partsUsed | 0x4l);
    altpiece3at21at32:
      if (partsUsed & 0x8l) goto altpiece4at21at32;
      if(!(puzzle & 0xd600000l))
        altDoSolve32(22, 0x400000l, puzzle | 0xd600000l, partsUsed | 0x8l);
    altpiece4at21at32:
      if (partsUsed & 0x10l) goto altpiece5at21at32;
      if (!(puzzle & 0x49200000l)) {
         if(!(puzzle & 0x69a00000l))
           altDoSolve32(22, 0x400000l, puzzle | 0x69a00000l, partsUsed | 0x10l);
         if(!(puzzle & 0xcb200000l))
           altDoSolve32(22, 0x400000l, puzzle | 0xcb200000l, partsUsed | 0x10l);
         if(!(puzzle & 0x59600000l))
           altDoSolve32(22, 0x400000l, puzzle | 0x59600000l, partsUsed | 0x10l);
      }
      if(!(puzzle & 0x9a600000l))
        altDoSolve32(22, 0x400000l, puzzle | 0x9a600000l, partsUsed | 0x10l);
    altpiece5at21at32:
      if (partsUsed & 0x20l) goto altpiece6at21at32;
      if (!(puzzle & 0x49200000l)) {
         if(!(puzzle & 0x69200000l))
           altDoSolve32(22, 0x400000l, puzzle | 0x69200000l, partsUsed | 0x20l);
         if(!(puzzle & 0x49600000l))
           altDoSolve32(22, 0x400000l, puzzle | 0x49600000l, partsUsed | 0x20l);
         if(!(puzzle & 0xc9200000l))
           altDoSolve32(22, 0x400000l, puzzle | 0xc9200000l, partsUsed | 0x20l);
      }
      if(!(puzzle & 0x92600000l))
        altDoSolve32(22, 0x400000l, puzzle | 0x92600000l, partsUsed | 0x20l);
    altpiece6at21at32:
      if (partsUsed & 0x40l) goto altpiece7at21at32;
      if (!(puzzle & 0x49200000l)) {
         if(!(puzzle & 0x49a00000l))
           altDoSolve32(22, 0x400000l, puzzle | 0x49a00000l, partsUsed | 0x40l);
         if(!(puzzle & 0x4d200000l))
           altDoSolve32(22, 0x400000l, puzzle | 0x4d200000l, partsUsed | 0x40l);
         if(!(puzzle & 0x4b200000l))
           altDoSolve32(22, 0x400000l, puzzle | 0x4b200000l, partsUsed | 0x40l);
         if(!(puzzle & 0x59200000l))
           altDoSolve32(22, 0x400000l, puzzle | 0x59200000l, partsUsed | 0x40l);
      }
    altpiece7at21at32:
      if (partsUsed & 0x80l) goto altpiece8at21at32;
      if(!(puzzle & 0x19600000l))
        altDoSolve32(22, 0x400000l, puzzle | 0x19600000l, partsUsed | 0x80l);
      if(!(puzzle & 0x1a600000l))
        altDoSolve32(22, 0x400000l, puzzle | 0x1a600000l, partsUsed | 0x80l);
    altpiece8at21at32:
      if (partsUsed & 0x200l) goto altpiece10at21at32;
      if (!(puzzle & 0x1a00000l)) {
         if(!(puzzle & 0x13a00000l))
           altDoSolve32(22, 0x400000l, puzzle | 0x13a00000l, partsUsed | 0x200l);
         if(!(puzzle & 0x7a00000l))
           altDoSolve32(22, 0x400000l, puzzle | 0x7a00000l, partsUsed | 0x200l);
         if(!(puzzle & 0x9e00000l))
           altDoSolve32(22, 0x400000l, puzzle | 0x9e00000l, partsUsed | 0x200l);
         if(!(puzzle & 0x19a00000l))
           altDoSolve32(22, 0x400000l, puzzle | 0x19a00000l, partsUsed | 0x200l);
      }
      if(!(puzzle & 0xf200000l))
        altDoSolve32(22, 0x400000l, puzzle | 0xf200000l, partsUsed | 0x200l);
    altpiece10at21at32:
      if (partsUsed & 0x400l) goto altpiece11at21at32;
      if (!(puzzle & 0x3200000l)) {
         if(!(puzzle & 0xb600000l))
           altDoSolve32(22, 0x400000l, puzzle | 0xb600000l, partsUsed | 0x400l);
         if(!(puzzle & 0x13600000l))
           altDoSolve32(22, 0x400000l, puzzle | 0x13600000l, partsUsed | 0x400l);
         if(!(puzzle & 0x1b200000l))
           altDoSolve32(22, 0x400000l, puzzle | 0x1b200000l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x3e00000l))
        altDoSolve32(22, 0x400000l, puzzle | 0x3e00000l, partsUsed | 0x400l);
      if(!(puzzle & 0xda00000l))
        altDoSolve32(22, 0x400000l, puzzle | 0xda00000l, partsUsed | 0x400l);
    altpiece11at21at32:
    break;
   case 22:
      if (partsUsed & 0x2l) goto altpiece2at22at32;
      if(!(puzzle & 0xf400000l))
        altDoSolve32(23, 0x800000l, puzzle | 0xf400000l, partsUsed | 0x2l);
    altpiece2at22at32:
      if (partsUsed & 0x4l) goto altpiece3at22at32;
      if(!(puzzle & 0x13c00000l))
        altDoSolve32(23, 0x800000l, puzzle | 0x13c00000l, partsUsed | 0x4l);
    altpiece3at22at32:
      if (partsUsed & 0x8l) goto altpiece4at22at32;
      if(!(puzzle & 0x7c00000l))
        altDoSolve32(23, 0x800000l, puzzle | 0x7c00000l, partsUsed | 0x8l);
    altpiece4at22at32:
      if (partsUsed & 0x10l) goto altpiece5at22at32;
      if(!(puzzle & 0xd3400000l))
        altDoSolve32(23, 0x800000l, puzzle | 0xd3400000l, partsUsed | 0x10l);
    altpiece5at22at32:
      if (partsUsed & 0x20l) goto altpiece6at22at32;
      if(!(puzzle & 0xd2400000l))
        altDoSolve32(23, 0x800000l, puzzle | 0xd2400000l, partsUsed | 0x20l);
    altpiece6at22at32:
      if (partsUsed & 0x40l) goto altpiece7at22at32;
      if(!(puzzle & 0x93400000l))
        altDoSolve32(23, 0x800000l, puzzle | 0x93400000l, partsUsed | 0x40l);
      if(!(puzzle & 0x9a400000l))
        altDoSolve32(23, 0x800000l, puzzle | 0x9a400000l, partsUsed | 0x40l);
    altpiece7at22at32:
      if (partsUsed & 0x200l) goto altpiece10at22at32;
      if(!(puzzle & 0xbc00000l))
        altDoSolve32(23, 0x800000l, puzzle | 0xbc00000l, partsUsed | 0x200l);
    altpiece10at22at32:
      if (partsUsed & 0x400l) goto altpiece11at22at32;
      if(!(puzzle & 0x1b400000l))
        altDoSolve32(23, 0x800000l, puzzle | 0x1b400000l, partsUsed | 0x400l);
    altpiece11at22at32:
    break;
   case 23:
      if (partsUsed & 0x2l) goto altpiece2at23at32;
      if(!(puzzle & 0x99800000l))
        altDoSolve32(24, 0x1000000l, puzzle | 0x99800000l, partsUsed | 0x2l);
      if(!(puzzle & 0xcc800000l))
        altDoSolve32(24, 0x1000000l, puzzle | 0xcc800000l, partsUsed | 0x2l);
    altpiece2at23at32:
      if (partsUsed & 0x4l) goto altpiece3at23at32;
      if(!(puzzle & 0x4b800000l))
        altDoSolve32(24, 0x1000000l, puzzle | 0x4b800000l, partsUsed | 0x4l);
      if(!(puzzle & 0x3c800000l))
        altDoSolve32(24, 0x1000000l, puzzle | 0x3c800000l, partsUsed | 0x4l);
    altpiece3at23at32:
      if (partsUsed & 0x8l) goto altpiece4at23at32;
      if(!(puzzle & 0xc9800000l))
        altDoSolve32(24, 0x1000000l, puzzle | 0xc9800000l, partsUsed | 0x8l);
      if(!(puzzle & 0x9c800000l))
        altDoSolve32(24, 0x1000000l, puzzle | 0x9c800000l, partsUsed | 0x8l);
    altpiece4at23at32:
      if (partsUsed & 0x80l) goto altpiece8at23at32;
      if(!(puzzle & 0x1e800000l))
        altDoSolve32(24, 0x1000000l, puzzle | 0x1e800000l, partsUsed | 0x80l);
      if(!(puzzle & 0x17800000l))
        altDoSolve32(24, 0x1000000l, puzzle | 0x17800000l, partsUsed | 0x80l);
      if(!(puzzle & 0x65800000l))
        altDoSolve32(24, 0x1000000l, puzzle | 0x65800000l, partsUsed | 0x80l);
      if(!(puzzle & 0x69800000l))
        altDoSolve32(24, 0x1000000l, puzzle | 0x69800000l, partsUsed | 0x80l);
    altpiece8at23at32:
      if (partsUsed & 0x200l) goto altpiece10at23at32;
      if(!(puzzle & 0x59800000l))
        altDoSolve32(24, 0x1000000l, puzzle | 0x59800000l, partsUsed | 0x200l);
      if(!(puzzle & 0x5c800000l))
        altDoSolve32(24, 0x1000000l, puzzle | 0x5c800000l, partsUsed | 0x200l);
    altpiece10at23at32:
      if (partsUsed & 0x400l) goto altpiece11at23at32;
      if (!(puzzle & 0x9800000l)) {
         if(!(puzzle & 0x1b800000l))
           altDoSolve32(24, 0x1000000l, puzzle | 0x1b800000l, partsUsed | 0x400l);
         if(!(puzzle & 0xf800000l))
           altDoSolve32(24, 0x1000000l, puzzle | 0xf800000l, partsUsed | 0x400l);
         if(!(puzzle & 0x2d800000l))
           altDoSolve32(24, 0x1000000l, puzzle | 0x2d800000l, partsUsed | 0x400l);
         if(!(puzzle & 0x4d800000l))
           altDoSolve32(24, 0x1000000l, puzzle | 0x4d800000l, partsUsed | 0x400l);
         if(!(puzzle & 0x1d800000l))
           altDoSolve32(24, 0x1000000l, puzzle | 0x1d800000l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x6c800000l))
        altDoSolve32(24, 0x1000000l, puzzle | 0x6c800000l, partsUsed | 0x400l);
    altpiece11at23at32:
    break;
   case 24:
      if (partsUsed & 0x2l) goto altpiece2at24at32;
      if(!(puzzle & 0x2f000000l))
        altDoSolve32(25, 0x2000000l, puzzle | 0x2f000000l, partsUsed | 0x2l);
    altpiece2at24at32:
      if (partsUsed & 0x4l) goto altpiece3at24at32;
      if(!(puzzle & 0xe9000000l))
        altDoSolve32(25, 0x2000000l, puzzle | 0xe9000000l, partsUsed | 0x4l);
    altpiece3at24at32:
      if (partsUsed & 0x8l) goto altpiece4at24at32;
      if(!(puzzle & 0x6b000000l))
        altDoSolve32(25, 0x2000000l, puzzle | 0x6b000000l, partsUsed | 0x8l);
    altpiece4at24at32:
      if (partsUsed & 0x80l) goto altpiece8at24at32;
      if(!(puzzle & 0xcb000000l))
        altDoSolve32(25, 0x2000000l, puzzle | 0xcb000000l, partsUsed | 0x80l);
      if(!(puzzle & 0xd3000000l))
        altDoSolve32(25, 0x2000000l, puzzle | 0xd3000000l, partsUsed | 0x80l);
    altpiece8at24at32:
      if (partsUsed & 0x200l) goto altpiece10at24at32;
      if (!(puzzle & 0xd000000l)) {
         if(!(puzzle & 0x9d000000l))
           altDoSolve32(25, 0x2000000l, puzzle | 0x9d000000l, partsUsed | 0x200l);
         if(!(puzzle & 0x3d000000l))
           altDoSolve32(25, 0x2000000l, puzzle | 0x3d000000l, partsUsed | 0x200l);
         if(!(puzzle & 0x4f000000l))
           altDoSolve32(25, 0x2000000l, puzzle | 0x4f000000l, partsUsed | 0x200l);
         if(!(puzzle & 0xcd000000l))
           altDoSolve32(25, 0x2000000l, puzzle | 0xcd000000l, partsUsed | 0x200l);
      }
      if(!(puzzle & 0x79000000l))
        altDoSolve32(25, 0x2000000l, puzzle | 0x79000000l, partsUsed | 0x200l);
    altpiece10at24at32:
      if (partsUsed & 0x400l) goto altpiece11at24at32;
      if (!(puzzle & 0x19000000l)) {
         if(!(puzzle & 0x5b000000l))
           altDoSolve32(25, 0x2000000l, puzzle | 0x5b000000l, partsUsed | 0x400l);
         if(!(puzzle & 0x9b000000l))
           altDoSolve32(25, 0x2000000l, puzzle | 0x9b000000l, partsUsed | 0x400l);
         if(!(puzzle & 0xd9000000l))
           altDoSolve32(25, 0x2000000l, puzzle | 0xd9000000l, partsUsed | 0x400l);
      }
      if(!(puzzle & 0x1f000000l))
        altDoSolve32(25, 0x2000000l, puzzle | 0x1f000000l, partsUsed | 0x400l);
      if(!(puzzle & 0x6d000000l))
        altDoSolve32(25, 0x2000000l, puzzle | 0x6d000000l, partsUsed | 0x400l);
    altpiece11at24at32:
    break;
   case 25:
      if (partsUsed & 0x2l) goto altpiece2at25at32;
      if(!(puzzle & 0x7a000000l))
        altDoSolve32(26, 0x4000000l, puzzle | 0x7a000000l, partsUsed | 0x2l);
    altpiece2at25at32:
      if (partsUsed & 0x4l) goto altpiece3at25at32;
      if(!(puzzle & 0x9e000000l))
        altDoSolve32(26, 0x4000000l, puzzle | 0x9e000000l, partsUsed | 0x4l);
    altpiece3at25at32:
      if (partsUsed & 0x8l) goto altpiece4at25at32;
      if(!(puzzle & 0x3e000000l))
        altDoSolve32(26, 0x4000000l, puzzle | 0x3e000000l, partsUsed | 0x8l);
    altpiece4at25at32:
      if (partsUsed & 0x200l) goto altpiece10at25at32;
      if(!(puzzle & 0x5e000000l))
        altDoSolve32(26, 0x4000000l, puzzle | 0x5e000000l, partsUsed | 0x200l);
    altpiece10at25at32:
      if (partsUsed & 0x400l) goto altpiece11at25at32;
      if(!(puzzle & 0xda000000l))
        altDoSolve32(26, 0x4000000l, puzzle | 0xda000000l, partsUsed | 0x400l);
    altpiece11at25at32:
    break;
   case 26:
      if (partsUsed & 0x80l) goto altpiece8at26at32;
      if(!(puzzle & 0xbc000000l))
        altDoSolve32(27, 0x8000000l, puzzle | 0xbc000000l, partsUsed | 0x80l);
      if(!(puzzle & 0xf4000000l))
        altDoSolve32(27, 0x8000000l, puzzle | 0xf4000000l, partsUsed | 0x80l);
    altpiece8at26at32:
      if (partsUsed & 0x400l) goto altpiece11at26at32;
      if (!(puzzle & 0x4c000000l)) {
         if(!(puzzle & 0xdc000000l))
           altDoSolve32(27, 0x8000000l, puzzle | 0xdc000000l, partsUsed | 0x400l);
         if(!(puzzle & 0x7c000000l))
           altDoSolve32(27, 0x8000000l, puzzle | 0x7c000000l, partsUsed | 0x400l);
         if(!(puzzle & 0xec000000l))
           altDoSolve32(27, 0x8000000l, puzzle | 0xec000000l, partsUsed | 0x400l);
      }
    altpiece11at26at32:
    break;
   case 27:
      if (partsUsed & 0x400l) goto altpiece11at27at32;
      if(!(puzzle & 0xf8000000l))
        altDoSolve32(28, 0x10000000l, puzzle | 0xf8000000l, partsUsed | 0x400l);
    altpiece11at27at32:
    break;
  }
}


